local attackBonusSettings = require("attackBonusSettings")
local civlua = require("civluaModified")
local func = require("functions")
local gen = require("generalLibrary")
local object = require("object")
local param = require("parameters")
local supplyLines = require("supplyLines")
local text = require("text")

local flag = require("flag")
local counter = require("counter")
local onTurn = require("onTurn")
local tables = require("tables")

-- local discreteEvents = require("discreteEvents")


local function round (decimal) --> integer
   return math.floor(decimal + 0.5)
end

-- ============================================================
-- moveUnitAdjacent TILE PREFERENCE FUNCTION ================ (by Prof. Garfield)
local function rankDestinations(unit,destTile)
    if (destTile.defender and destTile.defender ~=unit.owner) 
    or(destTile.city and destTile.city.owner ~= unit.owner) 
    or (not civ.canEnter(unit.type,destTile)) then
        return false
    end
    local southDifference = unit.location.y - destTile.y
    -- if destTile is further south than the unit.location,
    -- southDifferent will be negative, since south increases
    -- with y value.  Since smaller is better, this ranking
    -- prefers to move units south.
    return southDifference
end

-- FUNCTION USED IN LEADER ATTACK BONUSES (RANGED) SECTION BELOW (by Knighttime)
local function getTilesByDistance (tile, distance, includeOriginTile)
	local tileSet = { }
	for ycoord = tile.y - (distance * 2), tile.y + (distance * 2) do
		for xcoord = tile.x - (distance * 2), tile.x + (distance * 2) do
			local xdiff = math.abs(xcoord - tile.x)
			local ydiff = math.abs(ycoord - tile.y)
			if (xcoord + ycoord) % 2 == 0 and (xdiff + ydiff) <= (distance * 2) then
				local thisTile = civ.getTile(xcoord, ycoord, tile.z)
				if thisTile ~= nil and ((xdiff == 0 and ydiff == 0 and includeOriginTile) or (not(xdiff == 0 and ydiff == 0))) then
					table.insert(tileSet, thisTile)
				end
			end
		end
	end
	return tileSet
end

-- ===== SUPPLY LINES: =====
-- [[ Originally by Knighttime ]]
local baseUnitAttack = {}
for i = 0, civ.cosmic.numberOfUnitTypes - 1 do
	baseUnitAttack[i] = civ.getUnitType(i).attack
end
-- =========================


local unitAttackBonusDesc = { }

-- This function is provided to the helpkey module and will appear onscreen as notes for the corresponding unit
-- local function customUnitTextFunction (unit)
	-- local leaderBonusText = ""
	-- if unit.type.attack > 0 then
		-- leaderBonusText = "No leader attack bonus active."
		-- if unit.type.attack ~= baseUnitAttack[unit.type.id] then
			-- if unitAttackBonusDesc[unit.type.id] ~= nil and unitAttackBonusDesc[unit.type.id] ~= { } then
				-- leaderBonusText = unitAttackBonusDesc[unit.type.id]
			-- end
		-- end
	-- end
	-- return leaderBonusText
-- end


-- This file can be deleted if it is not used.
local register = {}

---&autoDoc onActivateUnit
--Registers a function to be called every time a unit is activated. The callback takes the unit activated as a parameter, and the source of unit activation. `source` is `true` if activated by keyboard or mouse click, `false` if activated by the game itself. `repeatMove` is `true` if it's a repeat activation caused by moving, `false` otherwise.
--If the function returns true, the unit activation will be cancelled.
--No further activation code will be executed, and the unit's type
--will temporarily be set to have 0 movement points.
--If the function returns function(unit), then the unit activation
--will be cancelled, and the function returned will be executed
--just before another unit is activated.  (You may wish to put
--the unit to sleep, for example.)
--Not returning anything is equivalent to returning nil, which is
--acceptable, and keeps the unit activation going.
function register.onActivateUnit(unit, source, repeatMove)
    -- civ.ui.text("unit activation separate file")
	
	
	-- ===== SUPPLY LINES: =====
	-- [[ Originally by Knighttime ]]
	-- Reset attack values of all unit types, to revert any changes made to the *previously* active unit:
	for i = 0, civ.cosmic.numberOfUnitTypes - 1 do
		civ.getUnitType(i).attack = baseUnitAttack[i]
	end
	attackBonusSettings.attackBonus(unit)
	local supplyStatus = supplyLines.getSupplyStatus(unit)
	-- Based on supply condition, determine message and adjust attack value if necessary:
	local suppliedText = "^Supply status determined from onActivateUnit\n^\n^" .. unit.owner.adjective .. " " .. unit.type.name .. " at " .. unit.x .. "," .. unit.y .. "," .. unit.z .. ": "
	if supplyStatus == nil then
		suppliedText = suppliedText .. "Not calculated."
	elseif supplyStatus == true then
		suppliedText = suppliedText .. "Supplied.\n^Unit has full attack strength."
	elseif supplyStatus == false then
		suppliedText = suppliedText .. "NOT SUPPLIED.\n^Unit attack strength reduced by 50%."
		-- unit.type.attack = math.ceil(unit.type.attack * 0.5)
		unit.type.attack = round(unit.type.attack * 0.5)
		if repeatMove == nil or repeatMove == false then
			if flag.value("Display Out of Supply Pop-Ups") == true then
				-- civ.ui.text("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at ".. tostring(text.coordinates(unit.location)) .. " has been cut off from supply! Its attack strength has been reduced by 50% while in unsupplied condition.")
				-- civ.ui.text("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at ".. tostring(text.coordinates(unit.location)) .. " is cut off from a supply source! Its attack strength will be reduced by 50% in any attack and its movement may come to an abrupt halt whenever attempting to move to a new location as long as it remains in an unsupplied condition!")
				if tables.alliedArtillery[unit.type.id] then
					-- text.simple("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at (".. tostring(text.coordinates(unit.location)) .. ") is cut off from a supply source! Its attack strength will be reduced by 50% in any attack, unless an artillery unit in which case it will be unable to generate munitions, and its movement may come to an abrupt end whenever attempting to relocate to a new position, and this as long as it remains in an unsupplied condition!","UNIT UNSUPPLIED!",object.monexone)
					text.simple("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at (".. tostring(text.coordinates(unit.location)) .. ") is cut off from a supply source! As an artillery unit it will be unable to generate munitions and its movement may come to an abrupt end whenever attempting to relocate to a new position, and this as long as it remains in an unsupplied condition!","UNIT UNSUPPLIED!",object.monexone)
				else
					text.simple("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at (".. tostring(text.coordinates(unit.location)) .. ") is cut off from a supply source! Its attack strength will be reduced by 50% in any attack and its movement may come to an abrupt end whenever attempting to relocate to a new position, and this as long as it remains in an unsupplied condition!","UNIT UNSUPPLIED!",object.monexone)
				end
			end
		end
	end
	print(suppliedText)
	if supplyLines.SHOW_SUPPLY_LINE_KEYCODE ~= nil and supplyStatus ~= nil then
		suppliedText = suppliedText .. "\n^\n^After closing this dialog box, press " .. supplyLines.SHOW_SUPPLY_LINE_KEYLABEL .. " to toggle a visual representation of supply line results."
	end

	-- Output message to the user:
	--	if unit.owner.isHuman then
	--		civ.ui.text(func.splitlines(suppliedText))
	--	end

	-- =========================
	-- GAME TIPS: APPEAR JUST ONCE THE FIRST TIME YOU ACTIVATE A UNIT TYPE WITH SPECIAL ABILITIES
	
	if flag.value("Display Game Tip Pop-Ups") == true then -- Players can turn In-Game Tip Pop-Ups on or off through the 'SCENARIO RULES AND GAME TIPS' menus (see gameTips.lua)
		-- ALLIED BOMBERS START UP TIPS
		if civ.getTurn() >= 2 then
			if unit.type == object.uB25Mitchell or unit.type == object.uB26Marauder or unit.type == object.uMartinBaltimore or unit.type == object.uBostonIII then
				gen.justOnce("Allied Bombers",function()
					text.simple(object.AlliedBombers,"TIP: Allied Bombers",object.mB24Bomber)
				end)
			end		
		end

		-- ALLIED FIGHTER-BOMBERS START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uA36Apache or unit.type == object.uHurricaneIIc then
				gen.justOnce("Allied Fighter-Bombers",function()
					text.simple(object.AlliedFighterBombers,"TIP: Allied Fighter-Bombers",object.mHawkerHurricaneIIc)
				end)
			end		
		end		

		-- ALLIED GENERALS START UP TIPS
		if civ.getTurn() >= 2 then
			gen.justOnce("Allied Generals",function()
				if unit.type == object.uGenMontgomery then
					text.simple(object.AlliedGenerals,"TIP: Allied Generals",object.mGenMontgomery)
				elseif unit.type == object.uGenPatton then
					text.simple(object.AlliedGenerals,"TIP: Allied Generals",object.mGeorgePatton)
				end	
			end)	
		end
		
		-- C-47 DAKOTA START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uC47Dakota then
				gen.justOnce("C-47 Dakota",function()
					text.simple(object.C47Dakota,"TIP: C-47 Dakota",object.mC47Dakota)
				end)
			end		
		end

		-- DUKW START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uDUKW then
				gen.justOnce("DUKW",function()
					text.simple(object.DUKW,"TIP: DUKW",object.mDUKWBeachead)
				end)
			end		
		end		
		
		-- HMS ILLUSTRIOUS CV CLASS START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uIllustriousClassCV then
				gen.justOnce("HMS Illustrious",function()
					text.simple(object.HMSIllustrious,"TIP: HMS Illustrious CV Class",object.mHMSIllustrious)
				end)
			end		
		end
		
		-- LANDING CRAFT START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uLandingCraft then
				gen.justOnce("Landing Craft",function()
					text.simple(object.LandingCraft,"TIP: Landing Craft",object.mLandingCraft)
				end)
			end		
		end

		-- LIBERTY SHIP START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uLibertyShip then
				gen.justOnce("Liberty Ship",function()
					text.simple(object.LibertyShip,"TIP: Liberty Ship",object.mLibertyShip)
				end)
			end		
		end
		
		-- MOUNTAIN UNIT START UP TIPS
		if civ.getTurn() >= 2 then
			if unit.type == object.uGoumierMarocain then
				gen.justOnce("Mountain units",function()
					text.simple(object.MountainUnits,"TIP: Mountain units",object.mGoumierMarocain)
				end)
			end		
		end

		-- OIL TANKER (F) UNIT START UP TIPS
		-- if civ.getTurn() >= 1 then
			-- if unit.type == object.uOilTankerF then
				-- gen.justOnce("Oil Tanker units",function()
					-- text.simple(object.oilTankers,"TIP: Oil Tanker units",object.mResupplyAtSea)
				-- end)
			-- end		
		-- end

		-- RECONNAISSANCE START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uM8Greyhound or unit.type == object.uDaimlerAC or unit.type == object.uP38Lightning or unit.type == object.uBeaufighterMkVI then
				gen.justOnce("Allied Reconnaissance",function()
					text.simple(object.reconnaissance,"TIP: Reconnaissance",object.mM8Greyhound)
				end)
			end		
		end	
		
		-- SUPPLY TRUCKS START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uUKTruck or unit.type == object.uUSTruck then
				gen.justOnce("Allied Supply Trucks",function()
					text.setLinesPerWindow(25)
						text.simple(object.supplyTrucks,"TIP: Allied Supply Trucks",object.mTrucks)
					text.setLinesPerWindow(12)	
				end)
			end		
		end	
		
		-- WACO GLIDER START UP TIPS
		if civ.getTurn() >= 1 then
			if unit.type == object.uWacoGlider then
				gen.justOnce("Waco Gliders",function()
					text.setLinesPerWindow(25)
						text.simple(object.WacoGliders,"TIP: Waco Gliders",object.mWacoGliders)
					text.setLinesPerWindow(12)	
				end)
			end		
		end

	end	
	-- END of GAME TIPS SECTION ================

	-- ===========================
	-- SEASONAL EFFECTS ON MOVEMENT (idea from HoF, barbarossa.lua, by John Petrovski)
	
	-- FALL/SPRING SEASON
	if  flag.value("FallSpring Months") == true then
		-- ALLIED AIR UNITS CAN BECOME GROUNDED DURING FALL/SPRING TURNS WHEN TRYING TO LIFT OFF EITHER FROM CITIES OR AIRBASES
		if unit.owner == object.pAllies
		and unit.type.domain == 1 
		and not tables.munitionsToBeDestroyed[unit.type.id] 
		and (unit.location.baseTerrain == object.bTownCenter or gen.hasAirbase(unit.location)) then
			if math.random(4) == 1 then 	-- "1" is arbitrary, we want a 25% chance or 1 in 4	
				civ.ui.text("Our " ..tostring(unit.type.name).. " is grounded thanks to poor weather! This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
				unit.moveSpent = 255
			end
		end	

		-- WACO GLIDERS CANNOT OPERATE DURING FALL/SPRING MONTHS
		if unit.owner == object.pAllies and (unit.location.baseTerrain == object.bTownCenter or (gen.hasAirbase(unit.location))) then
			if unit.type == object.uWacoGlider then
				civ.ui.text(tostring(unit.type.name).. " units cannot operate during winter months! This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
				unit.moveSpent = 255
			end
		end 
		
		-- ALLIED MECHANIZED GROUND UNITS CAN BECOME STUCK IN MUD DURING FALL/SPRING TURNS (not in cities, on highways, in Sardinia or Sicily or Southern Italy)
		-- SEE onEnter.lua file instead for Mechanized Penalties Effects
		-- if unit.owner == object.pAllies -- and unit.type.domain == 0  
		-- and tables.mechanizedSeasonalTerrainPenalties[unit.type.id]
		-- then
			-- if math.random(1,9) == 9 -- from 6
			-- and not unit.location.city 
			-- and not gen.hasRailroad(unit.location) 
			-- and unit.location.baseTerrain ~= object.bOcean
			-- and unit.location.baseTerrain ~= object.ltransportPoint 			
			-- -- and not tables.mechanizedSeasonalTerrainPenalties[unit.type.id] -- Only mechanized ground units
			-- and not tables.excludeFromSeasonalTerrainPenalties[unit.type.id]
			-- and not gen.inPolygon(unit.location,tables.sardiniaPolygon)
			-- and not gen.inPolygon(unit.location,tables.sicilyPolygon)
			-- and not gen.inPolygon(unit.location,tables.southernItalyPolygon)
			-- then -- "1" is arbitrary, we want a 20% chance or 1 in 5	
				-- civ.ui.text("Our " ..tostring(unit.type.name).. " is mirred in the mud thanks to the season! This unit is stuck in place at " .. unit.x .. "," .. unit.y .. "," .. unit.z .. "!")
				-- unit.moveSpent = 255
			-- end
		-- end 
		
		-- Engineers cannot build pontoon bridges during Fall/Spring (see consolidatedEvents.lua, engineers key 'shift.one')
	
	-- WINTER SEASON
	elseif flag.value("Winter Months") == true then
		-- ALLIED AIR UNITS CAN BECOME GROUNDED DURING WINTER TURNS WHEN TRYING TO LIFT OFF EITHER FROM CITIES OR AIRBASES
		if unit.owner == object.pAllies 
		and unit.type.domain == 1
		and not tables.munitionsToBeDestroyed[unit.type.id] 
		and (unit.location.baseTerrain == object.bTownCenter or (gen.hasAirbase(unit.location))) then
			if math.random(3) == 1 then -- "1" is arbitrary, we want a 33% chance or 1 in 3	
				civ.ui.text("Our " ..tostring(unit.type.name).. " is grounded thanks to poor winter weather! This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
				unit.moveSpent = 255
			end
		end

		-- WACO GLIDERS CANNOT OPERATE DURING WINTER MONTHS
		if unit.owner == object.pAllies and (unit.location.baseTerrain == object.bTownCenter or (gen.hasAirbase(unit.location))) then
			if unit.type == object.uWacoGlider then
				civ.ui.text(tostring(unit.type.name).. " units cannot operate during winter months! This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
				unit.moveSpent = 255
			end
		end

		-- ALLIED NAVAL UNITS CAN BECOME DAMAGED DURING WINTER TURNS IF MOVING AT SEA
		if unit.owner == object.pAllies and unit.location.baseTerrain == object.bOcean then
			if unit.type.domain == 2 and math.random(30) == 1 then 	-- "1" is arbitrary, we want a 3.33% chance or 1 in 30	
				civ.ui.text("Our " ..tostring(unit.type.name).. " suffered damage due to poor winter weather! Naval units are always safer in harbor during winter months!")
				unit.damage = math.min(unit.damage + param.navalMPWinterPenalty,unit.type.hitpoints-1)
			end
		end
		
		-- ALLIED MECHANIZED GROUND UNITS CAN BECOME STUCK IN SNOW DURING WINTER TURNS (not in cities, in Sardinia or Sicily or Southern Italy)
		-- SEE onEnter.lua file instead for Mechanized Penalties Effects
		-- if unit.owner == object.pAllies -- and unit.type.domain == 0  
		-- and tables.mechanizedSeasonalTerrainPenalties[unit.type.id]
		-- then 
			-- if math.random(1,12) == 12 -- from 8
			-- and not unit.location.city 
			-- and not gen.hasRoad(unit.location) 
			-- and not gen.hasRailroad(unit.location) 
			-- and unit.location.baseTerrain ~= object.bOcean	
			-- and unit.location.baseTerrain ~= object.ltransportPoint
			-- -- and tables.mechanizedSeasonalTerrainPenalties[unit.type.id] -- Only mechanized ground units
			-- and not tables.excludeFromSeasonalTerrainPenalties[unit.type.id] 
			-- and not gen.inPolygon(unit.location,tables.sardiniaPolygon)
			-- and not gen.inPolygon(unit.location,tables.sicilyPolygon)
			-- and not gen.inPolygon(unit.location,tables.southernItalyPolygon)
			-- then -- "1" is arbitrary, we want a 16% chance or 1 in 6	
				-- civ.ui.text("Our " ..tostring(unit.type.name).. " is stuck in the snow thanks to the season!' This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
				-- unit.moveSpent = 255
			-- end
		-- end 

		-- ATTRITION EFFECTS DUE TO COLD (OPTIONAL FOR EASTERN FRONT TYPE SCENARIOS)
		-- if (unit.type == object.uSchutzen or unit.type == object.uPanzergrenadier or unit.type == object.uFallschirm or unit.type == object.uATgun or unit.type == object.uRomanianInfantry or unit.type == object.uHungarianInfantry or unit.type == object.uBulgarianInfantry or unit.type == object.uSpanishInfantry or unit.type == object.uTurkishInfantry or unit.type == object.uCroatianInfantry) and unit.damage < 15 and math.random(1,4) == 4 and not civ.hasTech(object.tGermany, object.aWinterClothing) and not unit.location.city then 
			-- civ.ui.text("Our " ..tostring(unit.type.name).. " at " .. unit.x .. "," .. unit.y .. "," .. unit.z .. " has suffered attrition due to the cold!")
			-- unit.damage = unit.damage + 5
		-- end 
	
	end 
	-- END of SEASON EFFECTS ================
	
	-- FOR TESTING PURPOSES ONLY
	-- if unit.owner == object.pAxis and (unit.type == object.uPzkwVG or unit.type == object.uPzkwIVF or unit.type == object.uFanteriaVeterana) then 
		-- unit.moveSpent = unit.type.move
		-- unit.moveSpent = 255
	-- end
	
	-- =========================
	-- 1ST UK AIRBORNE AND 82ND US AIRBORNE UNITS CAN SUFFER DAMAGE DURING INITIAL SICILIAN AIR LANDINGS
	if civ.getTurn() <= 5 then
		if unit.owner == object.pAllies then
			if unit.type == object.u1stAirborne and (gen.inPolygon(unit.location,tables.sicilyPolygon)) and (gen.unitTypeOnTile(unit.location,object.uWacoGlider)) and unit.location.baseTerrain ~= object.bTownCenter then
				local randomNumber = math.random(1,3)
				if randomNumber == 1 then
						civ.playSound("AA.wav")
						text.simple("The British paratroopers meet fierce and unexpected friendly and enemy AA fire on their way to their drop zones and sustain heavy casualties!","Paratroopers Sustain Heavy Casualties!",object.monexone)
						unit.damage = unit.damage + 5
				elseif randomNumber == 2 then
						civ.playSound("AA.wav")
						text.simple("The British paratroopers meet mild resistance near their drop zones and sustained minimal casualties! They send out patrols which gather valuable intelligence that will help local commanders battle plans. 10 supplies are added to your theater depots!","Paratroopers Sustain Light Casualties!",object.monexone)
						unit.damage = unit.damage + 2
						object.pAllies.money = object.pAllies.money + 10
				elseif randomNumber == 3 then	
						text.simple("The British paratroopers meet light resistance and sustain no significant casualties! They send out aggressive patrols which gather valuable intelligence that will help shorten the battle for the island. 30 supplies are added to your theater depots!","Paratroopers Sustain No Casualties!",object.monexone)
						object.pAllies.money = object.pAllies.money + 30
				end
			end
		end 	
	end	

	if civ.getTurn() <= 5 then
		if unit.owner == object.pAllies then
			if unit.type == object.u82ndAirborne and (gen.inPolygon(unit.location,tables.sicilyPolygon)) and (gen.unitTypeOnTile(unit.location,object.uWacoGlider)) and unit.location.baseTerrain ~= object.bTownCenter then
				local randomNumber = math.random(1,3)
				if randomNumber == 1 then
					civ.playSound("AA.wav")
					text.simple("The American airborne troops meet fierce and unexpected friendly and enemy AA fire on their way to their drop zones and sustain heavy casualties!","Airborne Sustain Heavy Casualties!",object.monexone)
					unit.damage = unit.damage + 5
				elseif randomNumber == 2 then
					civ.playSound("AA.wav")
					text.simple("The American airborne troops meet mild resistance near their drop zones and sustained minimal casualties! They send out patrols which gather valuable intelligence that will help local commanders battle plans. 10 supplies are added to your theater depots!","Airborne Sustain Light Casualties!",object.monexone)
					unit.damage = unit.damage + 2
					object.pAllies.money = object.pAllies.money + 10
				elseif randomNumber == 3 then	
					text.simple("The American airborne troops meet light resistance and sustain no significant casualties! They send out aggressive patrols which gather valuable intelligence that will help shorten the battle for the island. 30 supplies are added to your theater depots!","Airborne Sustain No Casualties!",object.monexone)
					object.pAllies.money = object.pAllies.money + 30
				end
			end
		end 	
	end	
	
	-- =========================
	-- IF TRIESTE FELL TO COMMUNISTS TELEPORT ANY ALLIED UNIT OUT OF TRIESTE ZONE
	if unit.owner == object.pAllies and unit.type.domain == 0 and flag.value("Trieste Falls To Communists") == true and gen.inPolygon(unit.location,tables.triestePolygon) then
		civ.teleportUnit(unit, object.lGrassNNTrieste)
		text.simple("Your " ..tostring(unit.type.name).. " unit is located within Marshall Tito's Yugoslavian communist controlled zone and is forced to retreat north (" .. text.coordinates(unit.location) .. ")!","Allied Unit Forced to Retreat!",object.monexone)
	end	
	
	-- =========================
	-- DELETING ALLIED INFANTRY/ARTILLERY TYPE UNITS THAT WEREN'T WITDRAWN AS PER SCHEDULE
	-- Reason: It's possible that some units may not have been withdrawn because they were being tranported by trucks at the time
	-- See also onTribeBeginTurn.lua -- PARTIAL UNIT.TYPE DELETION FUNCTION section where I prevent withdrawing units from the 'unit.location ~= object.ltransportPoint' because this could cause an error when trying to unload the truck afterwards

	if civ.getTurn() >= 19 then
	-- UNITED STATES UNITS
		-- GI Veteran
		if unit.type == object.uGIVeteran then
			local GIVeteran = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uGIVeteran then
					GIVeteran = GIVeteran + 1
				end
			end	
			if 	GIVeteran > counter.value("# GI Veteran") then			
				civ.ui.text("The " .. object.uGIVeteran.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end	
		
		-- GI Regular
		if unit.type == object.uGIRegular then
			local GIRegular = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uGIRegular then
					GIRegular = GIRegular + 1
				end
			end	
			if 	GIRegular > counter.value("# GI Regular") then			
				civ.ui.text("The " .. object.uGIRegular.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
		
		-- US Ranger
		if unit.type == object.uRanger then
			local usRanger = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uRanger then
					usRanger = usRanger + 1
				end
			end	
			if 	usRanger > counter.value("# US Ranger") then			
				civ.ui.text("The " .. object.uRanger.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end
		end

		-- US 82nd Airborne
		if unit.type == object.u82ndAirborne then
			local us82ndAirborne = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.u82ndAirborne then
					us82ndAirborne = us82ndAirborne + 1
				end
			end	
			if 	us82ndAirborne > counter.value("# 82nd Airborne") then			
				civ.ui.text("The " .. object.u82ndAirborne.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
		
		-- US 442nd REGT
		if unit.type == object.uUS442ndRegt then
			local us442nd = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uUS442ndRegt then
					us442nd = us442nd + 1
				end
			end	
			if 	us442nd > counter.value("# US 442nd Regt") then			
				civ.ui.text("The " .. object.uUS442ndRegt.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end
		end		

		-- US M2A1 105mm Art
		if unit.type == object.uM2A1105mmArt then
			local m2A1105mmArt = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM2A1105mmArt then
					m2A1105mmArt = m2A1105mmArt + 1
				end
			end	
			if 	m2A1105mmArt > counter.value("# M2A1 105mm Art") then			
				civ.ui.text("The " .. object.uM2A1105mmArt.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end	
		
		-- US M1 155mm Art
		if unit.type == object.uM1155mmArt then
			local m1155mmArt = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM1155mmArt then
					m1155mmArt = m1155mmArt + 1
				end
			end	
			if 	m1155mmArt > counter.value("# M1 155mm Art") then			
				civ.ui.text("The " .. object.uM1155mmArt.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end		
		end

		-- US 57mm AT
		if unit.type == object.uM157mmAT then
			local m157mmAT = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM157mmAT then
					m157mmAT = m157mmAT + 1
				end
			end	
			if 	m157mmAT > counter.value("# M1 57mm AT") then			
				civ.ui.text("The " .. object.uM157mmAT.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end			
		end

	-- BRITISH UNITS
		-- British Veteran
		if unit.type == object.uBritishVeteran then
			local BritishVeteran = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishVeteran then
					BritishVeteran = BritishVeteran + 1
				end
			end	
			if 	BritishVeteran > counter.value("# British Veteran") then			
				civ.ui.text("The " .. object.uBritishVeteran.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
		
		-- British Regular
		if unit.type == object.uBritishRegular then
			local BritishRegular = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishRegular then
					BritishRegular = BritishRegular + 1
				end
			end	
			if 	BritishRegular > counter.value("# British Regular") then			
				civ.ui.text("The " .. object.uBritishRegular.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
			
		-- British MG
		if unit.type == object.uBritishMG then	
			local BritishMG = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishMG then
					BritishMG = BritishMG + 1
				end
			end	
			if 	BritishMG > counter.value("# British MG") then			
				civ.ui.text("The " .. object.uBritishMG.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end

		-- British PIAT
		if unit.type == object.uBritishPIAT then
			local BritishPIAT = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishPIAT then
					BritishPIAT = BritishPIAT + 1
				end
			end	
			if 	BritishPIAT > counter.value("# British PIAT") then			
				civ.ui.text("The " .. object.uBritishPIAT.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
		
		-- UK 1st Airborne
		if unit.type == object.u1stAirborne then
			local uk1stAirborne = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.u1stAirborne then
					uk1stAirborne = uk1stAirborne + 1
				end
			end	
			if 	uk1stAirborne > counter.value("# 1st Airborne") then			
				civ.ui.text("The " .. object.u1stAirborne.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
		
		-- UK 25pdr MKII Art
		if unit.type == object.u25pdrMKIIArt then
			local uk25pdrMKIIArt = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.u25pdrMKIIArt then
					uk25pdrMKIIArt = uk25pdrMKIIArt + 1
				end
			end	
			if 	uk25pdrMKIIArt > counter.value("# 25pdr MKII Art") then			
				civ.ui.text("The " .. object.u25pdrMKIIArt.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end			
		end
		
		-- UK 55 inch Art
		if unit.type == object.u55inchArt then
			local uk55inchArt = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.u55inchArt then
					uk55inchArt = uk55inchArt + 1
				end
			end	
			if 	uk55inchArt > counter.value("# 55inch Art") then			
				civ.ui.text("The " .. object.u55inchArt.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
		
		-- UK 6pdr AT
		if unit.type == object.u6pdrAT then
			local uk6pdrAT = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.u6pdrAT then
					uk6pdrAT = uk6pdrAT + 1
				end
			end	
			if 	uk6pdrAT > counter.value("# 6pdr AT") then			
				civ.ui.text("The " .. object.u6pdrAT.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end

	-- COMMONWEALTH UNITS
		-- 4th Indian Division
		if unit.type == object.u4thIndianDiv then
			local cw4thIndianDiv = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.u4thIndianDiv then
					cw4thIndianDiv = cw4thIndianDiv + 1
				end
			end	
			if 	cw4thIndianDiv > counter.value("# 4th Indian Div") then			
				civ.ui.text("The " .. object.u4thIndianDiv.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end

		-- Canadian Infantry
		if unit.type == object.uCdnInfantry then
			local cwCdnInfantry = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uCdnInfantry then
					cwCdnInfantry = cwCdnInfantry + 1
				end
			end	
			if 	cwCdnInfantry > counter.value("# Cdn Infantry") then			
				civ.ui.text("The " .. object.uCdnInfantry.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end

	-- FREE FRENCH UNITS		
		-- Infanterie Coloniale
		if unit.type == object.uInfanterieColoniale then
			local ffInfanterieColoniale = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uInfanterieColoniale then
					ffInfanterieColoniale = ffInfanterieColoniale + 1
				end
			end	
			if 	ffInfanterieColoniale > counter.value("# Infanterie Coloniale") then			
				civ.ui.text("The " .. object.uInfanterieColoniale.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end
			
		-- Tirailleur Algerien
		if unit.type == object.uTirailleurAlgerien then
			local ffTirailleurAlgerien = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uTirailleurAlgerien then
					ffTirailleurAlgerien = ffTirailleurAlgerien + 1
				end
			end	
			if 	ffTirailleurAlgerien > counter.value("# Tirailleur Algerien") then			
				civ.ui.text("The " .. object.uTirailleurAlgerien.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end			
		end

		-- Goumier Marocain
		if unit.type == object.uGoumierMarocain then
			local ffGoumierMarocain = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uGoumierMarocain then
					ffGoumierMarocain = ffGoumierMarocain + 1
				end
			end	
			if 	ffGoumierMarocain > counter.value("# Goumier Marocain") then			
				civ.ui.text("The " .. object.uGoumierMarocain.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end	
		
		-- Free French 105Art
		if unit.type == object.uFreeFrench105Art then
			local ff105Art = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uFreeFrench105Art then
					ff105Art = ff105Art + 1
				end
			end	
			if 	ff105Art > counter.value("# Free French 105Art") then			
				civ.ui.text("The " .. object.uFreeFrench105Art.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end

		-- USN 
		if unit.type == object.uLibertyShip then
			local usLibertyShip = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uLibertyShip then
					usLibertyShip = usLibertyShip + 1
				end
			end	
			if 	usLibertyShip > counter.value("# Liberty Ship") and unit.location.baseTerrain == object.bTownCenter then			
				civ.ui.text("The " .. object.uLibertyShip.name .." located at (" .. text.coordinates(unit.location) ..") was belatedly withdrawn from the theater as part of a previously scheduled departured!")
				civ.deleteUnit(unit)
			end	
		end	

	end	

	-- =========================
	-- PREVENT ALLIED PLAYER FROM STACKING MORE THAN 4 ARTILLERY PIECES IN FIRENZE
	-- REASON: to maintain play balance and prevent the human player from stacking a bunch of artillery   
	-- units in the city to pommel surrounding German defensive positions from a safe place

	if tables.alliedArtillery[unit.type.id] then
		local artilleryCount = 0
		for unit in civ.iterateUnits() do 
			if tables.alliedArtillery[unit.type.id] and unit.location == object.lFirenze then
				artilleryCount = artilleryCount + 1
			end
		end	
		if artilleryCount > 4 then			
			text.simple("The " .. unit.type.name .." located in Firenze exceeds the allowable artillery limit of 4 for the city and thus was relocated to an adjacent tile!","Overstacked Artillery in Firenze!",object.monexone)
			gen.moveUnitAdjacent(unit)
			-- gen.moveUnitAdjacent(unit,rankDestinations)
		end	
	end	
	
	-- =========================
	-- ALLIED GROUND UNITS SUFFER CASUALTIES WHILE TRANSITING ON MT. ETNA	
	if unit.owner == object.pAllies and (unit.location.x == 63 and unit.location.y == 157 and unit.location.z == 0) and math.random(1) == 2 then
		civ.ui.text("This unit suffered casualties while attempting to cross over the difficult terrain surrounding Mt. Etna!")
		-- unit.damage = unit.damage + 2
		unit.damage = math.min(unit.damage + 2,unit.type.hitpoints-1)
	end 

	-- ITALIAN PARTISAN UNITS RENDERED INACTIVE BY POLITICAL DISPUTES
	if unit.type == object.uPartigiani and flag.value("Italian CLN Partisans Activated") == true and (not gen.inPolygon(unit.location,tables.corsicaPolygon)) then
		if math.random(4) == 1 then 	-- "4" is arbitrary, we want a 25% chance or 1 in 4	
			civ.ui.text("The leaders of this " ..tostring(unit.type.name).. " unit are mirred in political disputes and unable to reach a consensus on their next actions!' The unit remains inactive at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
			unit.moveSpent = 255
		end
	end	
	
	-- MAKE SURE WHEN CONVERTING LANDING CRAFT TO BEACHHEAD THAT DUKW ISN'T HOMED TO A CITY
	if unit.type == object.uDUKW and unit.homeCity ~= nil then
		unit.homeCity = nil
	end	
	
	-- =========================
	-- FREEZE ITALIAN UNITS MOVEMENT SECTION
	
	-- PREVENT ITALIAN NAVAL UNITS STATIONED IN LA SPEZIA FROM MOVING OUT OF CITY PRIOR TO ITALY SURRENDERING
	-- Reason: To allow	Germans a chance of capturing Italian naval units when Italy surrenders to the Allies (see onTurn.lua "ITALIAN SURRENDER CONDITIONS" section)
	-- if civ.getTurn() <= 18 and flag.value("Italy Surrendered") == false then
		-- if unit.owner == object.pAxis and unit.type.domain == 2 and unit.location == object.lLaSpezia then
			-- -- civ.ui.text("Naval units in La Spezia frozen!")
			-- unit.moveSpent = 255
		-- end

	-- end 
	
	if civ.getTurn() >= 18 and flag.value("Italy Surrendered") == true then
		gen.justOnce("Regia Marina La Spezia Squadron moves",function()
			for unit in civ.iterateUnits() do 
				if unit.owner == object.pAxis and unit.type.domain == 2 and unit.location == object.lLaSpezia then
					civ.teleportUnit(unit, object.lsealane)
				end	
			end
			for unit in civ.iterateUnits() do 
				if unit.owner == object.pAxis and unit.type.domain == 2 and (unit.location.x >= 72 and unit.location.y >= 36 and unit.location.z == 0) then
					unit.gotoTile = civ.getTile(70,54,0) 
				end		
			end
			if flag.value("Display Axis Testing Messages") == true then
				civ.ui.text("Naval units in La Spezia on the move!")
			end
		end)	
	end 
	
	-- PREVENT ALL ITALIAN UNITS FROM MOVING AFTER SICILY CONQUERED
	-- Reason: to represent the ongoing armistice talks between the Italian Royalists and the Allies
	if flag.value("Italy Surrendered") == true and flag.value("Republic of Salo Created") == false then
		if unit.owner == object.pAxis and tables.italian_VivaIlRe[unit.type.id] then
			-- civ.ui.text("Naval units in La Spezia frozen!")
			print("Italian units canot move!")
			unit.moveSpent = 255
		end	
	end 


	-- =========================
	-- STACKING PENALTIES
	-- SEE discreteEvents.lua 'STACKING PENALTIES'
	-- SEE onEnterTile.lua 'STACKING LIMITATIONS'

	-- =========================
	-- RANDOM VETERAN PROMOTIONS (functionality replaced by lua templates rules_lst.txt @LSTPROMOTION section)	
	-- ALLIED AIR UNITS CAN GAIN VETERAN STATUS TO REPRESENT EXPERIENCE GAINED THROUGH COMBAT MISSIONS STARTING FROM TURN 10
	-- Additional conditions: 
	-- Air unit can't already be a veteran, or 
	-- Be located on the cities of Tunis or Valetta (to prevent newly created units from gaining veterancy)
	-- if civ.getTurn() >= 10 then
		-- if unit.owner == object.pAllies 
		-- and unit.type.domain == 1 
		-- and (unit.location.baseTerrain == object.bTownCenter or (gen.hasAirbase(unit.location))) 
		-- and ((unit.location.city ~= object.cTunis) and (unit.location.city ~= object.cValetta))
		-- and unit.veteran == false then
			-- if math.random(25) == 1 then -- "1" is arbitrary, we want a 4% chance or 1 in 25	
				-- civ.ui.text(func.splitlines("Our " ..tostring(unit.type.name).. " at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ") has gained veteran status due to its experience flying multiple combat missions!\n^\r^As such, moving forward its bomb loads will have veteran status as well."))
				-- unit.veteran = true
			-- end
		-- end
	-- end
	
	-- ===================================================
	-- COMBAT MODIFIERS 
	-- ===================================================	
	
	
	-- ===================================================
	-- LEADER ATTACK BONUSES (RANGED) - (by Knighttime)
	-- ===================================================
	
	-- Blank out the global variable containing attack bonus descriptions, and then initialize for this unit type:
	for key, _ in pairs(unitAttackBonusDesc) do
		unitAttackBonusDesc[key] = nil
	end
	unitAttackBonusDesc[unit.type.id] = { }
	
	-- Then calculate new attack bonus:
	-- METHOD FOR CALCULATING CUMULATIVE LEADER BONUSES:
	if unit.owner == object.pAllies and unit.type.domain == 0 or unit.type.domain == 1 then
		if unit.type.attack == 0 then
			print(unit.type.name .. " has 0 attack, leader attack bonus is irrelevant")
		else
			print("Testing for leader attack bonuses for " .. unit.type.name .. " (ID " .. unit.id .. ") at " .. unit.x .. "," .. unit.y)
			local startTimestamp = os.clock()
			
			-- Table of leader bonuses:
			-- Note that this is intentionally redefined here, for every unit activation, instead of once at the top of the file.
			-- This is so that individual rows can actually be deleted (directly below) if not applicable, without having a carryover effect to the next unit activation.
			local alliesLeaderBonuses = {
				Montgomery = 	{ leaderType = object.uGenMontgomery,	nationality = "UK", 	maxDistance = 4,	benefit = 1.3 },
				Leese =			{ leaderType = object.uGenLeese, 		nationality = "UK", 	maxDistance = 3, 	benefit = 1.2 },
				McCreery =		{ leaderType = object.uGenMcCreery, 	nationality = "UK", 	maxDistance = 3, 	benefit = 1.3 },
				Patton =		{ leaderType = object.uGenPatton, 		nationality = "US", 	maxDistance = 5, 	benefit = 1.4 },
				Clark =			{ leaderType = object.uGenClark, 		nationality = "US", 	maxDistance = 3, 	benefit = 1.1 },
				Truscott =		{ leaderType = object.uGenTruscott,		nationality = "US", 	maxDistance = 3, 	benefit = 1.2 },
			}
			
			local maxAllowedAttackFactor = 1.4
			local attackFactor = 1.0

			for leaderKey, leaderBonus in pairs(alliesLeaderBonuses) do
				-- local leaderType = leaderBonus.leaderName
				-- A leader cannot receive a bonus offered by the same unit type as itself:
				if leaderBonus.leaderType == unit.type then
					print("  Did not search for " .. leaderBonus.leaderType.name .. " (same unit type)")
				else
					if (leaderBonus.nationality == "UK" and tables.ukEligibleUnit[unit.type.id] == true) or
					   (leaderBonus.nationality == "US" and tables.usEligibleUnit[unit.type.id] == true)then
						print("  Searching for " .. leaderBonus.leaderType.name .. "...")
						
						-- A customized version of the Supply Lines module is used to determine if the leader is within range of the current unit:
						-- local leaderInRange = true
						local leaderInRange = false
						for _, rangeTile in ipairs(getTilesByDistance(unit.location, leaderBonus.maxDistance, true)) do
							for otherUnit in rangeTile.units do
								if otherUnit.type == leaderBonus.leaderType then
									leaderInRange = true
									break
								end
							end
							if leaderInRange then
								break
							end
						end
						
						if leaderInRange then
							attackFactor = math.min(attackFactor * leaderBonus.benefit, maxAllowedAttackFactor)
							table.insert(unitAttackBonusDesc[unit.type.id], leaderBonus.leaderType.name .. " leader attack bonus applied: +" .. tostring(round((leaderBonus.benefit - 1) * 100)) .. "%")
							local message = "    Attack factor of " .. leaderBonus.benefit .. " applied, total is now " .. attackFactor
							if attackFactor == maxAllowedAttackFactor then
								message = message .. " (max)"
							end
							print(message)
						end
					else
						print("  Unit is not eligible for " .. leaderKey .. " bonus")
					end
				end
			end
			
			if attackFactor == maxAllowedAttackFactor then
				table.insert(unitAttackBonusDesc[unit.type.id], "(Achieved maximum permitted bonus of +" .. tostring(round((maxAllowedAttackFactor - 1) * 100)) .. "%)")
			end
			
			print("  All searches completed in " .. string.format("%.4f", os.clock() - startTimestamp) .. " seconds")
			
			local newAttackValue = round(unit.type.attack * attackFactor)
			if newAttackValue ~= unit.type.attack then
				print("Changed attack of " .. unit.owner.adjective .. " " .. unit.type.name .. " from " .. unit.type.attack .. " to " .. newAttackValue .. " (attackFactor = " .. attackFactor .. ")")
				unit.type.attack = newAttackValue
			end
		end
	end
	
-- END LEADER BONUS ================
	
	
end
---&endAutoDoc

return register
