local gen = require("generalLibrary")
local func = require("functions")
local supplyLines = require("supplyLines")
local text = require("text")

local object = require("object")
local param = require("parameters")
local flag = require("flag")
local counter = require("counter")
local onTurn = require("onTurn")
local delayedAction = require("delayedAction")
local tables = require("tables")

-- local function hasNoCargo(unit) 
    -- for possibleCargo in unit.location.units do
        -- if possibleCargo.carriedBy == unit then
            -- return false
        -- end
    -- end
    -- return true
-- end

-- This file can be deleted if it is not used.
local register = {}


-- ============================================================
-- CHECK IF LIBERTY SHIP ALREADY EXISTS IN BEACHHEAD TILE FUNCTION ================ (by Prof. Garfield)
local function libertyShipBeachheadCheck(libertyShip,previousTile) -- (by Prof. Garfield)
    -- Do nothing if libertyShip isn't actually a libertyShip unit,
    -- or if there is no beachhead unit on the tile.
    if libertyShip.type ~= object.uLibertyShip or not gen.unitTypeOnTile(libertyShip.location, object.uBeachhead) then
        return
    end
    -- Check if there is another libertyShip on the tile
    for unitOnTile in libertyShip.location.units do
        if unitOnTile ~= libertyShip and unitOnTile.type == object.uLibertyShip then
            text.simple("The small coastal town port is already at capacity this turn and cannot receive another Liberty Ship at this time, until the one present departs, and therefore your incoming Liberty Ship unit, along with its cargo which is still onboard, are returned to their previous tile!","Coastal Town Port at Capacity!",object.monexone)
            libertyShip:teleport(previousTile)
            gen.spendMovementPoints(libertyShip, -1)
            return
        end
    end
    -- If we get here, there is no other libertyShip on the tile
    civ.playSound("Shipwhistle.wav")
	text.simple("By entering the beachhead tile your Liberty Ship expends all its remaining movement points in order to unload its cargo!","Liberty Ship Enters Beachhead!",object.mAnzioInvasion)            
    libertyShip.moveSpent = 255
    return
end

-- ============================================================
-- CHECK TILE OWNERSHIP FUNCTION ================ (by Prof. Garfield)
-- Used in TOWNS OF INTERESTS section below
-- Allows code to check if tile is owned by specific power before being able to create a unit on it
local function filterTilesByOwner(tileList,tribe)
    local filtered = {}
    local index = 1
    -- use ipairs to maintain order, in case
    -- randomization is not desired sometimes
    -- (but tileList must have integer indices)
    for _,tile in ipairs(tileList) do
        tile = gen.toTile(tile)
        if tile.owner ~= tribe then -- (modified by tootall: ~= to check that tile is NOT owned by selected tribe)
            filtered[index] = tile
            index = index+1
        end
    end
    return filtered
end

-- ============================================================
-- moveUnitAdjacent TILE PREFERENCE FUNCTION ================ (by Prof. Garfield)
local function rankDestinations(unit,destTile)
    if (destTile.defender and destTile.defender ~=unit.owner) 
    or(destTile.city and destTile.city.owner ~= unit.owner) 
    or (not civ.canEnter(unit.type,destTile)) then
        return false
    end
    local southDifference = unit.location.y - destTile.y
    -- if destTile is further south than the unit.location,
    -- southDifferent will be negative, since south increases
    -- with y value.  Since smaller is better, this ranking
    -- prefers to move units south.
    return southDifference
end

-- ============================================================
-- Allied Air List used to prevent ground units from entering their tiles
-- See 'PREVENT ALLIED GROUND UNITS FOM ENTERING TILES THAT CONTAIN AIR UNITS UNLESS ON AIRBASES OR TOWN CENTERS' below
local alliedAirUnitList = {
-- US
object.uP40Warhawk,
object.uP38Lightning,
object.uP51DMustang,
object.uA36Apache,
object.uB24Liberator,
object.uB25Mitchell,
object.uB26Marauder,
object.uB17Fortress,
-- UK
object.uBeaufighterMkVI,
object.uKittyhawk,
object.uSpitfireMkIX,
object.uHurricaneIIc,
object.uMartinBaltimore,
object.uVickersWellington,
object.uBostonIII,
object.uSunderlandIII,
-- RN
object.uSeafireMkII,
object.uFaireyBarracuda,
}

---&autoDoc onEnterTile
-- onEnterTile(unit,previousTile,previousDomainSpec)
-- executes when a unit successfully enters a tile (so not when it attacks
-- a unit or fails to enter a tile because it lacks movement points)
function register.onEnterTile(unit,previousTile,previousDomainSpec)
    -- if _global.eventTesting then
        -- civ.ui.text("onEnterTile.onEnterTile: "..unit.type.name.." has entered tile ("..text.coordinates(unit.location)..") from tile ("..text.coordinates(previousTile)..").")
    -- end

	-- ===== SUPPLY LINES: =====
	-- [[ Originally by Knighttime ]]
	local supplyStatus = supplyLines.getSupplyStatus(unit)
	-- Based on supply condition, determine message and adjust damage (HP) if necessary:
	local suppliedText = "^Supply status determined from onEnterTile\n^\n^" .. unit.owner.adjective .. " " .. unit.type.name .. " at " .. unit.x .. "," .. unit.y .. "," .. unit.z .. ": "
	if supplyStatus == nil then
		suppliedText = suppliedText .. "Not calculated."
	elseif supplyStatus == true then
		suppliedText = suppliedText .. "Supplied.\n^Unit is unchanged."
	elseif supplyStatus == false then
		if math.random(6) == 1 then
			suppliedText = suppliedText .. "NOT SUPPLIED.\n^Unit HP reduced by 1, from " .. unit.hitpoints .. " to " .. (unit.hitpoints - 1) .. "."
			unit.moveSpent = 255
			unit.damage = unit.damage + 1
			civ.ui.redrawTile(unit.location)
			-- civ.ui.text(func.splitlines("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at (".. tostring(text.coordinates(unit.location)) .. ") has been cut off from supply! \n^\r^It receives 1 point of damage (HP reduced by 1) per tile while moving in an unsupplied condition \n^\r^AND will be subject to a 50% combat penalty if it attempts to attack enemy units."))
			text.simple("The ".. tostring(unit.owner.adjective) .. " ".. tostring(unit.type.name) .. " at (".. tostring(text.coordinates(unit.location)) .. ") has expended all its remaining supplies for this turn! It received 1 HP point of damage and can no longer move!","UNIT OUT OF SUPPLIES!",object.monexone)
		end
	end
	print(suppliedText)
	if supplyLines.SHOW_SUPPLY_LINE_KEYCODE ~= nil and supplyStatus ~= nil then
		suppliedText = suppliedText .. "\n^\n^After closing this dialog box, press " .. supplyLines.SHOW_SUPPLY_LINE_KEYLABEL .. " to toggle a visual representation of supply line results."
	end

	-- Output message to the user:
	--	if unit.owner.isHuman then
	--		civ.ui.text(func.splitlines(suppliedText))
	--	end

	-- =========================
	-- SPECIAL EFFECTS SECTION
	-- =========================
	
	-- TEST TO SEE IF CAN GET TRAIN TYPE UNIT TO MOVE ONLY ALONG TERRAIN THAT INCLUDES RAILROADS
	-- if unit.type == object.uUKTruck and (not gen.hasRailroad(unit.location)) then
		-- civ.ui.text(func.splitlines("The " .. object.uUKTruck.name .. " can only move along rail lines. There is no railroad in tile (" ..text.coordinates(unit.location).. ") and therefore the unit was returned to its last tile, unless it was on its last movement point, in which case it will remain on that last tile. On the next turn you will have to return it to a track to continue your movement!"))
		-- unit:teleport(previousTile)
	-- end
	
	-- ==========================	
	-- FUEL SHORTAGES (from HoF by John Petrovski)
	-- The ground units for the Allies that cost fuel to move
	-- This only effects the Allies as the AI would be hopeless
	if unit.owner == object.pAllies and tables.fuelReliantUnit[unit.type.id] then
		-- if object.pAllies.money <= param.lowFuelLevel and object.pAllies.money >= param.criticalFuelLevel and math.random() < param.slightChanceOfNoMovement then 
			-- unit.moveSpent = 255
			-- civ.ui.text("We have run out of fuel and can no longer move our " ..tostring(unit.type.name).." forward!")
		-- elseif object.pAllies.money <= param.criticalFuelLevel and math.random() < param.greatChanceOfNoMovement then 
			-- unit.moveSpent = 255
			-- civ.ui.text("We have run out of fuel and can no longer move our " ..tostring(unit.type.name).." forward!")
		-- else 
			-- if (unit.type == object.uM4A1Sherman) then 
				-- object.pAllies.money = object.pAllies.money - param.largeFuelCost 
			-- end 
		-- end 
	end
	
	-- ===========================
	-- SEASONAL EFFECTS ON MOVEMENT (idea from HoF, barbarossa.lua, by John Petrovski)
	
	if  flag.value("FallSpring Months") == true then
		-- ALLIED MECHANIZED GROUND UNITS CAN BECOME STUCK IN MUD DURING FALL/SPRING TURNS (not in cities, on highways, in Sardinia or Sicily or Southern Italy)
		if unit.owner == object.pAllies -- and unit.type.domain == 0  
		-- and tables.terrainTypes[unit.type.id]
		and tables.mechanizedSeasonalTerrainPenalties[unit.type.id]
		then
			if math.random(1,9) == 9 -- from 6
			and not unit.location.city 
			-- and not gen.hasRoad(unit.location) 
			and not gen.hasRailroad(unit.location) 
			and unit.location.baseTerrain ~= object.bOcean
			and unit.location.baseTerrain ~= object.ltransportPoint 			
			-- and not tables.mechanizedSeasonalTerrainPenalties[unit.type.id] -- Only mechanized ground units
			and not tables.excludeFromSeasonalTerrainPenalties[unit.type.id]
			and not gen.inPolygon(unit.location,tables.sardiniaPolygon)
			and not gen.inPolygon(unit.location,tables.sicilyPolygon)
			and not gen.inPolygon(unit.location,tables.southernItalyPolygon)
			then -- "1" is arbitrary, we want a 20% chance or 1 in 5	
				civ.ui.text("Our " ..tostring(unit.type.name).. " is mirred in the mud thanks to the season! This unit is stuck in place at " .. unit.x .. "," .. unit.y .. "," .. unit.z .. "!")
				unit.moveSpent = 255
			end
		end 

	elseif flag.value("Winter Months") == true then
		-- ALLIED MECHANIZED GROUND UNITS CAN BECOME STUCK IN SNOW DURING WINTER TURNS (not in cities, in Sardinia or Sicily or Southern Italy)
		if unit.owner == object.pAllies -- and unit.type.domain == 0  
		and tables.mechanizedSeasonalTerrainPenalties[unit.type.id]
		then 
			if math.random(1,12) == 12 -- from 8
			and not unit.location.city 
			and not gen.hasRoad(unit.location) 
			and not gen.hasRailroad(unit.location) 
			and unit.location.baseTerrain ~= object.bOcean	
			and unit.location.baseTerrain ~= object.ltransportPoint
			-- and tables.mechanizedSeasonalTerrainPenalties[unit.type.id] -- Only mechanized ground units
			and not tables.excludeFromSeasonalTerrainPenalties[unit.type.id] 
			and not gen.inPolygon(unit.location,tables.sardiniaPolygon)
			and not gen.inPolygon(unit.location,tables.sicilyPolygon)
			and not gen.inPolygon(unit.location,tables.southernItalyPolygon)
			then -- "1" is arbitrary, we want a 16% chance or 1 in 6	
				civ.ui.text("Our " ..tostring(unit.type.name).. " is stuck in the snow thanks to the season!' This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
				unit.moveSpent = 255
			end
		end 
	
	end

	-- ==========================
	-- -- C-47 DAKOTA CODE TO ENSURE IT CAN ONLY LAND IN CITIES / AIRBASE TILES
	-- if unit.type == object.uC47Dakota and ((not gen.hasAirbase(unit.location)) and (unit.location.baseTerrain ~= object.bTownCenter)) then
		-- -- if math.random(1,2) == 2 then 
			-- if gen.isParadrop(unit.type) and not unit.location.city then
				-- unit:teleport(previousTile)
				-- gen.clearParadropped(unit)
				-- -- set cargoUnit.carriedBy = nil
				-- civ.ui.text("The "..tostring(unit.type.name).. " was unable to reach either an airbase or city and was therefore forced to return to its point of origin along with its cargo!")
			-- end
		-- -- else
			-- -- civ.deleteUnit(unit)
			-- -- civ.ui.text("The "..tostring(unit.type.name).. " was unable to reach either an airbase or city and therefore crash landed along with its cargo and was destroyed!")
		-- -- end
	-- end	
	
    if unit.type == object.uC47Dakota and ((not gen.hasAirbase(unit.location)) and (unit.location.baseTerrain ~= object.bTownCenter)) then
		unit:teleport(previousTile)
		unit.moveSpent = 255
		gen.clearParadropped(unit)
		for possibleCargo in unit.location.units do
			if possibleCargo.carriedBy == unit then
				possibleCargo.carriedBy = nil
			end
		end
		text.simple("The "..tostring(unit.type.name).. " was unable to reach either an airbase or city and was therefore forced to return to its point of origin along with its cargo!","C-47 Dakota Forced to Return to Base!",object.monexone)
    end	

	-- ==========================
	-- THE 8 TILES SURROUNDING MOUNT ETNA ARE CONSIDERED IMPASSABLE TO ALL MECHANIZED UNITS
	if unit.owner == object.pAllies 
	and tables.impassableTerrainUnit[unit.type.id]
	and gen.inPolygon(unit.location,tables.etnaPolygon) then
		text.simple("The "..tostring(unit.type.name).. " is prevented from entering the Mount Etna zone due to its inhospitable environment to mechanized units and therefore is compelled to return to its previous location!","Mount Etna Zone!", object.mMountEtna)
		unit:teleport(previousTile)
	end

	-- ==========================
	-- THIS "IMPASSABLE RIVERBED TILES" SECTION FOR THE PONTOON MECHANISM IS NO LONGER REQUIRED 
	-- (see 'ENGINEER: BUILDING PONTOON BRIDGES ON RIVERBED TILES' in discreteEvents.lua instead)
	--
	-- ALLIED MECHANIZED UNITS CAN ENTER IMPASSABLE RIVERBED TILES IF THERE'S A PONTOON BRIDGE ON THEM
	-- METHOD ONE
	-- if unit.owner == object.pAllies 
	-- -- and unit.type.domain == 0 
	-- and tables.impassableTerrainUnit[unit.type.id]
	-- -- and (unit.location.baseTerrain == object.bRiverbed or unit.location.baseTerrain == object.bTemperateRiverbed) 
	-- then
		-- if (not gen.unitTypeOnTile(unit.location,object.uPontoonBridge)) and (unit.location.baseTerrain ~= object.bRiverbed or unit.location.baseTerrain ~= object.bTemperateRiverbed or unit.location.baseTerrain == object.bApennines) then
			-- -- gen.removeCanCrossImpassableTerrain(object.uArmoredInfantry) 
			-- -- gen.removeCanCrossImpassableTerrain(object.uM5Stuart) 
			-- -- gen.removeCanCrossImpassableTerrain(object.uM4A1Sherman) 
			-- -- gen.removeCanCrossImpassableTerrain(object.uM4A1ShermanInd) 
			-- -- gen.removeCanCrossImpassableTerrain(object.uM4A376WSherman) 
			-- -- gen.removeCanCrossImpassableTerrain(object.uM10Wolverine) 
			-- gen.removeCanCrossImpassableTerrain(unit.type)
			-- civ.ui.text(func.splitlines("The " .. unit.type.name .. " trying to move into ("..text.coordinates(unit.location)..") can only enter a riverbed tile that contains a pontoon bridge and therefore is returned to its previous location!"))
			-- unit:teleport(previousTile)
		-- elseif (gen.unitTypeOnTile(unit.location,object.uPontoonBridge)) then	
			-- -- gen.giveCanCrossImpassableTerrain(object.uArmoredInfantry) 
			-- -- gen.giveCanCrossImpassableTerrain(object.uM5Stuart) 
			-- -- gen.giveCanCrossImpassableTerrain(object.uM4A1Sherman) 
			-- -- gen.giveCanCrossImpassableTerrain(object.uM4A1ShermanInd) 
			-- -- gen.giveCanCrossImpassableTerrain(object.uM4A376WSherman) 
			-- -- gen.giveCanCrossImpassableTerrain(object.uM10Wolverine) 			
			-- gen.giveCanCrossImpassableTerrain(unit.type) 
		-- end	
	-- end	

	-- METHOD TWO
	-- if unit.owner == object.pAllies 
	-- and tables.impassableTerrainUnit[unit.type.id]
	-- and (unit.location.baseTerrain == object.bRiverbed or unit.location.baseTerrain == object.bTemperateRiverbed)
	-- and (not gen.unitTypeOnTile(unit.location,object.uPontoonBridge)) 
	-- then
		-- civ.ui.text(func.splitlines("The " .. unit.type.name .. " trying to move into ("..text.coordinates(unit.location)..") can only enter a riverbed tile that contains a pontoon bridge and therefore is returned to its previous location after expending the associated terrain movement cost!"))
		-- unit:teleport(previousTile)	
		
		-- -- gen.removeCanCrossImpassableTerrain(unit.type)
		-- -- for id,_ in pairs(tables.impassableTerrainUnit) do
			-- -- gen.removeCanCrossImpassableTerrain(civ.getUnitType(id))
		-- -- end
	-- else
		-- -- gen.giveCanCrossImpassableTerrain(unit.type)
		-- for id,_ in pairs(tables.impassableTerrainUnit) do
			-- gen.giveCanCrossImpassableTerrain(civ.getUnitType(id))
			
		-- end
	-- end

	-- and prohibit tables.impassableTerrainUnit and Engineer units from entering Apennines tiles
	-- if unit.owner == object.pAllies and unit.type.id == tables.impassableTerrainUnit[unit.type.id] and unit.location.baseTerrain == object.bApennines then
	-- if unit.owner == object.pAllies 
	-- and (tables.impassableTerrainUnit[unit.type.id] or unit.type == object.uEngineer) 
	-- and unit.location.baseTerrain == object.bApennines then
		-- civ.ui.text(func.splitlines("The " .. unit.type.name .. " is prohibited from entering ".. object.bApennines.name .." tiles (at "..text.coordinates(unit.location)..") and therefore is returned to its previous location after expending the equivalent terrain movement cost!"))
		-- unit:teleport(previousTile)
	-- end	
	
	-- ==========================
	-- PREVENT ALLIED GROUND UNITS FROM ENTERING TILES THAT CONTAIN AIR UNITS UNLESS ON AIRBASES OR TOWN CENTERS
	-- Reason: to prevent the human player from using air units to bypass enemy zones of control
	-- Note: an air unit can still enter a tile that contains allied ground units, just not the other way around
	if unit.owner == object.pAllies and unit.type.domain == 0 then
		if gen.unitTypeOnTile(unit.location,alliedAirUnitList) and ((not gen.hasAirbase(unit.location)) and (unit.location.baseTerrain ~= object.bTownCenter)) then
			civ.playSound("NEG1.wav")
			text.simple("Ground units are prohibited from entering tiles that\n^contain aircraft unless located in a city or airbase!\n^\r^The " .. unit.type.name .. " that attempted to do so at tile\n^("..text.coordinates(unit.location)..") was returned to its previous location.\n^\r^On the other hand, note that air units themselves\n^may enter tiles that contain ground units.\n^\r^Note: this was done to prevent the human player\n^from bypassing enemy ZOC's using aircraft.","Illegal Ground Move!",object.monexone)
			unit:teleport(previousTile)
		end
	end

	-- ==========================
	-- PREVENT ALLIED DUKW UNITS FROM ENTERING APENNINE TILES
	-- Reason: to prevent the human player from trying to built supply line through mountainous terrain
	if unit.owner == object.pAllies and unit.type == object.uDUKW  and unit.location.baseTerrain == object.bApennines then
		civ.playSound("NEG1.wav")
		text.simple("DUKW units are prohibited from entering Apennines tiles!\n^\r^The " .. unit.type.name .. " that attempted to do so at tile\n^("..text.coordinates(unit.location)..") was returned to its previous location.","Illegal Ground Move!",object.monexone)
		unit:teleport(previousTile)
	end

	-- ==========================
	-- TELEPORT ANY ALLIED AND AXIS GROUND UNITS THAT ENTERS THE IMPASSABLE ZONE OF SOUTHERN FRANCE BACK TO TUNIS OR CUNEO
	-- Allies - Tunis
	if unit.owner == object.pAllies and unit.type.domain == 0 and gen.inPolygon(unit.location,tables.southernFrancePolygon) then
		civ.playSound("NEG1.wav")
		text.simple("Allied ground unit are prohibited from entering\n^within the zone of southern France!\n^\r^The " .. unit.type.name .. " that attempted to do so at tile\n^("..text.coordinates(unit.location)..") was transported back to the port of Tunis..","Illegal Ground Move!",object.monexone)
		civ.teleportUnit(unit, object.lTunis)
	end
	
	-- Axis - Cuneo
	if unit.owner == object.pAxis and unit.type.domain == 0 and gen.inPolygon(unit.location,tables.southernFrancePolygon) then
		civ.teleportUnit(unit, object.ltownCuneo)
	end	
	
	-- ==========================
	-- REMOVE LANDING CRAFT WHEN ENTERING VALETTA IN FIRST 3 TURNS
	-- Reason: To prevent players from bypassing the need to unload units from Liberty Ships onto the Beachhead tiles (with their limited unloading capacity)

	if civ.getTurn() <= 3 
	and	unit.owner == object.pAllies 
	and unit.type == object.uLandingCraft 
	and unit.location == object.lValetta then
		-- civ.ui.text(func.splitlines("The " .. unit.type.name .. " is prohibited from entering Valetta due to its limited docking capacity at this time and therefore is returned to its previous location after expending the equivalent movement cost!"))
		-- unit:teleport(previousTile)
		-- unit.moveSpent = 255
		text.simple("The " .. unit.type.name .. " has completed its mission for Operation Husky and is sent to a naval yard for repairs and refurbishing and you gain " .. param.lcValetta .. " supplies for its safe return!","Landing Craft Withdrawn!",object.monexone)
		object.pAllies.money = object.pAllies.money + param.lcValetta
		civ.deleteUnit(unit)
		-- civ.teleportUnit(unit, object.lNavalYard)
		-- gen.setToSleeping(unit)
		-- This line is required for tracking number of LC that were withdrawn (see onTurn.lua -- ALL LANDING CRAFT ARE WITHDRAWN)
		counter.add("# Landing Craft Withdrawn during 1st Invasion",1)
	end	
	
	-- ==========================
	-- TRUCKS AND MOTORIZED INFANTRY EXPEND EXTRA MP'S
	-- UK and US Trucks expend 1 TO 3 extra MP's per tile, depending on the season, if not moving along tiles that contain a road/highway or a town center  
	-- local unit = civ.getActiveUnit()
	local nonRoadTruckSummerMoveCost1 = 1
	local nonRoadTruckSummerMoveCost2 = 1 * totpp.movementMultipliers.aggregate
	local nonRoadTruckFallMoveCost1 = 3
	local nonRoadTruckFallMoveCost2 = 3 * totpp.movementMultipliers.aggregate
	local nonRoadTruckWinterMoveCost1 = 2
	local nonRoadTruckWinterMoveCost2 = 2 * totpp.movementMultipliers.aggregate
	-- local unitMoved = gen.moveRemaining(unit)
	if unit.owner.isHuman == true 
	and (unit.type == object.uUKTruck 
	or unit.type == object.uUSTruck
	or unit.type == object.uCdnMotInf 
	or unit.type == object.uSAMotInf 
	or unit.type == object.uFreeFrenchInfantry 
	or unit.type == object.uNZMotInf 
	or unit.type == object.uUS1stInfantry)
	and unit.location.baseTerrain ~= object.bTownCenter 
	and (not gen.hasRoad(unit.location)) and (not gen.hasRailroad(unit.location))
	-- and unitMoved >= nonRoadTruckMoveCost2 
	then 
		if flag.value("Summer Months") == true then
			unit.moveSpent = unit.moveSpent + totpp.movementMultipliers.aggregate*nonRoadTruckSummerMoveCost1
			civ.ui.text("The truck expended 1 extra MP to enter this tile")
		elseif flag.value("FallSpring Months") == true then
			unit.moveSpent = unit.moveSpent + totpp.movementMultipliers.aggregate*nonRoadTruckFallMoveCost1
			civ.ui.text("The truck expended 3 extra MP's to enter this tile")		
		elseif flag.value("Winter Months") == true then
			unit.moveSpent = unit.moveSpent + totpp.movementMultipliers.aggregate*nonRoadTruckWinterMoveCost1
			civ.ui.text("The truck expended 2 extra MP's to enter this tile")	
		end
	end	

	-- ==========================
	-- GROUND UNITS MAY NOT BOARD VESSELS ON OCEAN TILES OTHER THAN THOSE WITH A BEACHHEAD TILE PRESENT
	local oceanBoardingPenaltyCost1 = 1
	local oceanBoardingPenaltyCost2 = 1 * totpp.movementMultipliers.aggregate
	if unit.owner == object.pAllies 
	and unit.type.domain == 0 
	and unit.location.baseTerrain == object.bOcean
	and (not gen.unitTypeOnTile(unit.location,object.uBeachhead))
	
	then
		-- gen.spendMovementPoints(unit, -1)
		-- unit.moveSpent = unit.moveSpent + totpp.movementMultipliers.aggregate*oceanBoardingPenaltyCost1
		text.simple("Ground units may not embark vessels on ocean tiles other than\n^those located on a Beachhead unit!\n^\r^Your unit has expended all its movement points in attempting to\n^do so and been returned to its previous location.","Forbidden Ocean Move!",object.monexone)
		unit:teleport(previousTile)
	end

	-- ==========================
	-- LIBERTY SHIP RESTRICTIONS (see local function libertyShipBeachheadCheck(libertyShip,previousTile) ABOVE)
	-- TO PREVENT ALLIED PLAYER FROM UNLOADING MORE THAN 1 LIBERTY SHIP FROM SAME BEACHHEAD TILE IN A GIVEN TURN
	-- Reason: To represent the small shipping capacity of small coastal towns as opposed to major port cities
	
	if unit.owner.isHuman == true then
		libertyShipBeachheadCheck(unit,previousTile)
	end
	
	-- =========================
	-- STACKING LIMITATIONS (see discreteEvents.lua STACKING PENALITES section if you prefer to implement unit damages instead (currently commented out))
	-- Prevents human player from overstacking in tiles and returns offending unit to previous tile and removes 1 MP
	-- Different terrain types have different stacking limits 
	-- DUKW, Engineers, Generals, Pontoon Bridge, Supply Hub and Supply Truck units do NOT count against the stacking limit
		
	-- Player has the option to turn stacking limits off or on (set to on by default) 
	-- (UPDATE Sept 1, 2023: Stacking no longer optional as its too important a feature for overall game play and balance)
	
	if unit.owner.isHuman == true and flag.value("Stacking Limit Options") == true then	
        if unit.owner == object.pAllies and unit.type.domain == 0 then
  			local totalUnits = 0
            local stackingPenaltyMoveCost = 1
			for unitOnTile in unit.location.units do
                if unitOnTile.type.domain == 0 
				and not tables.excludeFromTerrainOccupancyLimit[unitOnTile.type.id] 
				-- and unit.location.baseTerrain ~= object.bTownCenter 
				then
                    totalUnits = totalUnits+1
                end
            end
            local terrainLimit = tables.terrainOccupancyLimit[unit.location.baseTerrain.type]
            if totalUnits > terrainLimit then
                text.simple("The " .. unit.type.name .. " trying to move into ("..text.coordinates(unit.location)..") caused a massive traffic jam and was forced to retreat to its previous location having expended " .. stackingPenaltyMoveCost .. " extra MP's in the process.\n^\r^The stacking limit for "..unit.location.baseTerrain.name.." terrain is "..tostring(terrainLimit).."!","Traffic Jam!",object.monexone)
                -- unit.damage = math.min(unit.damage + param.stackingLimitPenalty,unit.type.hitpoints-1)
                unit:teleport(previousTile)
                unit.moveSpent = unit.moveSpent + totpp.movementMultipliers.aggregate*stackingPenaltyMoveCost
                -- unit.moveSpent = 255
            end	
		end	
	end	

	-- ==========================
	-- PREVENT LIBERTY SHIPS FROM ENTERING PORT CITY IF IT ALREADY CONTAINS MAXIMUM GROUND UNITS
	-- NOTE: function doesn't work because transport unloads its cargo before moving the Liberty Ship back out to sea

	-- if unit.owner == object.pAllies and object.uLibertyShip and unit.location.baseTerrain == object.bTownCenter then
		-- local totalGroundUnitInCity = 0
		-- local stackingPenaltyMoveCost = 1
			-- for unitOnTile in unit.location.units do
                -- if unitOnTile.type.domain == 0 
				-- and not tables.excludeFromTerrainOccupancyLimit[unitOnTile.type.id] 
				-- -- and unit.location.baseTerrain ~= object.bTownCenter 
				-- then
				-- totalGroundUnitInCity = totalGroundUnitInCity + 1
			-- end
		-- end
		-- local terrainLimit = tables.terrainOccupancyLimit[unit.location.baseTerrain.type]
		-- if totalGroundUnitInCity >= tables.terrainOccupancyLimit[object.bTownCenter.type] and not hasNoCargo(unit) then
			-- civ.ui.text(func.splitlines("The " .. unit.type.name .. " trying to enter into ("..text.coordinates(unit.location)..") cannot do so because the city's stacking capacity is already at maximum and therefore was forced to retreat outside the harbor to its previous location having expended " .. stackingPenaltyMoveCost .. " extra MP's in the process.\n^\r^The stacking limit for "..unit.location.baseTerrain.name.." terrain is "..tostring(terrainLimit).."!"))
			-- -- unit.damage = math.min(unit.damage + param.stackingLimitPenalty,unit.type.hitpoints-1)
			-- unit:teleport(previousTile)
			-- unit.moveSpent = unit.moveSpent + totpp.movementMultipliers.aggregate*stackingPenaltyMoveCost		
		-- end
	-- end	

	-- end STACKING LIMITATIONS =========================	

	-- ==========================	
	-- FORTRESS TILE IMPROVEMENT REMOVED ONCE CAPTURED BY ALLIES
	if unit.owner == object.pAllies and unit.type.domain == 0 and (gen.hasFortress(unit.location)) then
		civ.playSound("LARGEXPL.wav")
		gen.removeFortress(unit.location)
		gen.chartTruthfully(unit.location,unit.owner)
		civ.ui.redrawTile(unit.location)
		text.simple("The " .. unit.type.name .. " located on tile (" .. text.coordinates(unit.location) .. ") has overrun and eliminated a German fortification!","Fortress Position Destroyed!",object.mAlliesAttack)
	end	

	-- ==========================
	-- ALLIED BEACHHEAD DESTROYED IF AXIS UNIT ENTERS ASSOCIATED TOWN TILE -- (code by Prof. Garfield)
	-- Also see townNames in tables.lua file for list of associated towns
    if unit.owner == object.pAxis and unit.type.domain == 0 and tables.townNames[gen.getTileID(unit.location)] then
        for possibleBeachhead in gen.nearbyUnits(unit.location, 1) do
            if possibleBeachhead.type == object.uBeachhead then
                civ.playSound("LARGEXPL.wav")
                text.simple("The capture of the town of ".. tables.townNames[gen.getTileID(unit.location)].." led to the destruction of the Allied beachhead. It should no longer be possible to unload cargo from your Liberty Ships from this location.","Allied Beachhead Destroyed!",object.monexone)
                gen.deleteUnit(possibleBeachhead)
            end
        end
    end

	-- ==========================
	-- NOTIFICATIONS WHEN AN ALLIED UNIT FIRST ENTERS EITHER THE GUSTAV OR GOTHIC LINES
	-- Gustav Line
	if unit.owner == object.pAllies and unit.type.domain == 0 and gen.inPolygon(unit.location,tables.gustavLinePolygon) then
		gen.justOnce("Allies Enter Gustav Line",function()
			civ.playSound("Partisan.wav")
			text.simple("Your "..tostring(unit.type.name).. " has just entered the German\n^Gustav Line Defensive Zone.\n^\r^The area is a deathtrap of enemy mine fields, bunkers\n^and artillery positions. Your forces should proceed with\n^care moving forward!\n^\r^On the other hand, if you can capture 4 of the Line's\n^red zones, you will have effectively breached its\n^perimeter and cause its collapse forcing the Germans\n^to abandon the entire position.\n^\r^The objective city of Pescara is considered amongst\n^one of the red zones.","Gustav Line Defensive Zone!", object.mGustavLine)
		end)
	end

	-- Gothic Line
	if unit.owner == object.pAllies and unit.type.domain == 0 and gen.inPolygon(unit.location,tables.gothicLinePolygon) then
		gen.justOnce("Allies Enter Gothic Line",function()
			civ.playSound("Partisan.wav")
			text.setLinesPerWindow(25)	
				text.simple("Your "..tostring(unit.type.name).. " has just entered the formidable\n^German Gothic Line Defensive Zone. The area is a deathtrap\n^of enemy mine fields, kasemattes and artillery positions.\n^\r^Your immediate goal is to secure the 19 objective points\n^required to prevent the sudden death victory loss conditon\n^prior to October III (turn #63). In this instance only,\n^capturing the towns of Leccio, Lucca, Massa, Pistoia\n^and San Marino each count as an objective point, in\n^addition to the normal city objective points.\n^\r^Your ultimate mission will be to breakthrough the\n^German defensive positions and break out into the\n^Po Valley and capture the remaining objective cities\n^which will allow you to complete your campaign's goals!\n^\r^If you can capture any 6 of the 7 following cities\n^(identified by the pink '*' on their left side) you will\n^have successfully breached the line: Bologna, Forli,\n^La Spezia, Modena, Parma, Ravenna, Rimini.\n^\r^Furthermore, Bologna is the headquarters for the GOTHIC\n^LINE wonder. If you capture it, German occupied cities\n^will no longer receive the benefits of the Anti-Tank\n^Defenses improvement (aka 'City Walls').","Gothic Line Defensive Zone!", object.mGothicLine)
			text.setLinesPerWindow(12)	
		end)
	end
	
	-- ==========================	
	-- TOWNS OF INTERESTS (as delimited by the towns whose name is in TURQUOISE)
	-- Capturing the following non-city towns with ground troops will provide you with extra benefits and/or generate Axis reaction forces
	
	if unit.owner == object.pAllies and unit.type.domain == 0 then
		-- Adrano
		if unit.location == object.ltownAdrano then 
			gen.justOnce("Capture of Adrano",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the town of Adrano opens up another possible route to capturing Messina and nets the Allies an extra " .. param.provincialCapital .. " supplies.","Adrano is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end

		-- Ancona
		if unit.location == object.ltownAncona then 
			gen.justOnce("Capture of Ancona",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the Marche capital town of Ancona nets the Allies an extra " .. param.provincialCapital .. " supplies. In addition, its nearby airbase can serve as a base of operations for your air units.","Ancona is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end

		-- Arezzo, Fano, Firenze or Urbino
		if unit.location == object.ltownArezzo or unit.location == object.ltownFano or unit.location == object.lFirenze or unit.location == object.ltownUrbino then
			gen.justOnce("Capture of Arezzo to Urbino",function()
				gen.createUnit(object.uKustenbatterie, object.pAxis, {{106,44,0}}, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.u88cmFlak37, object.pAxis, {{106,44,0}}, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{106,44,0},{103,39,0},{107,37,0}},object.pAllies), {count=1, randomize=false, veteran=true})
				gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{106,44,0},{103,39,0},{107,37,0}},object.pAllies), {count=1, randomize=false, veteran=true})
				gen.createUnit(object.uDo217E, object.pAxis, {{106,44,0},{100,30,0},{107,37,0}}, {count=1, randomize=true, veteran=false})				
				gen.createUnit(object.uFW190A, object.pAxis, {{106,44,0},{100,30,0},{107,37,0}}, {count=1, randomize=true, veteran=false})				
				if flag.value("German 10th Army Escaped") == true then
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{106,44,0},{103,39,0},{107,37,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uPzkwVIE, object.pAxis, filterTilesByOwner({{106,44,0},{103,39,0},{107,37,0}},object.pAllies), {count=1, randomize=false, veteran=true})
				end
				
				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans build a " .. object.uKustenbatterie.name .. " in Rimini.","" .. object.uKustenbatterie.name .. " in Rimini!",object.monexone)
				end	
			end)
		end

		-- Bolsena
		if unit.location == object.ltownBolsena then 
			gen.justOnce("Capture of Bolsena",function()
				local randomNumber = math.random(3)
				-- REACTION FORCE: create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{75,35,0},{84,46,0},{89,39,0}},object.pAllies), {count=2, randomize=false, veteran=false})
					gen.createUnit(object.uJu88, object.pAxis, {{89,39,0},{75,35,0},{103,39,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uMe110, object.pAxis, {{89,39,0},{75,35,0},{103,39,0}}, {count=1, randomize=true, veteran=true})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{75,35,0},{84,46,0},{89,39,0}},object.pAllies), {count=1, randomize=false, veteran=true})			
					end
				elseif randomNumber == 2 then
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{84,44,0},{93,49,0},{89,39,0}},object.pAllies), {count=1, randomize=false, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{75,35,0},{103,39,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{84,44,0},{93,49,0},{89,39,0}},object.pAllies), {count=1, randomize=false, veteran=true})			
					end
				elseif randomNumber == 3 then
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{83,51,0},{84,56,0},{89,39,0}},object.pAllies), {count=1, randomize=false, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{75,35,0},{103,39,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVIE, object.pAxis, filterTilesByOwner({{83,51,0},{84,56,0},{89,39,0}},object.pAllies), {count=1, randomize=false, veteran=true})			
					end
				end

				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction town of Bolsena! Random number: " ..randomNumber.. ".","Bolsena is captured!",object.monexone)
				end
			end)
		end	
		
		-- Bonifacio
		if unit.location == object.ltownBonifacio then 
			gen.justOnce("Capture of Bonifacio",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the town of Bonifacio activates Allied resistance cells on that island! In addition, an additional Goumier Marocain unit will be made available to you should you capture the city of Ajaccio.","Bonifacio is captured!",object.monexone)
				-- Create 3 Allied Partigiani in Corsica
				gen.createUnit(object.uPartigiani, object.pAllies, {{44,50,0},{45,45,0},{41,53,0},{48,48,0},{48,44}}, {count=3, randomize=true, scatter=true, veteran=false})
			end)
		end

		-- Castrovillari
		if unit.location == object.ltownCastrovillari then 
			gen.justOnce("Capture of Castrovillari",function()
				local randomNumber = math.random(3)
					civ.playSound("Cheers2.wav")
					text.simple("The capture of the important road junction town of Castrovillari nets the Allies an extra " .. param.provincialCapital .. " supplies!","Castrovillari is captured!",object.monexone)
					object.pAllies.money = object.pAllies.money + param.provincialCapital
				-- REACTION FORCE: Only on random number 1, create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{100,132,0},{104,140,0},{112,134,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, {{100,132,0},{104,140,0},{112,134,0}}, {count=1, randomize=true, veteran=false})

					if flag.value("Display Axis Testing Messages") == true then
						civ.playSound("Cheers2.wav")
						text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction town of Castrovillari! Random number: " ..randomNumber.. ".","Castrovillari is captured!",object.monexone)
					end	
				end
			end)
		end	

		-- Caltanissetta
		if unit.location == object.ltownCaltanissetta then 
			gen.justOnce("Capture of Caltanissetta",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the provincial town of Caltanissetta nets the Allies an extra " .. param.provincialCapital .. " supplies.","Caltanissetta is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
				local randomNumber = math.random(3)
				-- REACTION FORCE: Only on random number 1, create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{51,155,0},{48,150,0},{44,144,0},{44,140,0}},object.pAllies), {count=1, randomize=false, veteran=false})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{51,155,0},{48,150,0},{44,144,0},{44,140,0}},object.pAllies), {count=1, randomize=false, veteran=false})
				
					if flag.value("Display Axis Testing Messages") == true then
						civ.playSound("Cheers2.wav")
						text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction town of Caltanissetta! Random number: " ..randomNumber.. ".","Caltanissetta is captured!",object.monexone)
					end	
				end
			end)
		end

		-- Cassino
		if unit.location == object.ltownCassino then 
			gen.justOnce("Capture of Cassino",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the fortified town of Cassino serves as a great propaganda victory to the Allies and nets them an extra " .. param.provincialCapital .. " supplies.","Cassino is captured!",object.mMonteCassino)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end
		
		-- Catanzaro
		if unit.location == object.ltownCatanzaro then 
			gen.justOnce("Capture of Catanzaro",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the Calabrian capital town of Catanzaro nets the Allies an extra " .. param.provincialCapital .. " supplies.","Catanzaro is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end

		-- Cecina or Grosseto
		if unit.location == object.ltownCecina or unit.location == object.ltownGrosseto then
			gen.justOnce("Capture of Cecina or Grosseto",function()
				local randomNumber = math.random(3)
				-- REACTION FORCE: create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=2, randomize=false, veteran=true})
					gen.createUnit(object.uJu88, object.pAxis, {{89,39,0},{75,35,0},{77,23,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uDo217E, object.pAxis, {{89,39,0},{75,35,0},{77,23,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uMe110, object.pAxis, {{89,39,0},{75,35,0},{77,23,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=2, randomize=false, veteran=true})
					end
				elseif randomNumber == 2 then
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=2, randomize=false, veteran=true})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{75,35,0},{77,23,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					end
				elseif randomNumber == 3 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=2, randomize=false, veteran=true})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{75,35,0},{77,23,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVIE, object.pAxis, filterTilesByOwner({{84,46,0},{89,39,0},{75,35,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					end
				end

				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction towns of Cecina or Grosseto! Random number: " ..randomNumber.. ".","Cecina or Grosseto are captured!",object.monexone)
				end	
			end)		
		end

		-- Ceccina, Grosseto, Pisa or Siena
		if unit.location == object.ltownCecina or unit.location == object.ltownGrosseto or unit.location == object.ltownPisa or unit.location == object.ltownSiena then
			gen.justOnce("Capture of Cecina to Siena",function()
				gen.createUnit(object.uKustenbatterie, object.pAxis, {{75,35,0}}, {count=1, randomize=false, veteran=false})
				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans build a " .. object.uKustenbatterie.name .. " in Livorno.","" .. object.uKustenbatterie.name .. " in Livorno!",object.monexone)
				end	
			end)
		end

		-- Crotone 				
		if unit.location == object.ltownCrotone then 
			gen.justOnce("Capture of Crotone",function()

				local randomNumber = math.random(3)
				-- ALSO SEE 'function SeaMinesRemovedInCatania(argTable)' AT BOTTOM
				if randomNumber == 1 then
					delayedAction.doInFuture("BuildHarborinCrotone",{},civ.getTurn()+ 1 ,object.pAllies.id)
				elseif randomNumber == 2 then
					delayedAction.doInFuture("BuildHarborinCrotone",{},civ.getTurn()+ 2 ,object.pAllies.id)
				elseif randomNumber == 3 then
					delayedAction.doInFuture("BuildHarborinCrotone",{},civ.getTurn()+ 3 ,object.pAllies.id)
				end

				civ.playSound("Cheers2.wav")
				text.simple("The capture of the Calabrian regional town of Crotone\n^nets the Allies an extra " .. param.townOfCrotone .. " supplies.\n^\r^In addition, its nearby airbase can serve as a point of\n^transit for your air units.\n^\r^Finally, a harbor installation will be built in " .. randomNumber .. " week(s) on tile\n^(104,156,0) which will allow naval units that begin their turn in\n^this location to transit to other destinations without needing an\n^Oil Tanker to refuel!","Crotone is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.townOfCrotone
			end)
		end

		-- Foligno
		if unit.location == object.ltownFoligno then 
			gen.justOnce("Capture of Foligno",function()
				local randomNumber = math.random(3)
				-- REACTION FORCE: create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{93,49,0},{103,51,0},{91,53,0}},object.pAllies), {count=2, randomize=false, veteran=true})
					gen.createUnit(object.uDo217E, object.pAxis, {{89,39,0},{106,44,0},{103,39,0}}, {count=1, randomize=true, veteran=true})
					gen.createUnit(object.uFW190A, object.pAxis,{{89,39,0},{106,44,0},{103,39,0}}, {count=1, randomize=true, veteran=true})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{93,49,0},{103,51,0},{91,53,0}},object.pAllies), {count=1, randomize=false, veteran=false})
					end
				elseif randomNumber == 2 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{91,53,0},{100,58,0},{103,51,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uStugIIIG, object.pAxis, filterTilesByOwner({{91,53,0},{100,58,0},{103,51,0}},object.pAllies), {count=1, randomize=false, veteran=false})
					gen.createUnit(object.uFW190A, object.pAxis, {{89,39,0},{106,44,0},{103,39,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{91,53,0},{100,58,0},{103,51,0}},object.pAllies), {count=1, randomize=false, veteran=true})				
					end
				elseif randomNumber == 3 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{94,58,0},{100,58,0},{91,53,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{94,58,0},{100,58,0},{91,53,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{94,58,0},{100,58,0},{91,53,0}},object.pAllies), {count=1, randomize=false, veteran=false})
					gen.createUnit(object.uMe109G, object.pAxis, {{89,39,0},{106,44,0},{103,39,0}}, {count=1, randomize=true, veteran=false})
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{94,58,0},{100,58,0},{91,53,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					end
				end

				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction town of Foligno! Random number: " ..randomNumber.. ".","Foligno is captured!",object.monexone)
				end
			end)
		end	

		-- L'Aquila
		if unit.location == object.ltownLAquila then 
			gen.justOnce("Capture of L'Aquila",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the Abruzzian capital town of L'Aquila nets the Allies an extra " .. param.provincialCapital .. " supplies.","L'Aquila is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end

		-- Massa Zone -- Add bunker if Allies enter zone prior to turn 31
		if civ.getTurn() <= 30 and unit.owner == object.pAllies and unit.type.domain == 0 and gen.inPolygon(unit.location,tables.massaPolygonCoordinates) then 
			gen.justOnce("Allies Enter Massa Zone",function()
				if object.lhillNEMassa.defender and object.lhillNEMassa.defender ~= object.pAxis then
					for unit in object.lhillNEMassa.units do
						gen.moveUnitAdjacent(unit)
						-- gen.moveUnitAdjacent(unit,rankDestinations)
					end
				end

				local unit = gen.createUnit(object.uBunker, object.pAxis, {{80,26,0}}, {count=1, randomize=false, veteran=true})[1]
				if unit then
					gen.setToNoOrders(unit)
					gen.setToFortified(unit)
					gen.placeFortress(unit.location)
					unit.moveSpent = 255
				end
				
				if flag.value("Display Axis Testing Messages") == true then
					text.simple("AXIS TESTING MESSAGE:\n^The Allies Enter Massa Zone prior to turn 31.","Massa Zone Defense!",object.monexone)
				end
			end)
		end	
		

		-- MILANO - TORINO ZONE (objective tiles) ============================================================
		-- PART ONE: Axis forces react to Allies approaching Milano - Turino road junctions 
		if civ.getTurn() >= 30 and gen.inPolygon(unit.location,tables.milanoTorinoPolygon) then 
			gen.justOnce("Enter Milano - Turino Road Junctions Zone",function()
				if object.lobjectiveMilano.defender and object.lobjectiveMilano.defender ~= object.pAxis then -- (tile 74,56,0)
					for unit in object.lobjectiveMilano.units do
						gen.moveUnitAdjacent(unit)
					end
				end
			
				local randomNumber = math.random(3)
				-- REACTION FORCE: create local German and Italian reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uFanteriaVeterana, object.pAxis,{{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=3, randomize=true, veteran=false})
					gen.createUnit(object.uAB41, object.pAxis,{{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, {{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=1, randomize=false, veteran=false})				
				elseif randomNumber == 2 then
					gen.createUnit(object.uBersaglieri, object.pAxis,{{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uM1340, object.pAxis,{{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uMotInfanterie, object.pAxis, {{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=1, randomize=false, veteran=false})
				
				elseif randomNumber == 3 then
					gen.createUnit(object.uFanteriaMotorizzata, object.pAxis,{{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=2, randomize=true, veteran=false})
					gen.createUnit(object.uSemovente7518, object.pAxis,{{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=1, randomize=true, veteran=false})
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, {{58,2,0},{61,5,0},{64,8,0},{70,2,0},{75,1,0}}, {count=1, randomize=false, veteran=false})
				end

				civ.playSound("InfantryGer.wav")
				text.simple("The Germans along with Italian Fascisti react to the approach by the Allies to the important Milano and Turino road junctions!","Allies Approach Road Junctions!",object.monexone)
				if flag.value("Display Axis Testing Messages") == true then
					text.simple("Milano- Turino Reaction Forces, Random number: " ..randomNumber.. "","Milano - Turino Reaction Number!",object.monexone)
				end
				
			end)	
		end		

		--  PART TWO: 
		-- Allies secure Milano road junction
		if unit.location == object.lobjectiveMilano then 
			local randomNumber = math.random(1)
			-- ALSO SEE 'function alliesCapture_Milano(argTable)' AT BOTTOM
			if randomNumber == 1 then
				delayedAction.doInFuture("alliesMoveOnto_Milano",{},civ.getTurn()+ 1 ,object.pAllies.id)
			-- elseif randomNumber == 2 then
				-- delayedAction.doInFuture("alliesMoveOnto_Milano",{},civ.getTurn()+ 2 ,object.pAllies.id)
			end
			civ.playSound("Cheers2.wav")
			text.simple("The Allies have secured the road junction to Milano and send off a reconnaissance in force to capture the city (located on map 2) which should arrive in " ..randomNumber.. " week.","Allies Seize Milano Road Junction!",object.mM8Greyhound)
		end

		-- Allies secure Turino road junction
		if unit.location == object.lobjectiveTorino then 
			local randomNumber = math.random(1)
			-- ALSO SEE 'function alliesCapture_Torinno(argTable)' AT BOTTOM
			if randomNumber == 1 then
				delayedAction.doInFuture("alliesMoveOnto_Torino",{},civ.getTurn()+ 1 ,object.pAllies.id)
			-- elseif randomNumber == 2 then
				-- delayedAction.doInFuture("alliesMoveOnto_Torino",{},civ.getTurn()+ 2 ,object.pAllies.id)
			end
			civ.playSound("Cheers2.wav")
			text.simple("The Allies have secured the road junction to Turino and send off a reconnaissance in force to capture the city (located on map 2) which should arrive in " ..randomNumber.. " week.","Allies Seize Torino Road Junction!",object.mM8Greyhound)
		end

		-- ============================================================

		-- Ortona
		if unit.location == object.ltownOrtona then 
			gen.justOnce("Capture of Ortona",function()
				local randomNumber = math.random(3)
				-- REACTION FORCE: create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uInfanterieVeteranen, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{105,85,0}},object.pAllies), {count=2, randomize=false, veteran=false})
					gen.createUnit(object.uPanzerschreck, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{105,85,0}},object.pAllies), {randomize=false, veteran=false})
					gen.createUnit(object.uGrilleHsIG33, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{105,85,0}},object.pAllies), {randomize=false, veteran=false})					
				elseif randomNumber == 2 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{100,86,0}},object.pAllies), {count=2, randomize=false, veteran=true})
					gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{100,86,0}},object.pAllies), {count=1, randomize=false, veteran=false})					
				elseif randomNumber == 3 then
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{108,72,0}},object.pAllies), {count=1, randomize=false, veteran=true})
					gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{107,83,0},{105,87,0},{108,72,0}},object.pAllies), {count=1, randomize=false, veteran=true})					
				end

				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction town of Ortona! Random number: " ..randomNumber.. ".","Ortona is captured!",object.monexone)
				end
			end)
		end	

		-- Padova or Vicenza
		if civ.getTurn() >= 81 and unit.location == object.lPadova or unit.location == object.ltownVicenza and flag.value("Trieste is captured") == false then
			gen.justOnce("Race for Trieste",function()
				civ.playSound("Telegraph.wav")
				text.simple("As your "..tostring(unit.type.name).. " unit enters the italian province\n^of Veneto, Allied Intelligence reports that Tito's 4th\n^Yugoslavian Army is moving up the Neretva river\n^valley.\n^\r^At their current rate of progress they are expected to\n^reach and capture the critical objective city of Trieste\n^by the first week of May. The Allied High command\n^considers it is imperative, for post war relations, that\n^you arrive and liberate the city before that date.\n^\r^Failure to capture the city beforehand would prevent you\n^from acheiving a final decisive victory in your campaign.","Race for Trieste!",object.mMarshalTito)
			end)
		end	

		-- Perugia
		if unit.location == object.ltownPerugia then 
			gen.justOnce("Capture of Perugia",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the Umbrian capital town of Perugia nets the Allies an extra " .. param.provincialCapital .. " supplies.","Perugia is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end

		-- Pompei
		if civ.getTurn() <= 10 and unit.location == object.ltownPompei then 
			gen.justOnce("Capture of Pompei",function()
				gen.createUnit(object.uStugIIIG, object.pAxis, {{88,106,0}}, {count=1, randomize=false, veteran=false})
				gen.createUnit(object.uPzkwIVF, object.pAxis, {{88,106,0}}, {count=1, randomize=false, veteran=true})		

				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important town of Pompei! Random number: " ..randomNumber.. ".","Pompei is captured!",object.monexone)
				end
			end)	
		end
		
		-- Potenza
		if unit.location == object.ltownPotenza then 
			gen.justOnce("Capture of Potenza",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the Lucanian capital town of Potenza nets the Allies an extra " .. param.provincialCapital .. " supplies.","Potenza is captured!",object.monexone)
				object.pAllies.money = object.pAllies.money + param.provincialCapital
			end)
		end		
		
		-- Rimini Zone -- Add bunker if Allies enter zone prior to turn 31
		if civ.getTurn() <= 30 and unit.owner == object.pAllies and unit.type.domain == 0 and gen.inPolygon(unit.location,tables.riminiPolygonCoordinates) then 
			gen.justOnce("Allies Enter Massa Zone",function()
				if object.lhillSWRimini.defender and object.lhillSWRimini.defender ~= object.pAxis then
					for unit in object.lhillSWRimini.units do
						gen.moveUnitAdjacent(unit)
						-- gen.moveUnitAdjacent(unit,rankDestinations)
					end
				end

				local unit = gen.createUnit(object.uBunker, object.pAxis, {{104,46,0}}, {count=1, randomize=false, veteran=true})[1]
				if unit then
					gen.setToNoOrders(unit)
					gen.setToFortified(unit)
					gen.placeFortress(unit.location)
					unit.moveSpent = 255
				end
				
				if flag.value("Display Axis Testing Messages") == true then
					text.simple("AXIS TESTING MESSAGE:\n^The Allies Enter Rimini Zone prior to turn 31.","Massa Zone Defense!",object.monexone)
				end
			end)
		end	

		-- Salo
		if unit.location == object.ltownSalo then 
			gen.justOnce("Capture of Salo",function()
				civ.playSound("Cheers2.wav")
				text.simple("The capture of the town of Salo brings about the end\n^of Mussolini's 'Repubblica Sociale Italiana' puppet\n^state and nets the Allies an extra " .. param.townOfSalo .. " supplies.\n^\r^As a further consequence, the puppet state will no\n^longer be able to recruit new MSVN units.","Salo is captured!",object.mRepublicOfSalo)
				object.pAllies.money = object.pAllies.money + param.townOfSalo
				flag.setTrue("Republic of Salo Conquered")
			end)
		end		

		-- San Benedetto
		if unit.location == object.ltownSanBenedetto then  
			gen.justOnce("Capture of San Benedetto",function()
				local randomNumber = math.random(3)
				-- REACTION FORCE: create local German reaction force
				if randomNumber == 1 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{109,51,0},{103,51,0},{111,61,0}},object.pAllies), {count=3, randomize=true, veteran=true})	
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwIVH, object.pAxis, filterTilesByOwner({{109,51,0},{103,51,0},{111,61,0}},object.pAllies), {count=1, randomize=true, veteran=true})	
					end
				elseif randomNumber == 2 then
					gen.createUnit(object.uMotInfanterie, object.pAxis, filterTilesByOwner({{111,61,0},{108,64,0},{109,51,0}},object.pAllies), {count=2, randomize=false, veteran=true})	
					gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{111,61,0},{108,64,0},{109,51,0}},object.pAllies), {count=1, randomize=false, veteran=true})	
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVG, object.pAxis, filterTilesByOwner({{111,61,0},{108,64,0},{109,51,0}},object.pAllies), {count=1, randomize=false, veteran=true})	
					end
				elseif randomNumber == 3 then
					gen.createUnit(object.uPanzerGrenadier, object.pAxis, filterTilesByOwner({{106,44,0},{103,39,0},{107,37,0}},object.pAllies), {count=1, randomize=false, veteran=true})	
					if flag.value("German 10th Army Escaped") == true then
						gen.createUnit(object.uPzkwVIE, object.pAxis, filterTilesByOwner({{106,44,0},{103,39,0},{107,37,0}},object.pAllies), {count=1, randomize=false, veteran=true})			
					end
				end

				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Germans react to the capture of the important road junction town of San Benedetto! Random number: " ..randomNumber.. ".","San Benedetto is captured!",object.monexone)
				end
			end)
		end	

		-- TRASIMENE LINE - Grosseto to Ancona ==========================	
		-- If Allies enter any one of Ascoli, Civitavecchia, Fiumicino, Rieti, Pescara, ROMA, San Benedetto, Talamone, Tivoli or Viterbo
		if unit.location == object.ltownAscoli
		or unit.location == object.ltownCivitavecchia
		or unit.location == object.ltownFiumicino
		or unit.location == object.ltownRieti	
		or unit.location == object.lPescara
		or unit.location == object.lROMA
		or unit.location == object.ltownSanBenedetto
		or unit.location == object.ltownTalamone
		or unit.location == object.ltownTivoli
		or unit.location == object.ltownViterbo
		then

			gen.justOnce("Trasimene Line Triggered One",function()
				-- Add Defensive Line Units
				local randomNumber = math.random(3)
				flag.setTrue("Trasimene Defensive Line Triggered")

				-- OPTION ONE
				if randomNumber == 1 then
					if object.ltownAncona.defender and object.ltownAncona.defender ~= object.pAxis then
						for unit in object.ltownAncona.units do
							-- gen.moveUnitAdjacent(unit)
							gen.moveUnitAdjacent(unit,rankDestinations) -- In this case, use this moveUnitAdjcent to prevent units in coastal Ancona town being moved into adjacent ocean tiles
						end
					end
					if object.lhillSWFabriano.defender and object.lhillSWFabriano.defender ~= object.pAxis then 
						for unit in object.lhillSWFabriano.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.lhillSEMontalcino.defender and object.lhillSEMontalcino.defender ~= object.pAxis then 
						for unit in object.lhillSEMontalcino.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.ltownMontalcino.defender and object.ltownMontalcino.defender ~= object.pAxis then 
						for unit in object.ltownMontalcino.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.ltownPerugia.defender and object.ltownPerugia.defender ~= object.pAxis then 
						for unit in object.ltownPerugia.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end

					--if object.ltownAncona.defender == nil or object.ltownAncona.defender == object.pAllies then
						-- local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, filterTilesByOwner({{111,61,0}},object.pAllies), {count=1, randomize=false, veteran=true})[1]
						local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, {{111,61,0}}, {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						--- local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner({{111,61,0}},object.pAllies), {count=1, randomize=false, veteran=true})[1]
						local unit = gen.createUnit(object.uKasematte, object.pAxis, {{111,61,0}}, {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
					-- end
					
					local unit = gen.createUnit(object.u15cmsIG33, object.pAxis, filterTilesByOwner({{87,53,0},{83,51,0},{94,58,0}},object.pAllies), {count=1, randomize=true, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner({{89,43,0},{89,47,0},{84,46,0}},object.pAllies), {count=1, randomize=false, veteran=false})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					
					-- Widerstandsnest (9)
					local a = 0
					while( a < 9 )	do
						local placementLocations = {{74,52,0},{83,51,0},{87,53,0},{92,52,0},{94,58,0},{99,59,0},{103,55,0},{109,51,0},{73,47,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis, placementLocations , {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a uWiderstandsnest, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						a = a + 1
					end	
					
					-- Bunker (3)
					local b = 0
					while( b < 3 )	do
						local placementLocations = {{87,53,0},{83,51,0},{94,58,0},{103,55,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uBunker, object.pAxis, placementLocations, {count=1, randomize=true, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Bunker, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						b = b + 1
					end	
							
					-- Randomly Place Fortresses
					local randomNumber = math.random(3)
						if randomNumber == 1 then
							gen.placeFortressForce({83,51,0})
							gen.placeFortressForce({92,52,0})
							gen.placeFortressForce({103,55,0})
						elseif randomNumber == 2 then
							gen.placeFortressForce({74,52,0})
							gen.placeFortressForce({87,53,0})
							gen.placeFortressForce({99,59,0})						
						elseif randomNumber == 3 then
							gen.placeFortressForce({109,51,0})
							gen.placeFortressForce({94,58,0})
							gen.placeFortressForce({83,51,0})					
						end

				-- OPTION TWO
				elseif randomNumber == 2 then
					if object.lroughNW_Ancona.defender and object.lroughNW_Ancona.defender ~= object.pAxis then -- (tile 109,57,0)
						for unit in object.lroughNW_Ancona.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.lhillSSFirenze.defender and object.lhillSSFirenze.defender ~= object.pAxis then
						for unit in object.lhillSSFirenze.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.lhillNWSiena.defender and object.lhillNWSiena.defender ~= object.pAxis then
						for unit in object.lhillNWSiena.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.lhillNETerontola.defender and object.lhillNETerontola.defender ~= object.pAxis then
						for unit in object.lhillNETerontola.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.ltownUrbino.defender and object.ltownUrbino.defender ~= object.pAxis then
						for unit in object.ltownUrbino.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end

					local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, filterTilesByOwner({{109,57,0}},object.pAllies), {count=1, randomize=false, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner({{109,57,0}},object.pAllies), {count=1, randomize=false, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end

					local unit = gen.createUnit(object.u15cmsIG33, object.pAxis, filterTilesByOwner({{89,47,0},{83,45,0},{94,48,0}},object.pAllies), {count=1, randomize=true, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner({{80,36,0},{86,40,0},{89,41,0}},object.pAllies), {count=1, randomize=false, veteran=false})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					
					-- Widerstandsnest (9)
					local a = 0
					while( a < 9 )	do
						local placementLocations = {{77,39,0},{83,45,0},{89,47,0},{92,52,0},{103,51,0},{109,51,0},{94,48,0},{78,34,0},{82,34,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a uWiderstandsnest, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						a = a + 1
					end	
					
					-- Bunker (3)
					local b = 0
					while( b < 3 )	do
						local placementLocations = {{77,39,0},{83,45,0},{103,51,0},{94,48,0},{89,47,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Bunker, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						b = b + 1
					end	
							
					-- Randomly Place Fortresses
					local randomNumber = math.random(3)
						if randomNumber == 1 then
							gen.placeFortressForce({77,39,0})
							gen.placeFortressForce({89,47,0})
							gen.placeFortressForce({109,51,0})
						elseif randomNumber == 2 then
							gen.placeFortressForce({83,45,0})
							gen.placeFortressForce({94,48,0})
							gen.placeFortressForce({103,51,0})						
						elseif randomNumber == 3 then
							gen.placeFortressForce({78,34,0})
							gen.placeFortressForce({92,52,0})
							gen.placeFortressForce({103,51,0})					
						end

				-- OPTION THREE				
				elseif randomNumber == 3 then
					if object.lroughSE_Macerata.defender and object.lroughSE_Macerata.defender ~= object.pAxis then -- (tile 109,65,0)
						for unit in object.lroughSE_Macerata.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.lhillNWBolsena.defender and object.lhillNWBolsena.defender ~= object.pAxis then
						for unit in object.lhillNWBolsena.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.lhillSWFabriano.defender and object.lhillSWFabriano.defender ~= object.pAxis then 
						for unit in object.lhillSWFabriano.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end	
					if object.ltownOrvieto.defender and object.ltownOrvieto.defender ~= object.pAxis then
						for unit in object.ltownOrvieto.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					if object.ltownPerugia.defender and object.ltownPerugia.defender ~= object.pAxis then 
						for unit in object.ltownPerugia.units do
							gen.moveUnitAdjacent(unit)
							-- gen.moveUnitAdjacent(unit,rankDestinations)
						end
					end
					
					local unit = gen.createUnit(object.u105cmleFH18, object.pAxis, filterTilesByOwner({{109,65,0}},object.pAllies), {count=1, randomize=false, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner({{109,65,0}},object.pAllies), {count=1, randomize=false, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end

					local unit = gen.createUnit(object.u15cmsIG33, object.pAxis, filterTilesByOwner({{83,51,0},{91,53,0},{103,51,0}},object.pAllies), {count=1, randomize=true, veteran=true})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end
					local unit = gen.createUnit(object.uKasematte, object.pAxis, filterTilesByOwner({{77,39,0},{84,44,0},{94,48,0}},object.pAllies), {count=1, randomize=true, veteran=false})[1]
					if unit then -- if the unit was created, it will be a Kasematte, but we have to check the unit was actually created, and is not nil
						gen.setToFortified(unit)
						-- gen.placeFortress(unit.location)
						unit.moveSpent = 255
						gen.setMoved(unit)
					end

					
					-- Widerstandsnest (8)
					local a = 0
					while( a < 9 )	do
						local placementLocations = {{74,52,0},{83,51,0},{84,60,0},{89,61,0},{99,59,0},{94,58,0},{111,61,0},{91,53,0},{103,51,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uWiderstandsnest) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uWiderstandsnest, object.pAxis,placementLocations , {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a uWiderstandsnest, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						a = a + 1
					end	
					
					-- Bunker (3)
					local b = 0
					while( b < 3 )	do
						local placementLocations = {{83,51,0},{91,53,0},{111,61,0},{103,51,0}}
						for key,tileCoord in pairs(placementLocations) do
							if gen.unitTypeOnTile(gen.toTile(tileCoord),object.uBunker) then
								placementLocations[key] = nil
							end
						end
						gen.makeArrayOneToN(placementLocations)
						local unit = gen.createUnit(object.uBunker, object.pAxis,placementLocations , {count=1, randomize=false, veteran=true})[1]
						if unit then -- if the unit was created, it will be a Bunker, but we have to check the unit was actually created, and is not nil
							gen.setToFortified(unit)
							-- gen.placeFortress(unit.location)
							unit.moveSpent = 255
							gen.setMoved(unit)
						end
						b = b + 1
					end	
							
					-- Randomly Place Fortresses
					local randomNumber = math.random(3)
						if randomNumber == 1 then
							gen.placeFortressForce({84,60,0})
							gen.placeFortressForce({99,59,0})
							gen.placeFortressForce({111,61,0})
						elseif randomNumber == 2 then
							gen.placeFortressForce({74,52,0})
							gen.placeFortressForce({89,61,0})
							gen.placeFortressForce({103,51,0})						
						elseif randomNumber == 3 then
							gen.placeFortressForce({83,51,0})
							gen.placeFortressForce({91,53,0})
							gen.placeFortressForce({111,61,0})					
						end

				end
			
				if flag.value("Display Axis Testing Messages") == true then
					civ.playSound("Cheers2.wav")
					text.simple("AXIS TESTING MESSAGE:\n^The Trasimene Line defenses (#" .. randomNumber .. ") are now deployed!","Trasimene Line Deployed!",object.monexone)
				end	
			end)
		end

	end
end
---&endAutoDoc

-- ==============================
-- DELAYED ACTION FUNCTIONS
-- ==============================

local function HarborBuiltInCrotone(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Crotone harbor installation was built on tile (104,156,0). As previously indicated, naval units that begin their turn in the installation will automatically be fueled!","Harbor Built In Crotone!",object.mDocksBuilt)
	local tileToChange = civ.getTile(104,156,0)
	tileToChange.terrain = object.tHarbor
end 
delayedAction.makeFunctionDelayable("BuildHarborinCrotone",HarborBuiltInCrotone)

local function alliesCapture_Milano(argTable)
	civ.playSound("Cathedrl.wav")
	object.cMilano.owner = object.pAllies
	text.simple("The Allies capture the major Italian city of Milano along with the Alfa Romeo factories which were producing armored vehicles for the German war effort!","Milano Falls to the Allies!",object.mAlliesCaptureCity)			
end 
delayedAction.makeFunctionDelayable("alliesMoveOnto_Milano",alliesCapture_Milano)

local function alliesCapture_Torino(argTable)
	civ.playSound("Cathedrl.wav")
	object.cTurino.owner = object.pAllies
	text.simple("The Allies capture the major Italian city of Turino along with the RIV Workshops which were producing war materials for the German war effort!","Turino Captured by the Allies!",object.mAlliesCaptureCity)			
end 
delayedAction.makeFunctionDelayable("alliesMoveOnto_Torino",alliesCapture_Torino)

return register