local gen = require("generalLibrary")
local param = require("parameters")
local object = require("object")
local text = require("text")
local func = require("functions")

local flag = require("flag")
local counter = require("counter")

local onTurn = require("onTurn")
local tables = require("tables")
local discreteEvents = require("discreteEventsRegistrar")

local aiRangedUnits = require("aiRangedUnits")

-- this file can be deleted if it is not being used
local register = {}

local ukArmoredToBeRemoved = {}
ukArmoredToBeRemoved[object.uShermanIIa.id] = 1 
ukArmoredToBeRemoved[object.uBritishMotorTroops.id] = 1 
ukArmoredToBeRemoved[object.uBishop25pdr.id] = 1 

local alliedNavalUnits = {}
alliedNavalUnits[object.uTribalClassDD.id] = 1
alliedNavalUnits[object.uLeanderClassCL.id] = 1
alliedNavalUnits[object.uYorkClassCA.id] = 1
alliedNavalUnits[object.uQEClassBB.id] = 1
-- alliedNavalUnits[object.uSClass.id] = 1
alliedNavalUnits[object.uIllustriousClassCV.id] = 1
alliedNavalUnits[object.uLibertyShip.id] = 1

local alliedNavalUnitList = {
object.uTribalClassDD,
object.uLeanderClassCL,
object.uYorkClassCA,
object.uQEClassBB,
-- object.uSClass,
object.uIllustriousClassCV,
object.uLibertyShip,
}

-- PARTIAL UNIT.TYPE WITHDRAWAL FUNCTION ================ (by Prof. Garfield)
local function withdrawTroops(unitType,quantity,divisionName)
	local deletedUnits = 0
	local unitTypeStillInTheatre = false
	local messageTitle = "Supreme Allied Command"
	for unit in civ.iterateUnits() do
		if unit.type == unitType and deletedUnits < quantity and unit.location ~= object.ltransportPoint then
			civ.deleteUnit(unit)
			deletedUnits = deletedUnits +1
		elseif unit.type == unitType and deletedUnits == quantity then
			unitTypeStillInTheatre = true
		end
	end
	if deletedUnits == 0 then
		-- no message, since no units were deleted
		return
	elseif deletedUnits == 1 then
		if unitTypeStillInTheatre then
			text.simple(text.substitute("The %STRING1 Division withdraws 1 regiment!",{divisionName}),messageTitle)
			return
		else
			text.simple(text.substitute("The %STRING1 Division withdraws its last regiment!",{divisionName}),messageTitle)
			return
		end
	else
		if unitTypeStillInTheatre then
			text.simple(text.substitute("The %STRING1 Division withdraws %STRING2 regiment!",{divisionName, deletedUnits}),messageTitle)
			return
		else
			text.simple(text.substitute("The %STRING1 Division withdraws its last %STRING2 regiments!",{divisionName,deletedUnits}),messageTitle)
			return
		end
	end
end

-- ==============================================
-- BEGIN RETAINED FULL MP FOR UNIT THAT ARE DAMAGED FUNCTION (by Prof. Garfield)
-- Also be sure to include the following two lines in the header of the file
-- local gen = require("generalLibrary")
-- local discreteEvents = require("discreteEventsRegistrar")
-- IMPORTANT: see specialMovementUnits in section below to set which units that will benefit from this fucntion
-- Limitations: 
-- At very high damage, unit movement is still limited, since unitType.move can't be set above 255. 
-- Also, code assumes you aren't changing unit type movement allowances with other code.

local function expectedMoveAllowance(unit,proposedMaxMove)
    if unit.type.domain == 2 then
        -- apply nuclear power
        if unit.owner:hasTech(civ.getTech(59)) then
            proposedMaxMove = proposedMaxMove+totpp.movementMultipliers.aggregate
        end
        -- apply magellan's and lighthouse
        if gen.isWonderActiveForTribe(12,unit.owner) then
            proposedMaxMove = proposedMaxMove+2*totpp.movementMultipliers.aggregate
        end
        if gen.isWonderActiveForTribe(3,unit.owner) and not gen.isCoastal(unit.type) then
            proposedMaxMove = proposedMaxMove + totpp.movementMultipliers.aggregate
        end
    end
    local moveAllowance = (unit.hitpoints*proposedMaxMove)//unit.type.hitpoints
    local moveMult = totpp.movementMultipliers.aggregate
    if moveAllowance % moveMult > 0 then
        moveAllowance = moveAllowance - moveAllowance % moveMult + moveMult
    end
    if unit.type.domain == 0 or unit.type.domain >= 3 then
        return math.min(math.max( moveAllowance,moveMult),moveAllowance)
    elseif unit.type.domain == 1 then
        return moveAllowance
    elseif unit.type.domain == 2 then
        return math.min(math.max( moveAllowance,2*moveMult),moveAllowance)
    end
end


-- IMPORTANT: This is where you set which units will benefit from the full movement allowance
-- specialMovementUnits[unit.type.id] = {rulesMovement,minMovement}
local specialMovementUnits = {
    -- [gen.original.uGalleon.id] = {4,4}, -- galleon has 4 movement in rules, will have max movement of 4
	[object.uOilTankerF.id] = {25,25}, -- oil tanker has 25 movement in rules, will have max movement of 25
	[object.uOilTankerE.id] = {25,25}, -- oil tanker has 25 movement in rules, will have max movement of 25
	[object.uUSTruck.id] = {7,7}, -- US truck has 7 movement in rules, will have max movement of 7
	[object.uUKTruck.id] = {7,7}, -- UK truck has 7 movement in rules, will have max movement of 7
}

local function giveUnitThisMovement(unit,minAllowance)
    if gen.maxMoves(unit) >= minAllowance then
        return
    end
    if unit.type.domain == 2 then
        -- apply nuclear power
        if unit.owner:hasTech(civ.getTech(59)) then
            minAllowance = minAllowance+totpp.movementMultipliers.aggregate
        end
        -- apply magellan's and lighthouse
        if gen.isWonderActiveForTribe(12,unit.owner) then
            minAllowance = minAllowance+2*totpp.movementMultipliers.aggregate
        end
        if gen.isWonderActiveForTribe(3,unit.owner) and not gen.isCoastal(unit.type) then
            minAllowance = minAllowance + totpp.movementMultipliers.aggregate
        end
    end
    local highMax = 255 -- this is known to be higher than the correct MaxMove
    local lowMax = minAllowance -- this is known to be lower than correct maxMove
    local proposedMaxMove = math.floor(0.5*(highMax+lowMax))
    local expectedAllowance = expectedMoveAllowance(unit,proposedMaxMove)
    for loopCount = 0,255 do
        if expectedAllowance == minAllowance or proposedMaxMove == 255 then
            unit.type.move = proposedMaxMove
            return
        elseif expectedAllowance < minAllowance then
            lowMax = proposedMaxMove
            proposedMaxMove = math.ceil(0.5*(lowMax+highMax))
            expectedAllowance = expectedMoveAllowance(unit,proposedMaxMove)
        else
            highMax = proposedMaxMove
            proposedMaxMove = math.floor(0.5*(lowMax+highMax))
            expectedAllowance = expectedMoveAllowance(unit,proposedMaxMove)
        end
    end
    error("giveUnitThisMovement: failed to find a solution")
end

function discreteEvents.onActivateUnit(unit,source,repeatMove)
    for id,movementTable in pairs(specialMovementUnits) do
        civ.getUnitType(id).move = movementTable[1]*totpp.movementMultipliers.aggregate
    end
    if not specialMovementUnits[unit.type.id] then
        return
    end
    giveUnitThisMovement(unit,specialMovementUnits[unit.type.id][2]*totpp.movementMultipliers.aggregate)
end

-- END RETAINED FULL MP FUNCTION ==============================================

-- CHECK IF NAVAL UNITS ARE WITHIN RANGE OF OIL TANKER FUNCTION ================ (by Prof. Garfield)
-- OPTION ONE FUNCTION
local function unitTypeNearTile(tile,unitTypeTable,excludeFromCheck,checkRadius,searchingTribe)
    if civ.isUnitType(unitTypeTable) then
        unitTypeTable = {unitTypeTable}
    end
    local exclusionSet = {}
    if civ.isUnit(excludeFromCheck) then
        ---@cast excludeFromCheck unitObject
        exclusionSet[excludeFromCheck.id] = true
    elseif type(excludeFromCheck) == "table" then
        for key,unit in pairs(excludeFromCheck) do
            if not civ.isUnit(unit) then
        error("gen.unitTypeOnTile: Arg #3 must be a unit or table of units.  For key: "..tostring(key)..", Received: "..tostring(unit))
            end
            exclusionSet[unit.id] = true
        end
    elseif excludeFromCheck ~= nil then
        error("gen.unitTypeOnTile: Arg #3 must be a unit or table of units.  Received: "..tostring(excludeFromCheck))
    end
    for unit in gen.nearbyUnits(tile,checkRadius,tile.z) do
        if unit.owner == searchingTribe and not exclusionSet[unit.id] then
            for __,unitType in pairs(unitTypeTable) do
                if unit.type == unitType then
                    return true
                end
            end
        end
    end
    return false
end

-- OPTION TWO FUNCTION
-- local function getNearbyOceanUnit(tile,unitTypeTable,excludeFromCheck,checkRadius,searchingTribe)
    -- if civ.isUnitType(unitTypeTable) then
        -- unitTypeTable = {unitTypeTable}
    -- end
    -- local exclusionSet = {}
    -- if civ.isUnit(excludeFromCheck) then
        -- ---@cast excludeFromCheck unitObject
        -- exclusionSet[excludeFromCheck.id] = true
    -- elseif type(excludeFromCheck) == "table" then
        -- for key,unit in pairs(excludeFromCheck) do
            -- if not civ.isUnit(unit) then
        -- error("getNearbyOceanUnits: Arg #3 must be a unit or table of units.  For key: "..tostring(key)..", Received: "..tostring(unit))
            -- end
            -- exclusionSet[unit.id] = true
        -- end
    -- elseif excludeFromCheck ~= nil then
        -- error("getNearbyOceanUnits: Arg #3 must be a unit or table of units.  Received: "..tostring(excludeFromCheck))
    -- end
    -- for unit in gen.nearbyUnits(tile,checkRadius,tile.z) do
        -- if unit.owner == searchingTribe and not exclusionSet[unit.id] and unit.location.baseTerrain == object.bOcean then
            -- for __,unitType in pairs(unitTypeTable) do
                -- if unit.type == unitType then
                    -- return unit
                -- end
            -- end
        -- end
    -- end
    -- return false
-- end

---&autoDoc onTribeTurnBegin
function register.onTribeTurnBegin(turn,tribe)
    -- if _global.eventTesting then
        -- civ.ui.text("onTribeTurnBegin for turn "..turn.." and tribe "..tribe.name.." separate file")
    -- end

	-- FOR TESTING: Display current German treasury supply level
	-- if turn >= 2 and tribe == object.pAxis and flag.value("Display Axis Testing Messages") == true then
		-- text.simple("ON TRIBE BEGIN TURN:The German theater depots currently have " .. object.pAxis.money .. " supplies!","German Theater Supplies!",object.m10thArmy)
	-- end


	-- ALLIED AIR UNITS CAN BECOME GROUNDED DURING FALL/SPRING OR WINTER TURNS WHEN TRYING TO LIFT OFF EITHER FROM CITIES OR AIRBASES
	-- FALL/SPRING SEASON
	-- if flag.value("FallSpring Months") == true then
		-- for unit in civ.iterateUnits() do 
			-- if unit.owner == object.pAllies 
			-- and not tables.munitionsToBeDestroyed[unit.type.id] 
			-- and (unit.location.baseTerrain == object.bTownCenter or (gen.hasAirbase(unit.location))) 
			-- and unit.type.domain == 1	
			-- then
				-- if math.random(4) == 1 then 	-- "1" is arbitrary, we want a 25% chance or 1 in 4	
					-- civ.ui.text("Our " ..tostring(unit.type.name).. " is grounded thanks to poor weather! This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
					-- unit.moveSpent = 255
				-- end
			-- end	
		-- end	
	-- -- WINTER SEASON
	-- elseif flag.value("Winter Months") == true then
		-- for unit in civ.iterateUnits() do 
			-- if unit.owner == object.pAllies 
			-- and not tables.munitionsToBeDestroyed[unit.type.id] 
			-- and (unit.location.baseTerrain == object.bTownCenter or (gen.hasAirbase(unit.location))) 
			-- and unit.type.domain == 1	
			-- then
				-- if math.random(3) == 1 then 	-- "1" is arbitrary, we want a 33% chance or 1 in 3	
					-- civ.ui.text("Our " ..tostring(unit.type.name).. " is grounded thanks to poor weather! This unit is stuck in place at (" .. unit.x .. "," .. unit.y .. "," .. unit.z .. ")!")
					-- unit.moveSpent = 255
				-- end
			-- end	
		-- end	
	-- end

	-- ============================	
	if flag.value("Ships At Sea Penalty Option") == true then -- Player has the option to turn naval penalties off or on (set to on by default)
	
		-- OPTION ONE: 50% CHANCE THAT FUELED OIL TANKER WITH BECOME UN-FUELED
		-- Uses unitTypeNearTile function in header section above
		
		-- NAVAL UNITS AT SEA PENALTIES IF WITHOUT FUELED OIL TANKERS
		-- if turn >= 1 and tribe == object.pAllies then
			-- for unit in civ.iterateUnits() do
				-- if unit.type.domain == 2 
				-- and not unitTypeNearTile(unit.location,object.uOilTankerF,nil,3,object.pAllies) 
				-- and unit.type ~= object.uWacoGlider
				-- and unit.type ~= object.uC47Dakota
				-- and unit.location.baseTerrain ~= object.bTownCenter then -- this line works with function unitTypeNearTile()
					-- -- gen.spendMovementPoints(unit,10) -- use this if you want to set a specific number of MP'S
					-- gen.spendMovementPoints(unit, math.floor(0.5*(gen.maxMoves(unit)/totpp.movementMultipliers.aggregate))) -- use this if you want to set a % of MP'S and avoid fractions
					-- gen.setMoved(unit) -- Set the unit to moved to prevent it from healing before inflicting the damage
					-- unit.damage = math.min(unit.damage + param.navalUnsuppliedHPPenalty,unit.type.hitpoints-1)
				-- end
			
			-- end
		-- -- civ.ui.text("Penalty " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
 		-- text.simple("You have ships at sea that have not been refueled at the beginning of this turn and thus suffer a 1 HP penalty and 50% MP loss!","Unfueled Allied Naval Vessels!",object.mResupplyAtSea)
		-- end	

		-- -- UPDATE OIL TANKERS FUEL STATUS DEPENDING IF AT SEA OR IN PORT
		-- if turn >= 1 then
			-- for unit in civ.iterateUnits() do
				-- -- Oil Tanker starting turn in port
				-- if unit.owner == object.pAllies and unit.type == object.uOilTankerE and unit.location.baseTerrain == object.bTownCenter then 
					-- text.simple("The " .. unit.type.name .. " at (" .. text.coordinates(unit.location) .. ") has refueled and is ready to support any naval task force at sea!","Refueled Allied Oil Tanker!",object.monexone)
					-- gen.replaceUnit(unit,object.uOilTankerF)
				-- end
			-- end	

			-- for unit in civ.iterateUnits() do 
				-- -- Oil Tanker starting turn at sea
				-- if unit.owner == object.pAllies 
				-- and unit.type == object.uOilTankerF 
				-- and unit.location.baseTerrain == object.bOcean 
				-- and unitTypeNearTile(unit.location,alliedNavalUnitList,nil,param.oilTankerRange,object.pAllies) -- this line was changed
				-- -- and gen.unitTypeOnTile(unit.location,alliedNavalUnitList)
				-- then 
					-- -- There a 50 % chance the tanker will have expended its fuel
					-- if math.random(1,2) == 2 then
						-- text.simple("The " .. unit.type.name .. " at (" .. text.coordinates(unit.location) .. ") has expended all its fuel stock and must return to port to refuel!","Allied Oil Tanker to Refuel!",object.monexone)
						-- gen.replaceUnit(unit,object.uOilTankerE)
					-- end
				-- end
			-- end	

		-- end

		-- RESTORE RANGE (MP'S) TO DAMAGED OIL TANKERS USING 'restoreRangeFn' FUNCTION
		-- if turn >= 1 and tribe == object.pAllies then
			-- for activeUnit in civ.iterateUnits() do
				-- if activeUnit.owner == object.pAllies and (activeUnit.type == object.uOilTankerF or activeUnit.type == object.uOilTankerE) and activeUnit.location.baseTerrain == object.bOcean then 
					-- restoreRangeFn(activeUnit)
				-- end
			-- end
		-- end

	-- OPTION TWO: ONLY ONE FUELED OIL TANKER IN SAME TILE AS ANOTHER MAY BECOME UN-FUELED
	-- Uses unitTypeNearTile function in header section above

        -- NAVAL UNITS AT SEA PENALTIES IF WITHOUT FUELED OIL TANKERS
        if turn >= 2 and tribe == object.pAllies then
            local unitOutOfFuelCount = 0 -- pg change
            for unit in civ.iterateUnits() do
                if alliedNavalUnits[unit.type.id] -- pg change
                and unit.owner == object.pAllies -- pg change 
                and not unitTypeNearTile(unit.location,object.uOilTankerF,nil,3,object.pAllies)-- this line works with function unitTypeNearTile() 
                and not gen.unitTypeOnTile(unit.location,object.uBeachhead)
				and unit.location.baseTerrain ~= object.bTownCenter 
				and (unit.location.terrain ~= object.tHarbor or unit.location ~= object.lHarborCrotone)
				then 
                    -- gen.spendMovementPoints(unit,10) -- use this if you want to set a specific number of MP'S
                    gen.spendMovementPoints(unit, math.floor(0.5*(gen.maxMoves(unit)/totpp.movementMultipliers.aggregate))) -- use this if you want to set a % of MP'S and avoid fractions
                    gen.setMoved(unit) -- Set the unit to moved to prevent it from healing before inflicting the damage
                    unit.damage = math.min(unit.damage + param.navalUnsuppliedHPPenalty,unit.type.hitpoints-1)
                    unitOutOfFuelCount = unitOutOfFuelCount + 1 -- pg change
                end
            
            end
            -- civ.ui.text("Penalty " .. unit.type.name .. " at " .. unit.location.x .. "," .. unit.location.y .. "," .. unit.location.z)
            if unitOutOfFuelCount > 0 then -- pg change
                 text.simple("You have "..tostring(unitOutOfFuelCount).." ships at sea that have not been refueled at the beginning of this turn and thus suffer a 1 HP penalty and 50% MP loss!","Unfueled Allied Naval Vessels!",object.mResupplyAtSea)
            end
        

		-- UPDATE OIL TANKERS FUEL STATUS DEPENDING IF AT SEA OR IN PORT
			-- Empty Oil Tanker starting turn in port
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and unit.type == object.uOilTankerE and unit.location.baseTerrain == object.bTownCenter then 
					text.simple("The " .. unit.type.name .. " at (" .. text.coordinates(unit.location) .. ") has refueled and is ready to support any naval task force at sea!","Refueled Allied Oil Tanker!",object.monexone)
					gen.replaceUnit(unit,object.uOilTankerF)
					-- counter.add("# Oil Tanker (F)",1)
					-- counter.subtract("# Oil Tanker (E)",1)
					-- counter.add("# Total Naval Units",1)	
				end
			end

			-- Fueled Oil Tanker starting turn at sea
			local tankerUnfuelledTiles = {} --pg change
            for unit in civ.iterateUnits() do 
                if unit.owner == object.pAllies 
                and unit.type == object.uOilTankerF 
                and unit.location.baseTerrain == object.bOcean 
                and not gen.unitTypeOnTile(unit.location,object.uBeachhead)
				and unitTypeNearTile(unit.location,alliedNavalUnitList,nil,param.oilTankerRange,object.pAllies) -- this line was changed
                and not tankerUnfuelledTiles[gen.getTileID(unit.location)] --pg change
                -- and gen.unitTypeOnTile(unit.location,alliedNavalUnitList)
                then 
                    tankerUnfuelledTiles[gen.getTileID(unit.location)] = true -- pg change
                    -- There a 50 % chance the tanker will have expended its fuel
                    if math.random(1,2) == 2 then
                        text.simple("The " .. unit.type.name .. " at (" .. text.coordinates(unit.location) .. ") has expended all its fuel stock and must return to port to refuel!","Allied Oil Tanker to Refuel!",object.monexone)
                        gen.replaceUnit(unit,object.uOilTankerE)
						-- counter.add("# Oil Tanker (E)",1)
						-- counter.subtract("# Oil Tanker (F)",1)
						-- counter.add("# Total Naval Units",1)	
                    end
                end
            end  

		-- RESTORE RANGE (MP'S) TO DAMAGED OIL TANKERS USING 'restoreRangeFn' FUNCTION
			-- for activeUnit in civ.iterateUnits() do
				-- if activeUnit.owner == object.pAllies and (activeUnit.type == object.uOilTankerF or activeUnit.type == object.uOilTankerE) and activeUnit.location.baseTerrain == object.bOcean then 
					-- restoreRangeFn(activeUnit)
				-- end
			-- end

		end	
	end	

	-- ============================
	-- ALLIED UNIT WITHDRAWALS (also see onTurn.lua for ALLIED - REINFORCEMENTS PLACEMENT)
	-- ============================
	
	-- TEST WITHDRAWAL
		-- if turn == 1 and tribe == object.pAllies then -- Dec I, 1943 
        -- withdrawTroops(object.uBritishVeteran,3,"Test")
    -- end	

    -- 1ST UK AIRBORNE WITHDRAWS
	if turn == 19 then -- Nov III, 1943
		gen.justOnce("Withdraw 1st Airborne",function()
			withdrawTroops(object.u1stAirborne,3,"1st UK Airborne")

			counter.subtract("# 1st Airborne",3)
			counter.subtract("# Total UK Infantry",3)
		end)	
	end

	-- 325G and 505 Rgt OF 82ND AIRBORNE WITHDRAWS
	if turn == 19 then -- Nov III, 1943
		gen.justOnce("Withdraw 82nd Airborne",function()	
			withdrawTroops(object.u82ndAirborne,2,"82nd US Airborne")
	
			counter.subtract("# 82nd Airborne",2)
			counter.subtract("# Total US Infantry",2) 
		end)
	end		

	-- 9TH US INFANTRY DIVISION WITHDRAWS
	if turn == 21 then -- Dec I, 1943 
		gen.justOnce("Withdraw 9th US Infantry",function()
			withdrawTroops(object.uGIVeteran,3,"9th US Infantry")
			counter.subtract("# GI Veteran",3)
			counter.subtract("# Total US Infantry",3) 
		end)	
	end	

	-- 7TH BRITISH ARMORED DIVISION WITHDRAWS
	if turn == 22 then -- Dec II, 1943
		gen.justOnce("Withdraw 7th British Armored Division",function()
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and ukArmoredToBeRemoved[unit.type.id] then 
					civ.deleteUnit(unit)
				end
			end
			text.simple("The 7th British Armored Division is withdrawn from the theater and sent to England to prepare for the invasion of France!","7th British Armored Division Withdrawn!",object.monexone)
			flag.setFalse("British Armored Division Deployed")
			counter.subtract("# Sherman IIa",3)
			counter.subtract("# Bren Carrier",2)
			counter.subtract("# Bishop 25pdr",1)
			
			counter.subtract("# Total UK Armored",5)
			counter.subtract("# Total UK Artillery",1)
		end)	
	end	
	
    -- 50TH AND 51ST BRITISH INFANTRY DIVISIONS WITHDRAW
	if turn == 23 then -- Dec III, 1943
		gen.justOnce("Withdraw 51st UK Infantry",function()
			withdrawTroops(object.uBritishRegular,6,"50th and 51st British Infantry")
			counter.subtract("# British Regular",6)
			counter.subtract("# Total UK Infantry",6) 
		end)
    end

	-- 4TH BRITISH TANK BRIGADE WITHDRAWS 
	if turn == 27 then -- Jan III, 1944
		gen.justOnce("Withdraw 4th British Tank Brigade",function()
			local a = 0
			for unit in civ.iterateUnits() do 
				if unit.type == object.uShermanIIaBde then
					if a < 2 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end    
			civ.ui.text("The 4th British Tank Brigade withdraws " .. tostring(a).. " tank battalion(s) to be redeployed to England!")
			counter.subtract("# Sherman IIa Bde",2)
			counter.subtract("# Total UK Armored",2)
		end)
	end	

    -- 46TH AND 56TH BRITISH INFANTRY DIVISIONS AND SUPPORTING ELEMENTS OF V CORPS WITHDRAW
	-- if turn == 35 then -- Mar III, 1944
		-- gen.justOnce("Withdraw 46th and 56th UK Infantry",function()
			-- withdrawTroops(object.uBritishRegular,6,"46th and 56th British Infantry")
			-- counter.subtract("# British Regular",6)
			-- counter.subtract("# Total UK Infantry",6) 
		-- end)
	-- end	

	if turn == 35 then -- Mar III, 1944
		gen.justOnce("Withdraw 46th and 56th UK Infantry",function()	 
			local a = 0
			local b = 0
			local c = 0
			local d = 0
			local e = 0	
			local f = 0	
			local g = 0			
			local penaltyCost = 0
			local totalUnitsToWithdrawInf = 8
			local totalUnitsToWithdrawMech = 5
				  			
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishRegular and unit.location ~= object.ltransportPoint then
					if a < 6 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishPIAT and unit.location ~= object.ltransportPoint then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishMG and unit.location ~= object.ltransportPoint then
					if c < 1 then
						civ.deleteUnit(unit)
						c = c + 1
					end
				end    
			end			
			for unit in civ.iterateUnits() do 
				if unit.type == object.uDaimlerAC then
					if d < 1 then
						civ.deleteUnit(unit)
						d = d + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.u25pdrMKIIArt and unit.location ~= object.ltransportPoint then
					if e < 2 then
						civ.deleteUnit(unit)
						e = e + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.u55inchArt and unit.location ~= object.ltransportPoint then
					if f < 1 then
						civ.deleteUnit(unit)
						f = f + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.u6pdrAT and unit.location ~= object.ltransportPoint then
					if g < 1 then
						civ.deleteUnit(unit)
						g = g + 1
					end
				end    
			end			
			
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			-- object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - (a + b + c + d + e + f + g + h)) * param.witdrawalInfantryPenalty)
			-- penaltyCost = (totalUnitsToWithdraw - (a + b + c + d + e + f + g + h)) * param.witdrawalInfantryPenalty
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdrawInf - (a + b + c)) * param.witdrawalInfantryPenalty) - ((totalUnitsToWithdrawMech - (d + e + f + g)) * param.witdrawalMechanizedPenalty)
			penaltyCost = ((totalUnitsToWithdrawInf - (a + b + c)) * param.witdrawalInfantryPenalty) + ((totalUnitsToWithdrawMech - (d + e + f + g)) * param.witdrawalMechanizedPenalty)
			
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", a total of " .. totalUnitsToWithdrawInf + totalUnitsToWithdrawMech .. " units from the British V Corps and 46th and 56th Infantry divisions were scheduled to be withdrawn from the theater in preparation for the invasion of Germany! " .. (totalUnitsToWithdrawInf - (a + b + c)) + (totalUnitsToWithdrawMech - (d + e + f + g)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 21st Army Group",object.m21stArmyGrp)

			counter.subtract("# British Regular",6)
			counter.subtract("# British PIAT",1)
			counter.subtract("# British MG",1)
			counter.subtract("# Daimler AC",1)
			counter.subtract("# 25pdr MKII Art",2)
			counter.subtract("# 55inch Art",1)
			counter.subtract("# 6pdr AT",1)
			
			counter.subtract("# Total UK Infantry",8)
			counter.subtract("# Total UK Armored",1)
			counter.subtract("# Total UK Artillery",4)
		end)
	end
	
	-- 504 Rgt OF 82ND AIRBORNE WITHDRAWS 
	if turn == 35 then -- Mar III, 1944
		gen.justOnce("Withdraw 504-82nd Airborne",function()
			withdrawTroops(object.u82ndAirborne,1,"82nd US Airborne")
			counter.subtract("# 82nd Airborne",1)
			counter.subtract("# Total US Infantry",1)
		end)	
    end		
	
    -- ROYAL NAVY BATTLESHIP DIVISIONS WITHDRAWN
	if turn == 41 then -- May I, 1944
		gen.justOnce("Withdraw RN Battleships",function() 
			for unit in civ.iterateUnits() do 
				if unit.type == object.uQEClassBB then
					civ.deleteUnit(unit)
				end    
			end
			civ.playSound("Shipwhistle.wav")
			text.simple("The Royal Navy withdraws its remaining battleship divisions from the Italian theater in order to go support the upcoming Operation Overlord, the invasion of northern France!","Royal Navy Battleships withdrawn!",object.mForceH)
			counter.subtract("# QE Class BB",2)
			counter.subtract("# Total Naval Units",2) -- BB's don't count against Total Naval units
		end)	
	end

	-- 36TH INFANTRY DIVISION, 1 X US BAZOOKA, 2 X 105MM ART. AND 1 X 155MM ART. REGIMENT WITHDRAWS
	if turn == 44 then -- May IV, 1944 (#44)
		gen.justOnce("Withdraw 36th US Infantry Division",function()
			local a = 0
			local b = 0
			local c = 0
			local d = 0
			local e = 0
			-- local penaltyCost = 0
			local penaltyCost1 = 0
			local penaltyCost2 = 0
			local totalUnitsToWithdraw1 = 4
			local totalUnitsToWithdraw2 = 3
			for unit in civ.iterateUnits() do 
				-- make sure that the unit isn't located on the transportPoint tile which would mean it is currently being transported on a truck
				if unit.type == object.uGIRegular and unit.location ~= object.ltransportPoint then
					if a < 3 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uUSBazooka and unit.location ~= object.ltransportPoint then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM2A1105mmArt and unit.location ~= object.ltransportPoint then
					if c < 2 then
						civ.deleteUnit(unit)
						c = c + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM1155mmArt and unit.location ~= object.ltransportPoint then
					if d < 1 then
						civ.deleteUnit(unit)
						d = d + 1
					end
				end    
			end		
			-- Truck section below commented out till can figure how to remove truck that isn't transporting cargo.
			-- for unit in civ.iterateUnits() do 
				-- if unit.type == object.uUSTruck then
					-- if e < 1 then
						-- civ.deleteUnit(unit)
						-- e = e + 1
					-- end
				-- end    
			-- end	

			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			-- object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - (a + b + c + d)) * param.witdrawalInfantryPenalty)
			-- penaltyCost = (totalUnitsToWithdraw - (a + b + c + d)) * param.witdrawalInfantryPenalty
			object.pAllies.money = object.pAllies.money - (((totalUnitsToWithdraw1 - (a + b)) * param.witdrawalInfantryPenalty) + ((totalUnitsToWithdraw2 - (c + d)) * param.witdrawalMechanizedPenalty))
			penaltyCost1 = (totalUnitsToWithdraw1 - (a + b)) * param.witdrawalInfantryPenalty 
			penaltyCost2 = (totalUnitsToWithdraw2 - (c + d)) * param.witdrawalMechanizedPenalty
			
			-- text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the 36th US Infantry Division, 1 US Bazooka and 3 artillery regiments are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - (a + b + c + d)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the 36th US Infantry Division, 1 US Bazooka and 3 artillery regiments are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw1 - (a + b)) + (totalUnitsToWithdraw2 - (c + d)) .. " units were missing for a penalty of " .. penaltyCost1 + penaltyCost2 .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# GI Regular",3)
			counter.subtract("# US Bazooka",1)
			counter.subtract("# M2A1 105mm Art",2)
			counter.subtract("# M1 155mm Art",1)
			-- counter.subtract("# US Truck",1)
			
			-- counter.subtract("# Total US Armored",1)
			counter.subtract("# Total US Infantry",4) 
			counter.subtract("# Total US Artillery",3)
		end)
	end	
	
	-- 1ST SSF AND 6615 RANGER REGIMENTS WITDRAW
	if turn == 44 then -- May IV, 1944
		gen.justOnce("Withdraw Ranger regiments",function()
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 2
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and unit.type == object.uRanger and unit.location ~= object.ltransportPoint then 
					civ.deleteUnit(unit)
					a = a + 1				
				end
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " all Ranger regiments have been withdrawn! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# US Ranger",2)
			counter.subtract("# Total US Infantry",2) 
		end)	
	end

	-- 3RD AND 45TH INFANTRY DIVISIONS WITHDRAW
	if turn == 45 then -- Jun I, 1944
		gen.justOnce("Withdraw 3rd and 45th US Infantry Divisions",function()
			local a = 0
			local b = 0
			local c = 0
			local d = 0
			local e = 0	
			local f = 0	-- don't count against withdrawal cost
			-- local penaltyCost = 0
			-- local totalUnitsToWithdraw = 10
			local penaltyCost1 = 0
			local penaltyCost2 = 0
			local totalUnitsToWithdraw1 = 6
			local totalUnitsToWithdraw2 = 4			
			
			for unit in civ.iterateUnits() do 
				if unit.type == object.uGIVeteran and unit.location ~= object.ltransportPoint then
					if a < 6 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM10Wolverine then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM4A1ShermanInd then
					if c < 1 then
						civ.deleteUnit(unit)
						c = c + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM8Greyhound then
					if d < 1 then
						civ.deleteUnit(unit)
						d = d + 1
					end
				end    
			end			
			for unit in civ.iterateUnits() do 
				if unit.type == object.uM157mmAT and unit.location ~= object.ltransportPoint then
					if e < 1 then
						civ.deleteUnit(unit)
						e = e + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				-- to prevent this occuring while they are transporting cargo at sea only delete Liberty Ships that are in port
				if unit.type == object.uLibertyShip and unit.location.baseTerrain == object.bTownCenter then 
					if f < 1 then
						civ.deleteUnit(unit)
						f = f + 1
					end
				end    
			end	

			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			-- object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - (a + b + c + d + e)) * param.witdrawalInfantryPenalty)
			-- penaltyCost = (totalUnitsToWithdraw - (a + b + c + d + e)) * param.witdrawalInfantryPenalty
			object.pAllies.money = object.pAllies.money - (((totalUnitsToWithdraw1 - (a)) * param.witdrawalInfantryPenalty) + ((totalUnitsToWithdraw2 - (b + c + d + e)) * param.witdrawalMechanizedPenalty))
			penaltyCost1 = (totalUnitsToWithdraw1 - (a)) * param.witdrawalInfantryPenalty 
			penaltyCost2 = (totalUnitsToWithdraw2 - (b + c + d + e)) * param.witdrawalMechanizedPenalty		
			-- text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the 3rd and 45th US Infantry Divisions along with supporting elements of the US IV Corps and one Liberty Ship are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - (a + b + c + d + e)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the 3rd and 45th US Infantry Divisions along with supporting elements of the US IV Corps and one Liberty Ship are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw1 - (a)) + (totalUnitsToWithdraw2 - (b + c + d + e)) .. " units were missing for a penalty of " .. penaltyCost1 + penaltyCost2 .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			
			flag.setTrue("Veteran US Infantry units withdrawn")
			counter.subtract("# GI Veteran",6)
			counter.subtract("# M8 Greyhound",1)
			counter.subtract("# M10 Wolverine",1)
			counter.subtract("# M4A1 Ind",1)
			counter.subtract("# M1 57mm AT",1)
			
			counter.subtract("# Total US Infantry",6) 
			counter.subtract("# Total US Armored",3)
			counter.subtract("# Total US Artillery",1)
			
			counter.subtract("# Liberty Ship",1)
			counter.subtract("# Total Naval Units",1)
		end)
	end	
	
	-- 1ST FREE FRENCH MECHANIZED DIVISION WITHDRAWS 
	if turn == 47 then -- Jun III, 1944 (#47)
		gen.justOnce("Withdraw 1st Free French Motorized Division",function()
			local a = 0
			local b = 0 
			local c = 0 -- don't count against withdrawal cost
			local penaltyCost = 0
			local totalUnitsToWithdraw = 6
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and tables.ffMechanizedToBeRemoved[unit.type.id] then 
					civ.deleteUnit(unit)
					a = a + 1
				end
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uFreeFrench105Art and unit.location ~= object.ltransportPoint then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end	
			for unit in civ.iterateUnits() do 
				-- to prevent this occuring while they are transporting cargo at sea only delete Liberty Ships that are in port
				if unit.type == object.uLibertyShip and unit.location.baseTerrain == object.bTownCenter then
					if c < 1 then
						civ.deleteUnit(unit)
						c = c + 1
					end
				end    
			end				
			
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - (a + b)) * param.witdrawalMechanizedPenalty)
			penaltyCost = (totalUnitsToWithdraw - (a + b)) * param.witdrawalMechanizedPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the 1st Free French Motorized Division is withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - (a + b)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# Free French Infantry",3)
			counter.subtract("# FreeFrenchM10",2)
			counter.subtract("# Free French 105Art",1)
			
			counter.subtract("# Total Free French",6)
			
			counter.subtract("# Liberty Ship",1)
			counter.subtract("# Total Naval Units",1)			
		end)	
	end		

	-- 4 GOUMIERS MAROCAIN REGIMENTS WITHDRAW
	if turn == 49 then -- Jul I, 1944
		gen.justOnce("Withdraw Goumier Marocain Regiments",function()
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 4
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and unit.type == object.uGoumierMarocain and unit.location ~= object.ltransportPoint then 
					civ.deleteUnit(unit)
					a = a + 1
				end
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw .. " " .. object.uGoumierMarocain.name .. " are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			flag.setFalse("Goumier Marocain units arrive") 
			counter.subtract("# Goumier Marocain",4)
			counter.subtract("# Total Free French",4)
		end)	
	end	

	-- 2ND MAROCCAN AND 3RD ALGERIAN INFANTRY DIVISIONS WITHDRAW  
	if turn == 50 then -- Jul II, 1944
		gen.justOnce("Withdraw Tirailleur Algerien Regiments",function()
			local a = 0
			local b = 0
			local penaltyCost1 = 0
			local penaltyCost2 = 0
			local totalUnitsToWithdraw1 = 6
			local totalUnitsToWithdraw2 = 1
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and unit.type == object.uTirailleurAlgerien and unit.location ~= object.ltransportPoint then 
					if a < 6 then						
						civ.deleteUnit(unit)
						a = a + 1
					end	
				end
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uFreeFrench105Art and unit.location ~= object.ltransportPoint then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end	
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			-- object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			-- penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			object.pAllies.money = object.pAllies.money - (((totalUnitsToWithdraw1 - (a)) * param.witdrawalInfantryPenalty) + ((totalUnitsToWithdraw2 - (b)) * param.witdrawalMechanizedPenalty))
			penaltyCost1 = (totalUnitsToWithdraw1 - (a)) * param.witdrawalInfantryPenalty 
			penaltyCost2 = (totalUnitsToWithdraw2 - (b)) * param.witdrawalMechanizedPenalty					
			-- text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw .. " " .. object.uTirailleurAlgerien.name .. " are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw1 .. " " .. object.uTirailleurAlgerien.name .. " and " .. totalUnitsToWithdraw2 .. " " .. object.uFreeFrench105Art.name .." are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw1 - (a)) + (totalUnitsToWithdraw2 - (b)) .. " units were missing for a penalty of " .. penaltyCost1 + penaltyCost2 .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			
			flag.setFalse("Goumier Marocain units arrive") 
			counter.subtract("# Tirailleur Algerien",6)
			counter.subtract("# Free French 105Art",1)			
			counter.subtract("# Total Free French",7)
		end)
	end

	-- 4TH MAROCCAN AND 9TH COLONIALE INFANTRY DIVISIONS WITHDRAW  
	if turn == 51 then -- Jul III, 1944
		gen.justOnce("Withdraw 4th Maroccan and 9th Coloniale Divisions",function()
			local a = 0
			local b = 0
			-- local penaltyCost = 0
			-- local totalUnitsToWithdraw = 6
			local penaltyCost1 = 0
			local penaltyCost2 = 0
			local totalUnitsToWithdraw1 = 6
			local totalUnitsToWithdraw2 = 1			
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and (unit.type == object.uTirailleurAlgerien or unit.type == object.uInfanterieColoniale) and unit.location ~= object.ltransportPoint then 
					if a < 6 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uFreeFrench105Art and unit.location ~= object.ltransportPoint then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end	
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			-- object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			-- penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			object.pAllies.money = object.pAllies.money - (((totalUnitsToWithdraw1 - (a)) * param.witdrawalInfantryPenalty) + ((totalUnitsToWithdraw2 - (b)) * param.witdrawalMechanizedPenalty))
			penaltyCost1 = (totalUnitsToWithdraw1 - (a)) * param.witdrawalInfantryPenalty 
			penaltyCost2 = (totalUnitsToWithdraw2 - (b)) * param.witdrawalMechanizedPenalty								
			-- text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the final " .. totalUnitsToWithdraw .. " " .. object.uTirailleurAlgerien.name .. " and " .. object.uInfanterieColoniale.name .. " are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", the final " .. totalUnitsToWithdraw1 .. " " .. object.uTirailleurAlgerien.name .. ", " .. object.uInfanterieColoniale.name .. " and " .. totalUnitsToWithdraw2 .. " " .. object.uFreeFrench105Art.name .." are withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw1 - a) + (totalUnitsToWithdraw2 - b) .. " units were missing for a penalty of " .. penaltyCost1 + penaltyCost2 .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)

			flag.setFalse("Free French units arrive") 
			flag.setFalse("Tirailleur Algérien units arrive")
			counter.subtract("# Infanterie Coloniale",3)
			counter.subtract("# Tirailleur Algerien",3)
			counter.subtract("# Free French 105Art",1)			
			counter.subtract("# Total Free French",7)
		end)
	end	

	-- US AIR FORCE FIGHTER WITHRAWALS 
	if turn == 52 then -- July IV, 1944 (#52)
		gen.justOnce("Withdraw US Air Force Fighters",function()
			local a = 0
			local b = 0
			local c = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 3
			for unit in civ.iterateUnits() do 
				if unit.type == object.uP40Warhawk then
					if a < 1 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uP38Lightning then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end 
			for unit in civ.iterateUnits() do 
				if unit.type == object.uA36Apache then
					if c < 1 then
						civ.deleteUnit(unit)
						c = c + 1
					end
				end    
			end 		
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - (a + b + c)) * param.witdrawalAirPenalty)
			penaltyCost = (totalUnitsToWithdraw - (a + b + c)) * param.witdrawalAirPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", a total of " .. totalUnitsToWithdraw .. " aircraft, a " .. object.uP40Warhawk.name .. ", a " .. object.uP38Lightning.name .. " and a " .. object.uA36Apache.name .. " units are scheduled to be withdrawn this turn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - (a + b + c)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# P-40",1)
			counter.subtract("# P-38",1)
			counter.subtract("# A-36",1)

			counter.subtract("# Total US Fighters",2)
			counter.subtract("# Total US Bombers",1)
		end)
	end	
	
	-- US AIR FORCE 1ST GROUP OF BOMBER WITHRAWALS 
	if turn == 53 then -- August I, 1944 (#53)
		gen.justOnce("Withdraw US Air Force 1st Bomber Group",function()
			local a = 0
			local b = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 2
			for unit in civ.iterateUnits() do 
				if unit.type == object.uB25Mitchell then
					if a < 1 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uB26Marauder then
					if b < 1 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end 
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - (a + b)) * param.witdrawalAirPenalty)
			penaltyCost = (totalUnitsToWithdraw - (a + b)) * param.witdrawalAirPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", a total of " .. totalUnitsToWithdraw .. " aircraft, a " .. object.uB25Mitchell.name .. " and a " .. object.uB26Marauder.name .. " are scheduled to be withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - (a + b)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# B-25",1)
			counter.subtract("# B-26",1)
			counter.subtract("# Total US Bombers",2)
		end)	
	end	

	-- US AIR FORCE 2ND GROUP OF BOMBER WITHRAWALS 
	if turn == 54 then -- August II, 1944 (#54)
		gen.justOnce("Withdraw US Air Force 2nd Bomber Group",function()
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 1
			for unit in civ.iterateUnits() do 
				if unit.type == object.uB17Fortress then
					if a < 1 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - (totalUnitsToWithdraw - a) * param.witdrawalAirPenalty * 2
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalAirPenalty
			text.simple("As per the Special Bulletin # " .. counter.value("Special Bulletin #") .. ", " .. totalUnitsToWithdraw .. " " .. object.uB17Fortress.name .. " is scheduled to be withdrawn from the theater in preparation for the invasion of southern France! " .. (totalUnitsToWithdraw - a) .. " unit was missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# B-17",1)
			counter.subtract("# Total US Bombers",1)
		end)	
	end	

	-- US 442nd REGIMENTS WITDRAWS
	if turn == 60 then -- Sept IV, 1944
		gen.justOnce("Withdraw 442nd regiment",function()
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 1
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and unit.type == object.uUS442ndRegt and unit.location ~= object.ltransportPoint then 
					civ.deleteUnit(unit)
					a = a + 1				
				end
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			text.simple("The US 442nd regiment has been withdrawn! " .. (totalUnitsToWithdraw - a) .. " unit was missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 6th Army Group",object.m6thArmyGrp)
			counter.subtract("# US 442nd Regt",1)
			counter.subtract("# Total US Infantry",1) 
		end)	
	end

    -- 4TH BRITISH INFANTRY DIVISIONS WITHDRAWS
	-- if turn == 70 then -- Dec II, 1944
        -- withdrawTroops(object.uBritishRegular,3,"4th British Infantry")
		-- param.maxtotalBritishInfantryReg = param.maxtotalBritishInfantryReg - 3 -- maxtotalBritishInfantryReg should equal 9 of 9 now
		-- param.UKInfantry = param.maxtotalBritishInfantryVet - param.maxtotalBritishInfantryReg - param.maxtotalBritishPIAT - param.maxtotalBritishMG - param.maxtotalCommandos - param.maxtotal1stAirborne
    -- end
	
	if turn == 70 then -- Dec II, 1944
		gen.justOnce("Withdraw 4th British Infantry Division",function()	
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 3
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishRegular and unit.location ~= object.ltransportPoint then
					if a < 3 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw .. " " .. object.uBritishRegular.name .. " are withdrawn from the theater! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 21st Army Group",object.m21stArmyGrp)
			
			counter.subtract("# British Regular",3)
			counter.subtract("# Total UK Infantry",3)
		end)	
	end	

    -- GREEK MOUNTAIN BREIGADE WITHDRAWS
	if turn == 71 then -- Dec III, 1944
		gen.justOnce("Withdraw Greek Mtn Brigade",function()	
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 1
			for unit in civ.iterateUnits() do 
				if unit.type == object.uGreekMtnBde and unit.location ~= object.ltransportPoint then
					if a < 1 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw .. " " .. object.uGreekMtnBde.name .. " is withdrawn from the theater! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to Greece",object.mAFHQ)
			
			counter.subtract("# Greek Mtn Bde",1)
			counter.subtract("# Total Brazil et AL",1)
		end)	
	end		

    -- 4TH INDIAN INFANTRY DIVISION WITHDRAWS
	-- if turn == 72 then -- Dec IV, 1944
        -- withdrawTroops(object.u4thIndianDiv,3,"4th Indian Infantry")
		-- flag.setFalse("4th Indian Infantry units arrive") 
		-- param.maxtotal4thIndianDiv = param.maxtotal4thIndianDiv - 3 -- maxtotal4thIndianDiv should equal 0 of 0 now
		-- param.indian = param.maxtotalIndianInfantry - param.maxtotal4thIndianDiv - param.maxtotalGurkhas - param.maxtotalIndian25pdrArt
    -- end	
	
	if turn == 72 then -- Dec IV, 1944
		gen.justOnce("Withdraw 4th Indian Infantry Division",function()		
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 3
			for unit in civ.iterateUnits() do
				if unit.owner == object.pAllies and unit.type == object.u4thIndianDiv and unit.location ~= object.ltransportPoint then 
					civ.deleteUnit(unit)
					a = a + 1
				end
			end
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalInfantryPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw .. " regiments of the " .. object.u4thIndianDiv.name .. " are shipped to Greece to help stabilise the country after the Axis withdraw! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to Greece",object.mAFHQ)
			counter.subtract("# 4th Indian Div",3)
			counter.subtract("# Total Indian",3) 
		end)	
	end
	
	-- 25TH BRITISH TANK BRIGADE WITHDRAWS
	if turn == 73 then -- Jan I, 1945
		gen.justOnce("Withdraw 25th British Tank Brigade",function()	
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 2
			for unit in civ.iterateUnits() do 
				if unit.type == object.uShermanIIaBde then
					if a < 2 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end    
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalMechanizedPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalMechanizedPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " the " .. totalUnitsToWithdraw .. " " .. object.uShermanIIaBde.name .. " are withdrawn from the theater! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 21st Army Group",object.m21stArmyGrp)
			counter.subtract("# Sherman IIa Bde",2)
			counter.subtract("# Total UK Armored",2)
		end)	
	end	

	-- 1ST CANADIAN TANK BRIGADE WITHDRAWS
	if turn == 74 then -- Jan I, 1945
		gen.justOnce("Withdraw 1st Canadian Tank Brigade",function()	
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 2
			for unit in civ.iterateUnits() do 
				if unit.type == object.uCdnShermanIII then
					if a < 2 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end    
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalMechanizedPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalMechanizedPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", " .. totalUnitsToWithdraw .. " " .. object.uCdnShermanIII.name .. " were scheduled to be withdrawn from the theater in preparation for the invasion of Germany! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 21st Army Group",object.m21stArmyGrp)
			counter.subtract("# Cdn Sherman",2)
			counter.subtract("# Total Canadian",2)
		end)	
    end	

	-- 1ST CANADIAN AND 1ST BRITISH INFANTRY DIVISIONS WITHDRAW
	if turn == 75 then -- Jan III, 1945 (#75)
		gen.justOnce("Withdraw 1st Canadian and British Infantry Divisions",function()	 
			local a = 0
			local b = 0
			local penaltyCost = 0
			local totalUnitsToWithdrawInf = 6
			for unit in civ.iterateUnits() do 
				if unit.type == object.uCdnInfantry and unit.location ~= object.ltransportPoint then
					if a < 3 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end
			for unit in civ.iterateUnits() do 
				if unit.type == object.uBritishVeteran and unit.location ~= object.ltransportPoint then
					if b < 3 then
						civ.deleteUnit(unit)
						b = b + 1
					end
				end    
			end	
					
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdrawInf - (a + b)) * param.witdrawalInfantryPenalty)
			penaltyCost = (totalUnitsToWithdrawInf - (a + b)) * param.witdrawalInfantryPenalty
			-- object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdrawInf - (a + b + c + d)) * param.witdrawalInfantryPenalty) - ((totalUnitsToWithdrawMech - (e + f + g + h)) * param.witdrawalMechanizedPenalty)
			-- penaltyCost = ((totalUnitsToWithdrawInf - (a + b + c + d)) * param.witdrawalInfantryPenalty) + ((totalUnitsToWithdrawMech - (e + f + g + h)) * param.witdrawalMechanizedPenalty)
			
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", a total of " .. totalUnitsToWithdrawInf .. " units from the 1st Canadian and 1st British Infantry divisions were scheduled to be withdrawn from the theater in preparation for the invasion of Germany, including 3 " .. object.uCdnInfantry.name .. " and 3 " .. object.uBritishVeteran.name .. "! " .. (totalUnitsToWithdrawInf - (a + b)) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 21st Army Group",object.m21stArmyGrp)
			counter.subtract("# Cdn Infantry",3)
			counter.subtract("# Total Canadian",3)

			counter.subtract("# British Veteran",3)
			counter.subtract("# Total UK Infantry",3)

		end)	
	end	

	-- 5TH CANADIAN ARMORED DIVISION WITHDRAWS 
	if turn == 76 then -- Jan IV, 1945 (#76)
		gen.justOnce("Withdraw 5th Canadian Armored Division",function()	 
			local a = 0
			local penaltyCost = 0
			local totalUnitsToWithdraw = 6
			for unit in civ.iterateUnits() do 
				if unit.type == object.uCdnMotInf or unit.type == object.uCdnMotorTroops or unit.type == object.uCdnShermanIII or unit.type == object.uCdnM7105mm then
					if a < 6 then
						civ.deleteUnit(unit)
						a = a + 1
					end
				end    
			end    
			-- Calculate how many units were missing from the total to be withdrawn and subtract equivalent supplies from the treasury
			object.pAllies.money = object.pAllies.money - ((totalUnitsToWithdraw - a) * param.witdrawalMechanizedPenalty)
			penaltyCost = (totalUnitsToWithdraw - a) * param.witdrawalMechanizedPenalty
			text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. ", a total of " .. totalUnitsToWithdraw .. " units from the 5th Canadian Armored Divison were scheduled to be withdrawn from the theater in preparation for the invasion of Germany, including 1 " .. object.uCdnMotInf.name .. ", 1 " .. object.uCdnMotorTroops.name .. ", 3 " .. object.uCdnShermanIII.name .. " and 1 " .. object.uCdnM7105mm.name .. "! " .. (totalUnitsToWithdraw - a) .. " units were missing for a penalty of " .. penaltyCost .. " supplies!","Reassigned to 21st Army Group",object.m21stArmyGrp)
			flag.setTrue("Canadian units withdrawn")
			flag.setFalse("Cdn 5th Armored units arrive")
			counter.subtract("# Cdn Sherman",3)
			counter.subtract("# Cdn Mot Inf",1)
			counter.subtract("# Cdn Bren Carrier",1)
			counter.subtract("# Cdn M7 105mm",1)
			
			counter.subtract("# Total Canadian",6)
		end)	
	end		
	
	-- ============================
	-- REMOVE HOME CITY FROM AXIS UNITS (EXCEPTION: ReggimentoCostiero )
	for activeUnit in civ.iterateUnits() do
		if activeUnit.owner == object.pAxis and activeUnit.type ~= object.uReggimentoCostiero then
			if activeUnit.homeCity ~= nil then
				print(activeUnit.type.name .. " (" .. activeUnit.owner.adjective .. ") home city changed from " .. activeUnit.homeCity.name .. " to NONE")
				activeUnit.homeCity = nil
			end
		end	
	end
	
	for activeUnit in civ.iterateUnits() do
		if activeUnit.owner == object.pAllies and (not tables.alliedAirUnit[activeUnit.type.id]) then
			if activeUnit.homeCity ~= nil then
				print(activeUnit.type.name .. " (" .. activeUnit.owner.adjective .. ") home city changed from " .. activeUnit.homeCity.name .. " to NONE")
				activeUnit.homeCity = nil
			end
		end	
	end	
	
	-- ============================
	-- METHOD 1: DELETE UNUSED MUNITIONS AT THE BEGINNING OF EACH TURN
	-- if turn >= 1 then 
		-- for unit in civ.iterateUnits() do 
			-- if (unit.type == object.uHeavyShell or unit.type == object.u105mmShell or unit.type == object.u155mmShell or unit.type == object.u250lbBomb or unit.type == object.u500lbBomb or unit.type == object.uFlak) 
			-- and unit.owner == object.pAxis then	
				-- civ.deleteUnit(unit) 
			-- end 
		-- end
	-- end 

	-- METHOD 2: DELETE UNUSED AXIS MUNITONS AT THE BEGINNING OF EACH TURN by calling the tables.munitionsToBeDestroyed table (also see onTurn.lua for deleting ALLIED munitions)
	if turn >= 1 then 
		for activeUnit in civ.iterateUnits() do
		   if activeUnit.owner == object.pAxis and tables.munitionsToBeDestroyed[activeUnit.type.id] then 
				print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
				civ.deleteUnit(activeUnit)
			end
		end
	end	
	
	-- if turn >= 1 then 
		-- for activeUnit in civ.iterateUnits() do
		   -- if activeUnit.owner == object.pAxis and activeUnit.type == object.uFritzX and (not gen.hasAirbase(activeUnit.location) or activeUnit.location.baseTerrain ~= object.bTownCenter) then 
				-- print("Destroyed " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
				-- civ.deleteUnit(activeUnit)
			-- end
		-- end
	-- end	


	-- ============================
	-- AI RANGED UNITS MODULE
	-- 2 following lines of code are required to work with aiRangedUnits.lua module.
	-- At top of this file 'local aiRangedUnits = require("aiRangedUnits")' plus the following uncommented line here:
	aiRangedUnits.processRangedUnits(tribe)
	
end
---&endAutoDoc

return register
