local flag = require("flag")
local counter = require("counter")
local text = require("text")
local gen = require("generalLibrary")

local object = gen.makeDataTable({},"object")
-- This line forbids reassignment of keys of the object table
-- This should prevent errors
gen.forbidReplacement(object)

-- Civilization Advances
-- recommended key prefix 'a'

object.aEarlyRadarSystem        = civ.getTech(0)
object.aNotUsed                 = civ.getTech(1)
object.aUSNavy                  = civ.getTech(2)
object.aRegiaAeronautica        = civ.getTech(3)
object.aKriegsmarine            = civ.getTech(4)
object.aNotUsed5                = civ.getTech(5)   --Automobile
object.aEngland                 = civ.getTech(6)
object.aRailTransport           = civ.getTech(7)   --Bridge Building
object.aYaltaConference         = civ.getTech(8)
object.aPrintingPress           = civ.getTech(9)
object.aMinorPowers             = civ.getTech(10)
object.aItalia                  = civ.getTech(11)
object.aCommonwealth            = civ.getTech(12)
object.aDeutschland             = civ.getTech(13)
object.aTACAirForce             = civ.getTech(14)
object.aAxis                    = civ.getTech(15)
object.aBritishArtillery        = civ.getTech(16)
object.aNotUsed17               = civ.getTech(17)
object.aNotUsed18               = civ.getTech(18)
object.aItalianArmy             = civ.getTech(19)
object.aBritishEightArmy        = civ.getTech(20)
object.aUnitedStates            = civ.getTech(21)
object.aUrbanism                = civ.getTech(22)
object.aOKH                     = civ.getTech(23)
object.aNotUsed24               = civ.getTech(24)
object.aCanada                  = civ.getTech(25)
object.aIndia                   = civ.getTech(26)
object.aNewZealand              = civ.getTech(27)
object.aNotUsed27               = civ.getTech(28)
object.aMunitions               = civ.getTech(29)
object.aSACAirForce             = civ.getTech(30)
object.aNotUsed31               = civ.getTech(31)
object.aNotUsed32               = civ.getTech(32)
object.aBattleofItaly           = civ.getTech(33)
object.aNotUsed34               = civ.getTech(34)
object.aNotUsed35               = civ.getTech(35)
object.aUSArmor                 = civ.getTech(36)
object.aAllies                  = civ.getTech(37)
object.aNotUsed38               = civ.getTech(38)
object.aBritishArmor            = civ.getTech(39)
object.aUSAAF                   = civ.getTech(40)
object.aUSFifthArmy             = civ.getTech(41)
object.aPanzerkorps             = civ.getTech(42)
object.aWeaponsDesign           = civ.getTech(43)
object.aUSArtillery             = civ.getTech(44)
object.aNotUsed45               = civ.getTech(45)
object.aNavalBase               = civ.getTech(46)
object.aRegioEsercito           = civ.getTech(47)
object.aIndustrialism           = civ.getTech(48)
object.aCivilDefense            = civ.getTech(49)
object.aNotUsed50               = civ.getTech(50)
object.aUSArmy                  = civ.getTech(51)
object.aBritishArmy             = civ.getTech(52)
object.aNotUsed53               = civ.getTech(53)
object.aNotUsed54               = civ.getTech(54)
object.aWarOffice               = civ.getTech(55)
object.aMassMedia               = civ.getTech(56)
object.aRoyalNavy               = civ.getTech(57)
object.aNotUsed58               = civ.getTech(58)
object.aNotUsed59               = civ.getTech(59)
object.aNotUsed60               = civ.getTech(60)
object.aItalianArtillery        = civ.getTech(61)
object.aNotUsed62               = civ.getTech(62)
object.aImpossibleTech          = civ.getTech(63)
object.aNotUsed64               = civ.getTech(64)
object.aSouthAfrica             = civ.getTech(65)
object.aNotUsed66               = civ.getTech(66)
object.aNotUsed67               = civ.getTech(67)
object.aFreeFrench              = civ.getTech(68)
object.aPoland                  = civ.getTech(69)
object.aWarRationing            = civ.getTech(70)
object.aNotUsed71               = civ.getTech(71)
object.aGermanArtillery         = civ.getTech(72)
object.aLuftwaffe               = civ.getTech(73)
object.aNotUsed74               = civ.getTech(74)
object.aRoyalNavalArm           = civ.getTech(75)
object.aTroopCarrierCommand     = civ.getTech(76)
object.aFortifications          = civ.getTech(77)
object.aRegiaMarina             = civ.getTech(78)
object.aWehrmacht               = civ.getTech(79)
object.aFlakBatteries           = civ.getTech(80)
object.aRoyalAirForce           = civ.getTech(81)
object.aConditioning            = civ.getTech(82)
object.a25thRoyalEngineer       = civ.getTech(83)
object.aCommerce                = civ.getTech(84)
object.aNotUsed85               = civ.getTech(85)
object.aWaffenSS                = civ.getTech(86)
object.aItalianMechanized       = civ.getTech(87)
object.aNotUsed88               = civ.getTech(88)
object.aBattlePlan              = civ.getTech(89)
object.aBraziletal              = civ.getTech(90)
object.aBritishArmDiv           = civ.getTech(91)
object.aAir                     = civ.getTech(92)
object.aGround                  = civ.getTech(93)
object.aNaval                   = civ.getTech(94)
object.aAdvRadarResearch        = civ.getTech(95)
object.aAdvRadarSystem          = civ.getTech(96)
object.aOperationDiadem         = civ.getTech(97)
object.aOperationGrapeshot      = civ.getTech(98)
object.aExtraAdvance7           = civ.getTech(99)

-- Map Locations (tiles/squares)
-- recommended key prefix 'l'

object.lobjectiveMilano  	= civ.getTile(70,0,0)
object.lobjectiveTorino 	= civ.getTile(62,0,0)

-- object.lPantelleria 		= civ.getTile(18,148,0)
object.lAirbaseCatania 		= civ.getTile(59,161,0)
object.lAirbaseMessina 		= civ.getTile(73,153,0)
object.lAirbasePantelleria 	= civ.getTile(18,148,0)
object.lPantelleriaNorth 	= civ.getTile(21,147,0) -- island used to place Supply Hub unit on 1st turn
object.lAirbaseNW_Roma 		= civ.getTile(78,72,0)
object.lAirbaseTrapani 		= civ.getTile(33,139,0)
object.lAirbaseValetta 		= civ.getTile(39,177,0)

object.lseamine1	 		= civ.getTile(79,155,0)  -- To prevent early invasion attempt of Italian mainland
object.lseamine2	 		= civ.getTile(88,152,0)  -- To prevent early invasion attempt of Italian mainland
object.lseamine3	 		= civ.getTile(89,109,0)  -- To prevent early invasion attempt of Italian mainland
object.lseamine4	 		= civ.getTile(92,114,0)  -- To prevent early invasion attempt of Italian mainland
object.lseamine5	 		= civ.getTile(104,158,0) -- To prevent early invasion attempt of Italian mainland
object.lseamine6	 		= civ.getTile(130,138,0) -- To prevent early invasion attempt of Italian mainland

object.lsealane	 			= civ.getTile(72,36,0)   -- To prevent early invasion attempt of Italian mainland

object.ltransportPoint 		= civ.getTile(129,185,0) -- Tile where truck transported units temporarily reside

object.lbeachBonifacio 		= civ.getTile(40,58,0)

object.lBHLicata			= civ.getTile(42,162,0)
object.lHarborCrotone		= civ.getTile(104,156,0)

object.lfortNW_Cassino		= civ.getTile(90,90,0)   -- Gustav Line Objective
object.lfortS_Firenzuola	= civ.getTile(94,42,0)
object.lfortSE_Firenzuola	= civ.getTile(97,45,0)
object.lfortNE_Formia		= civ.getTile(88,94,0)   -- Gustav Line Objective
object.lfortNE_Isernia		= civ.getTile(100,90,0)  -- Gustav Line Objective
object.lfortNNE_Isernia		= civ.getTile(100,86,0)  -- Gustav Line Objective
object.lfortNW_Ortona		= civ.getTile(105,87,0)  -- Gustav Line Objective
object.lfortS_Rimini		= civ.getTile(106,48,0)
object.lfortSW_Rimini		= civ.getTile(101,47,0)

object.lGrassNNTrieste		= civ.getTile(137,25,0)

object.lhillNWBolsena		= civ.getTile(84,60,0)
object.lhillCivitavecchia	= civ.getTile(77,67,0)
object.lhillNECassino		= civ.getTile(92,92,0)
object.lhillSCuneo			= civ.getTile(57,9,0)
object.lhillSWFabriano	    = civ.getTile(99,59,0)
object.lhillNEFirenze		= civ.getTile(91,37,0)
object.lhillSSFirenze		= civ.getTile(89,47,0)
object.lhillNEFirenzuola	= civ.getTile(96,32,0)
object.lhillNEMassa 		= civ.getTile(80,26,0)
object.lhillSEMontalcino	= civ.getTile(87,53,0)
object.lhillSWRimini  		= civ.getTile(104,46,0)
object.lhillSSEROMA  		= civ.getTile(82,80,0)
object.lhillNWSiena  		= civ.getTile(83,45,0)
object.lhillNETerontola		= civ.getTile(92,52,0)
object.lhillSWValmontone	= civ.getTile(83,83,0)

object.loceanCivitavecchia1	= civ.getTile(74,68,0)
object.loceanCivitavecchia2	= civ.getTile(75,71,0)
object.loceanTalamone		= civ.getTile(73,57,0)

object.lroughNW_Ancona		= civ.getTile(109,57,0)
object.lroughSW_Firenze		= civ.getTile(88,40,0)
object.lroughSE_Macerata	= civ.getTile(109,65,0)
object.lroughS_Rimini		= civ.getTile(106,48,0)
object.lroughSW_Rimini		= civ.getTile(105,85,0)
object.lroughSE_ROMA		= civ.getTile(81,79,0)

object.ltownAdrano			= civ.getTile(60,158,0)  -- Town of interest
object.ltownAncona			= civ.getTile(111,61,0)  -- Town of interest
object.ltownAnzio			= civ.getTile(78,84,0)
object.ltownArezzo			= civ.getTile(93,49,0)
object.ltownAscoli			= civ.getTile(104,72,0)  -- 10th Army Cut Off & Trasimene Line optional
object.ltownBarietta		= civ.getTile(117,117,0) -- Barbara Line optional
object.ltownBolsena 		= civ.getTile(85,61,0)   -- Town of interest
object.ltownBonifacio 		= civ.getTile(38,58,0)   -- Town of interest
object.ltownCaltanissetta	= civ.getTile(47,155,0)  -- Town of interest
object.ltownCassino 		= civ.getTile(91,93,0)   -- Gustav Line Objective
object.ltownCastelnovo		= civ.getTile(86,24,0)
object.ltownCastrovillari	= civ.getTile(100,138,0) -- Town of interest
object.ltownCatanzaro		= civ.getTile(94,156,0)  -- Town of interest
object.ltownCecina			= civ.getTile(75,39,0)   -- Town of interest
object.ltownCivitavecchia	= civ.getTile(75,67,0)   -- Trasimene Line optional
object.ltownCrotone			= civ.getTile(103,157,0) -- Town of interest
object.ltownCuneo 			= civ.getTile(58,2,0)
object.ltownFabriano 		= civ.getTile(100,58,0)
object.ltownFano 			= civ.getTile(109,51,0)
object.ltownFiumicino		= civ.getTile(77,75,0)   -- Trasimene Line optional
object.ltownFoligno			= civ.getTile(95,63,0)   -- Town of interest
object.ltownFormia 			= civ.getTile(87,95,0)
object.ltownFrosinone		= civ.getTile(88,86,0)
object.ltownGiolaTauro		= civ.getTile(80,156,0)
object.ltownGrosseto		= civ.getTile(74,52,0)   -- Town of interest
object.ltownLagonegro		= civ.getTile(99,127,0)  -- Barbara Line optional
object.ltownLAquila			= civ.getTile(97,79,0)   -- Town of interest
object.ltownLavello			= civ.getTile(110,116,0) -- Barbara Line optional
object.ltownLeccio			= civ.getTile(94,42,0)   -- 2nd Sudden Death Condition
object.ltownLicata			= civ.getTile(43,161,0)
object.ltownLidoDiRoma		= civ.getTile(77,77,0)
object.ltownLucca			= civ.getTile(82,30,0)	 -- 2nd Sudden Death Condition
object.ltownMassa 			= civ.getTile(79,27,0)   -- 2nd Sudden Death Condition
object.ltownMonopoli		= civ.getTile(126,130,0)
object.ltownMontalcino		= civ.getTile(83,51,0)
object.ltownOrtona	 		= civ.getTile(107,87,0)  -- Town of interest
object.ltownOrvieto	 		= civ.getTile(89,61,0) 
object.ltownOstuni			= civ.getTile(128,138,0)
object.ltownPerugia			= civ.getTile(94,58,0)   -- Town of interest
object.ltownPisa	 		= civ.getTile(78,34,0)
object.ltownPistoia 		= civ.getTile(88,34,0)   -- 2nd Sudden Death Condition
object.ltownPizzo			= civ.getTile(88,154,0)
object.ltownPompei			= civ.getTile(90,110,0)  -- Barbara Line optional & Town of interest
object.ltownPortoscuso		= civ.getTile(6,86,0)
object.ltownPotenza			= civ.getTile(105,123,0) -- Barbara Line optional & Town of interest
object.ltownPracchia 		= civ.getTile(87,31,0)
object.ltownRieti			= civ.getTile(92,72,0)   -- 10th Army Cut Off & Trasimene Line optional
object.ltownSalerno			= civ.getTile(93,113,0)  -- Barbara Line optional
object.ltownSalo			= civ.getTile(103,7,0)
object.ltownSanBenedetto	= civ.getTile(108,72,0)  -- 10th Army Cut Off & Trasimene Line optional & Town of interest
object.ltownSanMarino 		= civ.getTile(104,46,0)  -- 2nd Sudden Death Condition
object.ltownSapri			= civ.getTile(95,129,0)  -- Barbara Line optional
object.ltownSciacci			= civ.getTile(36,148,0)
object.ltownSiena			= civ.getTile(84,46,0)
object.ltownSolignano		= civ.getTile(83,19,0)
object.ltownTalamone		= civ.getTile(74,56,0)
object.ltownTerontola		= civ.getTile(91,53,0)
object.ltownTerracina		= civ.getTile(82,92,0)
object.ltownTermoli			= civ.getTile(111,95,0)  -- Barbara Line optional
object.ltownTerni 			= civ.getTile(91,67,0)
object.ltownTivoli			= civ.getTile(85,77,0)   -- 10th Army Cut Off & Trasimene Line optional
object.ltownUrbino	 		= civ.getTile(103,51,0)
object.ltownValmontone 		= civ.getTile(84,82,0)   -- 10th Army Cut Off 
object.ltownVergato 		= civ.getTile(93,29,0) 
object.ltownVicenza	 		= civ.getTile(112,16,0) 
object.ltownViterbo 		= civ.getTile(82,66,0) 


-- object.lCataniaDock			= civ.getTile(63,161,0)
-- object.lSClass				= civ.getTile(69,171,0)
object.lroughTile1				= civ.getTile(1,137,0)
object.lroughTile2				= civ.getTile(80,24,0)
object.lroughTile3				= civ.getTile(3,137,0)

-- Locations of cities starting the scenario owned by the Barbarians:


-- Locations of cities starting the scenario owned by the Axis:

object.lVerona                  =civ.getTile(106,14,0)
object.lROMA                    =civ.getTile(80,76,0)
object.lNapoli                  =civ.getTile(88,106,0)
object.lPalermo                 =civ.getTile(44,140,0)
object.lGenova                  =civ.getTile(72,12,0)
object.lBologna                 =civ.getTile(100,30,0)
object.lBari                    =civ.getTile(123,125,0)
object.lCatania                 =civ.getTile(62,162,0)
object.lFirenze                 =civ.getTile(89,39,0)
object.lBrescia                 =civ.getTile(98,4,0)
object.lMessina                 =civ.getTile(73,155,0)
object.lPadova                  =civ.getTile(115,21,0)
object.lTaranto                 =civ.getTile(120,138,0)
object.lTrieste                 =civ.getTile(137,31,0)
object.lCagliari                =civ.getTile(14,90,0)
object.lFoggia                  =civ.getTile(111,107,0)
object.lLivorno                 =civ.getTile(75,35,0)
object.lParma                   =civ.getTile(93,17,0)
object.lReggioCalabria          =civ.getTile(73,159,0)
object.lReggioEmilia            =civ.getTile(93,21,0)
object.lCremona                 =civ.getTile(91,9,0)
object.lFerrara                 =civ.getTile(105,27,0)
object.lForli                   =civ.getTile(103,39,0)
object.lLaSpezia                =civ.getTile(77,23,0)
object.lRavenna                 =civ.getTile(107,37,0)
object.lSiracusa                =civ.getTile(59,169,0)
object.lMantova                 =civ.getTile(101,17,0)
object.lModena                  =civ.getTile(96,24,0)
object.lVenezia                 =civ.getTile(119,25,0)
object.lPiacenza                =civ.getTile(87,9,0)
object.lPavia                   =civ.getTile(84,2,0)
object.lAlessandria             =civ.getTile(75,1,0)
object.lSavona                  =civ.getTile(66,10,0)
object.lTrapani                 =civ.getTile(32,136,0)
object.lPescara                 =civ.getTile(107,83,0)
object.lUdine                   =civ.getTile(136,20,0)
object.lAjaccio                 =civ.getTile(40,48,0)
object.lBrindisi                =civ.getTile(130,142,0)
object.lRimini                  =civ.getTile(106,44,0)
object.lSassari                 =civ.getTile(22,64,0)
object.lBastia                  =civ.getTile(53,47,0)
object.lOlbia                   =civ.getTile(36,68,0)
object.lTurino                  =civ.getTile(62,0,1)
object.lMilano                  =civ.getTile(70,0,1)

-- Locations of cities starting the scenario owned by the Japanese:


-- Locations of cities starting the scenario owned by the Germans:


-- Locations of cities starting the scenario owned by the Spanish:


-- Locations of cities starting the scenario owned by the Americans:


-- Locations of cities starting the scenario owned by the Allies:

object.lValetta                 =civ.getTile(40,180,0)
object.lTunis                   =civ.getTile(0,136,0)

-- Locations of cities starting the scenario owned by the Indians:


-- Cities
-- recommended key prefix 'c'
-- It is not recommended to put cities into this list if the city
-- can be destroyed. This list returns an error if 'nil' is the value
-- associated with the key (see bottom of file), so that could cause
-- a problem if a city in this list is destroyed.  Also, if another
-- city is founded, the ID number of the city might get reused, causing
-- more confusion.  An alternate way to reference a city is by using
-- object.lRome.city when you actually need the city (and suitably guarding
-- against nil values)

--Find these by entering "for city in civ.iterateCities() do print(city.id, city.name) end" in the console

-- All the cities existing when you made generated this template are listed here.
-- If you wish to use these objects, change the line if false then below to if true then
-- It is your job to eliminate any cities that could be destroyed from this list if you use it
-- If you are not sure, it is recommended to reference cities from their locations instead
--

if true then

--Cities starting the scenario owned by the Barbarians:


--Cities starting the scenario owned by the Axis:

object.cVerona                  =civ.getTile(106,14,0).city
object.cROMA                    =civ.getTile(80,76,0).city
object.cNapoli                  =civ.getTile(88,106,0).city
object.cPalermo                 =civ.getTile(44,140,0).city
object.cGenova                   =civ.getTile(72,12,0).city
object.cBologna                 =civ.getTile(100,30,0).city
object.cBari                    =civ.getTile(123,125,0).city
object.cCatania                 =civ.getTile(62,162,0).city
object.cFirenze                 =civ.getTile(89,39,0).city
object.cBrescia                 =civ.getTile(98,4,0).city
object.cMessina                 =civ.getTile(73,155,0).city
object.cPadova                  =civ.getTile(115,21,0).city
object.cTaranto                 =civ.getTile(120,138,0).city
object.cTrieste                 =civ.getTile(137,31,0).city
object.cCagliari                =civ.getTile(14,90,0).city
object.cFoggia                  =civ.getTile(111,107,0).city
object.cLivorno                 =civ.getTile(75,35,0).city
object.cParma                   =civ.getTile(93,17,0).city
object.cReggioCalabria          =civ.getTile(73,159,0).city
object.cReggioEmilia            =civ.getTile(93,21,0).city
object.cCremona                 =civ.getTile(91,9,0).city
object.cFerrara                 =civ.getTile(105,27,0).city
object.cForli                   =civ.getTile(103,39,0).city
object.cLaSpezia                =civ.getTile(77,23,0).city
object.cRavenna                 =civ.getTile(107,37,0).city
object.cSiracusa                =civ.getTile(59,169,0).city
object.cMantova                 =civ.getTile(101,17,0).city
object.cModena                  =civ.getTile(96,24,0).city
object.cVenezia                 =civ.getTile(119,25,0).city
object.cPiacenza                =civ.getTile(87,9,0).city
object.cPavia                   =civ.getTile(84,2,0).city
object.cAlessandria             =civ.getTile(75,1,0).city
object.cSavona                  =civ.getTile(66,10,0).city
object.cTrapani                 =civ.getTile(32,136,0).city
object.cPescara                 =civ.getTile(107,83,0).city
object.cUdine                   =civ.getTile(136,20,0).city
object.cAjaccio                 =civ.getTile(40,48,0).city
object.cBrindisi                =civ.getTile(130,142,0).city
object.cRimini                  =civ.getTile(106,44,0).city
object.cSassari                 =civ.getTile(22,64,0).city
object.cBastia                  =civ.getTile(53,47,0).city
object.cOlbia                   =civ.getTile(36,68,0).city
object.cTurino                  =civ.getTile(62,0,1).city
object.cMilano                  =civ.getTile(70,0,1).city

--Cities starting the scenario owned by the Japanese:


--Cities starting the scenario owned by the Germans:


--Cities starting the scenario owned by the Spanish:


--Cities starting the scenario owned by the Americans:


--Cities starting the scenario owned by the Allies:

object.cValetta                 =civ.getTile(40,180,0).city
object.cTunis                   =civ.getTile(0,136,0).city

--Cities starting the scenario owned by the Indians:


end

-- Unit Types
-- recommended key prefix 'u'

object.uFanteriaVeterana        = civ.getUnitType(0)
object.uEngineer                = civ.getUnitType(1)   --Engineers
object.uBersaglieri             = civ.getUnitType(2)
object.uBritishVeteran          = civ.getUnitType(3)
object.uGIVeteran               = civ.getUnitType(4)
object.uInfanterieVeteranen     = civ.getUnitType(5)
object.uWiderstandsnest         = civ.getUnitType(6)
object.uReggimentoCostiero      = civ.getUnitType(7)
object.uMSVN                    = civ.getUnitType(8)
object.uPartigiani              = civ.getUnitType(9)
object.uMarines                 = civ.getUnitType(10)
object.uParacadutisto           = civ.getUnitType(11)
object.uPrigionieridiguerra     = civ.getUnitType(12)
object.uFallschirmjager         = civ.getUnitType(13)
object.uGebirgsjager            = civ.getUnitType(14)
object.uSeaMines                = civ.getUnitType(15)
object.uBunker                  = civ.getUnitType(16)
object.uKasematte               = civ.getUnitType(17)
object.uPzkwVG                  = civ.getUnitType(18)
object.uPzkwVIE                 = civ.getUnitType(19)
object.uPanzerGrenadier         = civ.getUnitType(20)
object.uPanzerschreck           = civ.getUnitType(21)
object.uStugIIIG                = civ.getUnitType(22)
object.u37cmFlak36              = civ.getUnitType(23)
object.u88cmFlak37              = civ.getUnitType(24)
object.u105cmleFH18             = civ.getUnitType(25)
object.u15cmsIG33               = civ.getUnitType(26)
object.uMe109G                  = civ.getUnitType(27)
object.uFW190A                  = civ.getUnitType(28)
object.uMe110                   = civ.getUnitType(29)
object.uDo217E                  = civ.getUnitType(30)
object.uJu88                    = civ.getUnitType(31)
object.uSellaClassDD            = civ.getUnitType(32)
object.uGiusannoClassCL         = civ.getUnitType(33)
object.uLittorioClassBB         = civ.getUnitType(34)
object.uSottomarino             = civ.getUnitType(35)
object.uTypeIXC                 = civ.getUnitType(36)
object.uTribalClassDD           = civ.getUnitType(37)
object.uLeanderClassCL          = civ.getUnitType(38)
object.uYorkClassCA             = civ.getUnitType(39)
object.uQEClassBB               = civ.getUnitType(40)
object.uSClass                  = civ.getUnitType(41)
object.uIllustriousClassCV      = civ.getUnitType(42)
object.uSeafireMkII             = civ.getUnitType(43)
object.uFaireyBarracuda         = civ.getUnitType(44)
object.uLibertyShip             = civ.getUnitType(45)   --Nuclear Msl
object.uLandingCraft            = civ.getUnitType(46)
object.uBeachhead               = civ.getUnitType(47)   --Spy
object.uM8Greyhound             = civ.getUnitType(48)
object.uEliteJagdgeschwader     = civ.getUnitType(49)   --Freight
object.uC200Saetta              = civ.getUnitType(50)
object.uC202Folgore             = civ.getUnitType(51)
object.uSM79Sparviero           = civ.getUnitType(52)
object.uZ1007Alcione            = civ.getUnitType(53)
object.uP40Warhawk              = civ.getUnitType(54)
object.uP38Lightning            = civ.getUnitType(55)
object.uP51DMustang             = civ.getUnitType(56)
object.uA36Apache               = civ.getUnitType(57)
object.uB24Liberator            = civ.getUnitType(58)
object.uB25Mitchell             = civ.getUnitType(59)
object.uB26Marauder             = civ.getUnitType(60)
object.uB17Fortress             = civ.getUnitType(61)
object.uC47Dakota               = civ.getUnitType(62)
object.uBeaufighterMkVI         = civ.getUnitType(63)
object.uKittyhawk               = civ.getUnitType(64)
object.uSpitfireMkIX            = civ.getUnitType(65)
object.uHurricaneIIc            = civ.getUnitType(66)
object.uMartinBaltimore         = civ.getUnitType(67)
object.uVickersWellington       = civ.getUnitType(68)
object.uBostonIII               = civ.getUnitType(69)
object.uSunderlandIII           = civ.getUnitType(70)
object.uWacoGlider              = civ.getUnitType(71)
object.uGenPatton               = civ.getUnitType(72)
object.uGenClark                = civ.getUnitType(73)
object.uGenTruscott             = civ.getUnitType(74)
object.uDUKW                    = civ.getUnitType(75)
object.uArmoredInfantry         = civ.getUnitType(76)
object.uM5Stuart                = civ.getUnitType(77)
object.uM4A1Sherman             = civ.getUnitType(78)
object.uM4A376WSherman          = civ.getUnitType(79)
object.uM10Wolverine            = civ.getUnitType(80)
object.uRanger                  = civ.getUnitType(81)
object.u82ndAirborne            = civ.getUnitType(82)
object.uM157mmAT                = civ.getUnitType(83)
object.uM7105mmHMC              = civ.getUnitType(84)
object.uM12155mmHMC             = civ.getUnitType(85)
object.uUSBofors40mmAA          = civ.getUnitType(86)
object.uM2A1105mmArt            = civ.getUnitType(87)
object.uM1155mmArt              = civ.getUnitType(88)
object.uUSTruck                 = civ.getUnitType(89)
object.uGenMontgomery           = civ.getUnitType(90)
object.uGenLeese                = civ.getUnitType(91)
object.uBritishPIAT             = civ.getUnitType(92)
object.uBritishMG               = civ.getUnitType(93)
object.uDaimlerAC               = civ.getUnitType(94)
object.uShermanIIaBde           = civ.getUnitType(95)
object.uChurchillMkIV           = civ.getUnitType(96)
object.uShermanVCFirefly        = civ.getUnitType(97)
object.uUKTruck                 = civ.getUnitType(98)
object.uCommandos               = civ.getUnitType(99)
object.u1stAirborne             = civ.getUnitType(100)
object.u6pdrAT                  = civ.getUnitType(101)
object.uBritishMotorTroops      = civ.getUnitType(102)
object.uShermanIIa              = civ.getUnitType(103)
object.uBishop25pdr             = civ.getUnitType(104)
object.uUKBofors40mmAA          = civ.getUnitType(105)
object.u25pdrMKIIArt            = civ.getUnitType(106)
object.u55inchArt               = civ.getUnitType(107)
object.uFanteriaMotorizzata     = civ.getUnitType(108)
object.uM1340                   = civ.getUnitType(109)
object.uSemovente7518           = civ.getUnitType(110)
object.uAB41                    = civ.getUnitType(111)
object.u2065AA                  = civ.getUnitType(112)
object.u7546AA                  = civ.getUnitType(113)
object.u7527Art                 = civ.getUnitType(114)
object.u10528Art                = civ.getUnitType(115)
object.u14940Art                = civ.getUnitType(116)
object.uCdnMotInf               = civ.getUnitType(117)
object.uCdnShermanIII           = civ.getUnitType(118)
object.uCdnM7105mm              = civ.getUnitType(119)
object.uCdnMotorTroops          = civ.getUnitType(120)
object.uCdnInfantry             = civ.getUnitType(121)
object.uRadarStation            = civ.getUnitType(122)
object.uKustenbatterie          = civ.getUnitType(123)
object.uKruppK5                 = civ.getUnitType(124)
object.uPontoonBridge           = civ.getUnitType(125)
object.uSAMotInf                = civ.getUnitType(126)
object.uSAShermanIII            = civ.getUnitType(127)
object.uSASexton                = civ.getUnitType(128)
object.uSAMotorTroops           = civ.getUnitType(129)
object.uBraz1stInf              = civ.getUnitType(130)
object.uGurkhas                 = civ.getUnitType(131)
object.uIndianInfantry          = civ.getUnitType(132)
object.u4thIndianDiv            = civ.getUnitType(133)
object.uIndian25pdrArt          = civ.getUnitType(134)
object.uPolishInfantry          = civ.getUnitType(135)
object.uPolishShermanIII        = civ.getUnitType(136)
object.uPolish25pdrArt          = civ.getUnitType(137)
object.uInfanterieColoniale     = civ.getUnitType(138)
object.uGoumierMarocain         = civ.getUnitType(139)
object.uTirailleurAlgerien      = civ.getUnitType(140)
object.uFreeFrenchInfantry      = civ.getUnitType(141)
object.uFreeFrench105Art        = civ.getUnitType(142)
object.uFreeFrenchM10           = civ.getUnitType(143)
object.uNZMotInf                = civ.getUnitType(144)
object.uNZShermanIII            = civ.getUnitType(145)
object.uNZ25pdrArt              = civ.getUnitType(146)
object.uUS442ndRegt             = civ.getUnitType(147)
object.uGrilleHsIG33            = civ.getUnitType(148)
object.uPzkwIVF                 = civ.getUnitType(149)
object.uPzkwIVH                 = civ.getUnitType(150)
object.uMotInfanterie           = civ.getUnitType(151)
object.uSdkFz231                = civ.getUnitType(152)
object.uGreekMtnBde             = civ.getUnitType(153)
object.uSdkFz72AA               = civ.getUnitType(154)
object.uSSSdkFz72AA             = civ.getUnitType(155)
object.uSSPanzergrenadier       = civ.getUnitType(156)
object.uSSPanther               = civ.getUnitType(157)
object.uSSHummel                = civ.getUnitType(158)
object.uSSPzkwIV                = civ.getUnitType(159)
object.uSSMotInfanterie         = civ.getUnitType(160)
object.uSSSdkFz2342             = civ.getUnitType(161)
object.uJewishBde               = civ.getUnitType(162)
object.uGIRegular               = civ.getUnitType(163)
object.uUS1stInfantry           = civ.getUnitType(164)
object.uUS10thMountain          = civ.getUnitType(165)
object.uUSBazooka               = civ.getUnitType(166)
object.uM4A1ShermanInd          = civ.getUnitType(167)
object.uBritishRegular          = civ.getUnitType(168)
object.uCromwellMkVII           = civ.getUnitType(169)
object.uChurchillAVRE           = civ.getUnitType(170)
object.uHeavyShell              = civ.getUnitType(171)
object.u105mmShell              = civ.getUnitType(172)
object.u155mmShell              = civ.getUnitType(173)
object.uLightBombload           = civ.getUnitType(174)
object.uMediumBombload          = civ.getUnitType(175)
object.uHeavyBombload           = civ.getUnitType(176)
object.uLightFlak               = civ.getUnitType(177)
object.uHeavyFlak               = civ.getUnitType(178)
object.uMortarShells            = civ.getUnitType(179)
object.uTorpedo                 = civ.getUnitType(180)
object.uFritzX                  = civ.getUnitType(181)
object.uRailroadYard            = civ.getUnitType(182)
object.uDocks                   = civ.getUnitType(183)
object.uOilTankerF              = civ.getUnitType(184)
object.uOilTankerE              = civ.getUnitType(185)
object.uSupplyHub               = civ.getUnitType(186)
object.uGenMcCreery             = civ.getUnitType(187)
object.uBritish52ndMtn          = civ.getUnitType(188)

-- City Improvements
-- recommended key prefix 'i'
--          

object.iNothing                 = civ.getImprovement(0)
object.iHeadquarters            = civ.getImprovement(1)
object.iBarracks                = civ.getImprovement(2)
object.iGranary                 = civ.getImprovement(3)
object.iNewspaper               = civ.getImprovement(4)
object.iRailStation             = civ.getImprovement(5)
object.iDesignbureau            = civ.getImprovement(6)
object.iMilitaryTribunal        = civ.getImprovement(7)
object.iAntiTankDefenses        = civ.getImprovement(8)
object.iAqueduct                = civ.getImprovement(9)
object.iBank                    = civ.getImprovement(10)
object.iCinema                  = civ.getImprovement(11)
object.iUniversity              = civ.getImprovement(12)
object.iMassTransit             = civ.getImprovement(13)
object.iRadioStation            = civ.getImprovement(14)
object.iFactory                 = civ.getImprovement(15)
object.iManufacturingPlant      = civ.getImprovement(16)
object.iSDIDefense              = civ.getImprovement(17)
object.iSupplyDepot             = civ.getImprovement(18)
object.iElectricity             = civ.getImprovement(19)
object.iHydroPlant              = civ.getImprovement(20)
object.iNuclearPlant            = civ.getImprovement(21)
object.iStockExchange           = civ.getImprovement(22)
object.iHospital                = civ.getImprovement(23)
object.iIndustrialFarming       = civ.getImprovement(24)
object.iSuperhighways           = civ.getImprovement(25)
object.iResearchLab             = civ.getImprovement(26)
object.iAntiAircraftBattery     = civ.getImprovement(27)
object.iShoreBattery            = civ.getImprovement(28)
object.iSolarPlant              = civ.getImprovement(29)
object.iFishingPort             = civ.getImprovement(30)
object.iOffshorePlatform        = civ.getImprovement(31)
object.iAirport                 = civ.getImprovement(32)
object.iMilitaryPolice          = civ.getImprovement(33)
object.iNavalYard               = civ.getImprovement(34)
object.iTransporter             = civ.getImprovement(35)

-- Players (Tribes)
-- recommended key prefix 'p'

object.pBarbarians              = civ.getTribe(0)
object.pAxis                    = civ.getTribe(1)
object.pJapanese                = civ.getTribe(2)
object.pGermans                 = civ.getTribe(3)
object.pSpanish                 = civ.getTribe(4)
object.pAmericans               = civ.getTribe(5)
object.pAllies                  = civ.getTribe(6)
object.pIndians                 = civ.getTribe(7)

-- Wonders
-- recommended key prefix 'w'

object.wPyramids                = civ.getWonder(0)
object.wHangingGardens          = civ.getWonder(1)
object.wColossus                = civ.getWonder(2)
object.wRoyalNavy               = civ.getWonder(3)
object.wGreatLibrary            = civ.getWonder(4)
object.wOracle                  = civ.getWonder(5)
object.wGothicLine              = civ.getWonder(6)
object.wSunTzusWarAcademy       = civ.getWonder(7)
object.wKingRichardsCrusade     = civ.getWonder(8)
object.wMarcoPolosEmbassy       = civ.getWonder(9)
object.wMichelangelosChapel     = civ.getWonder(10)
object.wAlfaRomeoFactories      = civ.getWonder(11)
object.wPortodiNapoli           = civ.getWonder(12)
object.wCittadelVaticano        = civ.getWonder(13)
object.wLeonardosWorkshop       = civ.getWonder(14)
object.wBBCWorldServices        = civ.getWonder(15)
object.wRIVWorkshops	        = civ.getWonder(16)
object.wAdamSmithsTradingCo     = civ.getWonder(17)
object.wDarwinsVoyage           = civ.getWonder(18)
object.wStatueofLiberty         = civ.getWonder(19)
object.wEiffelTower             = civ.getWonder(20)
object.wWomensSuffrage          = civ.getWonder(21)
object.wHooverDam               = civ.getWonder(22)
object.wManhattanProject        = civ.getWonder(23)
object.wUnitedNations           = civ.getWonder(24)
object.wApolloProgram           = civ.getWonder(25)
object.wSETIProgram             = civ.getWonder(26)
object.wCureforCancer           = civ.getWonder(27)

-- Base Terrain
-- recommended prefix 'b'

object.bBeach                   =civ.getBaseTerrain(0,0)  --Drt
object.bRough                   =civ.getBaseTerrain(0,1)  --Pln
object.bTemperateGrassland      =civ.getBaseTerrain(0,2)  --Grs
object.bForest                  =civ.getBaseTerrain(0,3)  --For
object.bHills                   =civ.getBaseTerrain(0,4)  --Hil
object.bApennines               =civ.getBaseTerrain(0,5)  --Mou
object.bGrassland               =civ.getBaseTerrain(0,6)  --Tun
object.bImpassable              =civ.getBaseTerrain(0,7)  --Gla
object.bSwamp                   =civ.getBaseTerrain(0,8)  --Swa
object.bTownCenter              =civ.getBaseTerrain(0,9)  --Jun
object.bOcean                   =civ.getBaseTerrain(0,10)  --Oce
object.bRiverbed                =civ.getBaseTerrain(0,11)  --Bbb
object.bTemperateHills          =civ.getBaseTerrain(0,12)  --Ccc
object.bTemperateRough          =civ.getBaseTerrain(0,13)  --Ddd
object.bTemperateRiverbed       =civ.getBaseTerrain(0,14)  --Eee
object.bRiverCrossing           =civ.getBaseTerrain(0,15)  --Fff
object.bBeach_                  =civ.getBaseTerrain(1,0)  --Drt
object.bRough_                  =civ.getBaseTerrain(1,1)  --Pln
object.bTemperateGrassland_     =civ.getBaseTerrain(1,2)  --Grs
object.bForest_                 =civ.getBaseTerrain(1,3)  --For
object.bHills_                  =civ.getBaseTerrain(1,4)  --Hil
object.bApennines_              =civ.getBaseTerrain(1,5)  --Mou
object.bGrassland_              =civ.getBaseTerrain(1,6)  --Tun
object.bImpassable_             =civ.getBaseTerrain(1,7)  --Gla
object.bSwamp_                  =civ.getBaseTerrain(1,8)  --Swa
object.bTownCenter_             =civ.getBaseTerrain(1,9)  --Jun
object.bOcean_                  =civ.getBaseTerrain(1,10)  --Oce
object.bRiverbed_               =civ.getBaseTerrain(1,11)  --Bbb
object.bTemperateHills_         =civ.getBaseTerrain(1,12)  --Ccc
object.bTemperateRough_         =civ.getBaseTerrain(1,13)  --Ddd
object.bTemperateRiverbed_      =civ.getBaseTerrain(1,14)  --Eee
object.bRiverCrossing_          =civ.getBaseTerrain(1,15)  --Fff

-- Terrain
-- recommended prefix 't'

object.tBeach                   =civ.getTerrain(0,0,0)
object.tTownBeach               =civ.getTerrain(0,0,1) -- Fish Resource
object.tRocks                   =civ.getTerrain(0,0,2) -- Whale Resource
object.tRough                   =civ.getTerrain(0,1,0)
object.tTownRough               =civ.getTerrain(0,1,1) -- Fish Resource
object.tDocks                   =civ.getTerrain(0,1,2) -- Whale Resource
object.tTemperateGrassland      =civ.getTerrain(0,2,0)
object.tForest                  =civ.getTerrain(0,3,0)
object.tDeer                    =civ.getTerrain(0,3,1) -- Fish Resource
object.tSawmill                 =civ.getTerrain(0,3,2) -- Whale Resource
object.tHills                   =civ.getTerrain(0,4,0)
object.tTownHills               =civ.getTerrain(0,4,1) -- Fish Resource
object.tnone5                   =civ.getTerrain(0,4,2) -- Whale Resource
object.tApennines               =civ.getTerrain(0,5,0)
object.tMVesuvius               =civ.getTerrain(0,5,1) -- Fish Resource
object.tMEtna                   =civ.getTerrain(0,5,2) -- Whale Resource
object.tGrassland               =civ.getTerrain(0,6,0)
object.tTown                    =civ.getTerrain(0,6,1) -- Fish Resource
object.tSupplyDepot             =civ.getTerrain(0,6,2) -- Whale Resource
object.tImpassable              =civ.getTerrain(0,7,0)
object.tnone1                   =civ.getTerrain(0,7,1) -- Fish Resource
object.tnone6                   =civ.getTerrain(0,7,2) -- Whale Resource
object.tSwamp                   =civ.getTerrain(0,8,0)
object.tnone2                   =civ.getTerrain(0,8,1) -- Fish Resource
object.tnone7                   =civ.getTerrain(0,8,2) -- Whale Resource
object.tTownCenter              =civ.getTerrain(0,9,0)
object.tFactory                 =civ.getTerrain(0,9,1) -- Fish Resource
object.tFuelDepot               =civ.getTerrain(0,9,2) -- Whale Resource
object.tOcean                   =civ.getTerrain(0,10,0)
object.tFish                    =civ.getTerrain(0,10,1) -- Fish Resource
object.tHarbor                  =civ.getTerrain(0,10,2) -- Whale Resource
object.tRiverbed                =civ.getTerrain(0,11,0)
object.tTownRiver               =civ.getTerrain(0,11,1) -- Fish Resource
object.tBlownBridge1            =civ.getTerrain(0,11,2) -- Whale Resource
object.tTemperateHills          =civ.getTerrain(0,12,0)
object.tTownTHills              =civ.getTerrain(0,12,1) -- Fish Resource
object.tnone8                   =civ.getTerrain(0,12,2) -- Whale Resource
object.tTemperateRough          =civ.getTerrain(0,13,0)
object.tTownTRough              =civ.getTerrain(0,13,1) -- Fish Resource
object.tnone9                   =civ.getTerrain(0,13,2) -- Whale Resource
object.tTemperateRiverbed       =civ.getTerrain(0,14,0)
object.tTownTRiver              =civ.getTerrain(0,14,1) -- Fish Resource
object.tBlownBridge2            =civ.getTerrain(0,14,2) -- Whale Resource
object.tRiverCrossing           =civ.getTerrain(0,15,0)
object.tCrossing                =civ.getTerrain(0,15,1) -- Fish Resource
object.tCrossingTemp            =civ.getTerrain(0,15,2) -- Whale Resource
object.tBeach_                  =civ.getTerrain(1,0,0)
object.tTownBeach_              =civ.getTerrain(1,0,1) -- Fish Resource
object.tRocks_                  =civ.getTerrain(1,0,2) -- Whale Resource
object.tRough_                  =civ.getTerrain(1,1,0)
object.tTownRough_              =civ.getTerrain(1,1,1) -- Fish Resource
object.tnone3                   =civ.getTerrain(1,1,2) -- Whale Resource
object.tTemperateGrassland_     =civ.getTerrain(1,2,0)
object.tForest_                 =civ.getTerrain(1,3,0)
object.tDeer_                   =civ.getTerrain(1,3,1) -- Fish Resource
object.tSawmill_                =civ.getTerrain(1,3,2) -- Whale Resource
object.tHills_                  =civ.getTerrain(1,4,0)
object.tTownHills_              =civ.getTerrain(1,4,1) -- Fish Resource
object.tnone5_                  =civ.getTerrain(1,4,2) -- Whale Resource
object.tApennines_              =civ.getTerrain(1,5,0)
object.tMVesuvius_              =civ.getTerrain(1,5,1) -- Fish Resource
object.tMEtna_                  =civ.getTerrain(1,5,2) -- Whale Resource
object.tGrassland_              =civ.getTerrain(1,6,0)
object.tTown_                   =civ.getTerrain(1,6,1) -- Fish Resource
object.tSupplyDepot_            =civ.getTerrain(1,6,2) -- Whale Resource
object.tImpassable_             =civ.getTerrain(1,7,0)
object.tnone1_                  =civ.getTerrain(1,7,1) -- Fish Resource
object.tnone6_                  =civ.getTerrain(1,7,2) -- Whale Resource
object.tSwamp_                  =civ.getTerrain(1,8,0)
object.tnone2_                  =civ.getTerrain(1,8,1) -- Fish Resource
object.tnone7_                  =civ.getTerrain(1,8,2) -- Whale Resource
object.tTownCenter_             =civ.getTerrain(1,9,0)
object.tFactory_                =civ.getTerrain(1,9,1) -- Fish Resource
object.tFuelDepot_              =civ.getTerrain(1,9,2) -- Whale Resource
object.tOcean_                  =civ.getTerrain(1,10,0)
object.tFish_                   =civ.getTerrain(1,10,1) -- Fish Resource
object.tHarbor_                 =civ.getTerrain(1,10,2) -- Whale Resource
object.tRiverbed_               =civ.getTerrain(1,11,0)
object.tTownRiver_              =civ.getTerrain(1,11,1) -- Fish Resource
object.tBlownBridge1_           =civ.getTerrain(1,11,2) -- Whale Resource
object.tTemperateHills_         =civ.getTerrain(1,12,0)
object.tTownTHills_             =civ.getTerrain(1,12,1) -- Fish Resource
object.tnone8_                  =civ.getTerrain(1,12,2) -- Whale Resource
object.tTemperateRough_         =civ.getTerrain(1,13,0)
object.tTownTRough_             =civ.getTerrain(1,13,1) -- Fish Resource
object.tnone9_                  =civ.getTerrain(1,13,2) -- Whale Resource
object.tTemperateRiverbed_      =civ.getTerrain(1,14,0)
object.tTownTRiver_             =civ.getTerrain(1,14,1) -- Fish Resource
object.tBlownBridge2_           =civ.getTerrain(1,14,2) -- Whale Resource
object.tRiverCrossing_          =civ.getTerrain(1,15,0)
object.tCrossing_               =civ.getTerrain(1,15,1) -- Fish Resource
object.tCrossingTemp_           =civ.getTerrain(1,15,2) -- Whale Resource

-- =============================================
-- TEXT
-- =============================================
-- You might find it easier to keep much of your events text here
-- Remember, you can place %STRING1, %STRING2, etc. in your text and
-- use text.substitute to insert the actual values in the event, instead
-- of splitting the text into multiple parts
-- recommended prefix 'x'

-- SCENARIO INTRODUCTORY TEXT  =========================

object.introduction =

[[
Introduction.
^

]]

-- PLAYER OPTIONS TEXT =========================

object.attackerDefenderDisplay = 
[[
The Attacker/Defender display window can be deactivated by going to the 'Game Tips and Concepts' menu (keyboard '6'). 
^ 
Select the 'Set Attacker and Defender Display' check box and click 'OK'.
^ 
This will bring you to the 'Set Attacker and Defender Display On/Off' screen. 
^
Check the 'Turn Attack / Defense Display On/Off' and click 'OK'.
^
Depending on its current state it will turn the feature off or on. You will then get a pop-up message confirming which.
]]

-- SPECIAL EVENTS TEXT =========================

object.granSassoRaid =

[[
In a daring afternoon raid, an elite commando of Fallschirmjager successfully rescue Benito Mussolini in a high-risk glider mission atop the Gran Sasso d'Italia mountain range.
^
Adolf Hitler had personally ordered Hauptsturmfuhrer Otto Skorzeny to track Mussolini and simultaneously ordered the paratroop General Kurt Student to execute the liberation.
^
After German signals intelligence intercepted a coded Italian report which indicated that Mussolini was imprisoned somewhere in the Abruzzi mountains the raid was launched.
^
The former fascist dictator is flown directly to the Fuhrer's headquarters were the two agree to set up the Italian Social Republic with its seat of power in the italian town of Salo.
^
Diehard Italian fascists will regularly join the ranks of the 'Milizia Volontaria per la Sicurezza Nazionale', commonly known as the Blackshirts, as long as the town of Salo (103,7,0) remains in German hands.
]]

-- COMMAND DECISIONS TEXT =========================

object.operationAvalanche =

[[
You may now proceed to launch your 2nd Amphibious Operation, i.e. the invasion of the Italian mainland.
^
You have the option to invade either the coasts of the Mare Adriatico (east) or Tirreno (west). If you choose the former, the landing craft will deploy in the ports of Messina and Catania. If the latter, in the ports of Messina and Palermo. You must ensure the ports are free of sea mines first.
^
Once activated you will only have a total of THREE turns to complete your invasion before the landing craft are recalled, so be certain to have the troops deployed in said ports beforehand:
]]


object.operationBaytown =

[[
If you decide to activate Operation Baytown, you can instantaneoulsy transport (i.e. teleport) any three British Regulars, one Sherman IIa (Bde), one Daimler AC, one DUKW and one UK Truck (which you may pre-load) that are located in the port of Messina to the beach town of Giola Tauro (80,156).
^
Therefore, if you wish to teleport all these units they MUST be located in the port of Messina PRIOR to proceeding with the decision, otherwise only those present will be sent. The DUKW unit would allow you to place Supply Hubs to support operations in Calabria.
^
It costs 85 supplies to launch the operation.
]]

object.operationDiadem =

[[
If you decide to activate Operation Diadem, your munitions (105mm and 155mm Shells, Light / Medium and Heavy Bombloads), and all your ground troops will benefit from a +2 AF to all their attacks for this and the next three weeks. 
^
This is in addition to any other attack bonuses they may also be eligible for.
^
It costs 500 supplies to launch the operation.
]]

object.operationGrapeshot =

[[
If you decide to activate Operation Grapeshot, your munitions (105mm and 155mm Shells, Light / Medium and Heavy Bombloads), and all your ground troops will benefit from a +2 AF to all their attacks for this and the next three weeks. 
^
This is in addition to any other attack bonuses they may also be eligible for.
^
It costs 500 supplies to launch the operation.
]]

object.operationShingle =

[[
You may now proceed to launch your 3rd Amphibious Operation. 
^
You have the option to invade either the coasts of the Mare Adriatico or Tirreno. If you choose the former, the landing craft will deploy immediately in the port of Bari. If the latter, either in the port of Ostia or Napoli. You must control and ensure the ports are free of sea mines first.
^
Once activated you will only have a total of THREE turns to complete your invasion before the landing craft are permanently recalled, so be certain to have the troops deployed in said ports beforehand.
]]

object.operationSlapstick =

[[
If you decide to activate Operation Slapstick, one new 'York Class CA' vessel and two existing '1st Airborne' units may be automatically redeployed to Catania. Should you fail to have the required Para's, you may purchase them beforehand through the procurement menu.
^
You may load the British Para's on the York class vessel, sail and land them next to Taranto where a secret agreement with the Italians should allow your units to land next to the city and enter it without opposition.
^
The 8th Indian Division will arrive in the city as soon as the sea mines that are blocking the port have been removed by the Italians, which could take up to 3 turns. It costs 50 supplies to launch the operation.
]]

object.operationTranport1stFF =
[[
The 1st Free French Mechanized Division is being temporarily assigned to the Italian front to gain combat experience. 
^
It is slated to particpate in the upcoming invasion of France operations and therefore its time here will be short.
^
As such, you may expedite its tranport to the port of Napoli, provided it is free of sea mines, for the cost of 90 supplies. How do you wish to proceed?:
^
]]



-- GAME CONCEPTS =========================

object.aiRangedUnits =

[[
The following AI controlled units may fire munitions at your forces if they come within range: 
^
^Munitions vs ground or naval targets:
^
^German Fallschirmjager: 1 x Mortar shell within 2 tile
^German Widerstandsnest: 1 x Mortar shell within 2 tile
^German 10.5cm leFH18: 1 x 105mm shell within 2 tile
^German 15cm sIG33: 1 x 155mm shell within 2 tiles
^Italian 105/28 Art.: 1 x 105mm shell within 2 tile
^Italian 149/40 Art.: 1 x 155mm shell within 2 tiles
^Kustenbatterie: : 1 x 155mm shell within 2 tiles
^Krupp 5: : 1 x Heavy Shell within 3 tiles
^Munitions vs aircraft:
^
^German 3.7cm Flak 36: 1 x Light Flak shell within 2 tile
^German 8.8cm Flak 37: 1 x Heavy Flak shell within 3 tiles
^German Railroad Yard: 1 x Heavy Flak shell within 3 tiles
^Italian 20/65 AA.: 1 x Light Flak shell within 2 tile
^Italian 75/46 AA.: 1 x Heavy Flak shell within 3 tiles
]]

object.alliedMunitions =

[[
The following Allied controlled units may fire munitions if within range of enemy forces: 
^
^Munitions vs ground or naval targets:
^Artillery:
^
^25 pdr MKII Art.: 1 x 105mm shell within 2 tile for 3 supplies
^5.5 inch Art.: 1 x 155mm shell within 2 tiles for 4 supplies
^Free French 105 Art: 1 x 105mm shell within 2 tile for 3 supplies
^Indian 25 pdr Art.: 1 x 105mm shell within 2 tile for 3 supplies
^NZ 25 pdr Art.: 1 x 105mm shell within 2 tile for 3 supplies
^M1 155mm Art.: 1 x 155mm shell within 2 tiles for 4 supplies
^M2A1 105mm Art.: 1 x 105mm shell within 2 tile for 3 supplies
^Polish 25 pdr Art.: 1 x 105mm shell within 2 tile for 3 supplies
^
^Self-propelled Artillery:
^
^Bishop 25pdr: 1 x 105mm shell within 2 tiles for 3 supplies
^Cdn M7 105mm: 1 x 105mm shell within 2 tile for 3 supplies
^M7 105mm HMC: 1 x 105mm shell within 2 tile for 3 supplies
^M12 155mm HMC: 1 x 155mm shell within 2 tiles for 4 supplies
^SA Sexton: 1 x 105mm shell within 2 tile for 3 supplies
^
^Munitions vs aircraft:
^
^UK Bofors 40mm AA: 1 x Light Flak shell within 2 tile for 2 supplies
^US Bofors 40mm AA: 1 x Light Flak shell within 2 tile for 2 supplies]]


object.alliedvsAxisLosses =

[[
As the Mediterranean theater increasingly began to be superseded by Operation Overlord, resource allocations, both in material and manpower, became increasingly strictly regulated.
^
Manpower shortages, in particular, would come to burden Allied commanders as the campaign progressed and therefore the Allied player needs to track their percentage losses in relation to their Axis counterparts.
^
The higher the loss percentage the greater the weekly burden on the Mediterranean theater supply depots, as it translates into a higher demand for hospital and recruit training facilities. The following supply loss calculations will only begin once the Allies have incurred a minimum of 50 unit casualties:
^
^From 0 to 25.99% = 0 supplies lost
^Between 26 to 39.99% = 15 supplies lost
^Between 40 to 49.99% = 30 supplies lost
^Between 50 to 59.99% = 45 supplies lost
^Between 60 to 69.99% = 60 supplies lost
^Over 70% = 75 supplies lost
^
The calculation is Allied Killed / (German Killed + (Italian Killed /2)) = x %
^
You can always review your current loss percentage within the weekly status report menu by pressing on the '1' key
^
NOTE: Munitions are excluded from the tally and Italian losses only count for half in the equation.
]]

object.axisRailyards =

[[
Axis railroad yards play an important role in the Germans ability to funnel replacements to the Italian front.
^
They can only be located in the following cities:
^Alessandria, Bologna, Brescia, Firenze, Modena, Pavia, Udine, Verona. 
^
The cities are easily identified by a train icon on the top right corner of each city tile and the Germans
can only possess a maximum of five of them and no more than 1 may occupy any city at any given time.
^
At the beginning of every turn a random number between 1 and 9 is generated by the events file:
^
^- For numbers 1 to 3 the Germans receive one reinforcement
^- For number 4 they receive 2 reinforcements
^- For number 5 they receive 3 reinforcements
^- For number 6 they receive 4 reinforcements
^- For number 7 they receive 5 reinforcements
^- For number 8 they receive 6 reinforcements
^- For number 9 they receive 7 reinforcements
^
For each railroad yard the Allies manage to destroy they recieve 100 supplies and one German reinforcement is subtracted from the maximum possible (though they can never receive less than 1). 
^
Thus if no railyards were in play at the beginning of a given turn the Germans would be limited to a maximum of 3 reinforcement units (see the ReadMe #14 Game Concept Chart).
^
Each destroyed railroad yard will be rebuilt within 4 to 6 turns and randomly placed on one of the aforementioned cities.
^
Finally, each railroad yard possesses an inherent anti-aircraft unit and therefore can produce one Heavy Flak munition per turn when an Allied air unit is located next to it.]]

object.gameMenus =

[[
You may access the following menus/boards by pressing on the appropriate key: 
^
You may, at any time, access the weekly status report by pressing on the '1' key. 
^
You may, at any time, access the 'Total Units in Theather Report' by pressing on the '2' key. 
^
You may, at any time, access the 'Radar' menu by pressing on the '3' key. 
^
You may, at any time, access the 'Allied Replacement Procurement' menu by pressing on the '4' key. 
^
You may, at any time, access the 'Command Decision Required' menu by pressing on the '5' key. 
^
You may, at any time, access the 'Game Tips and Concepts' menu by pressing on the '6' key. 
^
You may, at any time, access the 'Utility Tool' menu by pressing on the '7' key. 
^
You may, at any time, access the 'Choose a setting to change' menu by pressing on the '0' key. 
]]

object.majorAirfields =

[[
Major Airfields.
^

]]

object.nationBonuses =

[[
The Italian campaign saw the Allies deploy forces from a multitude of friendly nations including Americans, Brazilians, British, Canadians, French, Greeks, Indians, New Zealanders, Polish and South Africans.
^
As one might expect, even though there was a high degree of standardization, supplying such an incredibly diverse multinational force remained a significant logistical challenge.
^
As such, the Allied High command strived to keep fighting units of each nationality within their own divisional/corps organizations which typically operated within a specific sector of the front. This served to increase their overall combat efficiency.
^
In terms of the scenario, any unit that is stacked with another unit of the same nationality will receive a +1 bonus to its AF or DF when attacking or defending.
^
Note: This bonus is not added to the attack or defense value when using the unit HELP Tab. It is only visible under the Display Combat power popup. 
]]

object.radars =

[[
This scenario has integrated the lua template's Radar module into its design.  
^
Essentially, the player can activate the radar capability of certain units by 
^pressing either the 'k' key or through the 'Radar Options' (menu 3). The  
^following radar capable units can search either for air or naval units:
^
^1) Tribal Class DD: when active press on the 'k' key and the unit will search for  
^any naval unit within 5 tiles of its position at a cost of 2 supplies and 3 MP's 
^2) Illustrious Class CV: when active press on the 'k' key and the unit will search  
^for any air unit within 6 tiles of its position at a cost of 2 supplies and 3 MP's
^3) Sunderland III: when active press on the 'k' key and the unit will search for  
^any naval unit within 5 tiles of its position at a cost of 2 supplies and 3 MP's 
^4) Radar Stations: access menu 3 and select 'Perform the radar scan for this turn'  
^and EACH of your radar station will search for any enemy aircraft within 
^10 tiles of its position at a cost of 2 supplies per radar station
^
The Allied player begins with the 'Early Radar System' technology which sets the 
^accuracy of radar searches at 40%, meaning search results may be off by a few tiles.
^
Developing the 'Adv. Radar Research' and then the 'Adv. Radar System' advances 
^will allow you to increase the accuracy of searches to 80%.

]]

object.seasons =

[[
There are three types of seasons in the game: Summer, Fall/Spring and Winter.
^
Summer lasts between the months of April III and October II and there are no adverse effects. 
^
Fall and Spring occurs on the months of October III to November IV and March I to April II, respectively, and have the following possible adverse effects:
^
^- All air units activated on a city or airbase have a 25% chance of being grounded for the turn
^- All motorized units, once activated, have a 11% chance to become immobilized for the turn
^- Glider units may not perform paradrops during Fall/Spring months
^- All Truck type units expend an extra 3 MP's per tile when moving off road
^
Winter lasts between the months of December and February and have the following possible adverse effects:
^
^- All air units activated on a city or airbase have a 33% chance of being grounded for the turn
^- All motorized units, once activated, have a 8% chance to become immobilized for the turn
^- All naval units have a 3% chance of suffering damage while moving at sea
^- Glider units may not perform paradrops during winter months
^- Truck units expend an extra 2 MP's per tile when moving off road
^
There is no need to use a bat file as the relevant changes and graphic files are automatically handled by lua. 
]]

object.fallSeason =

[[
The fall torrential rains and mud season has begun.
^
Fall occurs between the weeks of October III and November IV 
and has the following possible adverse effects:
^
^- All air units activated on a city or airbase have a 25% 
chance of being grounded for the turn
^- All motorized units, once activated, have a 11% chance to 
become immobilized for the turn unless located in a city or
a highway, in Sardinia, Sicily or Southern Italy.
^- Glider units may not perform paradrops during Fall/Spring 
months
^- There's a 33% chance that engineer units may fail to rebuild  
river crossings or pontoon bridges on riverbed tiles
^- All Truck type units expend an extra 3 MP's per tile when 
moving off road
^- Road/Highway mulitpliers are reduced to 1 and 3

]]

object.winterSeason =

[[
The winter season has begun.
^
Winter occurs between the weeks of December I and February IV 
and has the following possible adverse effects:
^
^- All air units activated on a city or airbase have a 33% 
chance of being grounded for the turn
^- All mechanized units, once activated, have a 8% chance to 
become immobilized for the turn unless located in a city, on a 
road, a highway, or in Sardinia, Sicily or Southern Italy.
^- Glider units may not perform paradrops during Winter 
months
^- There's a 33% chance that engineer units may fail to rebuild  
river crossings or pontoon bridges on riverbed tiles
^- Naval units have their MP's reduced by a third (except 
submarines and oil tankers)
^- There is a 3% chance naval units may suffer some small 
damage everytime they move to a different ocean tile
^- All Truck type units expend an extra 2 MP's per tile 
when moving off road
^- Road/Highway mulitpliers are reduced to 1 and 2
]]

object.springSeason =

[[
The Spring thaw season has begun.
^
Spring occurs between the weeks of March I to April II and has 
the following possible adverse effects:
^
^- All air units activated on a city or airbase have a 25% 
chance of being grounded for the turn
^- All ground units, once activated, have a 11% chance to 
become immobilized for the turn unless located in a city or
a highway.
^- Glider units may not perform paradrops during Fall/Spring 
months
^- There's a 33% chance that engineer units may fail to rebuild  
river crossings or pontoon bridges on riverbed tiles
^- All Truck type units expend an extra 3 MP's per tile when 
moving off road
]]

object.stackingLimits =

[[
A new ground unit stacking limit feature has been implemented within the scenario.
^
Essentially, a unit will be returned to its previous location and suffer a 1 MP penalty if it attempts to enter a tile that already contains the maximum allowed ground units for that terrain type:
^
^Limit 12 = Cities 
^Limit 6 = Grassland 
^Limit 5 = Beaches, Forest, Hills, River Crossing and Rough
^Limit 4 = Riverbed
^Limit 3 = Apennines and Swamp
^
This stacking limit does not apply to air, naval units, ground units located on transport ships, Commandos/ Gurkhas/ Rangers, DUKW, Engineer, Leaders, Munitions, Supply Hub or UK/US Truck units.
^
The event does not prevent your transport ships or scheduled reinforcements from entering a city that already has a maximum number of ground units in it.
]]

-- ^
-- Though it is not recommended, since it will affect play balance, you may at any time turn this feature off or on by going to the 'Game Tips and Concept' menu (key '6') and selecting the 'Set Stacking Limit Option' box.

object.supplyLines =

[[
This scenario uses a slighlty updated version of Knighttime's supply  
^module (many thanks to him for this). 
^
In essence, Allied ground units (1) must be within range of one of two 
^types of supply sources to remain in supply:
^- Within 12 tiles of a city that contains a 'Supply Depot' improvement (2)
^- Within 8 tiles of a 'Supply Hub' unit (see Game Tips: DUKW for creating 
^new 'Supply Depots' and 'Supply Hubs')
^ 
As long as a unit is within range of a supply source it is considered 
^in supply (3). 
^
Therefore a unit can move in and out of supply during the same  
^turn as it moves within or beyond a supply source.
^
A unit out of supply suffers two adverse effects:
^
^- It suffers a 50% attack penalty to any combat it initiates, unless an
^artillery unit in which case it will be unable to generate munitions.
^- For each tile it enters while moving out of supply, there is a  
^chance that it will 'run out' of supplies and will no longer be able to  
^move that turn and will suffer a 1 HP penalty in the process.
^
It is always possible to check a unit's current supply status  
^by pressing the 'TAB' key.
^Each Supply Hub costs 5 supplies per turn to maintain
^Each Supply Depot costs 10 supplies per turn to maintain
^
In addition, damaged Allied ground units which begins their turn in 
^a city with a Supply Depot will benefit from a +2 HP heal bonus.
^
Note 1: 1st Airborne, 82nd Airborne, Commandos, Goumier Marocain, 
^Greek Mtn Bde, Ranger, US 10th mountain and British 52nd Mtn units 
^are always in supply.
^Note 2: The Allied cities of Tunis and Valetta both begin the 
^scenario with a 'Supply Depot' improvement.
^Note 3: Supply Lines may not be traced through Swamps or Apennines 
^tiles.]]


-- GAME TIP TEXT =========================

object.AlliedBombers =

[[
American (B-17, B-24, B-25 and B-26) and British (Baltimore, Boston III and Wellington) bomber units can generate one light, medium or heavy bomb load per flight mission by pressing on the 'k' key.
^
These bombers are easily recognizable by the orange-yellow dot(s) on the top right corner of their unit icon (1 for light, 2 for medium and 3 for heavy bombloads).
^
A bomber unit will automatically be homed to the city it begins its turn in, which means it will be mission capable to deliver a bomb load to an enemy target.
^
Once a bomber has delivered its payload, it will automatically become un-homed and will need to return to a 'city' tile (NOT an airbase) if it wishes to reload. 
^
Each time a bomber releases its bomb load, 3 (for Light), 4 (for Medium) or 5 (for Heavy) supplies will be subracted from your theater depots (aka treasury).
^
A bomber will be unable to generate a bomb load should your theater depot fail to have the minimum required supplies in them. 
]]

object.AlliedFighterBombers =

[[
American A-36 Apache and British Hurricane IIc are fighter-bomber units and are easily recognizable by the red dot on the top right corner of their unit icon.
^
Fighter-bomber units can select to use 'one' of two methods of attack in a turn:
^1. They can use their inherent attack factor to strike at enemy ground forces, or
^2. They can press the 'k' key to generate a light bombload
^
The first method delivers a stronger attack punch but subjects the aircraft to potential damage, whereas the second method delivers a weaker attack punch but avoids combat damage.
^
Contrary to regular bombers, they do not need to be homed to a city to generate a light bombload, though each time they do so, 3 supplies will be subracted from your theater depots (aka treasury).
^
A fighter-bomber will be unable to generate a bomb load should your theater depot fail to have the minimum required supplies in them. 
]]

object.AlliedGenerals =

[[
There are up to six different Allied Army Generals, 3 American (Patton, Clark, Truscott) and 3 British (Montgomery, Leese, McCreery), that may appear in the campaign. 
^
There can never be more than one American and one British leader in play at the same time.
^
Each general may provide combat bonuses in % (the yellow number in the red box on the top right corner of the unit) to any ground unit(s) of its own nationality or colonial troops that are within its 'command' range (the yellow number in the blue box on the top right corner of the unit). 
^
Gen. Patton for example, has a command range of 5 tiles and gives a combat bonus of 40% to all American, Brazilian and Free French troops. Gen. Montgomery, for example provides a bonus to all British and Commonwealth troops.
^
Allied leaders that are defeated in combat are never permanently killed, but considered wounded instead. Whenever this occurs, the events will randomly determine how many turns before they can return. 
]]

object.C47Dakota =

[[
C-47 Dakota planes are only able to transport non-motorized infantry type units and have a maximum carrying capacity of 1 unit and a range of 28 tiles!
^
To load the appropriate unit onto a transport simply put it to sleep and press on 'p' to fly it to its selected destination.
^
Beware that these transport planes may only be flown from one airbase or city tile to another. 
^
Should you fail to select an airbase or city as your destination, re-selecting the transport after it landed will return it, along with its cargo, to its original location with no further ability to fly again this turn (it's cargo will be 'embedded' within the transport and only become availble the next turn).
]]

object.Docks =

[[
Supplies are a crucial component of your Italian campaign as they are needed for everything from generating munitions to procuring new combat units, and Docks play a vital role in maintaining this flow of resources for the Allied war effort. 
^
New Docks may be built by engineer units in any port city that contains a 'Harbor' resource *, provided it doesn't already contain one. The Allied controlled cities of Tunis and Valetta already begin the game with one each.
^
In essence, EACH Dock unit in the theater provides an extra 25 supplies per turn. As such, it is in your best interest to build them as soon as possible.
^
Note *: This includes the port cities of Bari, Catania, Livorno, Messina, Napoli, Palermo and Taranto
]]

object.DUKW =

[[
The DUKW units can only be created when Landing Craft are converted to Beachhead's.
^
They have one primary function and that is to establish supply sources for the Allied land forces. This can be done in two manners:
^
1. At a cost of 2 MP's and 15 supplies they can deploy a 'Supply Hub' unit if they are in a city or on any tile that contains either a road or highway by pressing on the 'k' key.
^
2. They can help build a 'Supply Depot' improvement in a city by pressing on the 'shift' + '3' key combination. To proceed, the DUKW must be located in a valid city that doesn't already contain a Supply Depot, have 120 supplies in the theater depot and at least 4 MP's.
^
You may only build 'Supply Depot' improvements in objective or port cities with a harbor resource. 
^
Once you accept to build a depot, a random number will determine how many turns it will take to complete the task (between 3 to 5 game weeks) while a 'Supply Depot' resource will automatically be built within the city limit which can provide some extra supplies (aka trade).
^
The DUKW unit will be temporarily removed while it is busy with the building operation and will return in the same city once it is done.
^
A Supply Hub unit can always be disbanded in one of two ways:
^
1. You can move a DUKW, UK or US Truck unit to its location and press on the 'u' key and this will not only disband the hub but return the initial 15 supplies cost.
^
2. You can go to the game's Orders menu and select Disband, though in this case you will not recuperate the 15 supplies.
]]

object.engineers_River =
[[
An engineer can rebuild a 'River Crossing' along with the road on a tile that has 
been demolished by Axis, by using the 'k' key.
^
To proceed, the engineer must be on a 'Riverbed' tile that contains a 'Blown Bridge' 
resource icon, have 20 supplies in the Allied Theater Depot and possess a minimum 
of 2 MP's.
^
Furthemore, they cannot rebuild a river crossing during fall or spring turns.
^
Failure to meet these minimum conditions will prevent your engineer from proceeding 
with the operation and require you to try again on another turn.
]]

object.engineers_SeaMines =
[[
An engineer can remove enemy sea mines that are blocking the entrance of a port city, 
by using the 'u' key, thereafter allowing friendly naval units to enter.
^
To proceed, the engineer must be located in the port city that is obstructed by sea mines, 
have 12 supplies in the Allied Theater Depot and possess a minimum of 4 MP's.
^
Failure to meet these minimum conditions will prevent your engineer from proceeding 
with the operation and require you to try again on another turn.
^
Once you accept to remove obstacles, a random number will determine how many turns it 
will take to complete the task (between 1 to 3 game weeks).
^
The engineer unit will be temporarily removed while it is busy with the de-mining operation 
and will return in the same city once it is done.
]]

object.engineers_Docks =
[[
An engineer can create a 'Dock' unit in any port city that has a 'Harbor' resource associated 
to it, by using the 'd' key. 
^
To proceed, the engineer must be located in the port city, which may not already contain 
a Dock, have 80 supplies in the Allied Theater Depot and possess a minimum of 4 MP's.
^
Failure to meet these minimum conditions will prevent your engineer from proceeding 
with the operation and require you to try again on another turn.
^
Once you accept to build a dock, a random number will determine how many turns it will 
take to build the dock (between 3 to 5 game weeks).
^
The engineer unit will be temporarily removed while it is busy with the construction and 
will return in the same city once it is done.
^
Finallly, a 'Docks' resource icon will appear next to the city to confirm that a dock is
in the process of being built and will remain there once the construction is completed as well.
]]

object.engineers_PontoonBridge =
[[
An engineer can can construct a 'Pontoon Bridge' on any 'RiverBed' tile, by using the 'shift' + '1' 
key combination, thereby allowing mechanized unit to enter and cross riverbeds.
^
To proceed, the engineer must be located on a simple 'RiverBed' tile, have 10 supplies in 
the Allied Theater Depot and possess a minimum of 2 MP's.
^
Furthemore, they cannot build a pontoon bridge during fall or spring turns.
^
Failure to meet these minimum conditions will prevent your engineer from proceeding 
with the operation and require you to try again on another turn.
]]

object.engineers_RadarStations =
[[
An engineer can can construct a Radar Station on any city (not towns) or airbase tile, by using 
the 'shift' + 'k' key combination,
^
To proceed, the engineer must be located in a city or on an airbase tile that doesn't already 
contain a Radar Station, have 15 supplies in the Allied Theater Depot and possess a minimum of 2 MP's.
^
Failure to meet these minimum conditions will prevent your engineer from proceeding 
with the operation and require you to try again on another turn.
]]


object.HMSIllustrious = 

[[
The Illustrious Class Carrier may only transport two types of naval aircraft, the Seafire Mk.II and Fairey Barracuda, and has a maximum carrying capacity of 2 air units.
^
To load the appropriate planes unto the carrier while in port simply put them to sleep and when the carrier exits the city the air plane(s) will automatically be loaded. These same two naval air unit types may of course fly to a carrier unit located on an ocean tile and they will automatically be able to land.
^ 
No other aircraft type will be able to board a carrier while in port or fly to same said carrier while at sea (it will merely fly over the unit).
^ 
There is one exception, a naval air unit could fly to a carrier at sea that already had a maximum capacity and still be able to land.
^ 
As such, it is strongly recommended you use a house rule to avoid taking actions that may trigger this exception.
]]

object.LandingCraft = 

[[
Landing Craft may carry up to 4 ground unit of any type, save DUKW's and Trucks, and are only permitted to unload their cargo either directly onto 'Beach' tiles (*) or when located inside a port city.
^
In addition, if located on an ocean 'BH' tile, you may convert your landing craft unit into a 'Beachhead' unit by pressing on the 'k' key. 
^ 
'Beachhead' units allow Liberty Ships that are situated in the same ocean 'BH' tile to unload their cargo onto an adjacent 'Town - Beach' tile. Should an Axis ground unit enter a town tile located next to a 'Beachhead' then that 'Beachhead' will be permanently destroyed.
^
Converting the landing craft will grant you a 'DUKW' unit which are required for placing 'Supply Hub' units.
^ 
Finally, the Allies may only create a MAXIMUM of 4 Beachhead units in TOTAL during the entire campaign and therefore you should plan to reserve at least one for each of your possible three amphibious invasion operations. 
^ 
(*) Exception: British Commando and US Ranger type units may disembark onto any type of terrain tile.
]]

object.LibertyShip = 

[[
Liberty Ships may carry any type of ground unit and have a maximum carrying capacity of 6 units!
^
They are only capable of unloading their cargo either directly from an ocean 'BH' tile that contains a 'Beachhead' unit onto an adjacent 'Town - Beach' tile or when located inside a port city.
^
Whenever a Liberty Ship enters a Beachhead tile it will lose all its remaining MP's and no other Liberty Ship will be allowed to enter said tile until the first ship departs on a following turn.
^
Should you attempt to unload your cargo onto an invalid location you will be prevented from proceeding with this action and may receive a message to that effect.
]]

object.MountainUnits = 

[[
The Goumiers Marocains, Greek Mtn Bde, 10th US Mountain and British 52nd Mountain are specialized mountain units and are easily identified by the 'blue' dot and stylized 'm' located on the top right corner of their unit.
^ 
They have the Alpine movement and Impassable terrain abilities and are always considered to be in supply.
^
In addition, they reduce the terrain multiplier of enemy units located either in 'Rough', 'Hills' or 'Apennines' terrain by 1.0, which makes them adept at mountain warfare.
]]

object.oilTankers = 

[[
A new supply at sea feature has been implemented within the scenario.
^
Essentially, any naval unit (exception: submarines) that BEGINS its turn at sea will be considered out of supply, unless it is within 3 tiles of a fully supplied oil tanker vessel or on a Beachhead tile.
^
There are two types of Oil Tankers in the game: 
^'Oil Tanker (F)': Fully fueled
^'Oil Tanker (E)': Empty
^
A naval unit out of supply suffers the following two effects:
^1. Its movement points are reduced by 50%
^2. It incurs a 1 HP loss
^
There is a 50% chance that an 'Oil Tanker (F)' which BEGINS its turn at sea within range of ANY other naval units will expend its fuel and be converted to an 'Oil Tanker (E)'. Should 2 or more oil tankers occupy the same tile, only one may become unfueled.
^
In this case, the only way to refuel the 'Oil Tanker (E)' is to return it to a port. Any such tanker which BEGINS its turn in a port will automatically be refuelled.  
^
Finally, an 'Oil Tanker (F)' can refurbish any damaged naval unit simply by moving it onto the damaged unit(s) location, pressing the 'k' key and then selecting the unit you wish to replenish.
^
At a costs of 15 supplies for each transfer, 5 HP's are subtracted from the tanker and then added to the damaged unit's health bars respectively.
]]

-- ^
-- You may, at any time, turn this feature off or on by going to the 'Game Tips and Concept' menu (key '6') and selecting the 'Set Ships At Sea Penalty Option' box.

object.reconnaissance =
[[
Daimler AC,  M8 Greyhound, Beaufighter Mk.VI, P-38 Lightning and Partigiani are reconnaissance type units and are easily identified by the letter 'R' displayed on their unit.
^
They have the ability to 'Spy' on the content of enemy cities.
^
To do so, they must be located next to the target city and then press on the 'k' key, which will them allow them to look inside the city.
^
In addition, American / Free French, British / Commonwealth mechanized / motor troops and tank units may benefit from an extra 1 or 2 attack factors in combat whenever their respective armored cars (M8 Greyhound or Daimler AC) are stacked in the same tile as they are. 
]]

object.supplyTrucks =

[[
The UK and US Truck units play a dual role in the scenario: they can both 
re-supply and/or transport units
^
A. RE-SUPPLY: 
^American and British ground units that are damaged may only be 'resupplied' by 
truck units of their OWN nationality.
^
^To resupply any damaged unit(s) simply move the truck unit onto the damaged  
^unit(s) location and press the 'k' key and then select the unit you wish to replenish. 
^
^At a cost of 3, 5, or 8 supplies for each transfer (for infantry, artillery and   
^armored types), 5 HP's are subtracted from the truck and then added to the 
^damaged unit's health bars respectively.
^
^B. TRANSPORT: 
^Each US and UK Truck unit may carry up to two non-mechanized artillery or 
^infantry units at any given time.
^
^To Load: you may load an active unit onto a truck that is located in the same  
^tile by simply pressing on the 'backspace' key.
^
^To Unload: you may unload a unit from an active truck unit by simply pressing  
^on the 'shift' + 'backspace' key combination.
^
^All passengers are automatically eliminated if the truck they are   
^being carried on is destroyed.
^
^C. TRUCK TYPE MOVEMENT PENALTY: 
^All truck and motorized infantry(*) type units have a natural high movement  
^capacity along road/highway tiles. 
^
^On the other hand they all suffer a +1 (Summer), +2 (Winter) and +3 MP (Fall/Spring)   
^penalty for EACH tile entered that isn't along a road/highway.
^
^(*) IMPORTANT: Allied motorized units are easily identifiable by the truck icon 
^seen on their top right and includes the Cdn Mot. Inf., NZ Mot. Inf., SA Mot. Inf., 
^Free French Infantry and US 1st Infantry.]]

object.Torpedoes = 
[[
Torpedoes can be launched either by Fairey Barracuda aircraft or Tribal Class DD vessels. 
^ 
In both instances, a torpedo can be generated by pressing on the 'u' key. 
^
They have a range of 1 tile and can only be used against naval units.
^
Neither the aircraft or vessels need to be homed to a city to launch a torpedo but a cost of 3 supplies will be subracted from your theater depots (aka treasury) for each one fired.
]]

object.WacoGliders =

[[
Waco Gliders are only able to carry 1st UK Airborne and 82nd Airborne units (all marked with the light blue 'Parachute' icon) and have a maximum carrying capacity of 1 unit and a range of 28 tiles (*)!
^
To load the appropriate unit unto a glider simply put it to sleep and press on 'p' to fly it to its designated drop zone.
^
The initial airborne operations in Sicily were frewn with peril as the crews and pilots were inexperienced and as such there is a chance that your paratroopers may suffer casualties upon landing (**). 
^
You 'MUST' select the airborne unit at the moment your glider lands at which time the casualty rate will be randomly determined. It is recommended you move off the glider tile immediately to avoid further casualties.
^
On the other hand, if your landings are successfull it may bring you rewards in terms of additional supplies. 
^
You could opt not to make any paradrops during this period but would of course forfeit any chance at getting the rewards or having the units participate in your initial invasion plans.  
^
^*  Note: Gliders may only operate during summer months (April to mid-October)
^** Note: The airborne units are only subject to this penalty within the first 5 turns. 
]]


object.gameCredits =
[[
I would like to extend my gratitude and credits to the following:
^
^- To Prof. Garfield not only for his creation of the lua template, which in my very 
^humble opinion is as significant a contribution as The NamelessOne's ToTPP project 
^itself, but for his continued support during my design process in answering my many   
^many technical questions along with furnishing some invaluable code solutions when 
^needed!
^
^- To Knighttime for providing an updated version of his supply and leader bonuses 
^modules and his creation of the AI Ranged munitions module, which were successfully 
^adapted to work within the lua template framework!
^
^- To Fairline not only for his amazing set of WWII graphics but for reviewing and re-skinning 
^many of the scenario's units to their more appropriate color schemes for the period!
^
^- To civ2units and John Petroski, whose wonderful Reformation and Hinge of Fate  
^scenarios, which they allowed me to play test for them, both served as great training  
^resources on how to use and work with the Prof.'s lua template and for some of 
^whose ideas I borrowed and customized for this scenario.
^
^- To Sorfox whose transport module was adapted for use in this scenario!
^
^- To TheNamelessOne for his continued support and development, over the years,   
^of his Test of Time Patch Project.
^
^Many thanks to you all for helping make this latest scenario of mine possible!]]

-- RECRUIT SPECIAL BRITISH UNIT TEXT ===================================

object.british52ndLowlandDivision =

[[
The British 52nd Lowland Division, trained in mountain warfare, was scheduled to participate in Operation Market-Garden but given the battle's ultimate failure, its mission was cancelled.
^
It can be made available to the Italian theather. If you decide to recruit the division, it will cost you 270 supplies and become available in 2 weeks time.
^
If you decline it will be permanetly assigned to France:
^ 
]]

-- end TEXT =========================


-- =============================================
-- IMAGES
-- =============================================
-- Images
-- For optimal integration with the image functionality of the
-- text module, it is recommended that you load all your
-- images here.
-- recommended prefix `m`
--
text.setImageTable(object,"object")-- The string "object" provides a name of the table for error messages.

--Images

-- object.mBulldozer 		= civ.ui.loadImage("Images\\Bulldozer.bmp")
object.m10thArmy			= civ.ui.loadImage("Images\\10thArmy.bmp")
object.m15thAirForce		= civ.ui.loadImage("Images\\15thAirForce.bmp")
object.m21stArmyGrp			= civ.ui.loadImage("Images\\21stArmyGrp.bmp")
object.m5thArmy				= civ.ui.loadImage("Images\\5thArmy.bmp")
object.m6thArmyGrp			= civ.ui.loadImage("Images\\6thArmyGrp.bmp")
object.mAFHQ				= civ.ui.loadImage("Images\\AFHQ.bmp")
object.mAlbertKesselring	= civ.ui.loadImage("Images\\AlbertKesselring.bmp")
object.mAlliedReplacements	= civ.ui.loadImage("Images\\AlliedReplacements.bmp")
object.mAlliesAttack		= civ.ui.loadImage("Images\\AlliesAttack.bmp")
object.mAlliesCaptureCity	= civ.ui.loadImage("Images\\AlliesCaptureCity.bmp")
object.mAlliesEnterRome		= civ.ui.loadImage("Images\\AlliesEnterRome.bmp")
object.mAlliesInMessina		= civ.ui.loadImage("Images\\AlliesInMessina.bmp")
object.mAnzioAnnie			= civ.ui.loadImage("Images\\AnzioAnnie.bmp")
object.mAnzioInvasion		= civ.ui.loadImage("Images\\AnzioInvasion.bmp")
object.mArmyGroupSud		= civ.ui.loadImage("Images\\ArmyGroupSud.bmp")
object.mB24Bomber			= civ.ui.loadImage("Images\\B24Bomber.bmp")
object.mBadoglio			= civ.ui.loadImage("Images\\Badoglio.bmp")
object.mBombedRailYard		= civ.ui.loadImage("Images\\BombedRailYard.bmp")
object.mBridgeBuilding		= civ.ui.loadImage("Images\\BridgeBuilding.bmp")
object.mBridgeDestroyed	  	= civ.ui.loadImage("Images\\BridgeDestroyed.bmp")
object.mBritish52ndMtn	    = civ.ui.loadImage("Images\\British52ndMtn.bmp")
object.mC47Dakota			= civ.ui.loadImage("Images\\C47Dakota.bmp")
object.mCapitalShipSunk		= civ.ui.loadImage("Images\\CapitalShipSunk.bmp")
-- object.mClarkEntersRome		= civ.ui.loadImage("Images\\ClarkEntersRome.bmp")
-- object.mClarkEntersRome2	= civ.ui.loadImage("Images\\ClarkEntersRome2.bmp")
-- object.mClarkEntersRome3	= civ.ui.loadImage("Images\\ClarkEntersRome3.bmp")
object.mCLN					= civ.ui.loadImage("Images\\CLN.bmp")
object.mCombat				= civ.ui.loadImage("Images\\Combat.bmp")
object.mDamagedHarbor		= civ.ui.loadImage("Images\\DamagedHarbor.bmp")
object.mDocksBuilt			= civ.ui.loadImage("Images\\DocksBuilt.bmp")
object.mDUKWBeachead		= civ.ui.loadImage("Images\\DUKWBeachead.bmp")
object.mEngineers 			= civ.ui.loadImage("Images\\Engineers.bmp")
object.mForceH				= civ.ui.loadImage("Images\\ForceH.bmp")
object.mGameConcept			= civ.ui.loadImage("Images\\GameConcept.bmp")
object.mGameConcept2		= civ.ui.loadImage("Images\\GameConcept2.bmp")
object.mGenMontgomery		= civ.ui.loadImage("Images\\GenMontgomery.bmp")
object.mGeorgePatton		= civ.ui.loadImage("Images\\GeorgePatton.bmp")
object.GMCTruck				= civ.ui.loadImage("Images\\GMCTruck.bmp")
object.mGothicLine			= civ.ui.loadImage("Images\\GothicLine.bmp")
object.mGoumierMarocain		= civ.ui.loadImage("Images\\GoumierMarocain.bmp")
object.mGrandCouncil		= civ.ui.loadImage("Images\\GrandCouncil.bmp")
object.mGustavLine			= civ.ui.loadImage("Images\\GustavLine.bmp")
object.mHaroldAlexander		= civ.ui.loadImage("Images\\HaroldAlexander.bmp")
object.mHawkerHurricaneIIc	= civ.ui.loadImage("Images\\HawkerHurricaneIIc.bmp")
object.mHenryWilson			= civ.ui.loadImage("Images\\HenryWilson.bmp")
object.mHMSIllustrious		= civ.ui.loadImage("Images\\HMSIllustrious.bmp")
object.mItalianAirfield		= civ.ui.loadImage("Images\\ItalianAirfield.bmp")
object.mItalianMonarchist	= civ.ui.loadImage("Images\\ItalianMonarchist.bmp")
object.mItaliansSurrender	= civ.ui.loadImage("Images\\ItaliansSurrender.bmp")
object.mItalySurrenders		= civ.ui.loadImage("Images\\ItalySurrenders.bmp")
object.mLandingCraft		= civ.ui.loadImage("Images\\LandingCraft.bmp")
object.mLibertyShip			= civ.ui.loadImage("Images\\LibertyShip.bmp")
object.mLuftflotte2			= civ.ui.loadImage("Images\\Luftflotte2.bmp")
object.mM8Greyhound			= civ.ui.loadImage("Images\\M8Greyhound.bmp")
object.mMarshalTito			= civ.ui.loadImage("Images\\MarshalTito.bmp")
object.mMajorOffensive		= civ.ui.loadImage("Images\\MajorOffensive.bmp")
object.mMarkClark			= civ.ui.loadImage("Images\\MarkClark.bmp")
object.mMarleneDietrich		= civ.ui.loadImage("Images\\MarleneDietrich.bmp")
object.mMineClearance		= civ.ui.loadImage("Images\\MineClearance.bmp")
object.mMinefield	    	= civ.ui.loadImage("Images\\Minefield.bmp")
object.mMonteCassino	    = civ.ui.loadImage("Images\\MonteCassino.bmp")
object.mMountEtna	    	= civ.ui.loadImage("Images\\MountEtna.bmp")
object.mMussoliniLiberated	= civ.ui.loadImage("Images\\MussoliniLiberated.bmp")
object.mNapoliDestruction	= civ.ui.loadImage("Images\\NapoliDestruction.bmp")
object.monexone				= civ.ui.loadImage("Images\\onexone.bmp")
object.mOperationDragoon1	= civ.ui.loadImage("Images\\OperationDragoon1.bmp")
object.mOperationOverlord	= civ.ui.loadImage("Images\\OperationOverlord.bmp")
object.mPalermoSurrenders	= civ.ui.loadImage("Images\\PalermoSurrenders.bmp")
object.mPolishCorps			= civ.ui.loadImage("Images\\PolishCorps.bmp")
object.mPontoonBridge		= civ.ui.loadImage("Images\\PontoonBridge.bmp")
object.mRadarStation		= civ.ui.loadImage("Images\\RadarStation.bmp")
object.mRailyard			= civ.ui.loadImage("Images\\Railyard.bmp")
object.mRegiaMarina			= civ.ui.loadImage("Images\\RegiaMarina.bmp")
object.mRegiaMarinaDestroyers = civ.ui.loadImage("Images\\RegiaMarinaDestroyers.bmp")
object.mRepublicOfSalo		= civ.ui.loadImage("Images\\RepublicOfSalo.bmp")
object.mResupplyAtSea		= civ.ui.loadImage("Images\\ResupplyAtSea.bmp")
object.mSicily				= civ.ui.loadImage("Images\\Sicily.bmp")
object.mSupplyDepot			= civ.ui.loadImage("Images\\SupplyDepot.bmp")
object.mTorpedoBomber		= civ.ui.loadImage("Images\\TorpedoBomber.bmp")
object.mTrucks				= civ.ui.loadImage("Images\\Trucks.bmp")
object.mWacoGliders			= civ.ui.loadImage("Images\\WacoGliders.bmp")

-- Seasons
object.mFall				= civ.ui.loadImage("Images\\Fall.bmp")
object.mSpring				= civ.ui.loadImage("Images\\Spring.bmp")
object.mSummer				= civ.ui.loadImage("Images\\Summer.bmp")
object.mWinter				= civ.ui.loadImage("Images\\Winter.bmp")

-- =============================================
-- COUNTERS AND FLAGS 
-- =============================================
-- Counter Definitions
counter.define("Command Decision #",0)
counter.define("Special Bulletin #",0)
counter.define("Beachhead #",0)
counter.define("# Landing Craft Withdrawn during 1st Invasion",0)
counter.define("# Landing Craft Available for 2nd Invasion",0)
counter.define("# Landing Craft Available for 3rd Invasion",0)

counter.define("# Supplies Spent in Campaign",0)
counter.define("# Supplies Spent on Munitions",0)
counter.define("# Total Allied Supplies Spent",1000)

counter.define("# Cities For Italian Surrender",0)

counter.define("# Gothic Line Ersatz Count",0) -- Max 8
counter.define("# Gothic Line Kasematte Count",0) -- Max 11
counter.define("# Gothic Line Fallschirmjager Count",0) -- Max 7
counter.define("# Gothic Line BKR back line Count",0) -- Max 10
counter.define("# Gothic Line BKR front line Count",0) -- Max 9
counter.define("# Gothic Line WSN back line Count",0) -- Max 5
counter.define("# Gothic Line WSN front line Count",0) -- Max 7
counter.define("# Gothic Line 105mm Art Count",0) -- Max 5
counter.define("# Gothic Line 155mm Art Count",0) -- Max 6
counter.define("# Gothic Line 37mm AA Count",0) -- Max 5
counter.define("# Gothic Line 88mm AA Count",0) -- Max 4

-- ALLIED AND AXIS UNIT KILLED COUNT
counter.define("# Total Allied Killed",0)
counter.define("# Total Axis Killed",0)
counter.define("# Total Germans Killed",0)
counter.define("# Total Italians Killed",0)
counter.define("# Total German Replacements Received",0)


-- ======================================================
-- These define the maximum allowable Allied units
-- ======================================================
-- AMERICANS REPLACEMENTS
counter.define("# P-40",2) -- max: 2
counter.define("# P-38",2) -- max: 2
counter.define("# P-51",1) -- max: 2

counter.define("# Total US Fighters",5)

counter.define("# A-36",2) -- max: 2
counter.define("# B-17",0) -- max: 2 or 4
counter.define("# B-24",0) -- max: 2 (with capture of any major airfield)
counter.define("# B-25",0) -- max: 3 or 2
counter.define("# B-26",0) -- max: 3 or 2

counter.define("# Total US Bombers",2)

counter.define("# M8 Greyhound",1) -- max: 2
counter.define("# Armored Infantry",2) -- max: 2
counter.define("# M5 Stuart",1) -- max: 1
counter.define("# M4A1",3) -- max: 3
counter.define("# M10 Wolverine",3) -- max: 4
counter.define("# M4A3 (76)",0) -- max: 1
counter.define("# M4A1 Ind",3) -- max: 4
counter.define("# US Truck",3) -- max: 4
counter.define("# Engineer",4) -- max: 4

counter.define("# Total US Armored",20)

counter.define("# 1st Infantry",3) -- max: 3
counter.define("# GI Veteran",6) -- max: 9
counter.define("# GI Regular",0) -- max: 18
counter.define("# US Bazooka",1) -- max: 2
counter.define("# US Ranger",1) -- max: 2
counter.define("# 82nd Airborne",1) -- max: 3
counter.define("# 10th Mountain",0) -- max: 3
counter.define("# US 442nd Regt",0) -- max: 1

counter.define("# Total US Infantry",12)

counter.define("# M7 105mm HMC",2) -- max: 2
counter.define("# M12 155mm HMC",1) -- max: 1
counter.define("# US Bofors 40mm AA",2) -- max: 2
counter.define("# M2A1 105mm Art",2) -- max: 4
counter.define("# M1 155mm Art",1) -- max: 2
counter.define("# M1 57mm AT",1) -- max: 2

counter.define("# Total US Artillery",9)

-- BRITISH REPLACEMENTS
counter.define("# Beaufighter MkVI",1) -- max: 1
counter.define("# Kittyhawk",2) -- max: 2
counter.define("# Spitfire MkIX",3) -- max: 3 
counter.define("# Seafire MkII",2)  -- max: 2

counter.define("# Total UK Fighters",8)

counter.define("# Hurricane IIc",1) -- max: 1
counter.define("# Martin Baltimore",1) -- max: 1
counter.define("# Vickers Wellington",0) -- max: 3
counter.define("# Boston III",0) -- max: 1
counter.define("# Sunderland III",1) -- max: 1
counter.define("# Fairey Barracuda",2) -- max: 2

counter.define("# Total UK Bombers",5)

counter.define("# Daimler AC",2) -- max: 2
counter.define("# Bren Carrier",0) -- max: 1
counter.define("# Sherman IIa Bde",4) -- max: 6
counter.define("# Sherman IIa",0) -- max: 3
counter.define("# Churchill Mk IV",0) -- max: 2
counter.define("# Cromwell Mk VII",0) -- max: 2
counter.define("# Sherman Firefly",0) -- max: 1
counter.define("# Churchill AVRE",0) -- max: 2
counter.define("# UK Truck",4) -- max: 5

counter.define("# Total UK Armored",10)

counter.define("# British Veteran",0) -- max: 3
counter.define("# British Regular",10) -- max: 21
counter.define("# British PIAT",2) -- max: 2
counter.define("# British MG",2) -- max: 2
counter.define("# Commandos",1) -- max: 1
counter.define("# 1st Airborne",1) -- max: 3
counter.define("# British 52nd Mountain",0) -- max: 3

counter.define("# Total UK Infantry",16)

counter.define("# Bishop 25pdr",0) -- max: 1
counter.define("# UK Bofors 40mm AA",2) -- max: 2
counter.define("# 25pdr MKII Art",4) -- max: 4
counter.define("# 55inch Art",2) -- max: 2
counter.define("# 6pdr AT",2) -- max: 2

counter.define("# Total UK Artillery",10)

-- ALLIED MINORS REPLACEMENTS

counter.define("# Cdn Mot Inf",0) -- max: 1
counter.define("# Cdn Sherman",2) -- max: 5
counter.define("# Cdn M7 105mm",0) -- max: 1
counter.define("# Cdn Bren Carrier",0) -- max: 1
counter.define("# Cdn Infantry",3) -- max: 3

counter.define("# Total Canadian",5)

counter.define("# Indian Infantry",0) -- max: 6
counter.define("# 4th Indian Div",0) -- max: 3
counter.define("# Gurkhas",0) -- max: 1
counter.define("# Indian 25pdr Art",0) -- max: 1

counter.define("# Total Indian",0)

counter.define("# NZ Mot Inf",0) -- max: 2
counter.define("# NZ Sherman",0) -- max: 3
counter.define("# NZ 25pdr Art",0) -- max: 1

counter.define("# Total New Zealand",0)

counter.define("# SA Mot Inf",0) -- max: 1
counter.define("# SA Sherman",0) -- max: 3
counter.define("# SA Sexton",0) -- max: 1
counter.define("# SA Bren Carrier",0) -- max: 1

counter.define("# Total South African",0)

counter.define("# Braz 1st Inf",0) -- max: 3
counter.define("# Greek Mtn Bde",0) -- max: 1
counter.define("# Jewish Bde",0) -- max: 1

counter.define("# Total Brazil et AL",0)

counter.define("# Infanterie Coloniale",0) -- max: 3
counter.define("# Goumier Marocain",0) -- max: 4
counter.define("# Tirailleur Algerien",0) -- max: 9
counter.define("# Free French Infantry",0) -- max: 3
counter.define("# Free French 105Art",0) -- max: 2
counter.define("# FreeFrenchM10",0) -- max: 2

counter.define("# Total Free French",0)

counter.define("# Polish Sherman",0) -- max: 2
counter.define("# Polish Infantry",0) -- max: 6
counter.define("# Polish 25pdr Art",0) -- max: 1

counter.define("# Total Polish",0)

-- NAVAL REPLACEMENTS (optional?)
counter.define("# Tribal Class DD",6) -- max: 7
counter.define("# Leander Class CL",6) -- max: 7
counter.define("# York Class CA",0) -- max: 3
counter.define("# QE Class BB",2) -- max: 2 (not in total naval unit count)
counter.define("# S-Class",3) -- max: 3
counter.define("# Illustrious Class CV",2) -- max: 2 (not in total naval unit count)
counter.define("# Liberty Ship",3) -- max: 7
-- counter.define("# Oil Tanker (F)",2) -- max: 3
-- counter.define("# Oil Tanker (E)",0) -- max: 3
counter.define("# Total Oil Tankers",2)
counter.define("# DUKW",0) -- max: 4 

counter.define("# Total Naval Units",24)

-- =================================================
-- Flag Definitions
-- =================================================
flag.define("Display Attacker and Defender Values",true)
flag.define("Display Game Tip Pop-Ups",true)
flag.define("Display Out of Supply Pop-Ups",true)
flag.define("Stacking Limit Options",true)
flag.define("Ships At Sea Penalty Option",true)

flag.define("Allied Naval unit count",false)

flag.define("Display Axis Messages",true)
flag.define("Display Axis Testing Messages",false) -- Only used during testing process, otherwise set to false

flag.define("Allies at War",true) -- Initialized at start
flag.define("Patton dismissed",false) -- Occurs as result of slapping incident
flag.define("Game ends March I, 1945",false) -- Occurs after slapping incident
flag.define("General Montgomery Reassigned",false) -- Occurs around Dec I, 1943
flag.define("General Clark Reassigned",false) -- Occurs around Dec I, 1944
flag.define("General Leese Reassigned",false) -- Can occur around Dec IV, 1944

flag.define("Germans evacuate Sicily",false)
flag.define("Sicily is Conquered",false) -- Set flag to true when all the island's cities are captured
flag.define("2nd Invasion Available",false) -- Set flag to true one turn after 'Sicily is Conquered' flag is set to true
flag.define("2nd Invasion Launched",false) 
flag.define("2nd Invasion Above Gustav",false) 
flag.define("Operation Baytown Activated",false) 
flag.define("Operation Slapstick Activated",false) 
flag.define("Operation Brimstone Launched",false) -- Occupation of Sardinia
flag.define("Operation Firebrand Launched",false) -- Occupation of Corsica
flag.define("3rd Invasion Available",false)
flag.define("3rd Invasion Launched",false)
flag.define("3rd Invasion Not-Launched",false)
flag.define("Operation Diadem Launched",false) -- Major Offensive
flag.define("Operation Diadem Ended",false) -- Major Offensive
flag.define("Operation Grapeshot Available",false) -- Major Offensive
flag.define("Operation Grapeshot Launched",false) -- Major Offensive
flag.define("Italian CLN Front Activated",false) 
flag.define("Italian CLN Partisans Activated",false)
flag.define("Polish Troops Demoralized After Yalta",false)
flag.define("Allies completed first SD milestone",false) 
flag.define("Allies completed second SD milestone",false) 
flag.define("Allied automatic victory",false) 

flag.define("Airfield Captured in Bastia",false)
flag.define("Airfield Captured in Foggia",false)
flag.define("Airfield Captured in Napoli",false)
flag.define("Airfield Captured in ROMA",false)

flag.define("Bologna is captured",false) 
flag.define("Firenze is captured",false) 
flag.define("Foggia is captured",false) 
flag.define("Livorno is captured",false)
flag.define("Messina is captured",false) 
flag.define("Napoli is captured",false)
flag.define("Ostia is captured",false) 
flag.define("Pescara is captured",false) 
flag.define("Rimini is captured",false) 
flag.define("Roma is captured",false) 
flag.define("Trieste is captured",false) 

flag.define("Docks being built in Bari",false)
flag.define("Docks being built in Catania",false)
flag.define("Docks being built in Livorno",false)
flag.define("Docks being built in Messina",false)
flag.define("Docks being built in Napoli",false)
flag.define("Docks being built in Palermo",false)
flag.define("Docks being built in Taranto",false)
flag.define("Docks being built in Tunis",true)
flag.define("Docks being built in Valetta",true)

flag.define("Docks built in Bari",false)
flag.define("Docks built in Catania",false)
flag.define("Docks built in Livorno",false)
flag.define("Docks built in Messina",false)
flag.define("Docks built in Napoli",false)
flag.define("Docks built in Palermo",false)
flag.define("Docks built in Taranto",false)
flag.define("Docks built in Tunis",true)
flag.define("Docks built in Valetta",true)

flag.define("Sea Mines being removed from Bari",false)
flag.define("Sea Mines being removed from Brindisi",false)
flag.define("Sea Mines being removed from Catania",false)
flag.define("Sea Mines being removed from Genova",false)
flag.define("Sea Mines being removed from La Spezia",false)
flag.define("Sea Mines being removed from Livorno",false)
flag.define("Sea Mines being removed from Messina",false)
flag.define("Sea Mines being removed from Napoli",false)
flag.define("Sea Mines being removed from Palermo",false)
flag.define("Sea Mines being removed from Pescara",false)
flag.define("Sea Mines being removed from Rimini",false)
flag.define("Sea Mines being removed from Siracusa",false)
flag.define("Sea Mines being removed from Taranto",false)
flag.define("Sea Mines being removed from Trapani",false)
flag.define("Sea Mines being removed from Venezia",false)

flag.define("Sea Mines removed from Bari",false)
flag.define("Sea Mines removed from Brindisi",false)
flag.define("Sea Mines removed from Catania",false)
flag.define("Sea Mines removed from Genova",false)
flag.define("Sea Mines removed from La Spezia",false)
flag.define("Sea Mines removed from Livorno",false)
flag.define("Sea Mines removed from Messina",false)
flag.define("Sea Mines removed from Napoli",false)
flag.define("Sea Mines removed from Palermo",false)
flag.define("Sea Mines removed from Pescara",false)
flag.define("Sea Mines removed from Rimini",false)
flag.define("Sea Mines removed from Siracusa",false)
flag.define("Sea Mines removed from Taranto",false)
flag.define("Sea Mines removed from Trapani",false)
flag.define("Sea Mines removed from Venezia",false)

flag.define("Supply Depot being built in Bari",false)
flag.define("Supply Depot being built in Bologna",false)
flag.define("Supply Depot being built in Catania",false)
flag.define("Supply Depot being built in Firenze",false)
flag.define("Supply Depot being built in Foggia",false)
flag.define("Supply Depot being built in Genova",false)
flag.define("Supply Depot being built in Livorno",false)
flag.define("Supply Depot being built in Messina",false)
flag.define("Supply Depot being built in Napoli",false)
flag.define("Supply Depot being built in Palermo",false)
flag.define("Supply Depot being built in Pescara",false)
flag.define("Supply Depot being built in Rimini",false)
flag.define("Supply Depot being built in Roma",false)
flag.define("Supply Depot being built in Taranto",false)

flag.define("Supply Depot built in Bari",false)
flag.define("Supply Depot built in Bologna",false)
flag.define("Supply Depot built in Catania",false)
flag.define("Supply Depot built in Firenze",false)
flag.define("Supply Depot built in Foggia",false)
flag.define("Supply Depot built in Genova",false)
flag.define("Supply Depot built in Livorno",false)
flag.define("Supply Depot built in Messina",false)
flag.define("Supply Depot built in Napoli",false)
flag.define("Supply Depot built in Palermo",false)
flag.define("Supply Depot built in Pescara",false)
flag.define("Supply Depot built in Rimini",false)
flag.define("Supply Depot built in Roma",false)
flag.define("Supply Depot built in Taranto",false)

flag.define("1st Infantry units withdrawn",false) -- Set flag to true when 1st Infantry withdrawn on turn #21 (Dec I, 43)
flag.define("10th Mountain units arrive",false) -- Set flag to true when 10th Mountain arrives on turn #75 (Jan III, 45)
flag.define("82nd Airborne units withdrawn",false)
flag.define("Ranger units withdrawn",false)
flag.define("M4A3 76(W) enters US armor inventory",false)
flag.define("Veteran US Infantry units withdrawn",false)
flag.define("US Armored Division is Active",true)

flag.define("1st Airborne units withdrawn",false)
flag.define("British Armored Division Deployed",false)
flag.define("Commandos units withdrawn",false)
flag.define("Churchill AVRE enters UK armor inventory",false)
flag.define("Churchill MkIV enters UK armor inventory",false)
flag.define("Sherman Firefly enters UK armor inventory",false)
flag.define("Veteran UK Infantry units withdrawn",false)
flag.define("British 52nd Mountain Deployed",false)

flag.define("Seafire MkII carrier aircraft withdrawn",false)
flag.define("Fairey Barracuda carrier aircraft withdrawn",false)

flag.define("Canadian units withdrawn",false)
flag.define("Cdn 5th Armored units arrive",false)
flag.define("4th Indian Infantry units arrive",false)
flag.define("10th Indian Infantry units arrive",false)
flag.define("Indian Infantry units arrive",false)
flag.define("Gurkhas units arrived",false)
flag.define("New Zealand units arrive",false)
flag.define("South African units arrive",false)

flag.define("Brazil Infantry units arrive",false)
flag.define("Greek Infantry units arrive",false)
flag.define("Jewish Infantry units arrive",false)

flag.define("Goumier Marocain units arrive",false)
flag.define("Tirailleur Algérien units arrive",false)
flag.define("Free French units arrive",false)

flag.define("Polish units arrive",false)


flag.define("Summer Months",true) 
flag.define("FallSpring Months",false) 
flag.define("Winter Months",false) 

-- flag.define("Summer 1943",true) 
-- flag.define("Winter 1943-1944",false) 
-- flag.define("Summer 1944",false) 
-- flag.define("Winter 1944-1945",false) 
-- flag.define("Summer 1945",false) 

flag.define("1st SS Pz Div triggered",false)
flag.define("2nd SS Pz Div triggered",false)
flag.define("16th SS PzGr Div triggered",false)
flag.define("16th Pz Div triggered",false)
flag.define("24th Pz Div triggered",false)
flag.define("26th Pz Div triggered",false)
flag.define("3rd PzGr Div triggered",false)
flag.define("90th PzGr Div triggered",false)

flag.define("3rd Invasion Units, 1st turn, triggered",false)
flag.define("3rd Invasion Units, 2nd turn, triggered",false)
flag.define("3rd Invasion Units, 3rd turn, triggered",false)
flag.define("No 3rd Invasion Units, 1st turn, triggered",false)
flag.define("No 3rd Invasion Units, 2nd turn, triggered",false)
flag.define("No 3rd Invasion Units, 3rd turn, triggered",false)

flag.define("Adriatic Command triggered",false)
flag.define("Ligurian Command triggered",false)
flag.define("Ersatz Reinforcements Triggered",false)


flag.define("Gothic Line Construction Begun",false)
flag.define("Gustav Line is breached",false)
flag.define("Gothic Line is breached",false)
flag.define("German 10th Army Cutoff",false)
flag.define("German 10th Army Escaped",false)
flag.define("Italian Prisonners Can Be Captured",false) 
flag.define("Mussolini Dismissed and Arrested",false) 
flag.define("Republic of Salo Created",false) 
flag.define("Republic of Salo Conquered",false) 
flag.define("Regia Marina Main Battle Fleet Sorties",false)
flag.define("Trasimene Defensive Line Triggered",false) 
flag.define("Italy Surrendered",false) 
flag.define("Italy Surrendered Level 1",false) 
-- flag.define("Italy Surrendered Level 2",false) 
-- flag.define("Italy Surrendered Level 3",false) 
flag.define("Trieste Falls To Communists",false) 

-- this will give you an if you try to access a key not entered into
-- the object table, which could be helpful for debugging, but it
-- means that no nil value can ever be returned for table object
-- If you need that ability, comment out this line
gen.forbidNilValueAccess(object)

return object
