
local versionNumber = 1
local fileModified = true -- set this to true if you change this file for your scenario
-- if another file requires this file, it checks the version number to ensure that the
-- version is recent enough to have all the expected functionality
-- if you set fileModified to true, the error generated if this file is out of date will
-- warn you that you've modified this file



-- This file is meant to group scenario parameters, so that there is a single
-- place where important values are recorded.  It is recommended that you define
-- a parameter value in one place and always refer to that place, so that if
-- you decide to make a change, only one value has to be changed instead of
-- searching for every possible place that value is relevant.

---@module "generalLibrary"
local gen = require("generalLibrary"):minVersion(1)
local text = require("text")
--local eventTools = require("eventTools")
---@module "discreteEvents"
local discreteEvents = require("discreteEventsRegistrar"):minVersion(1)

text.setMoney("%STRING1 gold")
-- text.setMoney("$%STRING1,000")
-- text.setMoneu("%STRING1 Pounds", "%STRING1 Pound")
-- used to determine how text.money(amount)--> string
-- will work
-- if second argument is supplied, that is used when the amount is 1

text.setDigitGroupSeparator(",")
-- text.setDigitGroupSeparator("")
-- text.setDigitGroupSeparator(".")
-- used to determine how text.groupDigits(integer)-->string
-- will work

text.setVeteranTitle("Veteran")
-- sets the string that is returned by text.getVeteranTitle()
--

text.setShortVeteranTitle("Vet")
-- sets the string that is returned by text.getShortVeteranTitle()

--[[ This eventTools functionality has been replaced by the activateUnitBackstop
--      in events.lua that was made available by onGetFormatted date
--      and by the civ.scen.onCityProcessingComplete
--      This code hasn't been removed yet, just in case something breaks and 
--      eventTools still needed

-- The event tools have functionality to guarantee that a tribe
-- will have an active unit.  This ensures that after production
-- events and promotions to new unit types work properly
-- (and, maybe, other stuff too).
-- The unit that is guaranteeing unit activation will be deleted
-- as soon as it is activated.
-- Make sure to choose a unit type that has more than 0 movement points
-- Also, change activationLocations below
--
eventTools.setGuaranteeUnitActivationType(nil)
-- these locations are used for creating units that will
-- guarantee unit activation
-- If possible, choose an out of the way area of the map
-- if none is available, you'll have to write a function
-- to find a place
local activationLocations = {
[0] = civ.getTile(0,0,0) ,
[1] = civ.getTile(0,0,0),
[2] = civ.getTile(0,0,0),
[3] = civ.getTile(0,0,0),
[4] = civ.getTile(0,0,0),
[5] = civ.getTile(0,0,0),
[6] = civ.getTile(0,0,0),
[7] = civ.getTile(0,0,0),
}
local function activationLocationFunction(tribe)
    return activationLocations[tribe.id]
end
eventTools.setGuaranteeActivationUnitLocationFunction(activationLocationFunction)

-- This is the end of the eventTools code for this file
--]]


function discreteEvents.onScenarioLoaded()
    civ.scen.compatibility.activateUnitEveryMove = true
end


-- This function registers what happens when Lua code causes a
-- unit to be destroyed because it is 'out of range/fuel'.  This could be
-- because events expended the last movement points of an aircraft,
-- or because land/sea units have been given a range using Lua.  
-- (No code is run when the standard game detects an aircraft to be
-- out of range)<br><br>
-- If `textOrFunction` is a string, the text is shown when a unit is 
-- lost due to being out of range, with %STRING1 substitutes for the unit type's name.  `title` provides the title for the box.
--
-- If `textOrFunction` is a `function(unit) -> void`, the
-- function is exectued, and is trusted to generate the 
-- the function is trusted to generate a suitable message.
--gen.setOutOfRangeMessage(textOrFunction,title)
local message = 
[[Your aircraft has run out of fuel.
Fighter and Missile units must return to
a city or Carrier at the end of each
turn.  Bomber units must return at the end
of their second turn.]]
gen.setOutOfRangeMessage(message,"Civ Rules: Fuel")


local param = {}

-- ======================================================
-- These define the various movement, operational costs or rewards
-- ======================================================

--These define the fuel (aka treasury) level
param.lowFuelLevel = 50 -- At this point, decent chance units won't move
param.criticalFuelLevel = 50 -- After this point, very strong chance units won't move
param.slightChanceOfNoMovement = .5 
param.greatChanceOfNoMovement = .75 

-- HP RESUPPLY COST
param.resupplyCostArt = 5 -- this is the cost to resupply an artillery unit through the HP Transfer function
param.resupplyCostInf = 3 -- this is the cost to resupply a land unit through the HP Transfer function
param.resupplyCostMot = 8 -- this is the cost to resupply a mechanized land unit through the HP Transfer function
param.resupplyOceanCost = 15 -- this is the cost to resupply a naval unit through the HP Transfer function

param.supplyDepotHeal = 2 

--MOVEMENT COSTS
param.largeShipFuelCost = 8
param.mediumShipFuelCost = 4
param.smallShipFuelCost = 2
param.largeFuelCost = 5 
param.mediumFuelCost = 3
param.smallFuelCost = 1  

-- OPERATIONAL COSTS
param.buildRadarStation = 15
param.repairRiverCrossings = 20
param.beachHeadCreation = 25
param.removeSeaMines = 12
param.buildDocks = 80
param.buildSupplyDepot = 120
param.buildpontoonBridge = 10 -- SEE consolidatedEvents.lua

-- param.removeSeaMinesCatania = 75
-- param.removeSeaMinesSiracusa = 25

param.costPerSupplyHub = 5
param.buildSupplyHub = 15
param.decommissionSupplyHub = 15


-- SUPPLY BONUSES
param.suppliesPerDocks = 25

param.bastiaAirfield = 10 -- 1
param.foggiaAirfield = 5 -- 3
param.napoliAirfield = 5 -- 5
param.romaAirfield = 10 -- 7

param.lcValetta = 5 

-- CASUALTY % PENALTIES
param.casualties26Plus = 15 -- If modify values here also need to change text in 'object.alliedvsAxisLosses' in object.lua file
param.casualties40Plus = 30
param.casualties50Plus = 45
param.casualties60Plus = 60
param.casualties70Plus = 75

-- SPECIAL OPERATIONS COSTS
param.baytownOpsCost = 85 -- If modify value here also need to change text in 'object.operationBaytown' in object.lua file
param.slapstickOpsCost = 50 -- If modify value here also need to change text in 'object.operationSlapstick' in object.lua file
param.teleportCostSardegna = 10

param.ff1stDivisionTranportCost = 90 -- If modify value here also need to change text in 'object.operationTranport1stFF' in object.lua file 

param.recruitBritish52ndMtn = 270

param.operationDiademCost = 500 -- if modify value here also need to change text in 'object.operationDiadem' in object.lua file
param.operationGrapeshotCost = 500

-- PENALTY COSTS
param.pattonReprimandCost = 300 -- 300
param.clarkTakesCommandCost = 200
param.witdrawalInfantryPenalty = 24 -- For reassignment to US 6th Army Group (Operation Dragoon)
param.witdrawalMechanizedPenalty = 70 -- For reassignment to US 6th Army Group (Operation Dragoon)
param.witdrawalAirPenalty = 120 -- For reassignment to US 6th Army Group (Operation Dragoon)

param.CapitalShipSunk = 125 -- onUnitKilled.lua
param.beachheadDestroyed = 50 -- onUnitKilled.lua

-- STACKING AND WINTER HP PENALTY COSTS
param.stackingLimitPenalty = 1 -- SEE discreteEvents.lua
param.minenfeldPenalty = 2  -- SEE discreteEvents.lua

param.navalMPWinterPenalty = 2 -- SEE onActivateUnit.lua
param.navalUnsuppliedHPPenalty = 1 -- SEE onTribeTurnBegin.lua

param.oilTankerRange = 3
param.fuelTankerEmptyChance = 2

-- CITY OR OBJECTIVE CAPTURE REWARDS
param.cityOfBari = 45
param.captureFirenze = 120
param.captureLivorno = 80
param.captureNapoli = 300
param.capturePalermo = 60
param.captureRoma = 450
param.captureSicily = 180
param.gustavLineBreached = 200

param.provincialCapital = 30
param.townOfCrotone = 50
param.townOfSalo = 120

param.capturePrigionieridiguerra = 10
param.sunkLittoriBB = 25

param.destroyedRailYard = 100

-- MUNITIONS COSTS
param.mm105ShellCost = 3
param.mm155ShellCost = 4
param.LightBombloadCost = 3 -- If modify value here also need to change text in 'object.object.AlliedBombers' in object.lua file 
param.MediumBombloadCost = 4 -- If modify value here also need to change text in 'object.object.AlliedBombers' in object.lua file
param.HeavyBombloadCost = 5 -- If modify value here also need to change text in 'object.object.AlliedBombers' in object.lua file
param.LightFlakCost = 2
param.HeavyFlakCost = 3
-- param.mortarShellCost = 2
param.TorpedoCost = 3
param.HeavyShellCost = 6

-- AXIS REWARDS FOR EACH ALLIED UNIT destroyed
param.alliedUnitKilled = 20

-- GERMANS GET EXTRA SUPPLIES EACH TIMES ALLIES SPEND 1000 SUPPLIES ON REPLACEMENTS/MUNITIONS (aka gold)
param.bonusGermanAir = 100

-- COST OF GERMAN AIR REPLACEMENTS
param.extraGermanAir = 1000
param.extraGermanAir_Me109 = 435
param.extraGermanAir_FW190 = 480
param.extraGermanAir_Me110 = 435
param.extraGermanAir_Do217 = 960
param.extraGermanAir_Ju88 = 870

-- ======================================================
-- These define the available Allied LC units
-- ======================================================

-- -- NAVAL UNITS
param.availableLandingCraft2ndInvasion = 3 -- Number to subtract after 1st invasion
param.availableLandingCraft3rdInvasion = 2 -- Number to subtract after 2nd invasion

-- ======================================================
-- These define allowable builds replacement cost per unit
-- ======================================================
-- AMERICAN REPLACEMENT COSTS
param.replacementP38Cost = 80
param.replacementP40Cost = 68
param.replacementP51Cost = 84

param.replacementA36Cost = 66
param.replacementB17Cost = 156
param.replacementB24Cost = 150
param.replacementB25Cost = 106
param.replacementB26Cost = 108

param.replacementM8Cost = 48
param.replacementArmInfCost = 54
param.replacementM5Cost = 62
param.replacementM4A1Cost = 82
param.replacementM4A3Cost = 86
param.replacementM10Cost = 60
param.replacementM4A1IndCost = 76
param.replacementUSTruckCost = 52
param.replacementEngineerCost = 40

param.replacement1stInfCost = 42
param.replacementGIVetCost = 24
param.replacementGIRegCost = 16
param.replacementBazookaCost = 28
param.replacementRangerCost = 36
param.replacement82ndCost = 32
param.replacement10thMtnCost = 32
param.replacement442ndCost = 42

param.replacement105HMCCost = 60
param.replacement155HMCCost = 84
param.replacementUSBoforsCost = 40
param.replacement105ArtCost = 38
param.replacement155ArtCost = 64
param.replacement57ATCost = 30

-- BRITISH REPLACEMENT COSTS
param.replacementBeaufighterCost = 74
param.replacementKittyhawkCost = 68
param.replacementSpitfireMkVCost = 74
param.replacementSeafireMkIICost = 74

param.replacementHurricaneIIcCost = 62
param.replacementMartinBaltimoreCost = 80
param.replacementVickersWellingtonCost = 112
param.replacementBostonIIICost = 106
param.replacementSunderlandIIICost = 102
param.replacementFaireyBarracudaCost = 68

param.replacementDaimlerACCost = 48
param.replacementBrenCarrierCost = 44
param.replacementShermanIIIBdeCost = 76
param.replacementShermanIIICost = 80
param.replacementChurchillMkIVCost = 100
param.replacementCromwellCost = 82 
param.replacementShermanFireflyCost = 94
param.replacementChurchillAVRECost = 130
param.replacementUKTruckCost = 52

param.replacementBritishInfantryVetCost = 20
param.replacementBritishInfantryRegCost = 16
param.replacementBritishPIATCost = 26
param.replacementBritish52ndMtnCost = 26
param.replacementBritishMGCost = 18
param.replacementCommandosCost = 36
param.replacement1stAirborneCost = 32
param.replacementGurkhasCost = 32

param.replacementBishop25pdrCost = 56
param.replacementUKBofors40mmAACost = 40
param.replacement25pdrMKIIArtCost = 36
param.replacement55inchArtCost = 64
param.replacement6pdrATCost = 30

-- ALLIED MINOR REPLACEMENT COSTS
-- Canadian
param.replacementCdnMotInfCost = 36
param.replacementCdnShermanCost = 76
param.replacementCdnM7105mmCost = 60
param.replacementCdnBrenCarrierCost = 46
param.replacementCdnInfantryCost = 26

-- Indian
param.replacementIndianInfantryCost = 16
param.replacement4thIndianDivCost = 20
param.replacementIndian25pdrArtCost = 36

-- New Zealand
param.replacementNZMotInfCost = 36
param.replacementNZShermanCost = 76
param.replacementNZ25pdrArtCost = 36

-- South Africa
param.replacementSAMotInfCost = 36
param.replacementSAShermanCost = 76
param.replacementSASextonCost = 52
param.replacementSABrenCarrierCost = 46

-- Brazil et Al.
param.replacementBraz1stInfCost = 16 
param.replacementGreekMtnBdeCost = 30 
param.replacementJewishBdeCost = 26

-- Free French
param.replacementInfanterieColonialeCost = 16
param.replacementGoumierMarocainCost = 24
param.replacementTirailleurAlgrienCost = 20
param.replacementFreeFrenchInfantryCost = 36
param.replacementFreeFrench105ArtCost = 38
param.replacementFreeFrenchM10Cost = 60

-- Polish
param.replacementPolishShermanCost = 76
param.replacementPolishInfantryCost = 26
param.replacementPolish25pdrArtCost = 36

-- RN & USN REPLACEMENT COSTS
param.replacementDestroyersCost = 172 -- 86
param.replacementLightCruisersCost = 228 -- 114
param.replacementHeavyCruisersCost = 268 -- 134
param.replacementSubmarinesCost = 116 -- 58
param.replacementLibertyShipsCost = 98 -- 49
param.replacementDUKWsCost = 40 -- 20
param.replacementOilTankersCost = 98 -- 49


gen.versionFunctions(param,versionNumber,fileModified,"LuaParameterFiles".."\\".."parameters.lua")
return param
