--[[
aiRangedUnits.lua
by Knighttime
]]

local gen = require("generalLibrary")
local object = require("object")

-- The following table can be updated freely, adding/modifying/deleting rows as necessary, as long as the same pattern is followed.
-- Also checkout 'aiRangedUnits.processRangedUnits(tribe)' found in onTribeTurnBegin.lua file
local RANGED_UNIT_DATA_TABLE = {
	-- German:
	-- Flak Shells
	[object.u37cmFlak36.id]		= { projectileType = object.uLightFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	[object.u88cmFlak37.id]		= { projectileType = object.uHeavyFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	[object.uRailroadYard.id]	= { projectileType = object.uHeavyFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	-- [object.uSdkFz72AA.id]	= { projectileType = object.uLightFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	-- [object.uSSSdkFz72AA.id]	= { projectileType = object.uLightFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	-- Artillery Shells
	[object.uWiderstandsnest.id]= { projectileType = object.uMortarShells,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.uFallschirmjager.id]= { projectileType = object.uMortarShells,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	-- [object.uBunker.id]			= { projectileType = object.uMortarShells,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.uKasematte.id]		= { projectileType = object.u105mmShell,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.u105cmleFH18.id]	= { projectileType = object.u105mmShell,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.u15cmsIG33.id]		= { projectileType = object.u155mmShell,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.uKustenbatterie.id]	= { projectileType = object.u155mmShell,	quantityFired = 2,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.uKruppK5.id]		= { projectileType = object.uHeavyShell,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	-- Italian:
	-- Flak Shells
	[object.u2065AA.id]			= { projectileType = object.uLightFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	[object.u7546AA.id]			= { projectileType = object.uHeavyFlak,		quantityFired = 1,	targetDomain0 = false,	targetDomain1 = true,	targetDomain2 = false },
	-- Artillery Shells
	[object.u10528Art.id]		= { projectileType = object.u105mmShell,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
	[object.u14940Art.id]		= { projectileType = object.u155mmShell,	quantityFired = 1,	targetDomain0 = true,	targetDomain1 = false,	targetDomain2 = true },
}

local function getAdjacentTiles (tile, includeOriginTile) --> table (of tile and possibly boolean objects)
	local tileSet = { }
	local mapWidth, mapHeight, mapQuantity = civ.getAtlasDimensions()
	for ycoord = tile.y - 2, tile.y + 2 do
		for xcoord = tile.x - 2, tile.x + 2 do
			local xdif = math.abs(xcoord - tile.x)
			local ydif = math.abs(ycoord - tile.y)
			if civ.game.rules.flatWorld == false then
				if xcoord < 0 then
					xcoord = xcoord + mapWidth
				elseif xcoord > mapWidth then
					xcoord = xcoord - mapWidth
				end
			end
			if (xcoord + ycoord) % 2 == 0 and (xdif + ydif) <= 2 then
				local thisTile = civ.getTile(xcoord, ycoord, tile.z)
				if thisTile ~= nil and ((xdif == 0 and ydif == 0 and includeOriginTile) or (not(xdif == 0 and ydif == 0))) then
					table.insert(tileSet, thisTile)
				else
					table.insert(tileSet, false)
				end
			end
		end
	end
	return tileSet
end

local function getValidProjectileTargets (rangedUnit, rangedUnitData) --> table (of table objects)
	local sourceTile = rangedUnit.location
	local projectileRange = rangedUnitData.projectileType.move / totpp.movementMultipliers.aggregate
	local tilesEncountered = { }
	local tileId = gen.getTileID(sourceTile)
	tilesEncountered[tileId] = 0
	local tilesNotYetConsidered = { }
	table.insert(tilesNotYetConsidered, { tile = sourceTile, path = { } })
	local validTargets = { }

	while #tilesNotYetConsidered > 0 do
		local consideringTile = tilesNotYetConsidered[1]["tile"]
		local consideringTilePath = {table.unpack(tilesNotYetConsidered[1]["path"])}
		local consideringTileDistance = #consideringTilePath
		table.insert(consideringTilePath, consideringTile)
		table.remove(tilesNotYetConsidered, 1)
		
		for potentialTarget in consideringTile.units do
			if potentialTarget.owner ~= rangedUnit.owner then
				if (potentialTarget.type.domain == 0 and rangedUnitData.targetDomain0 == true) or
				   (potentialTarget.type.domain == 1 and rangedUnitData.targetDomain1 == true) or
				   (potentialTarget.type.domain == 2 and rangedUnitData.targetDomain2 == true)
				then
					print("    Detected " .. potentialTarget.owner.adjective .. " " .. potentialTarget.type.name .. " (ID " .. potentialTarget.id .. ") at " .. consideringTile.x .. "," .. consideringTile.y)
					table.insert(validTargets, {target = potentialTarget, distance = consideringTileDistance})
				end
			end
		end
		
		if projectileRange > consideringTileDistance and (consideringTile.city == nil or consideringTile == sourceTile) then
			for _, adjTile in ipairs(getAdjacentTiles(consideringTile, false)) do
				if civ.isTile(adjTile) then
					tileId = gen.getTileID(adjTile)
					if tilesEncountered[tileId] == nil or tilesEncountered[tileId] > (consideringTileDistance + 1) then
						tilesEncountered[tileId] = consideringTileDistance + 1
						table.insert(tilesNotYetConsidered, { tile = adjTile, path = consideringTilePath })
					end
				end
			end
		end
	end

	return validTargets
end

local function firingRoutine (rangedUnit) --> void
	print("Processing ranged unit: " .. rangedUnit.owner.adjective .. " " .. rangedUnit.type.name .. " (ID " .. rangedUnit.id .. ") at " .. rangedUnit.x .. "," .. rangedUnit.y .. "," .. rangedUnit.z)
	local rangedUnitData = RANGED_UNIT_DATA_TABLE[rangedUnit.type.id]
	local targets = getValidProjectileTargets(rangedUnit, rangedUnitData)
	print("  Targets found: " .. tostring(#targets))
	if #targets > 0 then
		for i = 1, rangedUnitData.quantityFired do
			-- Probability of actually firing depends on percent of HP remaining
			local fireChance = (rangedUnit.hitpoints / rangedUnit.type.hitpoints) * 100
			local randomNumber = math.random(100)
			print("    Firing chance = " .. fireChance .. "%, randomNumber = " .. randomNumber)
			if randomNumber <= fireChance then
				local projectile = civ.createUnit(rangedUnitData.projectileType, rangedUnit.owner, rangedUnit.location)
				projectile.homeCity = nil
				print("  Created " .. projectile.type.name .. " at " .. projectile.x .. "," .. projectile.y .. "," .. projectile.z)
			else
				print("    Not permitted to fire")
			end
		end
	end
end

local function processRangedUnits (tribe) --> void
	-- Per specifications, this module only handles ranged units belonging to AI tribes.
	-- (The human player will fire projectiles manually with a key press, and there are no barbarians in the scenario.)
	if tribe.id > 0 and tribe.isHuman == false then
		print("Processing ranged units for " .. tribe.name .. "...")
		for unit in civ.iterateUnits() do
			if unit.owner == tribe and RANGED_UNIT_DATA_TABLE[unit.type.id] ~= nil then
				firingRoutine(unit)
			end
		end
	end
end

print("aiRangedUnits.lua parsed successfully at " .. os.date("%c"))
print("    Lines found: " .. debug.getinfo(1).currentline + 4)

return {
	processRangedUnits = processRangedUnits,
}
