local gen = require("generalLibrary")
local param = require("parameters")
local object = require("object")
local text = require("text")
local func = require("functions")
local flag = require("flag")
local counter = require("counter")
local onTurn = require("onTurn")


local delayedAction = require("delayedAction")

-- INITIAL 15TH ARMY COMMANDER NAME (can change overtime)
local commanderAFHQ_Base = onTurn.commanderAFHQ_Base
local commander15thArmy_Base = onTurn.commander15thArmy_Base

local function smallNestedMenu(menuType)
	-- MENU FOR COMMAND DECISIONS (see also consolidatedEvents.lua (KEY PRESS '5'))
	
    menuType = menuType or "menu1" -- menu1 is the default menu
    -- MENU 1 (INTRO)
	if menuType == "menu1" then
		local menuText = "Select A Command Decision!"
		local menuTable = {"Activate 2nd Invasion","Activate 3rd Invasion","Launch Major Offensive","Exit Menu"}
		local menuImage = civ.ui.loadImage("Images\\AnzioInvasion.bmp")
		local menuTitle = "Command Decision Required!"
		local choice = text.menu(menuTable,menuText,menuTitle,menuImage)
		if choice == 1 then
			return smallNestedMenu("menu2")
		
		elseif choice == 2 then
			return smallNestedMenu("menu3")
			
		elseif choice == 3 then
			return smallNestedMenu("menu4")

		elseif choice == 4 then
			return
		end
	end

	-- MENU 2: 2ND INVASION OPTIONS
	if menuType == "menu2" then
		if flag.value("2nd Invasion Available") == true and civ.getTurn() <= 13 then
			local menuText = "Select 2nd Invasion Options!"
			local menuTable = {"Activate Baytown","Activate Slapstick","Activate 2nd Invasion","Return to Main Menu","Exit Menu"}
			local choice = text.menu(menuTable,menuText,"Activate 2nd Invasion")
	
			-- OPERATION BAYTOWN: the amphibious invasion of the southern tip of Italy			
			if choice == 1 then
				if flag.value("Operation Baytown Activated") == false then
					-- local menuText = "Select 2nd Invasion Options!"
					local menuText = object.operationBaytown 
					local menuTable = {"Return to Main Menu","Yes, proceed","No, don't proceed and permanently cancel the mission","Exit Menu"}
					local menuImage = civ.ui.loadImage("Images\\AnzioInvasion.bmp")
					local menuTitle = "Activate Operation Baytown?"
					local choice = text.menu(menuTable,menuText,menuTitle,menuImage)			
					-- local choice = text.menu(menuTable,menuText,"Activate Slapstick")
						if choice == 1 then
							return  smallNestedMenu("menu1")
						elseif choice == 2 then
							-- Check to see if you have the necessary supplies
							if object.pAllies.money >= param.baytownOpsCost then
								local a = 0
								local b = 0
								local c = 0
								local d = 0
								local e = 0
								-- First remove any Axis unit that may be located in Giola Tauro
								for activeUnit in civ.iterateUnits() do
									if activeUnit.owner == object.pAxis and activeUnit.location == object.ltownGiolaTauro then 
									print("Teleported " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
									civ.teleportUnit(activeUnit, object.lNapoli)
									end
								end
								-- Then teleport the British units located in Messina to Giola Tauro
								for unit in civ.iterateUnits() do 
									-- Count 3 Br. Infantry
									if unit.type == object.uBritishRegular and (unit.location.city == object.cMessina) then
										if a < 3 then
											civ.teleportUnit(unit, object.ltownGiolaTauro)
											a = a + 1
										end
									end    
									-- Count 1 Br. Sherman
									if unit.type == object.uShermanIIaBde and (unit.location.city == object.cMessina) then
										if b < 1 then
											civ.teleportUnit(unit, object.ltownGiolaTauro)
											b = b + 1
										end
									end 
									-- Count 1 Daimler AC
									if unit.type == object.uDaimlerAC and (unit.location.city == object.cMessina) then
										if c < 1 then
											civ.teleportUnit(unit, object.ltownGiolaTauro)
											c = c + 1
										end
									end 
									-- Count 1 UK Truck
									if unit.type == object.uUKTruck and (unit.location.city == object.cMessina) then
										if d < 1 then
											civ.teleportUnit(unit, object.ltownGiolaTauro)
											d = d + 1
										end
									end 
									-- Count 1 DUKW
									if unit.type == object.uDUKW and (unit.location.city == object.cMessina) then
										if e < 1 then
											civ.teleportUnit(unit, object.ltownGiolaTauro)
											e = e + 1
										end
									end 
								end 
								-- gen.createUnit(object.uSupplyHub, object.pAllies, {{80,156,0}}, {randomize=false, veteran=false})	
								civ.ui.text("The British land at total of " .. tostring(a + b + c + d + e) .. " ground units at Giola Tauro!")
								object.pAllies.money = object.pAllies.money - param.baytownOpsCost
								flag.setTrue("Operation Baytown Activated")
								return  smallNestedMenu("menu2")
							else
								civ.ui.text(func.splitlines("You have insufficient reserves in your theather depots to cover the cost of " .. param.baytownOpsCost .. " supplies!\n^\r^You may try again when you've amassed the necessary supplies."))
								smallNestedMenu("menu2")
							end	
						
						elseif choice == 3 then
							text.simple("Operation Baytown has permanently been cancelled!","Baytown Cancelled!",object.monexone)
							flag.setTrue("Operation Baytown Activated")
							return  smallNestedMenu("menu2")

						elseif choice == 4 then
							return
						end
				elseif flag.value("Operation Baytown Activated") == true then
					text.simple("You have already opted to launch or cancel Operation Baytown!","Baytown Launched/Cancelled!",object.monexone)
					return smallNestedMenu("menu2")			
				end

			-- OPERATION SLAPTICK: the special naval/airborne operation to capture the port of Taranto
			elseif choice == 2 then
				if flag.value("Operation Slapstick Activated") == false then
					-- local menuText = "Select 2nd Invasion Options!"
					local menuText = object.operationSlapstick
					local menuTable = {"Return to Main Menu","Yes, proceed","No, don't proceed and permanently cancel the mission","Exit Menu"}
					local menuImage = civ.ui.loadImage("Images\\AnzioInvasion.bmp")
					local menuTitle = "Activate Operation Slapstick?"
					local choice = text.menu(menuTable,menuText,menuTitle,menuImage)			
					-- local choice = text.menu(menuTable,menuText,"Activate Slapstick")
						if choice == 1 then
							return  smallNestedMenu("menu1")
						elseif choice == 2 then
							-- Check to see if you have the necessary supplies
							if object.pAllies.money >= param.slapstickOpsCost then
								-- First remove any Axis unit that may be located in Taranto
								for activeUnit in civ.iterateUnits() do
									if activeUnit.owner == object.pAxis and activeUnit.type.domain == 2 and activeUnit.location == object.lTaranto then 
										print("Teleported " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
										civ.teleportUnit(activeUnit, object.lPescara)
									elseif activeUnit.owner == object.pAxis and (activeUnit.type.domain == 0 or activeUnit.type.domain == 1) and activeUnit.location == object.lTaranto then
										print("Teleported " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
										civ.teleportUnit(activeUnit, object.lFoggia)									
									end
								end
								-- Secondly, if exist, teleport any British Para's into Catania
								for activeUnit in civ.iterateUnits() do
								   if activeUnit.owner == object.pAllies and activeUnit.type == object.u1stAirborne then 
										civ.teleportUnit(activeUnit, object.lCatania )
										print("Teleport " .. activeUnit.type.name .. " at " .. activeUnit.location.x .. "," .. activeUnit.location.y .. "," .. activeUnit.location.z)
					
								   end
								end
								gen.createUnit(object.uYorkClassCA, object.pAllies, {{62,162,0}}, {randomize=false, veteran=false})
								civ.ui.text("The British " .. object.u1stAirborne.name .. " units and the " .. object.uYorkClassCA.name .. " were deployed to Catania and are ready to sail for the port of Taranto!")
								object.pAllies.money = object.pAllies.money - param.slapstickOpsCost
								flag.setTrue("Operation Slapstick Activated")
								
								counter.add("# York Class CA",1)
								counter.add("# Total Naval Units",1)			
								
								return  smallNestedMenu("menu2")
							else
								civ.ui.text(func.splitlines("You have insufficient reserves in your theather depots to cover the cost of " .. param.slapstickOpsCost .. " supplies!\n^\r^You may try again when you've amassed the necessary supplies."))
								smallNestedMenu("menu2")
							end	
								
						elseif choice == 3 then
							text.simple("Operation Slapstick has permanently been cancelled!","Slapstick Cancelled!",object.monexone)
							flag.setTrue("Operation Slapstick Activated")
							return  smallNestedMenu("menu2")

						elseif choice == 4 then
							return
						end
				elseif flag.value("Operation Slapstick Activated") == true then
					text.simple("You have already opted to launch or cancel Operation Slapstick!","Slapstick Launched/Cancelled!",object.monexone)
					return smallNestedMenu("menu2")			
				end

				
			-- 2ND INVASION: the primary amphibious invasion operation of the Italian mainland
			elseif choice == 3 
			-- and flag.value("2nd Invasion Launched") == false 
			then
				if flag.value("2nd Invasion Launched") == false then
					local menuText = object.operationAvalanche
					local menuTable = {"Return to Main Menu","Yes, proceed with invasion of the Adriatico coast","Yes, proceed with invasion of the Tirreno coast","Exit Menu"}
					local menuImage = civ.ui.loadImage("Images\\AnzioInvasion.bmp")
					local menuTitle = "Activate Invasion of Italian Mainland?"
					local choice = text.menu(menuTable,menuText,menuTitle,menuImage)	
						if choice == 1 then
							return  smallNestedMenu("menu1")
							
						elseif choice == 2 then
							local a = 0
							for activeUnit in civ.iterateUnits() do
								-- if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
									-- if a < param.maxtotalLandingCraft then
									if a < counter.value("# Landing Craft Available for 2nd Invasion") then
										gen.createUnit(object.uLandingCraft, object.pAllies, {{73,155,0}, {62,162,0}}, {randomize=true, veteran=false})
										-- gen.createUnit(object.uLandingCraft, object.pAllies, {{0,136,0}, {40,180,0}}, {randomize=true, veteran=false})
										a = a + 1
									end
								-- end
							end
							-- Setting the terrain to Ocean (type = 10) allows designer to place Waco Gliders (which is a domain = 2 (sea) unit) on a land airbase tile
								gen.setTerrainType(object.lAirbaseCatania,10) 
								gen.setTerrainType(object.lAirbaseMessina,10) 
								gen.createUnit(object.uWacoGlider, object.pAllies, {{73,153,0}, {59,161,0}}, {count=1, randomize=false, veteran=false})
								gen.createUnit(object.uC47Dakota, object.pAllies, {{73,153,0}, {59,161,0}}, {count=1, randomize=false, veteran=false})
							-- Immediately after placing Waco Glider on airbase tile, reset terrain to its original 'Grassland' (type = 6)
								gen.setTerrainType(object.lAirbaseCatania,6) 
								gen.setTerrainType(object.lAirbaseMessina,6)
							
							civ.playSound("Shipwhistle.wav")	
							text.simple("The " .. object.uLandingCraft.name .. " units were deployed to Catania and Messina and are ready to embark troops!","The Landing Craft Are Deployed!",object.mLandingCraft)
							flag.setTrue("2nd Invasion Launched")
							
							-- See DELAYED ACTIONS SECTION FOR 2ND and 3RD INVASIONS below
							delayedAction.doInFuture("secondInvasionLC_warning",{},civ.getTurn()+ 1 ,object.pAllies.id)
							delayedAction.doInFuture("secondInvasionLC_deployed",{},civ.getTurn()+ 2 ,object.pAllies.id)
							
							return  smallNestedMenu("menu2")
							
						elseif choice == 3 then
							local a = 0
							for activeUnit in civ.iterateUnits() do
								-- if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
									-- if a < param.maxtotalLandingCraft then
									if a < counter.value("# Landing Craft Available for 2nd Invasion") then
										gen.createUnit(object.uLandingCraft, object.pAllies, {{73,155,0}, {44,140,0}}, {randomize=true, veteran=false})
										-- gen.createUnit(object.uLandingCraft, object.pAllies, {{0,136,0}, {40,180,0}}, {randomize=true, veteran=false})
										a = a + 1
									end
								-- end
							end
							-- Setting the terrain to Ocean (type = 10) allows designer to place Waco Gliders (which is a domain = 2 (sea) unit) on a land airbase tile
								gen.setTerrainType(object.lAirbaseCatania,10) 
								gen.setTerrainType(object.lAirbaseMessina,10) 
								gen.createUnit(object.uWacoGlider, object.pAllies, {{73,153,0}, {59,161,0}}, {count=1, randomize=false, veteran=false})
								gen.createUnit(object.uC47Dakota, object.pAllies, {{73,153,0}, {59,161,0}}, {count=1, randomize=false, veteran=false})
							-- Immediately after placing Waco Glider on airbase tile, reset terrain to its original 'Grassland' (type = 6)
								gen.setTerrainType(object.lAirbaseCatania,6) 
								gen.setTerrainType(object.lAirbaseMessina,6)							
							
							civ.playSound("Shipwhistle.wav")
							text.simple("The " .. object.uLandingCraft.name .. " units were deployed to Messina and Palermo and are ready to embark troops!","The Landing Craft Are Deployed!",object.mLandingCraft)
							flag.setTrue("2nd Invasion Launched")
							
							-- See DELAYED ACTIONS SECTION FOR 2ND and 3RD INVASIONS below
							delayedAction.doInFuture("secondInvasionLC_warning",{},civ.getTurn()+ 2 ,object.pAllies.id)
							delayedAction.doInFuture("secondInvasionLC_deployed",{},civ.getTurn()+ 3 ,object.pAllies.id)
							
							return  smallNestedMenu("menu2")
							
						-- elseif choice == 3 then
							-- return  smallNestedMenu("menu2")
						elseif choice == 4 then
							return
						end
				elseif flag.value("2nd Invasion Launched") == true then
					text.simple("You have already opted to launch the 2nd Invasion!","2nd Invasion Launched!",object.monexone)
					return smallNestedMenu("menu2")			
				end					

			elseif choice == 4 then		
				return smallNestedMenu("menu1")
			
			elseif choice == 5 then		
				return
			end
		
		else
			text.simple("You must conquer Sicily before being able to have access to the 2nd invasion options and cannot use them any later than October I, 1943 (turn #13)!","Conquer Sicily!",object.mAFHQ)
		end
	end
    
	-- MENU 3: 3RD INVASION OPTIONS
	if menuType == "menu3" then
		-- text.setLinesPerWindow(20)
		if (flag.value("3rd Invasion Available") == false and flag.value("3rd Invasion Launched") == false) or counter.value("Beachhead #") > 4 then
			text.simple("You may not have access to the 3rd Invasion option prior to Week I of November, 1943 or if you've already placed a maximum of 4 beachhead units!","3rd Invasion Option!",object.mAFHQ)
			return smallNestedMenu("menu1")
		elseif flag.value("3rd Invasion Available") == true and flag.value("3rd Invasion Launched") == false then
			local menuText = object.operationShingle
			local menuTable = {"Return to Main Menu","Yes, proceed with the Mare Adriatico (Bari) option","Yes, proceed with the Mare Tirreno (Napoli) option","Yes, proceed with the Mare Tirreno (Ostia) option","No, don't proceed and permanently cancel the mission","Exit Menu"}
			local menuImage = civ.ui.loadImage("Images\\AnzioInvasion.bmp")
			local menuTitle = "Activate 3rd Amphibious Invasion Operation?"
			local choice = text.menu(menuTable,menuText,menuTitle,menuImage)	
			if choice == 1 then
				return smallNestedMenu("menu1")
			elseif choice == 2 and flag.value("3rd Invasion Launched") == false then
				if flag.value("Sea Mines removed from Bari") == true then
					local a = 0
					for activeUnit in civ.iterateUnits() do
						-- if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
						-- if a < param.maxtotalLandingCraft then
							if a < counter.value("# Landing Craft Available for 3rd Invasion") then
								gen.createUnit(object.uLandingCraft, object.pAllies, {{123,125,0}}, {randomize=false, veteran=false})
								-- gen.createUnit(object.uLandingCraft, object.pAllies, {{0,136,0}}, {randomize=true, veteran=false})
								a = a + 1
							end
						-- end
					end
					
					civ.playSound("Shipwhistle.wav")
					text.simple("The ".. counter.value("# Landing Craft Available for 3rd Invasion") .. " " .. object.uLandingCraft.name .. " units were deployed to Bari and are ready to embark troops!","The Landing Craft Are Deployed!",object.mLandingCraft)
					flag.setTrue("3rd Invasion Launched")
					
					-- See DELAYED ACTIONS SECTION FOR 2ND and 3RD INVASIONS below
					delayedAction.doInFuture("thirdInvasionLC_warning",{},civ.getTurn()+ 2 ,object.pAllies.id)
					delayedAction.doInFuture("thirdInvasionLC_deployed",{},civ.getTurn()+ 3 ,object.pAllies.id)
					
					return smallNestedMenu("menu1")
				else
					text.simple("The sea mines must be removed from the port of Bari before you can proceed with the invasion plans!","Clear Sea Mines from Bari!",object.monexone)
					return smallNestedMenu("menu1")
				end				
				
			elseif choice == 3 and flag.value("3rd Invasion Launched") == false then
				if flag.value("Sea Mines removed from Napoli") == true then
					local a = 0
					for activeUnit in civ.iterateUnits() do
						-- if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
						-- if a < param.maxtotalLandingCraft then
							if a < counter.value("# Landing Craft Available for 3rd Invasion") then
								gen.createUnit(object.uLandingCraft, object.pAllies, {{88,106,0}}, {randomize=false, veteran=false})
								-- gen.createUnit(object.uLandingCraft, object.pAllies, {{40,180,0}}, {randomize=true, veteran=false})
								a = a + 1
							end
						-- end
					end
					
					civ.playSound("Shipwhistle.wav")
					text.simple("The ".. counter.value("# Landing Craft Available for 3rd Invasion") .. " " .. object.uLandingCraft.name .. " units were deployed to Napoli and are ready to embark troops!","The Landing Craft Are Deployed!",object.mLandingCraft)
					flag.setTrue("3rd Invasion Launched")

					-- See DELAYED ACTIONS SECTION FOR 2ND and 3RD INVASIONS below
					delayedAction.doInFuture("thirdInvasionLC_warning",{},civ.getTurn()+ 2 ,object.pAllies.id)
					delayedAction.doInFuture("thirdInvasionLC_deployed",{},civ.getTurn()+ 3 ,object.pAllies.id)

					return smallNestedMenu("menu1")
				else
					text.simple("The sea mines must be removed from the port of Napoli before you can proceed with the invasion plans!","Clear Sea Mines from Napoli!",object.monexone)
					return smallNestedMenu("menu1")
				end		

			elseif choice == 4 then
				if flag.value("Ostia is captured") == true then
					local a = 0
					for activeUnit in civ.iterateUnits() do
						-- if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
						-- if a < param.maxtotalLandingCraft then
							if a < counter.value("# Landing Craft Available for 3rd Invasion") then
								gen.createUnit(object.uLandingCraft, object.pAllies, {{36,68,0}}, {randomize=false, veteran=false})
								a = a + 1
							end
						-- end
					end
					
					civ.playSound("Shipwhistle.wav")
					text.simple("The ".. counter.value("# Landing Craft Available for 3rd Invasion") .. " " .. object.uLandingCraft.name .. " units were deployed to Ostia and are ready to embark troops!","The Landing Craft Are Deployed!",object.mLandingCraft)
					flag.setTrue("3rd Invasion Launched")

					-- See DELAYED ACTIONS SECTION FOR 2ND and 3RD INVASIONS below
					delayedAction.doInFuture("thirdInvasionLC_warning",{},civ.getTurn()+ 2 ,object.pAllies.id)
					delayedAction.doInFuture("thirdInvasionLC_deployed",{},civ.getTurn()+ 3 ,object.pAllies.id)

					return smallNestedMenu("menu1")
				else
					text.simple("The port of Ostia must be under your control before you can proceed with the invasion plans!","Control Ostia!",object.monexone)
					return smallNestedMenu("menu1")
				end	
				
			elseif choice == 5 then
				civ.ui.text("The 3rd Invasion operation has been permanently cancelled!")
				flag.setTrue("3rd Invasion Launched")
				return smallNestedMenu("menu1")
			
			elseif choice == 6 then
				return
			end

		elseif flag.value("3rd Invasion Available") == true and flag.value("3rd Invasion Launched") == true then
			text.simple("You have already opted to launch/cancel your 3rd amphibious invasion!","Invasion Option Taken/Cancelled!",object.monexone)
			return smallNestedMenu("menu1")
		end			
    end
	
	-- MENU 4: MAJOR OFFENSIVE OPERATIONS
	if menuType == "menu4" then
		if civ.getTurn() >= 41 then -- May I, 1944 (#41)
			local menuText = "Select Major Offensive Option!"
			local menuTable = {"Launch Operation Diadem", "Launch Operation Grapeshot", "Return to Main Menu","Exit Menu"}
			local choice = text.menu(menuTable,menuText,"Launch Major Offensive")
	
			-- OPERATION DIADEM: 			
			if choice == 1 then
				if (civ.getTurn() >= 41 and civ.getTurn() <= 57) and flag.value("Operation Diadem Launched") == false then
					-- local menuText = "Select 2nd Invasion Options!"
					local menuText = object.operationDiadem 
					local menuTable = {"Return to Main Menu","Yes, proceed with Diadem","No, don't proceed and permanently cancel the mission","Exit Menu"}
					local menuImage = civ.ui.loadImage("Images\\MajorOffensive.bmp")
					local menuTitle = "Activate Operation Diadem?"
					local choice = text.menu(menuTable,menuText,menuTitle,menuImage)			
					-- local choice = text.menu(menuTable,menuText,"Activate Diadem")
						if choice == 1 then
							return  smallNestedMenu("menu1")
						elseif choice == 2 then
							-- Check to see if you have the necessary supplies
							if object.pAllies.money >= param.operationDiademCost then
								civ.playSound("ArtilleryB.wav")
								text.simple("The Allies have launched Operation Diadem and starting this and the following three weeks both your munitions and ground troops will benefit from a +2 AF to all attacks!","Operation Diadem Underway!",object.mMajorOffensive)
								object.pAllies.money = object.pAllies.money - param.operationDiademCost
								civ.giveTech(object.pAllies, object.aOperationDiadem)
								flag.setTrue("Operation Diadem Launched")
								
								delayedAction.doInFuture("operationDiadem_launched",{},civ.getTurn()+ 3 ,object.pAllies.id)
								return  -- smallNestedMenu("menu4")
							else
								civ.ui.text(func.splitlines("You have insufficient reserves in your theater depots to cover the cost of " .. param.operationDiademCost .. " supplies!\n^\r^You may try again when you've amassed the necessary supplies."))
								smallNestedMenu("menu1")
							end	
						
						elseif choice == 3 then
							text.simple("Operation Diadem has permanently been cancelled!","Diadem Cancelled!",object.monexone)
							flag.setTrue("Operation Diadem Launched")
							return  smallNestedMenu("menu1")

						elseif choice == 4 then
							return
						end
				elseif flag.value("Operation Diadem Launched") == true then
					text.simple("You have already opted to launch or cancel Operation Diadem!","Diadem Launched/Cancelled!",object.monexone)
					return smallNestedMenu("menu4")			
				end

			-- OPERATION GRAPESHOT: 
			elseif choice == 2 then
				-- if civ.getTurn() >= 85 and flag.value("Operation Grapeshot Launched") == false then
				if flag.value("Operation Grapeshot Available") == false and flag.value("Operation Grapeshot Launched") == false then 
					text.simple("You cannot access the Operation Grapeshot option till 16 weeks after launching Operation Diadem!","Grapeshot Unavailable!",object.monexone)
					return smallNestedMenu("menu4")		
				elseif flag.value("Operation Grapeshot Available") == true and flag.value("Operation Grapeshot Launched") == false then
					-- local menuText = "Select 2nd Invasion Options!"
					local menuText = object.operationGrapeshot
					local menuTable = {"Return to Main Menu","Yes, proceed with Grapeshot","No, don't proceed and permanently cancel the mission","Exit Menu"}
					local menuImage = civ.ui.loadImage("Images\\MajorOffensive.bmp")
					local menuTitle = "Activate Operation Grapeshot?"
					local choice = text.menu(menuTable,menuText,menuTitle,menuImage)			
					-- local choice = text.menu(menuTable,menuText,"Activate Grapeshot")
						if choice == 1 then
							return  smallNestedMenu("menu1")
						elseif choice == 2 then
							-- Check to see if you have the necessary supplies
							if object.pAllies.money >= param.operationGrapeshotCost then
								civ.playSound("ArtilleryB.wav")
								text.simple("The Allies have launched Operation Grapeshot and starting this and the following three weeks both your munitions and ground troops will benefit from a +2 AF to all attacks!","Operation Grapeshot Underway!",object.mMajorOffensive)
								object.pAllies.money = object.pAllies.money - param.operationGrapeshotCost
								civ.giveTech(object.pAllies, object.aOperationGrapeshot)
								flag.setTrue("Operation Grapeshot Launched")
								
								delayedAction.doInFuture("operationGrapeshot_launched",{},civ.getTurn()+ 3 ,object.pAllies.id)
								return  -- smallNestedMenu("menu1")
							else
								civ.ui.text(func.splitlines("You have insufficient reserves in your theater depots to cover the cost of " .. param.operationGrapeshotCost .. " supplies!\n^\r^You may try again when you've amassed the necessary supplies."))
								smallNestedMenu("menu1")
							end	
								
						elseif choice == 3 then
							text.simple("Operation Grapeshot has permanently been cancelled!","Grapeshot Cancelled!",object.monexone)
							flag.setTrue("Operation Grapeshot Launched")
							return  smallNestedMenu("menu1")

						elseif choice == 4 then
							return
						end
				-- elseif civ.getTurn() <= 84 then 
					-- if civ.getTurn() <= 84 then 
						-- text.simple("You cannot access the Operation Grapeshot option prior to Apr I, 1945 (turn #85)","Grapeshot Unavailable!",object.monexone)
						-- return smallNestedMenu("menu4")					
					-- elseif flag.value("Operation Grapeshot Launched") == true then
						-- text.simple("You have already opted to launch or cancel Operation Grapeshot!","Grapeshot Launched/Cancelled!",object.monexone)
						-- return smallNestedMenu("menu4")
					-- end	
				elseif flag.value("Operation Grapeshot Available") == true and flag.value("Operation Grapeshot Launched") == true then
					text.simple("You have already opted to launch or cancel Operation Grapeshot!","Grapeshot Launched/Cancelled!",object.monexone)
					return smallNestedMenu("menu1")
				end

			elseif choice == 3 then		
				return smallNestedMenu("menu1")
			
			elseif choice == 4 then		
				return
			end
		
		else
			text.simple("You cannot access the Major Offensive options prior to May I, 1944 (turn #41)!","Major Offensive Unavailable!",object.mAFHQ)
		end
	end	

	-- MENU 5: SARDEGNA OPERATION
	-- if menuType == "menu5" then
		-- if flag.value("Italy Surrendered") == false and flag.value("Operation Brimstone Launched") == false then
			-- civ.ui.text("You may not have access to Operation Brimstone prior to Italy surrendering!")
			-- return smallNestedMenu("menu1")
		-- elseif flag.value("Italy Surrendered") == true and flag.value("Operation Brimstone Launched") == false then 
			-- local menuText = "Select One Allied Option!"
			-- local menuTable = {"Launch Special Sardegna Operation","Return to Main Menu","Exit Menu"}
			-- local menuTitle = "Activate Operation Brimstone?"
			-- local choice = text.menu(menuTable,menuText,menuTitle)

			-- if choice == 1 then
				-- return  smallNestedMenu("menu5")
			-- elseif choice == 2 then		
				-- return smallNestedMenu("menu1")
			
			-- elseif choice == 3 then		
				-- return
			-- end
		-- elseif flag.value("Italy Surrendered") == true and flag.value("Operation Brimstone Launched") == true then
			-- civ.ui.text("You have already opted to launch/cancel Operation Brimstone!")
			-- return smallNestedMenu("menu1")		
		-- end	
    -- end	

end

-- =====================================================
-- DELAYED ACTIONS SECTION FOR 2ND and 3RD INVASIONS 
-- =====================================================

-- 2ND INVASION
-- 2ND AMPHIBIOUS INVASION LANDING CRAFT WARNING
local function secondInvasionLC_warned(argTable)
	civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. onTurn.commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. onTurn.commander15thArmy_Base .. ", (15th Army Group)\n^\r^At the beginning of next turn, the week of " .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ",\n^all 2nd invasion landing craft will be withdrawn from operations and be returning\n^to port for refurbishing!\n^\r^As such, you MUST unload any cargo from these vessels prior to the end of\n^THIS turn, otherwise they will returned to Messina!\n^\r^Notice: This does not include landing craft which have been converted to\n^Beachhead units!"))
end 
delayedAction.makeFunctionDelayable("secondInvasionLC_warning",secondInvasionLC_warned)	
	
	
-- ALL 2ND AMPHIBIOUS INVASION LANDING CRAFT ARE WITHDRAWN
local function secondInvasionLC_withdraw(argTable)
	-- Calculate how many landing craft are still in play at begining of 4th turn to determine the number that will be available for 3rd invasion
	local totalLandingCraft = 0
	local landingCraftFor3rdInvasion = 0
	local a = 0
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAllies and unit.type == object.uLandingCraft then
			totalLandingCraft = totalLandingCraft + 1
		end
	end	
	-- param.maxtotalLandingCraft = totalLandingCraft - param.availableLandingCraft3rdInvasion
	-- print("Total Landing Craft: There will be " .. param.maxtotalLandingCraft .. " available for the next invasion")

	landingCraftFor3rdInvasion = totalLandingCraft - param.availableLandingCraft3rdInvasion
	
	while( a < landingCraftFor3rdInvasion )	do
		counter.add("# Landing Craft Available for 3rd Invasion",1)
		a = a + 1
	end
	print("Total Landing Craft: There will be " .. tostring(counter.value("# Landing Craft Available for 3rd Invasion")) .. " available for the next invasion")

	-- Then teleport any unit still on board landing craft to Messina...
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAllies 
		and unit.type.domain == 0 
		and unit.location.baseTerrain == object.bOcean
		and (gen.unitTypeOnTile(unit.location,object.uLandingCraft)) then
			civ.teleportUnit(unit, object.lMessina)
		end
	end
	-- and then remove all remaining landing craft from the map
	for activeUnit in civ.iterateUnits() do
	   if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
			civ.deleteUnit(activeUnit)
		end
	end
	text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " all landing craft for the 2nd Invasion have returned to port! A total of " .. tostring(counter.value("# Landing Craft Available for 3rd Invasion")) .. " landing craft will be available should you decide to proceed with a 3rd amphibious invasion, which will not be possible before the Week I of November, 1943.","Special Bulletin",object.mAFHQ)

end 
delayedAction.makeFunctionDelayable("secondInvasionLC_deployed",secondInvasionLC_withdraw)	

-- 3RD INVASION
-- 3RD AMPHIBIOUS INVASION LANDING CRAFT WARNING
local function thirdInvasionLC_warned(argTable)
	civ.ui.text(func.splitlines("****************** SPECIAL BULLETIN #".. counter.add("Special Bulletin #",1) .. " ******************\n^\r^Date: ".. onTurn.displayWeekMonthYear() .. "\n^\r^From: ".. onTurn.commanderAFHQ_Base .. ", (AFHQ)\n^To: ".. onTurn.commander15thArmy_Base .. ", (15th Army Group)\n^\r^At the beginning of next turn, the week of " .. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+1)) .. ",\n^all 3rd invasion landing craft will be withdrawn from operations and be\n^returning to port for refurbishing!\n^\r^As such, you MUST unload any cargo from these vessels prior to the\n^end of THIS turn, otherwise they will returned to Messina!!\n^\r^Notice: This does not include landing craft which have been converted\n^to Beachhead units!"))
end 
delayedAction.makeFunctionDelayable("thirdInvasionLC_warning",thirdInvasionLC_warned)	
	
	
-- ALL 3RD AMPHIBIOUS INVASION LANDING CRAFT ARE WITHDRAWN
local function thirdInvasionLC_withdraw(argTable)
	-- Simply teleport any unit still on board landing craft to Messina...
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAllies 
		and unit.type.domain == 0 
		and unit.location.baseTerrain == object.bOcean
		and (gen.unitTypeOnTile(unit.location,object.uLandingCraft)) then
			civ.teleportUnit(unit, object.lMessina)
		end
	end
	-- and then remove all remaining landing craft from the map
	for activeUnit in civ.iterateUnits() do
	   if activeUnit.owner == object.pAllies and activeUnit.type == object.uLandingCraft then 
			civ.deleteUnit(activeUnit)
		end
	end
	text.simple("As per the Special Bulletin #" .. counter.value("Special Bulletin #") .. " all landing craft for the 3rd Invasion have returned to port and are being re-assigned in preparation for the invasion of France!","Special Bulletin",object.mAFHQ)

end 
delayedAction.makeFunctionDelayable("thirdInvasionLC_deployed",thirdInvasionLC_withdraw)	


-- OPERATION DIADEM COMES TO AN END
local function operationDiadem_completed(argTable)
	civ.playSound("Telegraph.wav")
	text.simple("Operation Diadem has ended and as such your units no longer benefit from the extra combat bonuses! Furthermore, you will have the opportunity to launch a second major offensive, Operation Grapeshot, any time after 16 weeks from now have elapsed (starting from ".. tostring(onTurn.displayWeekMonthYear(civ.getTurn()+16)) ..").","Operation Diadem Has Ended!",object.mAFHQ)
	civ.takeTech(object.pAllies, object.aOperationDiadem)
	flag.setTrue("Operation Diadem Ended")
	delayedAction.doInFuture("operationGrapeshot_Availability",{},civ.getTurn()+ 16 ,object.pAllies.id)
end 
delayedAction.makeFunctionDelayable("operationDiadem_launched",operationDiadem_completed)	

-- OPERATION GRAPESHOT BECOMES AVAILABLE
local function operationGrapeshot_Available(argTable)
	civ.playSound("Telegraph.wav")
	text.simple("Operation Grapeshot is now available to be launched at your convenience!","Operation Grapeshot Is Now Available!",object.mAFHQ)
	flag.setTrue("Operation Grapeshot Available")
end 
delayedAction.makeFunctionDelayable("operationGrapeshot_Availability",operationGrapeshot_Available)	

-- OPERATION GRAPESHOT COMES TO AN END
local function operationGrapeshot_completed(argTable)
	civ.playSound("Telegraph.wav")
	text.simple("Operation Grapeshot has ended and as such your units no longer benefit from the extra combat bonuses!","Operation Grapeshot Has Ended!",object.mAFHQ)
	civ.takeTech(object.pAllies, object.aOperationGrapeshot)
end 
delayedAction.makeFunctionDelayable("operationGrapeshot_launched",operationGrapeshot_completed)	

return {smallNestedMenu = smallNestedMenu}

