
local versionNumber = 1
local fileModified = true -- set this to true if you change this file for your scenario
-- if another file requires this file, it checks the version number to ensure that the
-- version is recent enough to have all the expected functionality
-- if you set fileModified to true, the error generated if this file is out of date will
-- warn you that you've modified this file

--
--
-- This file allows for a scenario designer to work with several
-- "event triggers" or "execution points" in the same file, instead
-- of spreading the events over several files.
--
-- This is for three purposes:
--  1.  For scenarios with relatively few events, this file may be more convenient.  
--  2.  For those starting their journey into Lua events, working with a single
--      file may be less intimidating.
--  3.  If an event requires the interaction of several triggers, it is more
--      convenient to work in one file.
--
-- However, beware the fact that as you write more events, having them in a
-- single file is likely to make them disorganized and hard to follow.  This
-- was the one of the reasons the template was separated into so many files
-- in the first place.
--

-- ===============================================================================
--
--          Require Lines etc.
--
-- ===============================================================================
-- This section is for the 'require' lines for this file, and anything
-- else that must be at the top of the file.
local consolidator = require("consolidator"):minVersion(1)
local gen = require("generalLibrary"):minVersion(1)
local param = require("parameters")
local object = require("object")
local text = require("text")
local diplomacy = require("diplomacy")
local delayedAction = require("delayedAction")
local calendar = require("calendar")
local keyboard = require("keyboard")
local flag = require("flag")
local counter = require("counter")
local civlua = require("civluaModified")
local func = require("functions")

local transport = require("transport")
local munitionsSettings = require("munitionsSettings")
local supplyLines = require("supplyLines")

-- local onActivateUnit = require("onActivateUnit")
local onTurn = require("onTurn")
local tables = require("tables")

local procurement = require("procurement")
-- local procurement3 = require("procurement3")

local printparamList = require("printparamList")
local gameTips = require("gameTips")
local commandDecision = require("commandDecision")


-- local invasion = require("invasion")

local events = {}


--local beachheadTiles = {}
--beachheadTiles = civ.getTile(45,165,0)

-- local beachheadTiles = {{45,165,0}}

local function beachheadTiles (tile)
	if (tile.x == 77 and tile.y == 85 and tile.z == 0 or -- Anzio
		tile.x == 57 and tile.y == 173 and tile.z == 0 or -- Avola
		tile.x == 74 and tile.y == 40 and tile.z == 0 or -- Cecina
		tile.x == 74 and tile.y == 66 and tile.z == 0 or -- Civitavecchia
		tile.x == 104 and tile.y == 158 and tile.z == 0 or -- Crotone
		tile.x == 110 and tile.y == 50 and tile.z == 0 or -- Fano
		tile.x == 45 and tile.y == 165 and tile.z == 0 or -- Gela
		tile.x == 79 and tile.y == 155 and tile.z == 0 or -- Giola Tauro
		tile.x == 42 and tile.y == 162 and tile.z == 0 or -- Licata
		tile.x == 30 and tile.y == 138 and tile.z == 0 or -- Marsala
		tile.x == 77 and tile.y == 27 and tile.z == 0 or -- Massa
		tile.x == 127 and tile.y == 129 and tile.z == 0 or -- Monopoli
		tile.x == 40 and tile.y == 138 and tile.z == 0 or -- Monreale
		tile.x == 130 and tile.y == 138 and tile.z == 0 or -- Ostuni
		tile.x == 89 and tile.y == 109 and tile.z == 0 or -- Pompei
		tile.x == 88 and tile.y == 152 and tile.z == 0 or -- Pizzo
		tile.x == 92 and tile.y == 114 and tile.z == 0 or -- Salerno
		tile.x == 112 and tile.y == 94 and tile.z == 0 or -- Termoli
		tile.x == 81 and tile.y == 93 and tile.z == 0)  -- Terracina
	then 	
		return true
	else
		return false
	end
end

-- local consolidatedEvents = {}

-- ===============================================================================
--
--          Helper Functions
--
-- ===============================================================================
--      If you write a helper function that is used for multiple events,
--      It is recommended that you store them in the 'helper' table here.
--      This will make it easier to move them to their own file later, if
--      you decide to migrate the events to individual files
local helper = {}
function helper.sampleHelperFunction()
    print("I am a sample helper function")
end


-- ===============================================================================
--
--          Execution Points
--
-- ===============================================================================
--      You can delete any unneeded function here.  It won't cause an error.
--      If you want to add a function back in, you can copy the relevant code
--      from consolidator.lua in the LuaCore directory


-- Before being able to create a Beachhead, also double check that the Landing Craft isn't carrying any passengers (associated with keyboard.k)
local function hasNoCargo(unit) 
    for possibleCargo in unit.location.units do
        if possibleCargo.carriedBy == unit then
            return false
        end
    end
    return true
end

-- DUKW units can create a Supply Hub unit in cities, beach town or tiles that contain a road/highway (associated with keyboard.k)
local function supplyDUKW(unit)
	local activeUnit = civ.getActiveUnit()
	local buildSupplyHubMoneyCost = param.buildSupplyHub
	local supplyHubCreationMoveCost1 = 2
	local supplyHubCreationMoveCost2 = 2 * totpp.movementMultipliers.aggregate
	local activeUnitMoved = gen.moveRemaining(activeUnit)
	if activeUnit and civ.getActiveUnit().type == object.uDUKW 
	and (gen.hasRoad(activeUnit.location) 
	or gen.hasRailroad(activeUnit.location) 
	or activeUnit.location.baseTerrain == object.bTownCenter
	or activeUnit.location.terrain == object.tTownBeach)	
	and activeUnit.owner.money >= buildSupplyHubMoneyCost
	and activeUnitMoved >= supplyHubCreationMoveCost2 then 
		activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*supplyHubCreationMoveCost1
		activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyHub
		gen.createUnit(object.uSupplyHub,object.pAllies,activeUnit.location)
		text.simple("A supply hub was created by a DUKW at (" .. activeUnit.x .. "," ..activeUnit.y .. "," .. activeUnit.z .. ")!","Supply Hub Created!",object.monexone)
		-- counter.add("# DUKW",1)
		-- counter.add("# Total Naval Units",1)
	elseif activeUnit and civ.getActiveUnit().type == object.uDUKW then
		text.simple("The DUKW must be on a road/highway, city center or town beach, have at least " .. supplyHubCreationMoveCost1 .. " MP's remaining and " .. param.buildSupplyHub .. " supplies in the theater depot to create a supply hub!","Supply Hub Requirements not Met!",object.monexone)
	end
end

-- Photo Recon function (from HoF by John Petroski) (associated with keyboard.k)
local function photoRecon(unit)
	if (civ.getActiveUnit().type == object.uM8Greyhound or civ.getActiveUnit().type == object.uDaimlerAC or civ.getActiveUnit().type == object.uBeaufighterMkVI or civ.getActiveUnit().type == object.uP38Lightning or civ.getActiveUnit().type == object.uPartigiani) then
	civ.playSound("Spysound.wav")
		for _,tile in pairs(gen.getAdjacentTiles(civ.getActiveUnit().location)) do
			if tile.city and tile.city.owner ~= civ.getActiveUnit().owner then
				text.simple("Our reconnaissance unit has brought back information about "..tile.city.name..".  Click on the city to see the information.","Reconnaissance")
				gen.setCityInvestigated(tile.city)
			end
		end
	end
end 

-- ============================================================================ [BEGIN]
-- SUPPLY TRUCK LAND FUNCTION (from HoF by John Petroski) (associated with keyboard.k)
-- Supply Truck function where supply type units can reinforce damaged Allied units 


-- UK Supply Trucks
local function canReceiveUKHP(unit)
    -- if unit.type.domain <= 1 and unit.damage > 0 then
	if (tables.ukEligibleUnitArt[unit.type.id] or tables.ukEligibleUnitInf[unit.type.id] or tables.ukEligibleUnitMot[unit.type.id] ) and unit.damage > 0 then
        return true
    else
        return false
    end
end

local function transferUKHP(truck)
	local hpTransfer = 5 -- might want to set equal to param.hpTransfer or something like that
	local minHpForTransfer = hpTransfer +1 -- just in case you want to make this higher
	if truck.hitpoints < minHpForTransfer then
		text.simple(text.substitute("Our %STRING1 can not resupply any UK units in the field because it has less than %STRING2 HP.  Rest it a few turns and allow supplies to restock before attempting to resupply units.",{truck.type.name,minHpForTransfer}),"Game Tips: UK & US Trucks")
		return
	end
	local offset = 3
	local choiceMenu = {}
	for unit in truck.location.units do
		if canReceiveUKHP(unit) then
			local selectionInfo = unit.type.name..""
			if unit.veteran then
				selectionInfo = selectionInfo.." ("..text.getVeteranTitle().."), "
			else
				selectionInfo = selectionInfo..", "
			end
			selectionInfo = selectionInfo..tostring(unit.hitpoints).." of "..tostring(unit.type.hitpoints)..", "
			local hC = "None"
			if unit.homeCity then
				hC = unit.homeCity.name
			end
			selectionInfo = selectionInfo..hC
			choiceMenu[unit.id+offset] = selectionInfo
		end
	end
	choiceMenu[1] = "Do not give supplies to any unit."
	local menuText = text.substitute("Our %STRING1 unit (%STRING2 of %STRING3 HP) can supply another UK unit on this tile, by giving it %STRING4 of its hitpoints.  This will cost artillery "..tostring(param.resupplyCostArt)..", infantry "..tostring(param.resupplyCostInf).. " or armored units "..tostring(param.resupplyCostMot).." supplies. What unit should we supply?",{truck.type.name,truck.hitpoints,truck.type.hitpoints,hpTransfer})
	local menuTitle = "UK Supply Trucks"
	local choice = text.menu(choiceMenu,menuText,menuTitle)
	if choice == 1 then
		return
	end
	local receiverUnit = civ.getUnit(choice-offset)
	receiverUnit.damage = math.max(receiverUnit.damage-hpTransfer,0)
	truck.damage = truck.damage+hpTransfer
		if tables.ukEligibleUnitArt[receiverUnit.type.id] then 
			object.pAllies.money = object.pAllies.money - param.resupplyCostArt
			text.simple("Our "..receiverUnit.type.name.." has received " ..hpTransfer.. " replacements at a cost of " ..tostring(param.resupplyCostArt).. " supplies.")
		elseif tables.ukEligibleUnitInf[receiverUnit.type.id] then
			object.pAllies.money = object.pAllies.money - param.resupplyCostInf
			text.simple("Our "..receiverUnit.type.name.." has received " ..hpTransfer.. " replacements at a cost of " ..tostring(param.resupplyCostInf).. " supplies.")
		elseif tables.ukEligibleUnitMot[receiverUnit.type.id] then
			object.pAllies.money = object.pAllies.money - param.resupplyCostMot
			text.simple("Our "..receiverUnit.type.name.." has received " ..hpTransfer.. " replacements at a cost of " ..tostring(param.resupplyCostMot).. " supplies.") 
		end
	return
end

-- US Supply Trucks
local function canReceiveUSHP(unit)
    -- if unit.type.domain <= 1 and unit.damage > 0 then
	if (tables.usEligibleUnitArt[unit.type.id] or tables.usEligibleUnitInf[unit.type.id] or tables.usEligibleUnitMot[unit.type.id] ) and unit.damage > 0 then
        return true
    else
        return false
    end
end

local function transferUSHP(truck)
    local hpTransfer = 5 -- might want to set equal to param.hpTransfer or something like that
    local minHpForTransfer = hpTransfer +1 -- just in case you want to make this higher
    if truck.hitpoints < minHpForTransfer then
        text.simple(text.substitute("Our %STRING1 can not resupply any US units in the field because it has less than %STRING2 HP.  Rest it a few turns and allow supplies to restock before attempting to resupply units.",{truck.type.name,minHpForTransfer}),"Game Tips: UK & US Trucks")
        return
    end
    local offset = 3
    local choiceMenu = {}
    for unit in truck.location.units do
        if canReceiveUSHP(unit) then
            local selectionInfo = unit.type.name..""
            if unit.veteran then
                selectionInfo = selectionInfo.." ("..text.getVeteranTitle().."), "
            else
                selectionInfo = selectionInfo..", "
            end
            selectionInfo = selectionInfo..tostring(unit.hitpoints).." of "..tostring(unit.type.hitpoints)..", "
            local hC = "None"
            if unit.homeCity then
                hC = unit.homeCity.name
            end
            selectionInfo = selectionInfo..hC
            choiceMenu[unit.id+offset] = selectionInfo
        end
    end
    choiceMenu[1] = "Do not give supplies to any unit."
   local menuText = text.substitute("Our %STRING1 unit (%STRING2 of %STRING3 HP) can supply another US unit on this tile, by giving it %STRING4 of its hitpoints.  This will cost artillery "..tostring(param.resupplyCostArt)..", infantry "..tostring(param.resupplyCostInf).. " or armored units "..tostring(param.resupplyCostMot).." supplies. What unit should we supply?",{truck.type.name,truck.hitpoints,truck.type.hitpoints,hpTransfer})
    local menuTitle = "US Supply Trucks"
    local choice = text.menu(choiceMenu,menuText,menuTitle)
    if choice == 1 then
        return
    end
	local receiverUnit = civ.getUnit(choice-offset)
	receiverUnit.damage = math.max(receiverUnit.damage-hpTransfer,0)
	truck.damage = truck.damage+hpTransfer
		if tables.usEligibleUnitArt[receiverUnit.type.id] then 
			object.pAllies.money = object.pAllies.money - param.resupplyCostArt
			text.simple("Our "..receiverUnit.type.name.." has received " ..hpTransfer.. " replacements at a cost of " ..tostring(param.resupplyCostArt).. " supplies.")
		elseif tables.usEligibleUnitInf[receiverUnit.type.id] then
			object.pAllies.money = object.pAllies.money - param.resupplyCostInf
			text.simple("Our "..receiverUnit.type.name.." has received " ..hpTransfer.. " replacements at a cost of " ..tostring(param.resupplyCostInf).. " supplies.")
		elseif tables.usEligibleUnitMot[receiverUnit.type.id] then
			object.pAllies.money = object.pAllies.money - param.resupplyCostMot
			text.simple("Our "..receiverUnit.type.name.." has received " ..hpTransfer.. " replacements at a cost of " ..tostring(param.resupplyCostMot).. " supplies.") 
		end
	return
end

-- Supply Tanker function where naval tanker units can reinforce damaged Allied naval units at sea
local function canReceiveSeaHP(unit)
    if unit.type.domain == 2 and unit.type ~= object.uOilTankerF and unit.damage > 0 then
        return true
    else
        return false
    end
end

local function transferSeaHP(tanker)
    local hpSeaTransfer = 5 -- might want to set equal to param.hpSeaTransfer or something like that
    local minHpForTransfer = hpSeaTransfer +1 -- just in case you want to make this higher
    if tanker.hitpoints < minHpForTransfer then
        text.simple(text.substitute("Our %STRING1 can not resupply any naval units at sea because it has less than %STRING2 HP.  Rest it a few turns and allow supplies to restock before attempting to resupply units.",{tanker.type.name,minHpForTransfer}),"Hinge of Fate Concepts: Resupply")
        return
    end
    local offset = 3
    local choiceMenu = {}
    for unit in tanker.location.units do
        if canReceiveSeaHP(unit) then
            local selectionInfo = unit.type.name..""
            if unit.veteran then
                selectionInfo = selectionInfo.." ("..text.getVeteranTitle().."), "
            else
                selectionInfo = selectionInfo..", "
            end
            selectionInfo = selectionInfo..tostring(unit.hitpoints).." of "..tostring(unit.type.hitpoints)..", "
            local hC = "None"
            if unit.homeCity then
                hC = unit.homeCity.name
            end
            selectionInfo = selectionInfo..hC
            choiceMenu[unit.id+offset] = selectionInfo
        end
    end
    choiceMenu[1] = "Do not give supplies to any unit."
    local menuText = text.substitute("Our %STRING1 unit (%STRING2 of %STRING3 HP) can supply another naval unit on this tile, by giving it %STRING4 of its hitpoints.  This will cost "..tostring(param.resupplyOceanCost).." supplies. What unit should we supply?",{tanker.type.name,tanker.hitpoints,tanker.type.hitpoints,hpSeaTransfer})
    local menuTitle = "Supply Tanker"
    local choice = text.menu(choiceMenu,menuText,menuTitle)
    if choice == 1 then
        return
    end
    local receiverUnit = civ.getUnit(choice-offset)
    receiverUnit.damage = math.max(receiverUnit.damage-hpSeaTransfer,0)
    tanker.damage = tanker.damage+hpSeaTransfer
	object.pAllies.money = object.pAllies.money - param.resupplyOceanCost
    text.simple("Our "..receiverUnit.type.name.." has received extra supplies.")
    return
end
-- END SUPPLY SEA AND TRUCK LAND FUNCTION ============================================================== 

-- COUNT EACH INDIVIDUAL ALLIED UNIT EACH TURN FUNCTION (associated with keyboard.four) ================
local function getAlliedTotalsDetails ()
	local alliedUnits = { }
		for unit in civ.iterateUnits() do
			-- AMERICAN UNITS
			if unit.type == object.uP40Warhawk then
				alliedUnits[1] = (alliedUnits[1] or 0) + 1
			end
			if unit.type == object.uP38Lightning then
				alliedUnits[2] = (alliedUnits[2] or 0) + 1
			end
			if unit.type == object.uP51DMustang then
				alliedUnits[3] = (alliedUnits[3] or 0) + 1
			end
			
			if unit.type == object.uA36Apache then
				alliedUnits[4] = (alliedUnits[4] or 0) + 1
			end
			if unit.type == object.uB17Fortress then
				alliedUnits[5] = (alliedUnits[5] or 0) + 1
			end
			if unit.type == object.uB24Liberator then
				alliedUnits[6] = (alliedUnits[6] or 0) + 1
			end
			if unit.type == object.uB25Mitchell then
				alliedUnits[7] = (alliedUnits[7] or 0) + 1
			end
			if unit.type == object.uB26Marauder then
				alliedUnits[8] = (alliedUnits[8] or 0) + 1
			end

			if unit.type == object.uM8Greyhound then
				alliedUnits[9] = (alliedUnits[9] or 0) + 1
			end
			if unit.type == object.uArmoredInfantry then
				alliedUnits[10] = (alliedUnits[10] or 0) + 1
			end
			if unit.type == object.uM5Stuart then
				alliedUnits[11] = (alliedUnits[11] or 0) + 1
			end
			if unit.type == object.uM4A1Sherman then
				alliedUnits[12] = (alliedUnits[12] or 0) + 1
			end
			if unit.type == object.uM4A376WSherman then
				alliedUnits[13] = (alliedUnits[13] or 0) + 1
			end
			if unit.type == object.uM10Wolverine then
				alliedUnits[14] = (alliedUnits[14] or 0) + 1
			end	

			if unit.type == object.uUS1stInfantry then
				alliedUnits[15] = (alliedUnits[15] or 0) + 1
			end
			if unit.type == object.uGIVeteran then
				alliedUnits[16] = (alliedUnits[16] or 0) + 1
			end
			if unit.type == object.uGIRegular then
				alliedUnits[17] = (alliedUnits[17] or 0) + 1
			end
			if unit.type == object.uUSBazooka then
				alliedUnits[18] = (alliedUnits[18] or 0) + 1
			end
			if unit.type == object.uRanger then
				alliedUnits[19] = (alliedUnits[19] or 0) + 1
			end
			if unit.type == object.u82ndAirborne then
				alliedUnits[20] = (alliedUnits[20] or 0) + 1
			end			
			if unit.type == object.uUS10thMountain then
				alliedUnits[21] = (alliedUnits[21] or 0) + 1
			end	
			
			if unit.type == object.uM7105mmHMC then
				alliedUnits[22] = (alliedUnits[22] or 0) + 1
			end
			if unit.type == object.uM12155mmHMC then
				alliedUnits[23] = (alliedUnits[23] or 0) + 1
			end
			if unit.type == object.uUSBofors40mmAA then
				alliedUnits[24] = (alliedUnits[24] or 0) + 1
			end
			if unit.type == object.uM2A1105mmArt then
				alliedUnits[25] = (alliedUnits[25] or 0) + 1
			end
			if unit.type == object.uM1155mmArt then
				alliedUnits[26] = (alliedUnits[26] or 0) + 1
			end
			
			-- BRITISH UNITS
			if unit.type == object.uBeaufighterMkVI then
				alliedUnits[27] = (alliedUnits[27] or 0) + 1
			end
			if unit.type == object.uKittyhawk then
				alliedUnits[28] = (alliedUnits[28] or 0) + 1
			end
			if unit.type == object.uSpitfireMkIX then
				alliedUnits[29] = (alliedUnits[29] or 0) + 1
			end

			if unit.type == object.uHurricaneIIc then
				alliedUnits[30] = (alliedUnits[30] or 0) + 1
			end
			if unit.type == object.uMartinBaltimore then
				alliedUnits[31] = (alliedUnits[31] or 0) + 1
			end			
			if unit.type == object.uVickersWellington then
				alliedUnits[32] = (alliedUnits[32] or 0) + 1
			end
			if unit.type == object.uBostonIII then
				alliedUnits[33] = (alliedUnits[33] or 0) + 1
			end			
			if unit.type == object.uSunderlandIII then
				alliedUnits[34] = (alliedUnits[34] or 0) + 1
			end				
			
			if unit.type == object.uDaimlerAC then
				alliedUnits[35] = (alliedUnits[35] or 0) + 1
			end
			if unit.type == object.uBritishMotorTroops then
				alliedUnits[36] = (alliedUnits[36] or 0) + 1
			end			
			if unit.type == object.uShermanIIa then
				alliedUnits[37] = (alliedUnits[37] or 0) + 1
			end
			if unit.type == object.uChurchillMkIV then
				alliedUnits[38] = (alliedUnits[38] or 0) + 1
			end			
			if unit.type == object.uShermanVCFirefly then
				alliedUnits[39] = (alliedUnits[39] or 0) + 1
			end	

			if unit.type == object.uBritishVeteran then
				alliedUnits[40] = (alliedUnits[40] or 0) + 1
			end
			if unit.type == object.uBritishRegular then
				alliedUnits[41] = (alliedUnits[41] or 0) + 1
			end			
			if unit.type == object.uBritishPIAT then
				alliedUnits[42] = (alliedUnits[42] or 0) + 1
			end
			if unit.type == object.uBritishMG then
				alliedUnits[43] = (alliedUnits[43] or 0) + 1
			end			
			if unit.type == object.uCommandos then
				alliedUnits[44] = (alliedUnits[44] or 0) + 1
			end				
			if unit.type == object.u1stAirborne then
				alliedUnits[45] = (alliedUnits[45] or 0) + 1
			end	

			if unit.type == object.uBishop25pdr then
				alliedUnits[46] = (alliedUnits[46] or 0) + 1
			end			
			if unit.type == object.uUKBofors40mmAA then
				alliedUnits[47] = (alliedUnits[47] or 0) + 1
			end				
			if unit.type == object.u25pdrMKIIArt then
				alliedUnits[48] = (alliedUnits[48] or 0) + 1
			end	
			if unit.type == object.u55inchArt then
				alliedUnits[49] = (alliedUnits[49] or 0) + 1
			end	
			
			-- ALLIED MINOR UNITS
			-- Canadian
			if unit.type == object.uCdnMotInf then
				alliedUnits[50] = (alliedUnits[50] or 0) + 1
			end
			if unit.type == object.uCdnShermanIII then
				alliedUnits[51] = (alliedUnits[51] or 0) + 1
			end			
			if unit.type == object.uCdnM7105mm then
				alliedUnits[52] = (alliedUnits[52] or 0) + 1
			end
			if unit.type == object.uCdnMotorTroops then
				alliedUnits[53] = (alliedUnits[53] or 0) + 1
			end			
			if unit.type == object.uCdnInfantry then
				alliedUnits[54] = (alliedUnits[54] or 0) + 1
			end				
			-- alliedUnits[55] = Use to equal to 'uCdn25pdrArt' which has been removed
			
			-- Indian	
			if unit.type == object.uIndianInfantry then
				alliedUnits[56] = (alliedUnits[56] or 0) + 1
			end			
			if unit.type == object.u4thIndianDiv then
				alliedUnits[57] = (alliedUnits[57] or 0) + 1
			end
			if unit.type == object.uGurkhas then
				alliedUnits[58] = (alliedUnits[58] or 0) + 1
			end			
			if unit.type == object.uIndian25pdrArt then
				alliedUnits[59] = (alliedUnits[59] or 0) + 1
			end				
			-- New Zealand
			if unit.type == object.uNZMotInf then
				alliedUnits[60] = (alliedUnits[60] or 0) + 1
			end
			if unit.type == object.uNZShermanIII then
				alliedUnits[61] = (alliedUnits[61] or 0) + 1
			end			
			if unit.type == object.uNZ25pdrArt then
				alliedUnits[62] = (alliedUnits[62] or 0) + 1
			end					
			-- South African	
			if unit.type == object.uSAMotInf then
				alliedUnits[63] = (alliedUnits[63] or 0) + 1
			end			
			if unit.type == object.uSAShermanIII then
				alliedUnits[64] = (alliedUnits[64] or 0) + 1
			end
			if unit.type == object.uSASexton then
				alliedUnits[65] = (alliedUnits[65] or 0) + 1
			end			
			if unit.type == object.uSAMotorTroops then
				alliedUnits[66] = (alliedUnits[66] or 0) + 1
			end	
			-- Free French	
			if unit.type == object.uInfanterieColoniale then
				alliedUnits[67] = (alliedUnits[67] or 0) + 1
			end	
			if unit.type == object.uGoumierMarocain then
				alliedUnits[68] = (alliedUnits[68] or 0) + 1
			end			
			if unit.type == object.uTirailleurAlgerien then
				alliedUnits[69] = (alliedUnits[69] or 0) + 1
			end
			if unit.type == object.uFreeFrenchInfantry then
				alliedUnits[70] = (alliedUnits[70] or 0) + 1
			end			
			if unit.type == object.uFreeFrench105Art then
				alliedUnits[71] = (alliedUnits[71] or 0) + 1
			end	
			if unit.type == object.uFreeFrenchM10 then
				alliedUnits[72] = (alliedUnits[72] or 0) + 1
			end	
			-- Polish	
			if unit.type == object.uPolishShermanIII then
				alliedUnits[73] = (alliedUnits[73] or 0) + 1
			end			
			if unit.type == object.uPolishInfantry then
				alliedUnits[74] = (alliedUnits[74] or 0) + 1
			end			
			if unit.type == object.uPolish25pdrArt then
				alliedUnits[75] = (alliedUnits[75] or 0) + 1
			end	

			-- LATE UK ARMORED ADDITIONS
			if unit.type == object.uShermanIIaBde then
				alliedUnits[76] = (alliedUnits[76] or 0) + 1
			end
			if unit.type == object.uCromwellMkVII then
				alliedUnits[77] = (alliedUnits[77] or 0) + 1
			end

			-- Brazilian et Al.	
			if unit.type == object.uBraz1stInf then
				alliedUnits[78] = (alliedUnits[78] or 0) + 1
			end	
			if unit.type == object.uGreekMtnBde then
				alliedUnits[79] = (alliedUnits[79] or 0) + 1
			end	
			if unit.type == object.uJewishBde then
				alliedUnits[80] = (alliedUnits[80] or 0) + 1
			end	

			-- LATE US ARMORED ADDITIONS
			if unit.type == object.uM4A1ShermanInd then
				alliedUnits[81] = (alliedUnits[81] or 0) + 1
			end	

			-- LATE ARTILLERY ADDITIONS	
			if unit.type == object.uM157mmAT then
				alliedUnits[82] = (alliedUnits[82] or 0) + 1
			end	
			if unit.type == object.u6pdrAT then
				alliedUnits[83] = (alliedUnits[83] or 0) + 1
			end				

			-- LATE SPECIAL ADDITIONS	
			if unit.type == object.uUSTruck then
				alliedUnits[84] = (alliedUnits[84] or 0) + 1
			end	
			if unit.type == object.uUKTruck then
				alliedUnits[85] = (alliedUnits[85] or 0) + 1
			end	
			if unit.type == object.uEngineer then
				alliedUnits[86] = (alliedUnits[86] or 0) + 1
			end	

			-- LATE CARRIER AIRCRAFT ADDITIONS	
			if unit.type == object.uSeafireMkII then
				alliedUnits[87] = (alliedUnits[87] or 0) + 1
			end	
			if unit.type == object.uFaireyBarracuda then
				alliedUnits[88] = (alliedUnits[88] or 0) + 1
			end	

			-- LATE NAVAL UNITS ADDITIONS	
			if unit.type == object.uTribalClassDD then
				alliedUnits[89] = (alliedUnits[89] or 0) + 1
			end	
			if unit.type == object.uLeanderClassCL then
				alliedUnits[90] = (alliedUnits[90] or 0) + 1
			end	
			if unit.type == object.uYorkClassCA then
				alliedUnits[91] = (alliedUnits[91] or 0) + 1
			end				
			if unit.type == object.uQEClassBB then -- Cannot be rebuilt
				alliedUnits[92] = (alliedUnits[92] or 0) + 1
			end	
			if unit.type == object.uSClass then
				alliedUnits[93] = (alliedUnits[93] or 0) + 1
			end				
			if unit.type == object.uIllustriousClassCV then  -- Cannot be rebuilt
				alliedUnits[94] = (alliedUnits[94] or 0) + 1
			end	
			if unit.type == object.uLibertyShip then
				alliedUnits[95] = (alliedUnits[95] or 0) + 1
			end				
			if unit.type == object.uOilTankerE or unit.type == object.uOilTankerF then
				alliedUnits[96] = (alliedUnits[96] or 0) + 1
			end	
			if unit.type == object.uDUKW then
				alliedUnits[97] = (alliedUnits[97] or 0) + 1
			end	
			
			-- LATE SPECIAL MOUNTAIN ADDITIONS	
			if unit.type == object.uBritish52ndMtn then
				alliedUnits[98] = (alliedUnits[98] or 0) + 1
			end	

			-- LATE AVRE ADDITIONS	
			if unit.type == object.uChurchillAVRE then
				alliedUnits[99] = (alliedUnits[99] or 0) + 1
			end	
			if unit.type == object.uUS442ndRegt then
				alliedUnits[100] = (alliedUnits[100] or 0) + 1
			end				
			
		end
	return alliedUnits
end
-- END ================

---&autoDoc onActivateUnit
-- Unit Activation
-- Unit Activation
--Registers a function to be called every time a unit is activated. The callback takes the unit activated as a parameter, and the source of unit activation. `source` is `true` if activated by keyboard or mouse click, `false` if activated by the game itself. `repeatMove` is `true` if it's a repeat activation caused by moving, `false` otherwise.
--If the function returns true, the unit activation will be cancelled.
--No further activation code will be executed, and the unit's type
--will temporarily be set to have 0 movement points.
--If the function returns function(unit), then the unit activation
--will be cancelled, and the function returned will be executed
--just before another unit is activated.  (You may wish to put
--the unit to sleep, for example.)
--Not returning anything is equivalent to returning nil, which is
--acceptable, and keeps the unit activation going.
function events.onActivateUnit(unit,source,repeatMove)
    --civ.ui.text("Unit activation consolidated test")

end
---&endAutoDoc


---&autoDoc onBribeUnit
-- On Unit Bribery
function events.onBribeUnit(unit,previousOwner)
    --civ.ui.text("Bribe unit consolidated test")

end
---&endAutoDoc

-- On calculate City Yield not included here

-- On Can Build not included here

---&autoDoc onCentauriArrival
-- On Centauri Arrival
-- This is available with games started as an extended original game,
-- but not with games started as a standard game (I think, this hasn't been looked at too closely)
function events.onCentauriArrival(tribe)
    --civ.ui.text(tribe.name.." has reached Alpha Centauri.")

end
---&endAutoDoc


---&autoDoc onCityDestroyed
-- City destruction
function events.onCityDestroyed(city)
    --civ.ui.text("City destroyed consolidated test")

end
---&endAutoDoc

---&autoDoc onCityFounded
-- On City Founded
function events.onCityFounded(city)
    if _global.eventTesting then
        civ.ui.text("consolidated.onCityFounded for "..city.name)
    end
    -- the cityCancelled() function is executed if the player
    -- decides not to found the city after all
    -- (so you can undo terrain changes, etc.
    local function cityCancelled()
        if _global.eventTesting then
            civ.ui.text("consolidated.onCityFounded city cancelled for "..city.name)
        end
    end
    return cityCancelled
end
---&endAutoDoc

---&autoDoc onCityProcessed
-- On City Processed
function events.onCityProcessed(city)
    --civ.ui.text(city.name.." processed")

end
---&endAutoDoc

---&autoDoc onCityProduction
-- On city production (when a city produces a unit/improvement/wonder)
function events.onCityProduction(city,prod)
    --civ.ui.text(city.name.." has procuded something.")

end
---&endAutoDoc

---&autoDoc onCityTaken
-- On City Taken
-- (get conqueror by using city.owner)
function events.onCityTaken(city,defender)
    --civ.ui.text(city.name.." captured from the "..defender.name.." by the "..city.owner.name..". Consolidated Events.")
	           
end
---&endAutoDoc


-- On Initiate Combat not included here 
---&autoDoc onKeyPress
-- On key press
function events.onKeyPress(keyCode)
    --if keyCode == keyboard.backspace then
    --    civ.ui.text("backspace pressed")
    --end
	
	-- KEY PRESS 'd'
	-- ENGINEER: BUILDING DOCKS IN MAJOR PORTS CITIES
	-- Bari, Catania, Livorno, Messina, Napoli, Palermo and Taranto
	if keyCode == keyboard.d then
        if not civ.getActiveUnit() then
            return
        end	
	
		local activeUnit = civ.getActiveUnit()
		local buildDocks = {}
		buildDocks[object.uEngineer.id] = true
		local buildDocksMoneyCost = param.buildDocks
		local buildDocksMoveCost1 = 4 
		local buildDocksMoveCost2 = 4 * totpp.movementMultipliers.aggregate
		local activeUnitMoved = gen.moveRemaining(activeUnit)
		local engineerText
		
		if activeUnit and buildDocks[activeUnit.type.id] and activeUnit.location.baseTerrain == object.bTownCenter and activeUnit.owner.money >= buildDocksMoneyCost 
		and activeUnitMoved >= buildDocksMoveCost2 and (not gen.unitTypeOnTile(activeUnit.location,object.uDocks))
		and ((activeUnit.location.city == object.cBari and flag.value("Sea Mines removed from Bari") == true)
		or (activeUnit.location.city == object.cCatania and flag.value("Sea Mines removed from Catania") == true)
		or (activeUnit.location.city == object.cLivorno and flag.value("Sea Mines removed from Livorno") == true)
		or (activeUnit.location.city == object.cMessina and flag.value("Sea Mines removed from Messina") == true)
		or (activeUnit.location.city == object.cNapoli and flag.value("Sea Mines removed from Napoli") == true)
		or (activeUnit.location.city == object.cPalermo and flag.value("Sea Mines removed from Palermo") == true)
		or (activeUnit.location.city == object.cTaranto and flag.value("Sea Mines removed from Taranto") == true))
		then

			local menuOptions = {[1]="No, do not attempt to build a Docks.", [2]="Yes, spend "..param.buildDocks.." to build a Docks in this port."}
			local menuText = text.substitute("Our %STRING1 unit can attempt to build a Docks at the cost of %STRING2 and %STRING3 movement points.  Shall we proceed?",
			{activeUnit.type.name, text.money(buildDocksMoneyCost),buildDocksMoveCost1})
			local menuTitle = "Build docks in this major port?"
			
			local menuImage = civ.ui.loadImage("Images\\DamagedHarbor.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box

			if choice == 2 then
				-- CITY: BARI
				-- METHOD ONE: Engineer 'rolls die' once each turn to build and gets a random chance of success each time
				-- if activeUnit.location.city == object.cBari and flag.value("Sea Mines removed from Bari") == true then
					-- if math.random(1,5) == 5 then -- An engineer unit has a 20% chance of building a dock
						-- activeUnit.moveSpent = activeUnit.type.move
						-- activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
						-- gen.createUnit(object.uDocks, object.pAllies, {{123,125,0}}, {randomize=false, veteran=false})
						-- -- engineerText = civ.ui.text("A docks was successfully built at the port of Bari at the cost of " .. param.buildDocks .. " supplies!")
						-- engineerText = text.simple("A docks was successfully built at the port of Bari at the cost of " .. param.buildDocks .. " supplies!","Harbor Repaired", object.mDocksBuilt)
						-- flag.setTrue("Docks built in Bari")
					-- else
						-- engineerText = civ.ui.text("Your engineer unit failed to build a docks in the port of Bari and expended " .. tostring(param.buildDocks / 6) .. " supplies in its attempt. You may try again with this unit on another turn!")
						-- activeUnit.moveSpent = activeUnit.type.move
						-- activeUnit.owner.money = activeUnit.owner.money - (param.buildDocks / 6)
					-- end
				-- end	

				-- METHOD TWO: Engineer selects to build dock and event will randomly determine how many turns it will take to build dock and for engineer to return
				if activeUnit.location.city == object.cBari and flag.value("Sea Mines removed from Bari") == true 
				and flag.value("Docks being built in Bari") == false and flag.value("Docks built in Bari") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInBari(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInBari",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInBari",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInBari",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Bari")
					local tileToChange = civ.getTile(121,125,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Bari\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Bari!", object.monexone)
				elseif activeUnit.location.city == object.cBari and flag.value("Docks being built in Bari") == true and flag.value("Docks built in Bari") == false then
					engineerText = text.simple("The construction of a dock in Bari is already in progress!","Docks being built in Bari!", object.monexone)
				end	

				-- CITY: CATANIA
				if activeUnit.location.city == object.cCatania and flag.value("Sea Mines removed from Catania") == true 
				and flag.value("Docks being built in Catania") == false and flag.value("Docks built in Catania") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInCatania(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInCatania",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInCatania",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInCatania",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Catania")
					local tileToChange = civ.getTile(60,162,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Catania\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Catania!", object.monexone)
				elseif activeUnit.location.city == object.cCatania and flag.value("Docks being built in Catania") == true and flag.value("Docks built in Catania") == false then
					engineerText = text.simple("The construction of a dock in Catania is already in progress!","Docks being built in Catania!", object.monexone)
				end	
				
				-- CITY: LIVORNO
				if activeUnit.location.city == object.cLivorno and flag.value("Sea Mines removed from Livorno") == true 
				and flag.value("Docks being built in Livorno") == false and flag.value("Docks built in Livorno") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInLivorno(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInLivorno",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInLivorno",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInLivorno",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Livorno")
					local tileToChange = civ.getTile(76,34,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Livorno\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Livorno!", object.monexone)
				elseif activeUnit.location.city == object.cLivorno and flag.value("Docks being built in Livorno") == true and flag.value("Docks built in Livorno") == false then
					engineerText = text.simple("The construction of a dock in Livorno is already in progress!","Docks being built in Livorno!", object.monexone)
				end	

				-- CITY: MESSINA
				if activeUnit.location.city == object.cMessina and flag.value("Sea Mines removed from Messina") == true 
				and flag.value("Docks being built in Messina") == false and flag.value("Docks built in Messina") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInMessina(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInMessina",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInMessina",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInMessina",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Messina")
					local tileToChange = civ.getTile(71,155,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Messina\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Messina!", object.monexone)
				elseif activeUnit.location.city == object.cMessina and flag.value("Docks being built in Messina") == true and flag.value("Docks built in Messina") == false then
					engineerText = text.simple("The construction of a dock in Messina is already in progress!","Docks being built in Messina!", object.monexone)
				end	

				-- CITY: NAPOLI
				if activeUnit.location.city == object.cNapoli and flag.value("Sea Mines removed from Napoli") == true 
				and flag.value("Docks being built in Napoli") == false and flag.value("Docks built in Napoli") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInNapoli(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInNapoli",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInNapoli",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInNapoli",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Napoli")
					local tileToChange = civ.getTile(86,106,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Napoli\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Napoli!", object.monexone)
				elseif activeUnit.location.city == object.cNapoli and flag.value("Docks being built in Napoli") == true and flag.value("Docks built in Napoli") == false then
					engineerText = text.simple("The construction of a dock in Napoli is already in progress!","Docks being built in Napoli!", object.monexone)
				end	

				-- CITY: PALERMO
				if activeUnit.location.city == object.cPalermo and flag.value("Sea Mines removed from Palermo") == true 
				and flag.value("Docks being built in Palermo") == false and flag.value("Docks built in Palermo") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInPalermo(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInPalermo",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInPalermo",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInPalermo",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Palermo")
					local tileToChange = civ.getTile(45,141,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Palermo\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Palermo!", object.monexone)
				elseif activeUnit.location.city == object.cPalermo and activeUnit.location.city == object.cPalermo and flag.value("Docks being built in Palermo") == true and flag.value("Docks built in Palermo") == false then
					engineerText = text.simple("The construction of a dock in Palermo is already in progress!","Docks being built in Palermo!", object.monexone)
				end	

				-- CITY: TARANTO
				if activeUnit.location.city == object.cTaranto and flag.value("Sea Mines removed from Taranto") == true 
				and flag.value("Docks being built in Taranto") == false and flag.value("Docks built in Taranto") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function DocksBuiltInTaranto(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingDocksInTaranto",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingDocksInTaranto",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingDocksInTaranto",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.moveSpent = activeUnit.type.move
					activeUnit.owner.money = activeUnit.owner.money - param.buildDocks
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					flag.setTrue("Docks being built in Taranto")
					local tileToChange = civ.getTile(120,136,0)
					tileToChange.terrain = object.tDocks	
							
					engineerText = text.simple("The construction of a dock was begun in Taranto\n^and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Docks being built in Taranto!", object.monexone)
				elseif activeUnit.location.city == object.cTaranto and flag.value("Docks being built in Taranto") == true and flag.value("Docks built in Taranto") == false then
					engineerText = text.simple("The construction of a dock in Taranto is already in progress!","Docks being built in Taranto!", object.monexone)
				end	

			end
		-- else
		elseif activeUnit and buildDocks[activeUnit.type.id] then
			engineerText = text.simple("Your engineer must be located in a port with a 'Harbor' resource, that doesn't already contain a dock, has been freed of mines, possess at least " .. param.buildDocks .. " supplies and " .. buildDocksMoveCost1 .. " MP's to proceed!","Cannot Build Docks!", object.monexone)
		
		end
		return engineerText

	end

	-- KEY PRESS 'k'
	if keyCode == keyboard.k then
        if not civ.getActiveUnit() then
            return
        end

	--local activeUnit = civ.getActiveUnit()
		-- MUNITIONS CREATION
		if civ.getActiveUnit() then
			-- munitionsSettings.primaryAttack(civ.getActiveUnit()) - (not required here, handled by keyPressSettings.lua)
			photoRecon(civ.getActiveUnit())
			supplyDUKW(civ.getActiveUnit())
		end

		-- SUPPLY TRUCK HP TRANSFER
		-- UK supply truck
		local activeUnit = civ.getActiveUnit()
		if activeUnit and activeUnit.type == object.uUKTruck then
			transferUKHP(activeUnit)
		end
		-- US supply truck
		local activeUnit = civ.getActiveUnit()
		if activeUnit and activeUnit.type == object.uUSTruck then
			transferUSHP(activeUnit)
		end
		-- OIL TANKER HP TRANSFER
		local activeUnit = civ.getActiveUnit()
		if flag.value("Ships At Sea Penalty Option") == true then
			if activeUnit and activeUnit.type == object.uOilTankerF then
				transferSeaHP(activeUnit)
			end
		end
		
		-- LANDING CRAFT TO BEACHHEAD UNIT CONVERSION
		local activeUnit = civ.getActiveUnit()
		local beachHead = {}
		beachHead[object.uLandingCraft.id] = true
		local beachHeadMoneyCost = param.beachHeadCreation	
		
		if activeUnit and beachHead[activeUnit.type.id] and beachheadTiles(activeUnit.location) and activeUnit.owner.money >= beachHeadMoneyCost and hasNoCargo(activeUnit) and counter.value("Beachhead #") < 4 then
			text.simple("A new beachhead along with a DUKW unit have been created at (" .. activeUnit.x .. "," ..activeUnit.y .. "," .. activeUnit.z .. ").\n^\r^It will now be possible for you to unload units from\n^your Liberty Ships from this location.\n^\r^Otherwise, the only other point of disembarkment for\n^your transports are within a de-mined port.","Beachhead Established",object.mDUKWBeachead)
			-- civ.ui.text(func.splitlines("A beachhead has been created at (" .. activeUnit.x .. "," ..activeUnit.y .. "," .. activeUnit.z .. ").\n^\r^As a consequence, a new DUKW unit is also deployed at the same location."))
			
			-- civ.createUnit(object.uBeachhead,object.pAllies,activeUnit.location)
			gen.createUnit(object.uBeachhead, object.pAllies, activeUnit.location, {count=1, homeCity=nil, randomize=false, veteran=false})	
			civ.createUnit(object.uDUKW,object.pAllies,activeUnit.location)
			-- gen.createUnit(object.uDUKW, object.pAllies, activeUnit.location, {count=1, homeCity=nil, randomize=false, veteran=false})	
			
			civ.deleteUnit(activeUnit)
			activeUnit.owner.money = activeUnit.owner.money - beachHeadMoneyCost
			counter.add("Beachhead #",1)
			counter.add("# DUKW",1)
			counter.add("# Total Naval Units",1)
			
		elseif activeUnit and beachHead[activeUnit.type.id] then
			text.simple("The Landing Craft unit must be located in a yellow 'BH' tile and have unloaded its cargo to be able to create a Beachhead unit!\n^\r^Furthermore, the Allies may only support a maximum of 4 Beachhead units in total.","Landing Craft May Not Convert While Loaded!",object.monexone)
		end


		-- ENGINEERS: REBUILDING RIVER CROSSING
		local activeUnit = civ.getActiveUnit()
		local riverCrossings = {}
		riverCrossings[object.uEngineer.id] = true
		local crossingUpgradeMoneyCost = param.repairRiverCrossings
		local crossingUpgradeMoveCost1 = 2
		local crossingUpgradeMoveCost2 = 2 * totpp.movementMultipliers.aggregate
		-- local activeUnitMoved = activeUnit.type.move - activeUnit.moveSpent
		local activeUnitMoved = gen.moveRemaining(activeUnit)
		
		-- if activeUnit and riverCrossings[activeUnit.type.id] and (gen.hasRoad(activeUnit.location) or gen.hasRailroad(activeUnit.location)) and activeUnit.location.baseTerrain == object.bRiverbed and activeUnit.owner.money >= crossingUpgradeMoneyCost 
		-- if flag.value("Winter Months") == false
		if activeUnit and riverCrossings[activeUnit.type.id] 
		-- and (activeUnit.location.baseTerrain == object.bRiverbed or activeUnit.location.baseTerrain == object.bTemperateRiverbed) 
		and (activeUnit.location.terrain == object.tBlownBridge1 or activeUnit.location.terrain == object.tBlownBridge2)
		and activeUnit.owner.money >= crossingUpgradeMoneyCost 
		and activeUnitMoved >= crossingUpgradeMoveCost2 then
		-- and tile.terrainType & 0x0F == 0x0B -- Riverbed terrain
		   
			local menuOptions = {[1]="No, do not rebuild the river crossing.", [2]="Yes, spend ".. param.repairRiverCrossings .." supplies to rebuild this river crossing."}
			local menuText = text.substitute("Our %STRING1 unit can upgrade this riverbed to a crossing at the cost of %STRING2 and %STRING3 movement points.  Shall we proceed?",
			{activeUnit.type.name, text.money(crossingUpgradeMoneyCost),crossingUpgradeMoveCost1})
			local menuTitle = "Rebuild River Crossing?"
			
			local menuImage = civ.ui.loadImage("Images\\BridgeDestroyed.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
			--local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
			if choice == 2 then
				-- For Southern River Crossings
				if activeUnit.location.baseTerrain == object.bRiverbed then
					
					-- Summer Months
					if flag.value("Summer Months") == true then
						civ.playSound("Bldspcsh.wav")
						text.simple("The bridge on the river crossing located on tile\n^(" .. text.coordinates(activeUnit.location) .. ") has been repaired and traffic across\n^it can resume!","Bridge Repaired!",object.mBridgeBuilding)    
						activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*crossingUpgradeMoveCost1
						activeUnit.owner.money = activeUnit.owner.money - crossingUpgradeMoneyCost
						gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
						local tileToChange = civ.getTile(activeUnit.location.x,activeUnit.location.y,activeUnit.location.z)
						tileToChange.terrain = object.tCrossing
						gen.placeRoad(activeUnit.location)
						-- gen.placeRailroad (activeUnit.location)
						gen.chartTruthfully(activeUnit.location,activeUnit.owner)
						civ.ui.redrawTile(activeUnit.location) -- new line
					
					-- Fall/Spring/Winter Months: There a 1 in 3 chance engineers may fail to rebuild river crossings during non-summer months
					else
						if math.random(1,3) == 3 then
							civ.playSound("NEG1.wav")
							text.simple("Your engineer failed to repair the bridge on the\n^river crossing located on tile (" .. text.coordinates(activeUnit.location) .. ") due to\n^the bad weather conditions and has expended " .. crossingUpgradeMoveCost1 .. " MP's\n^and ".. math.floor(param.repairRiverCrossings/2) .." supplies in the process. You will need to try\n^again if you wish to succeed in your endeavour!","Bridge Not Repaired!",object.mFall)  
							activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*crossingUpgradeMoveCost1
							activeUnit.owner.money = activeUnit.owner.money - (crossingUpgradeMoneyCost/2)
						else
							civ.playSound("Bldspcsh.wav")
							text.simple("The bridge on the river crossing located on tile\n^(" .. text.coordinates(activeUnit.location) .. ") has been repaired and traffic across\n^it can resume!","Bridge Repaired!",object.mBridgeBuilding)    
							activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*crossingUpgradeMoveCost1
							activeUnit.owner.money = activeUnit.owner.money - crossingUpgradeMoneyCost
							gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
							local tileToChange = civ.getTile(activeUnit.location.x,activeUnit.location.y,activeUnit.location.z)
							tileToChange.terrain = object.tCrossing
							gen.placeRoad(activeUnit.location)
							-- gen.placeRailroad (activeUnit.location)
							gen.chartTruthfully(activeUnit.location,activeUnit.owner)
							civ.ui.redrawTile(activeUnit.location) -- new line
 						end
					end	
				
				-- For Northern River Crossings with Crossing Temp 'resource' added to base terrain tile
				elseif activeUnit.location.baseTerrain == object.bTemperateRiverbed then

					-- Summer Months
					if flag.value("Summer Months") == true then
						civ.playSound("Bldspcsh.wav")
						text.simple("The bridge on the river crossing located on tile\n^(" .. text.coordinates(activeUnit.location) .. ") has been repaired and traffic across\n^it can resume!","Bridge Repaired",object.mBridgeBuilding)
						activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*crossingUpgradeMoveCost1
						activeUnit.owner.money = activeUnit.owner.money - crossingUpgradeMoneyCost
						gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
						local tileToChange = civ.getTile(activeUnit.location.x,activeUnit.location.y,activeUnit.location.z)
						tileToChange.terrain = object.tCrossingTemp
						gen.placeRoad(activeUnit.location)
						-- gen.placeRailroad (activeUnit.location)
						gen.chartTruthfully(activeUnit.location,activeUnit.owner)
						civ.ui.redrawTile(activeUnit.location) -- new line	
					
					-- Fall/Spring/Winter Months
					else
						if math.random(1,3) == 3 then
							civ.playSound("NEG1.wav")
							text.simple("Your engineer failed to repair the bridge on the\n^river crossing located on tile (" .. text.coordinates(activeUnit.location) .. ") due to\n^the bad weather conditions and has expended " .. crossingUpgradeMoveCost1 .. " MP's\n^and ".. math.floor(param.repairRiverCrossings/2) .." supplies in the process. You will need to try\n^again if you wish to succeed in your endeavour!","Bridge Not Repaired!",object.mBridgeDestroyed)  
							activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*crossingUpgradeMoveCost1
							activeUnit.owner.money = activeUnit.owner.money - (crossingUpgradeMoneyCost/2)
						else
							civ.playSound("Bldspcsh.wav")
							text.simple("The bridge on the river crossing located on tile\n^(" .. text.coordinates(activeUnit.location) .. ") has been repaired and traffic across\n^it can resume!","Bridge Repaired",object.mBridgeBuilding)
							activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*crossingUpgradeMoveCost1
							activeUnit.owner.money = activeUnit.owner.money - crossingUpgradeMoneyCost
							gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
							local tileToChange = civ.getTile(activeUnit.location.x,activeUnit.location.y,activeUnit.location.z)
							tileToChange.terrain = object.tCrossingTemp
							gen.placeRoad(activeUnit.location)
							-- gen.placeRailroad (activeUnit.location)
							gen.chartTruthfully(activeUnit.location,activeUnit.owner)
							civ.ui.redrawTile(activeUnit.location) -- new line	
						end		
					end

				end	
			end
		elseif activeUnit and riverCrossings[activeUnit.type.id] then
			text.simple("The " .. object.uEngineer.name .. " unit can only upgrade to a river crossing provided the tile they are located on is a riverbed that contains a 'Blown Bridge' icon, it has at least " .. crossingUpgradeMoveCost1 .. " MP's remaining and the cost of ".. param.repairRiverCrossings .." supplies doesn't exceed the amount currently in your theater depots and its not a winter turn!","Unable to Build River Crossing!",object.mBridgeDestroyed)
		end
		
	end

	-- KEY PRESS 'u'
	if keyCode == keyboard.u then
        if not civ.getActiveUnit() then
            return
        end
		-- SECONDARY MUNITIONS FOR BARRACUDA CARRIER PLANE (not required here, handled by keyPressSettings.lua)
		-- local activeUnit = civ.getActiveUnit()
		-- if civ.getActiveUnit() then	
			 -- munitionsSettings.secondaryAttack(civ.getActiveUnit())
		-- end

		-- SUPPLY HUB DECOMMISSIONING BY DUKW OR TRUCK TYPE UNITS
		local activeUnit = civ.getActiveUnit()
		local supplyHub = {}
		supplyHub[object.uUSTruck.id] = true
		supplyHub[object.uUKTruck.id] = true
		supplyHub[object.uDUKW.id] = true
		local decommissionSupplyHubBonus = param.decommissionSupplyHub
		local decommissionMoveCost1 = 2	
		local decommissionMoveCost2 = 2 * totpp.movementMultipliers.aggregate
		local activeUnitMoved = gen.moveRemaining(activeUnit)
		
		if activeUnit and (civ.getActiveUnit().type == object.uUSTruck or civ.getActiveUnit().type == object.uUKTruck or civ.getActiveUnit().type == object.uDUKW) 
		and (gen.unitTypeOnTile(activeUnit.location,object.uSupplyHub)) 
		and activeUnitMoved >= decommissionMoveCost2 then
			local menuOptions = {[1]="No, do not decommission this supply hub.", [2]="Yes, decommission this supply hub and receive ".. param.decommissionSupplyHub .." supplies."}
			local menuText = text.substitute("Our %STRING1 unit can decommisssion this Supply Hub with %STRING2 movement points. Shall we proceed?",
			{activeUnit.type.name,decommissionMoveCost1,text.money(decommissionSupplyHubBonus)})
			local menuTitle = "Decommission Supply Hub?"
			
			local menuImage = civ.ui.loadImage("Images\\GMCTruck.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
			
			if choice == 2 then
				for unit in activeUnit.location.units do
					if unit.type == object.uSupplyHub then	
						text.simple("The " ..tostring(unit.type.name).. " at (".. unit.x .. "," .. unit.y .. "," .. unit.z ..") was dispanded! A total of ".. param.decommissionSupplyHub .." supplies were redistributed to the theater depots.","Supply Hub Decommissioned!",object.monexone)
						object.pAllies.money = object.pAllies.money + decommissionSupplyHubBonus
						civ.deleteUnit(unit)
					end 
				end
				if activeUnit.type == object.uUKTruck or activeUnit.type == object.uUSTruck or activeUnit.type == object.uDUKW then
					activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*decommissionMoveCost1
				end
			end

		elseif activeUnit and supplyHub[activeUnit.type.id] then
			text.simple("The " .. activeUnit.type.name .. " must be located on a tile that contains a supply hub and possess at least " .. decommissionMoveCost1 .. " MP's to decommission it!","Unable to Decommision Supply Hub!",object.monexone)
		end	
		
		-- ENGINEERS: CLEARING SEA MINES FROM DIFFERENT PORTS 
		local activeUnit = civ.getActiveUnit()
		local removeSeaMines = {}
		removeSeaMines[object.uEngineer.id] = true
		local removeSeaMinesMoneyCost = param.removeSeaMines
		local removeSeaMinesMoveCost1 = 4 
		local removeSeaMinesMoveCost2 = 4 * totpp.movementMultipliers.aggregate
		local activeUnitMoved = gen.moveRemaining(activeUnit)
		local engineerText
		if activeUnit and removeSeaMines[activeUnit.type.id] and activeUnit.location.baseTerrain == object.bTownCenter and activeUnit.owner.money >= removeSeaMinesMoneyCost 
		and activeUnitMoved >= removeSeaMinesMoveCost2 then

	
			local menuOptions = {[1]="No, do not clear mines from the harbor.", [2]="Yes, spend "..param.removeSeaMines.." to remove the sea mines from this port."}
			local menuText = text.substitute("Our %STRING1 unit can attempt to remove sea mines at the cost of %STRING2 and %STRING3 movement points. Shall we proceed?",
			{activeUnit.type.name, text.money(removeSeaMinesMoneyCost),removeSeaMinesMoveCost1})
			local menuTitle = "Remove mines from harbor?"
			
			local menuImage = civ.ui.loadImage("Images\\MineClearance.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
			--local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
			if choice == 2 then
				-- CITY: BARI
				-- METHOD ONE: Engineer 'rolls die' once each turn to remove obstacles and gets a random chance of success each time
				-- if activeUnit.location.city == object.cBari then
					-- if math.random(1,3) == 3 then -- An engineer unit has a 33% chance of removing sea mines
						-- for unit in civ.iterateUnits() do
							-- if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 123 and unit.location.y >= 123 and unit.location.x <= 125 and unit.location.y <= 127 and unit.location.z == 0) then
								-- civ.deleteUnit(unit)
								-- civ.ui.redrawTile(unit.location) 
								-- -- activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
							-- end
							-- gen.justOnce("engineerBariText",function ()
								-- activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
								-- activeUnit.moveSpent = activeUnit.type.move
								-- engineerText = civ.ui.text("The obstacles have been removed from the port of Bari at the cost of " .. param.removeSeaMines .. " supplies!")
								-- flag.setTrue("Sea Mines removed from Bari")
								-- print("Print Flag.value.Bari 2+: " .. tostring(flag.value("Sea Mines removed from Bari")))
							-- end)	
						-- end
					-- else
						-- engineerText = civ.ui.text("Your engineer unit failed to clear the water obstacles from the port of Bari and expended " .. tostring(param.removeSeaMines / 2) .. " supplies in its attempt. You may try again with this unit on another turn!")
						-- activeUnit.moveSpent = activeUnit.type.move
						-- activeUnit.owner.money = activeUnit.owner.money - (param.removeSeaMines / 2)
					-- end							
				-- end
	
				-- METHOD TWO: Engineer selects to remove obstacles and event will randomly determine how many turns it will take to complete the task and for the engineer to return
				if activeUnit.location.city == object.cBari and flag.value("Sea Mines being removed from Bari") == false and flag.value("Sea Mines removed from Bari") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInBari(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInBari",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInBari",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInBari",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Bari")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Bari has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Bari!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cBari and flag.value("Sea Mines being removed from Bari") == true and flag.value("Sea Mines removed from Bari") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Bari is already in progress!","Sea Mines being built in Bari!", object.mMineClearance)
				end	
	
				-- CITY: BRINDISI
				if activeUnit.location.city == object.cBrindisi and flag.value("Sea Mines being removed from Brindisi") == false and flag.value("Sea Mines removed from Brindisi") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInBrindisi(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInBrindisi",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInBrindisi",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInBrindisi",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Brindisi")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Brindisi has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Brindisi!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cBrindisi and flag.value("Sea Mines being removed from Brindisi") == true and flag.value("Sea Mines removed from Brindisi") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Brindisi is already in progress!","Sea Mines being built in Brindisi!", object.mMineClearance)
				end	

				-- CITY: CATANIA
				if activeUnit.location.city == object.cCatania and flag.value("Sea Mines being removed from Catania") == false and flag.value("Sea Mines removed from Catania") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInCatania(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInCatania",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInCatania",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInCatania",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Catania")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Catania has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Catania!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cCatania and flag.value("Sea Mines being removed from Catania") == true and flag.value("Sea Mines removed from Catania") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Catania is already in progress!","Sea Mines being built in Catania!", object.mMineClearance)
				end	

				-- CITY: GENOVA
				if activeUnit.location.city == object.cGenova and flag.value("Sea Mines being removed from Genova") == false and flag.value("Sea Mines removed from Genova") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInGenova(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInGenova",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInGenova",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInGenova",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Genova")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Genova has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Genova!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cGenova and flag.value("Sea Mines being removed from Genova") == true and flag.value("Sea Mines removed from Genova") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Genova is already in progress!","Sea Mines being built in Genova!", object.mMineClearance)
				end					

				-- CITY: LA SPEZIA
				if activeUnit.location.city == object.cLaSpezia and flag.value("Sea Mines being removed from La Spezia") == false and flag.value("Sea Mines removed from La Spezia") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInLaSpezia(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInLaSpezia",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInLaSpezia",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInLaSpezia",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from La Spezia")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^LaSpezia has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in LaSpezia!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cLaSpezia and flag.value("Sea Mines being removed from La Spezia") == true and flag.value("Sea Mines removed from La Spezia") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of LaSpezia is already in progress!","Sea Mines being built in LaSpezia!", object.mMineClearance)
				end	

				-- CITY: LIVORNO
				if activeUnit.location.city == object.cLivorno and flag.value("Sea Mines being removed from Livorno") == false and flag.value("Sea Mines removed from Livorno") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInLivorno(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInLivorno",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInLivorno",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInLivorno",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Livorno")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Livorno has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Livorno!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cLivorno and flag.value("Sea Mines being removed from Livorno") == true and flag.value("Sea Mines removed from Livorno") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Livorno is already in progress!","Sea Mines being built in Livorno!", object.mMineClearance)
				end	

				-- CITY: MESSINA
				-- SEE oncityTaken.lua; capture of Messina where sea mines are now removed

				-- CITY: NAPOLI
				if activeUnit.location.city == object.cNapoli and flag.value("Sea Mines being removed from Napoli") == false and flag.value("Sea Mines removed from Napoli") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInNapoli(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInNapoli",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInNapoli",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInNapoli",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Napoli")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Napoli has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Napoli!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cNapoli and flag.value("Sea Mines being removed from Napoli") == true and flag.value("Sea Mines removed from Napoli") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Napoli is already in progress!","Sea Mines being built in Napoli!", object.mMineClearance)
				end	

				-- CITY: PALERMO
				if activeUnit.location.city == object.cPalermo and flag.value("Sea Mines being removed from Palermo") == false and flag.value("Sea Mines removed from Palermo") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInPalermo(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInPalermo",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInPalermo",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInPalermo",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Palermo")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Palermo has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Palermo!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cPalermo and flag.value("Sea Mines being removed from Palermo") == true and flag.value("Sea Mines removed from Palermo") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Palermo is already in progress!","Sea Mines being built in Palermo!", object.mMineClearance)
				end	

				
				-- CITY: PESCARA
				if activeUnit.location.city == object.cPescara and flag.value("Sea Mines being removed from Pescara") == false and flag.value("Sea Mines removed from Pescara") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInPescara(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInPescara",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInPescara",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInPescara",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Pescara")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Pescara has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Pescara!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cPescara and flag.value("Sea Mines being removed from Pescara") == true and flag.value("Sea Mines removed from Pescara") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Pescara is already in progress!","Sea Mines being built in Pescara!", object.mMineClearance)
				end					
				
				-- CITY: RIMINI
				if activeUnit.location.city == object.cRimini and flag.value("Sea Mines being removed from Rimini") == false and flag.value("Sea Mines removed from Rimini") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInRimini(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInRimini",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInRimini",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInRimini",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Rimini")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Rimini has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Rimini!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cRimini and flag.value("Sea Mines being removed from Rimini") == true and flag.value("Sea Mines removed from Rimini") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Rimini is already in progress!","Sea Mines being built in Rimini!", object.mMineClearance)
				end	
				
				-- CITY: SIRACUSA
				if activeUnit.location.city == object.cSiracusa and flag.value("Sea Mines being removed from Siracusa") == false and flag.value("Sea Mines removed from Siracusa") == false then	
					local randomNumber = math.random(2)
					-- ALSO SEE 'function SeaMinesRemovedInSiracusa(argTable)' AT BOTTOM
					-- Plus make it a little more difficult to clear obstacles from this port, i.e. 2 or 3 turns instead of 1, 2 or 3
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInSiracusa",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInSiracusa",{},civ.getTurn()+ 3 ,object.pAllies.id)
					-- elseif randomNumber == 3 then
						-- delayedAction.doInFuture("SeaMinesBeingRemovedInSiracusa",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Siracusa")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Siracusa has begun and should be completed in " .. math.floor(tostring(randomNumber) + 1) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Siracusa!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cSiracusa and flag.value("Sea Mines being removed from Siracusa") == true and flag.value("Sea Mines removed from Siracusa") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Siracusa is already in progress!","Sea Mines being built in Siracusa!", object.mMineClearance)
				end	
				
				-- CITY: TARANTO
				-- see oncityTaken.lua function eightIndianArrives
				
				-- CITY: TRAPANI
				if activeUnit.location.city == object.cTrapani and flag.value("Sea Mines being removed from Trapani") == false and flag.value("Sea Mines removed from Trapani") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInTrapani(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInTrapani",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInTrapani",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInTrapani",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Trapani")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Trapani has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Trapani!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cTrapani and flag.value("Sea Mines being removed from Trapani") == true and flag.value("Sea Mines removed from Trapani") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Trapani is already in progress!","Sea Mines being built in Trapani!", object.mMineClearance)
				end	

				-- CITY: VENEZIA
				if activeUnit.location.city == object.cVenezia and flag.value("Sea Mines being removed from Venezia") == false and flag.value("Sea Mines removed from Venezia") == false then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SeaMinesRemovedInVenezia(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInVenezia",{},civ.getTurn()+ 1 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInVenezia",{},civ.getTurn()+ 2 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("SeaMinesBeingRemovedInVenezia",{},civ.getTurn()+ 3 ,object.pAllies.id)
					end

					activeUnit.owner.money = activeUnit.owner.money - param.removeSeaMines
					civ.deleteUnit(activeUnit)
					counter.subtract("# Engineer",1)
					counter.subtract("# Total US Armored",1)
					
					flag.setTrue("Sea Mines being removed from Venezia")
					engineerText = text.simple("The attempt to remove obstacles from the port of\n^Venezia has begun and should be completed in " .. tostring(randomNumber) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!","Sea Mines Being Removed in Venezia!", object.mMineClearance)
				
				elseif activeUnit.location.city == object.cVenezia and flag.value("Sea Mines being removed from Venezia") == true and flag.value("Sea Mines removed from Venezia") == false then
					engineerText = text.simple("The attempt to remove obstacles from the port of Venezia is already in progress!","Sea Mines being built in Venezia!", object.mMineClearance)
				end					
				
					
			end
		elseif activeUnit and removeSeaMines[activeUnit.type.id] then
			engineerText = text.simple("Your engineer must be located in a port, possess at least " .. param.removeSeaMines .. " supplies and " .. removeSeaMinesMoveCost1 .. " MP's to clear a harbor of all obstacles!","Unable to Clear Mines From Port!",object.monexone)
		
		end
		return engineerText
	end
	

	-- KEY PRESS 'shift.one'
	-- ENGINEER: BUILDING PONTOON BRIDGES ON RIVERBED TILES
	if keyCode == keyboard.shift.one then
        if not civ.getActiveUnit() then
            return
        end	
		
		-- if flag.value("Winter Months") == false then -- cannot build pontoon bridges during fall/spring turns
			local activeUnit = civ.getActiveUnit()
			local pontoonBridge = {}
			pontoonBridge[object.uEngineer.id] = true
			local buildpontoonBridgeCost = param.buildpontoonBridge
			local buildpontoonBridgeMoveCost1 = 2
			local buildpontoonBridgeMoveCost2 = 2 * totpp.movementMultipliers.aggregate
			-- local activeUnitMoved = activeUnit.type.move - activeUnit.moveSpent
			local activeUnitMoved = gen.moveRemaining(activeUnit)
			
			if activeUnit and pontoonBridge[activeUnit.type.id] 
			-- and (activeUnit.location.baseTerrain == object.bRiverbed or activeUnit.location.baseTerrain == object.bTemperateRiverbed)
			and ((activeUnit.location.baseTerrain == object.bRiverbed and activeUnit.location.terrain ~= object.tBlownBridge1)
			or (activeUnit.location.baseTerrain == object.bTemperateRiverbed and activeUnit.location.terrain ~= object.tBlownBridge2))
			and (not gen.unitTypeOnTile(activeUnit.location,object.uPontoonBridge))
			and activeUnit.owner.money >= buildpontoonBridgeCost 
			and activeUnitMoved >= buildpontoonBridgeMoveCost2 then
			-- and tile.terrainType & 0x0F == 0x0B -- Riverbed terrain
			   
				local menuOptions = {[1]="No, do not build a Pontoon Bridge.", [2]="Yes, spend ".. param.buildpontoonBridge .." supplies to build a Pontoon Bridge."}
				local menuText = text.substitute("Our %STRING1 unit can build a Pontoon Bridge at the cost of %STRING2 and %STRING3 movement points.  Shall we proceed?",
				{activeUnit.type.name, text.money(buildpontoonBridgeCost),buildpontoonBridgeMoveCost1})
				local menuTitle = "Build Pontoon Bridge?"
				
				local menuImage = civ.ui.loadImage("Images\\PontoonBridge.bmp")
				local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
				--local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
				if choice == 2 then
					
					-- Summer Months
					if flag.value("Summer Months") == true then
						activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*buildpontoonBridgeMoveCost1
						activeUnit.owner.money = activeUnit.owner.money - buildpontoonBridgeCost
						-- Check if Riverbed or Temperate Riverbed terrain
						if activeUnit.location.baseTerrain == object.bTemperateRiverbed then
							gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
							local tileToChange = civ.getTile(activeUnit.location.x,activeUnit.location.y,activeUnit.location.z)
							tileToChange.terrain = object.tCrossing
						elseif activeUnit.location.baseTerrain == object.bRiverbed then
							gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
						end
						civ.playSound("Bldspcsh.wav")
						text.simple("The " .. activeUnit.type.name .." unit built a Pontoon Bridge on tile\n^(".. text.coordinates(activeUnit.location) ..") and therefore mechanized units can\n^now cross this riverbed!\n^\r^Should enemy actions destroy the pontoon bridge\n^then the tile on which it resides will revert back\n^to an impassable riverbed for mobile units.","Pontoon Bridge Built!",object.mPontoonBridge)
						gen.createUnit(object.uPontoonBridge, object.pAllies, {{activeUnit.location.x, activeUnit.location.y, activeUnit.location.z}}, {veteran=false})
						civ.ui.redrawTile(activeUnit.location) -- new line
					
					-- Fall/Spring/Winter Months: There a 1 in 3 chance engineers may fail to build pontoon during non-summer months
					else
						if math.random(1,3) == 3 then
							civ.playSound("NEG1.wav")
							text.simple("Your engineer failed to build the pontoon on the\n^riverbed located on tile (" .. text.coordinates(activeUnit.location) .. ") due to the bad\n^weather conditions and has expended " .. buildpontoonBridgeMoveCost1 .. " MP's\n^and ".. math.floor(param.buildpontoonBridge/2) .." supplies in the process. You will need to try\n^again if you wish to succeed in your endeavour!","Pontoon Bridge Not Built!",object.mFall)  
							activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*buildpontoonBridgeMoveCost1
							activeUnit.owner.money = activeUnit.owner.money - (buildpontoonBridgeCost/2)
						else
							activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*buildpontoonBridgeMoveCost1
							activeUnit.owner.money = activeUnit.owner.money - buildpontoonBridgeCost
							-- Check if Riverbed or Temperate Riverbed terrain
							if activeUnit.location.baseTerrain == object.bTemperateRiverbed then
								gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
								local tileToChange = civ.getTile(activeUnit.location.x,activeUnit.location.y,activeUnit.location.z)
								tileToChange.terrain = object.tCrossing
							elseif activeUnit.location.baseTerrain == object.bRiverbed then
								gen.setTerrainType(activeUnit.location,15) -- Upgrade to River Crossing Terrain
							end
							civ.playSound("Bldspcsh.wav")
							text.simple("The " .. activeUnit.type.name .." unit built a Pontoon Bridge on tile\n^(".. text.coordinates(activeUnit.location) ..") and therefore mechanized units can\n^now cross this riverbed!\n^\r^Should enemy actions destroy the pontoon bridge\n^then the tile on which it resides will revert back\n^to an impassable riverbed for mobile units.","Pontoon Bridge Built!",object.mPontoonBridge)
							gen.createUnit(object.uPontoonBridge, object.pAllies, {{activeUnit.location.x, activeUnit.location.y, activeUnit.location.z}}, {veteran=false})
							civ.ui.redrawTile(activeUnit.location) -- new line
						end
					end	
				end
			elseif activeUnit and pontoonBridge[activeUnit.type.id] then
				text.simple("The engineer unit can only build a Pontoon Bridge provided the tile they are located on is a Riverbed or Temperate Riverbed, doesn't already contain a Pontoon Bridge or a Blown Bridge icon, has at least " .. buildpontoonBridgeMoveCost1 .. " MP's remaining and the cost of ".. param.buildpontoonBridge .." supplies doesn't exceed the amount currently in the theater depots!","Unable to Pontoon Bridge!",object.mPontoonBridge)
			end
		-- else
			-- text.simple("Engineer units cannot build pontoon bridges during winter turns.","Pontoon Bridge Building Prohibited",object.monexone)
		-- end
	end	

	-- KEY PRESS 'shift.two'
	-- SARDEGNA/CORSICA: AMERICAN OR FREE FRENCH INFANTRY UNITS MAY BE TELEPORTED TO SARDEGNA OR CORSICA AFTER ITALY SURRENDERS	
	if keyCode == keyboard.shift.two and flag.value("Italy Surrendered") == true then
        if not civ.getActiveUnit() then
            return
        end	

		local activeUnit = civ.getActiveUnit()
		local teleportCostSardegna = param.teleportCostSardegna
		local moveToSardegnaCost1 = 2 
		local moveToSardegnaCost2 = 2 * totpp.movementMultipliers.aggregate
		local activeUnitMoved = gen.moveRemaining(activeUnit)		
		
		if tables.alliedSardegnaUnit[activeUnit.type.id] 
		and activeUnit.location.baseTerrain == object.bTownCenter
		and activeUnit.owner.money >= teleportCostSardegna
		-- and (activeUnit.location == object.lPalermo or activeUnit.location == object.lTrapani ) then 
		-- and (activeUnit.location == object.lValetta or activeUnit.location == object.lTunis ) 
		and activeUnitMoved >= moveToSardegnaCost2 then 
			local menuOptions = {[1]="No, do not send unit", [2]="Yes, spend ".. param.teleportCostSardegna .." supplies to send unit to Cagliari", [3]="Yes, spend ".. param.teleportCostSardegna .." supplies to send unit to Olbia", [4]="Yes, spend ".. param.teleportCostSardegna .." supplies to send unit to Bonifacio", [5]="Yes, spend ".. param.teleportCostSardegna .." supplies to send unit to Palermo"}
			local menuText = text.substitute("Our %STRING1 unit can be transported to Sardegna at the cost of %STRING2 supplies.  Shall we proceed?",
			{activeUnit.type.name, text.money(teleportCostSardegna)})
			local menuTitle = "Tranport Infantry Unit to Sardegna?"
			
			local menuImage = civ.ui.loadImage("Images\\LibertyShip.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
			--local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
			
			-- Teleport infantry unit from Palermo/Trapani to Cagliari			
			if choice == 2 then
				if (activeUnit.location == object.lPalermo or activeUnit.location == object.lTrapani or activeUnit.location == object.lTunis) and object.cCagliari.owner == object.pAllies then
				-- if (activeUnit.location == object.lValetta or activeUnit.location == object.lTunis ) then
				-- activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*moveToSardegnaCost1
					civ.playSound("Cheers2.wav")
					text.simple("The " .. activeUnit.type.name .. " unit was safely transported to Cagliari!","Transported to Cagliari",object.mLibertyShip)
					activeUnit.owner.money = activeUnit.owner.money - teleportCostSardegna
					civ.teleportUnit(activeUnit, object.lCagliari)
				else
					civ.playSound("NEG1.wav")
					text.simple("The " .. activeUnit.type.name .. " unit must be located in either the port of Palermo, Trapani or Tunis and you must control Cagliari to be transported to that city!","Unable to Transport to Cagliari",object.mLibertyShip)
				end
			
			-- Teleport infantry unit from Cagliari, Palermo or Trapani to Olbia		
			elseif choice == 3 then
				if (activeUnit.location == object.lCagliari or activeUnit.location == object.lPalermo or activeUnit.location == object.lTrapani or activeUnit.location == object.lTunis)
				and object.cOlbia.owner == object.pAllies
				then
					-- activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*moveToSardegnaCost1
					civ.playSound("Cheers2.wav")
					text.simple("The " .. activeUnit.type.name .. " unit was safely transported to Olbia!","Transported to Olbia",object.mLibertyShip)
					activeUnit.owner.money = activeUnit.owner.money - teleportCostSardegna
					civ.teleportUnit(activeUnit, object.lOlbia)
				else
					civ.playSound("NEG1.wav")
					text.simple("The " .. activeUnit.type.name .. " unit must be located in either the port of Cagliari, Palermo, Trapani or Tunis and you must control Olbia to be transported to that city!","Unable to Transport to Olbia",object.mLibertyShip)
				end	
			
			-- Teleport infantry unit from Olbia to Bonifacio
			elseif choice == 4 then
				if (activeUnit.location == object.lOlbia) and object.lbeachBonifacio.defender == nil then
					-- activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*moveToSardegnaCost1
					civ.playSound("Cheers2.wav")
					text.simple("The " .. activeUnit.type.name .. " unit was safely transported to Bonifacio!","Transported to Bonifacio",object.mLibertyShip)
					activeUnit.owner.money = activeUnit.owner.money - teleportCostSardegna
					civ.teleportUnit(activeUnit, object.lbeachBonifacio)
				else
					civ.playSound("NEG1.wav")
					text.simple("The " .. activeUnit.type.name .. " unit must be located in the port of Olbia and the beach tile NE of Bonifacio must be free of enemy defenders for your unit to be transported there!","Unable to Transport to Bonifacio",object.mLibertyShip)
				end		
			-- end

			-- Teleport infantry unit from Ajaccio or Cagliari to Palermo
			elseif choice == 5 then
				if (activeUnit.location == object.lAjaccio or activeUnit.location == object.lCagliari) then
					-- activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*moveToSardegnaCost1
					civ.playSound("Cheers2.wav")
					text.simple("The " .. activeUnit.type.name .. " unit was safely transported to Palermo!","Transported to Bonifacio",object.mLibertyShip)
					activeUnit.owner.money = activeUnit.owner.money - teleportCostSardegna
					civ.teleportUnit(activeUnit, object.lPalermo)
				else
					civ.playSound("NEG1.wav")
					text.simple("The " .. activeUnit.type.name .. " unit must be located in the port of Ajaccio to be transported to Palermo!","Unable to Transport to Palermo",object.mLibertyShip)
				end		
			end
		
		elseif activeUnit and tables.alliedSardegnaUnit[activeUnit.type.id] then
			text.simple("The " .. activeUnit.type.name .. " infantry unit can only move to Sardegna if it is located in the cities of Palermo, Trapani or Tunis, or to Palermo if it is located in Ajaccio if it has at least " .. moveToSardegnaCost1 .. " MP's remaining and the cost of ".. param.teleportCostSardegna .." supplies doesn't exceed the amount currently in the theater depots!","Unable to Transport to Destination!",object.mLibertyShip)
		end

	end	

	-- KEY PRESS 'shift.three'
	-- DUKW'S BUILD SUPPLY DEPOT
	if keyCode == keyboard.shift.three and flag.value("Allies at War") == true then
        if not civ.getActiveUnit() then
            return
        end	

		local activeUnit = civ.getActiveUnit()
		local buildSupplyDepot = {}
		buildSupplyDepot[object.uDUKW.id] = true
		local buildSupplyDepotMoneyCost = param.buildSupplyDepot
		local buildSupplyDepotMoveCost1 = 4 
		local buildSupplyDepotMoveCost2 = 4 * totpp.movementMultipliers.aggregate
		local activeUnitMoved = gen.moveRemaining(activeUnit)
		local engineerText

		if activeUnit and buildSupplyDepot[activeUnit.type.id] 
		and activeUnit.location.baseTerrain == object.bTownCenter 
		and activeUnit.owner.money >= buildSupplyDepotMoneyCost 
		and activeUnitMoved >= buildSupplyDepotMoveCost2  
		and ((activeUnit.location.city == object.cBari and flag.value("Docks built in Bari") == true and (not civ.hasImprovement(object.cBari,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cBologna and (not civ.hasImprovement(object.cBologna,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cCatania and flag.value("Docks built in Catania") == true and (not civ.hasImprovement(object.cCatania,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cFirenze and (not civ.hasImprovement(object.cFirenze,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cFoggia and (not civ.hasImprovement(object.cFoggia,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cGenova and flag.value("Sea Mines removed from Genova") == true and (not civ.hasImprovement(object.cGenova,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cLivorno and flag.value("Docks built in Livorno") == true and (not civ.hasImprovement(object.cLivorno,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cMessina and flag.value("Docks built in Messina") == true and (not civ.hasImprovement(object.cMessina,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cNapoli and flag.value("Docks built in Napoli") == true and (not civ.hasImprovement(object.cNapoli,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cPalermo and flag.value("Docks built in Palermo") == true and (not civ.hasImprovement(object.cPalermo,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cPescara and (not civ.hasImprovement(object.cPescara,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cRimini and flag.value("Docks built in Rimini") == true and (not civ.hasImprovement(object.cRimini,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cROMA and (not civ.hasImprovement(object.cROMA,object.iSupplyDepot)))
		or (activeUnit.location.city == object.cTaranto and flag.value("Docks built in Taranto") == true and (not civ.hasImprovement(object.cTaranto,object.iSupplyDepot)))
		)
		then

			local menuOptions = {[1]="No, do not attempt to build a Supply Depot.", [2]="Yes, spend "..param.buildSupplyDepot.." to build a Supply Depot in this port."}
			local menuText = text.substitute("Our %STRING1 unit can attempt to build a Supply Depot at the cost of %STRING2 and %STRING3 movement points.  Shall we proceed?",
			{activeUnit.type.name, text.money(buildSupplyDepotMoneyCost),buildSupplyDepotMoveCost1})
			local menuTitle = "Build Supply Depot in this major port?"
			
			local menuImage = civ.ui.loadImage("Images\\SupplyDepot.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box

			if choice == 2 then
				-- CITY: BARI
				if activeUnit.location.city == object.cBari 
				and flag.value("Supply Depot being built in Bari") == false and flag.value("Supply Depot built in Bari") == false 
				-- and (not civ.hasImprovement(object.cBari,object.iSupplyDepot))
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInBari(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInBari",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInBari",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInBari",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(122,124,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Bari")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Bari and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Bari!", object.monexone)
				elseif activeUnit.location.city == object.cBari and flag.value("A Supply Depot being built in Bari") == true and flag.value("Supply Depot built in Bari") == false then
					engineerText = text.simple("The construction of a Supply Depot in Bari is already in progress!","Supply Depot being built in Bari!", object.monexone)
				end

				-- CITY: BOLOGNA
				if activeUnit.location.city == object.cBologna 
				and flag.value("Supply Depot being built in Bologna") == false and flag.value("Supply Depot built in Bologna") == false 
				-- and (not civ.hasImprovement(object.cBologna,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInBologna(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInBologna",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInBologna",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInBologna",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(101,33,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Bologna")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Bologna and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Bologna!", object.monexone)
				elseif activeUnit.location.city == object.cBologna and flag.value("A Supply Depot being built in Bologna") == true and flag.value("Supply Depot built in Bologna") == false then
					engineerText = text.simple("The construction of a Supply Depot in Bologna is already in progress!","Supply Depot being built in Bologna!", object.monexone)
				end

				-- CITY: CATANIA
				if activeUnit.location.city == object.cCatania 
				and flag.value("Supply Depot being built in Catania") == false and flag.value("Supply Depot built in Catania") == false 
				-- and (not civ.hasImprovement(object.cCatania,object.iSupplyDepot))
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInCatania(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInCatania",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInCatania",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInCatania",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(64,160,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Catania")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Catania and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Catania!", object.monexone)
				elseif activeUnit.location.city == object.cCatania and flag.value("A Supply Depot being built in Catania") == true and flag.value("Supply Depot built in Catania") == false then
					engineerText = text.simple("The construction of a Supply Depot in Catania is already in progress!","Supply Depot being built in Catania!", object.monexone)
				end
				
				-- CITY: FIRENZE
				if activeUnit.location.city == object.cFirenze 
				and flag.value("Supply Depot being built in Firenze") == false and flag.value("Supply Depot built in Firenze") == false 
				-- and (not civ.hasImprovement(object.cFirenze,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInFirenze(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInFirenze",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInFirenze",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInFirenze",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(87,39,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Firenze")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Firenze and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Firenze!", object.monexone)
				elseif activeUnit.location.city == object.cFirenze and flag.value("A Supply Depot being built in Firenze") == true and flag.value("Supply Depot built in Firenze") == false then
					engineerText = text.simple("The construction of a Supply Depot in Firenze is already in progress!","Supply Depot being built in Firenze!", object.monexone)
				end

				-- CITY: FOGGIA
				if activeUnit.location.city == object.cFoggia 
				and flag.value("Supply Depot being built in Foggia") == false and flag.value("Supply Depot built in Foggia") == false 
				-- and (not civ.hasImprovement(object.cFoggia,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInFoggia(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInFoggia",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInFoggia",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInFoggia",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(112,108,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Foggia")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Foggia and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Foggia!", object.monexone)
				elseif activeUnit.location.city == object.cFoggia and flag.value("A Supply Depot being built in Foggia") == true and flag.value("Supply Depot built in Foggia") == false then
					engineerText = text.simple("The construction of a Supply Depot in Foggia is already in progress!","Supply Depot being built in Foggia!", object.monexone)
				end

				-- CITY: GENOVA
				if activeUnit.location.city == object.cGenova 
				and flag.value("Supply Depot being built in Genova") == false and flag.value("Supply Depot built in Genova") == false 
				-- and (not civ.hasImprovement(object.cGenova,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInGenova(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInGenova",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInGenova",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInGenova",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(73,15,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Genova")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Genova and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Genova!", object.monexone)
				elseif activeUnit.location.city == object.cGenova and flag.value("A Supply Depot being built in Genova") == true and flag.value("Supply Depot built in Genova") == false then
					engineerText = text.simple("The construction of a Supply Depot in Genova is already in progress!","Supply Depot being built in Genova!", object.monexone)
				end
				
				-- CITY: LIVORNO
				if activeUnit.location.city == object.cLivorno 
				and flag.value("Supply Depot being built in Livorno") == false and flag.value("Supply Depot built in Livorno") == false 
				-- and (not civ.hasImprovement(object.cLivorno,object.iSupplyDepot)) 
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInLivorno(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInLivorno",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInLivorno",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInLivorno",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(76,38,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Livorno")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Livorno and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Livorno!", object.monexone)
				elseif activeUnit.location.city == object.cLivorno and flag.value("A Supply Depot being built in Livorno") == true and flag.value("Supply Depot built in Livorno") == false then
					engineerText = text.simple("The construction of a Supply Depot in Livorno is already in progress!","Supply Depot being built in Livorno!", object.monexone)
				end
				
				-- CITY: MESSINA
				if activeUnit.location.city == object.cMessina 
				and flag.value("Supply Depot being built in Messina") == false and flag.value("Supply Depot built in Messina") == false 
				-- and (not civ.hasImprovement(object.cMessina,object.iSupplyDepot)) 
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInMessina(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInMessina",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInMessina",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInMessina",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(71,153,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Messina")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Messina and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Messina!", object.monexone)
				elseif activeUnit.location.city == object.cMessina and flag.value("A Supply Depot being built in Messina") == true and flag.value("Supply Depot built in Messina") == false then
					engineerText = text.simple("The construction of a Supply Depot in Messina is already in progress!","Supply Depot being built in Messina!", object.monexone)
				end
				
				-- CITY: NAPOLI
				if activeUnit.location.city == object.cNapoli 
				and flag.value("Supply Depot being built in Napoli") == false and flag.value("Supply Depot built in Napoli") == false 
				-- and (not civ.hasImprovement(object.cNapoli,object.iSupplyDepot)) 
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInNapoli(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInNapoli",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInNapoli",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInNapoli",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(87,103,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Napoli")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Napoli and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Napoli!", object.monexone)
				elseif activeUnit.location.city == object.cNapoli and flag.value("A Supply Depot being built in Napoli") == true and flag.value("Supply Depot built in Napoli") == false then
					engineerText = text.simple("The construction of a Supply Depot in Napoli is already in progress!","Supply Depot being built in Napoli!", object.monexone)
				end

				-- CITY: PALERMO
				if activeUnit.location.city == object.cPalermo 
				and flag.value("Supply Depot being built in Palermo") == false and flag.value("Supply Depot built in Palermo") == false 
				-- and (not civ.hasImprovement(object.cPalermo,object.iSupplyDepot)) 
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInPalermo(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInPalermo",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInPalermo",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInPalermo",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(44,138,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Palermo")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in\n^Palermo and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Palermo!", object.monexone)
				elseif activeUnit.location.city == object.cPalermo and flag.value("Supply Depot being built in Palermo") == true and flag.value("Supply Depot built in Palermo") == false then
					engineerText = text.simple("The construction of a Supply Depot in Palermo is already in progress!","Supply Depot being built in Palermo!", object.monexone)
				end

				-- CITY: PESCARA
				if activeUnit.location.city == object.cPescara 
				and flag.value("Supply Depot being built in Pescara") == false and flag.value("Supply Depot built in Pescara") == false 
				-- and (not civ.hasImprovement(object.cPescara,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInPescara(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInPescara",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInPescara",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInPescara",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(107,85,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Pescara")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Pescara and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Pescara!", object.monexone)
				elseif activeUnit.location.city == object.cPescara and flag.value("A Supply Depot being built in Pescara") == true and flag.value("Supply Depot built in Pescara") == false then
					engineerText = text.simple("The construction of a Supply Depot in Pescara is already in progress!","Supply Depot being built in Pescara!", object.monexone)
				end

				-- CITY: RIMINI
				if activeUnit.location.city == object.cRimini 
				and flag.value("Supply Depot being built in Rimini") == false and flag.value("Supply Depot built in Rimini") == false 
				-- and (not civ.hasImprovement(object.cRimini,object.iSupplyDepot)) 
				then	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInRimini(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInRimini",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInRimini",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInRimini",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(106,42,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Rimini")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Rimini and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Rimini!", object.monexone)
				elseif activeUnit.location.city == object.cRimini and flag.value("A Supply Depot being built in Rimini") == true and flag.value("Supply Depot built in Rimini") == false then
					engineerText = text.simple("The construction of a Supply Depot in Rimini is already in progress!","Supply Depot being built in Rimini!", object.monexone)
				end

				-- CITY: ROMA
				if activeUnit.location.city == object.cROMA 
				and flag.value("Supply Depot being built in Roma") == false and flag.value("Supply Depot built in Roma") == false 
				-- and (not civ.hasImprovement(object.cROMA,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInRoma(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInRoma",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInRoma",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInRoma",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(82,78,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Roma")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Roma and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Roma!", object.monexone)
				elseif activeUnit.location.city == object.cROMA and flag.value("A Supply Depot being built in Roma") == true and flag.value("Supply Depot built in Roma") == false then
					engineerText = text.simple("The construction of a Supply Depot in Roma is already in progress!","Supply Depot being built in Roma!", object.monexone)
				end

				-- CITY: TARANTO
				if activeUnit.location.city == object.cTaranto 
				and flag.value("Supply Depot being built in Taranto") == false and flag.value("Supply Depot built in Taranto") == false 
				-- and (not civ.hasImprovement(object.cTaranto,object.iSupplyDepot)) 
				then	 	
					local randomNumber = math.random(3)
					-- ALSO SEE 'function SupplyDepotBuiltInTaranto(argTable)' AT BOTTOM
					if randomNumber == 1 then
						delayedAction.doInFuture("BuildingSupplyDepotInTaranto",{},civ.getTurn()+ 3 ,object.pAllies.id)
					elseif randomNumber == 2 then
						delayedAction.doInFuture("BuildingSupplyDepotInTaranto",{},civ.getTurn()+ 4 ,object.pAllies.id)
					elseif randomNumber == 3 then
						delayedAction.doInFuture("BuildingSupplyDepotInTaranto",{},civ.getTurn()+ 5 ,object.pAllies.id)
					end
					activeUnit.owner.money = activeUnit.owner.money - param.buildSupplyDepot
					local tileToChange = civ.getTile(123,139,0)
					tileToChange.terrain = object.tSupplyDepot
					civ.ui.redrawTile(tileToChange)
					
					civ.deleteUnit(activeUnit)
					counter.subtract("# DUKW",1)
					counter.subtract("# Total Naval Units",1)
					flag.setTrue("Supply Depot being built in Taranto")
							
					engineerText = text.simple("The construction of a Supply Depot was begun in \n^Taranto and should be completed in " .. tostring(randomNumber + 2) .. " turns.\n^\r^The '" .. activeUnit.type.name .. "' unit assigned to the task will return\n^once the job is completed!\n^\r^A supply depot resource has been added next\n^to the city.","Supply Depot being built in Taranto!", object.monexone)
				elseif activeUnit.location.city == object.cTaranto and flag.value("A Supply Depot being built in Taranto") == true and flag.value("Supply Depot built in Taranto") == false then
					engineerText = text.simple("The construction of a Supply Depot in Taranto is already in progress!","Supply Depot being built in Taranto!", object.monexone)
				end	
	
			end
		elseif activeUnit and buildSupplyDepot[activeUnit.type.id] then
				engineerText = text.simple("Your DUKW must be located either in an objective city or in a port with a 'Harbor' resource, that doesn't already contain a Supply Depot, does contain a dock if a port, doesn't have sea mines, possess at least " .. param.buildSupplyDepot .. " supplies and " .. buildSupplyDepotMoveCost1 .. " MP's to proceed!","Unable to Build Supply Depot!",object.monexone)
		end		
		return engineerText
	end	
	
	
	-- KEY PRESS 'shift.k'
	-- ENGINEER: BUILDING RADARS ON AIRBASE AND CITY TILES
	if keyCode == keyboard.shift.k then
        if not civ.getActiveUnit() then
            return
        end	
		
		local activeUnit = civ.getActiveUnit()
		local radarStation = {}
		radarStation[object.uEngineer.id] = true
		local buildRadarStationCost = param.buildRadarStation
		local buildRadarStationMoveCost1 = 2
		local buildRadarStationMoveCost2 = 2 * totpp.movementMultipliers.aggregate
		-- local activeUnitMoved = activeUnit.type.move - activeUnit.moveSpent
		local activeUnitMoved = gen.moveRemaining(activeUnit)
		
		if activeUnit and radarStation[activeUnit.type.id] and (gen.hasAirbase(activeUnit.location) or activeUnit.location.baseTerrain == object.bTownCenter) and activeUnit.owner.money >= buildRadarStationCost 
		and (not gen.unitTypeOnTile(activeUnit.location,object.uRadarStation))
		and activeUnitMoved >= buildRadarStationMoveCost2 then
		-- and tile.terrainType & 0x0F == 0x0B -- Riverbed terrain
		   
			local menuOptions = {[1]="No, do not build a Radar Station.", [2]="Yes, spend ".. param.buildRadarStation .." supplies to build a Radar Station."}
			local menuText = text.substitute("Our %STRING1 unit can build a Radar Station at the cost of %STRING2 and %STRING3 movement points.  Shall we proceed?",
			{activeUnit.type.name, text.money(buildRadarStationCost),buildRadarStationMoveCost1})
			local menuTitle = "Build Radar Station?"
			
			local menuImage = civ.ui.loadImage("Images\\RadarStation.bmp")
			local choice = text.menu(menuOptions,menuText,menuTitle,menuImage) -- If you want to include an image in the pop up box
			--local choice = text.menu(menuOptions,menuText,menuTitle) -- If you don't want to include an image in the pop up box
			if choice == 2 then
				civ.playSound("Bldspcsh.wav")
				text.simple("The engineer unit successfully built a " .. object.uRadarStation.name .. " on tile (".. text.coordinates(activeUnit.location) ..")!","Radar Station Built!",object.mRadarStation)
				activeUnit.moveSpent = activeUnit.moveSpent + totpp.movementMultipliers.aggregate*buildRadarStationMoveCost1
				activeUnit.owner.money = activeUnit.owner.money - buildRadarStationCost
				gen.createUnit(object.uRadarStation, object.pAllies, {{activeUnit.location.x, activeUnit.location.y, activeUnit.location.z}}, {veteran=false})
				-- gen.placeRoad(activeUnit.location)
				--gen.chartRailroad(activeUnit.location,activeUnit.owner) -- new line
				civ.ui.redrawTile(activeUnit.location) -- new line
				
			end
		elseif activeUnit and radarStation[activeUnit.type.id] then
			text.simple("The engineer unit can only build a Radar Station provided the tile they are located on is an airbase or a city (not a town), doesn't already contain a Radar Station, has at least " .. buildRadarStationMoveCost1 .. " MP's remaining and the cost of ".. param.buildRadarStation .." supplies doesn't exceed the amount currently in the theater depots!","Radar Station",object.mRadarStation)
		end

	end	
	

	-- KEY PRESS 'BACKSPACE'
	-- LOAD UNITS ON TRANSPORTS
	if keyCode == keyboard.backspace then
	local activeUnit = civ.getActiveUnit()
		if civ.getActiveUnit() then	
			transport.loadUnitsOnTruck(activeUnit)
		end
	end

	-- KEY PRESS 'SHIFT' + 'BACKSPACE'
	-- UNLOAD UNITS FROM TRANSPORTS
	if keyCode == keyboard.shift.backspace then
	local activeUnit = civ.getActiveUnit()
		if civ.getActiveUnit() then
			transport.unloadUnitsOffTruck(activeUnit)
		end
	end
	
	
	-- KEY PRESS '1'
	-- DISPLAYS WEEK TURN, NUMBER OF DOCKS, SUPPLY HUBS, RAILROAD YARDS AND OBJECTIVES CONTROLLED BY ALLIES FOR AUTOMATIC VICTORY
	if keyCode == keyboard.one then

		local commanderAFHQ_Base = onTurn.commanderAFHQ_1
		if civ.getTurn() >= 1 and civ.getTurn() <= 26 then
			commanderAFHQ_Base = onTurn.commanderAFHQ_1
		elseif civ.getTurn() >= 27 and civ.getTurn() <= 68 then
			commanderAFHQ_Base = onTurn.commanderAFHQ_2
		elseif civ.getTurn() >= 69 and civ.getTurn() <= 89 then
			commanderAFHQ_Base = onTurn.commanderAFHQ_3
		end

		-- local messageText = " " .. displayWeekMonthYear() .. ""
		-- local messageText = "************** " .. onTurn.displayWeekMonthYear() .. " (turn " .. civ.getTurn() .. " of 89) **************\n^\r^General " .. commanderAFHQ ..", here are the latest status reports:\r^"
		local messageText = ""
		if  flag.value("Game ends March I, 1945") == false then
			messageText = "*********** " .. onTurn.displayWeekMonthYear() .. " (turn " .. civ.getTurn() .. " of 89) ***********\n^\r^General " .. commanderAFHQ_Base ..", here are the latest status reports:\r^"
		else
			messageText = "*********** " .. onTurn.displayWeekMonthYear() .. " (turn " .. civ.getTurn() .. " of 81) ***********\n^\r^General " .. commanderAFHQ_Base ..", here are the latest status reports:\r^"
		end	
		
		-- TOTAL # OF BEACHEADS 
		messageText = messageText .. "\n^Total Beachheads: " .. counter.value("Beachhead #") .. " out of a maximum of 4 allowed"
		
		-- TOTAL # OF DOCKS
		local totalDocks = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAllies then
				if unit.type == object.uDocks then
					totalDocks = totalDocks + 1
				end
			end
		end		
		if totalDocks >= 0 then
			messageText = messageText .. "\n^Total Docks: " .. totalDocks .. " docks out of a possible 9 added " .. tostring(totalDocks * param.suppliesPerDocks) .. " supplies this week"
		end
		
		-- TOTAL PER MAJOR AIRFIELD
		local numberAirfields = 0
		local totalAirfields = 0
		
		if object.cBari.owner == object.pAllies then
			numberAirfields = numberAirfields + 1
			totalAirfields = totalAirfields + param.bastiaAirfield
		end
		if object.cFoggia.owner == object.pAllies then
			numberAirfields = numberAirfields + 1
			totalAirfields = totalAirfields + param.foggiaAirfield
		end
		if object.cNapoli.owner == object.pAllies then
			numberAirfields = numberAirfields + 1
			totalAirfields = totalAirfields + param.napoliAirfield
		end
		if object.cROMA.owner == object.pAllies then
			numberAirfields = numberAirfields + 1
			totalAirfields =  totalAirfields + param.romaAirfield
		end

		if numberAirfields >= 0 then
			messageText = messageText .. "\n^Total Airfields: " .. numberAirfields .. " airfields out of a possible 4 added " .. tostring(totalAirfields) .. " supplies this week"
		end


		-- TOTAL # OF SUPPLY HUBS
		local totalSupplyHubs = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAllies then
				if unit.type == object.uSupplyHub then
					totalSupplyHubs = totalSupplyHubs + 1
				end
			end
		end		
		if totalSupplyHubs >= 0 then
			messageText = messageText .. "\n^Total Supply Hubs: " .. totalSupplyHubs .. " supply hub(s) cost " .. tostring(totalSupplyHubs * param.costPerSupplyHub) .. " supplies per week"
		end

		-- TRACK ALLIED SUPPLIES SPENT DURING CAMPAIGN
		-- Spent on Replacements 
		-- See procurement.lua for the tracking of replacement costs
		if counter.value("# Supplies Spent in Campaign") >= 0 then
			messageText = messageText .. "\n^Total Supplies Spent on Replacements: " .. counter.value("# Supplies Spent in Campaign") .. ""
		end	
		-- Spent on Munitions
		-- see keyPressSettings.lua for the tracking of all generated munitions cost
		if counter.value("# Supplies Spent in Campaign") >= 0 then
			messageText = messageText .. "\n^Total Supplies Spent on Munitions: " .. counter.value("# Supplies Spent on Munitions") .. ""
		end	

		-- TOTAL # OF RAILROAD YARDS
		local totalRailroadYard = 0
		for unit in civ.iterateUnits() do
			if unit.owner == object.pAxis then
				if unit.type == object.uRailroadYard then
					totalRailroadYard = totalRailroadYard + 1
				end
			end
		end		
		if totalRailroadYard >= 0 then
			messageText = messageText .. "\n^\r^Total Axis Railroad yards: " .. totalRailroadYard .. " out 5 are active"
		end

		-- TOTAL GERMAN REPLACEMENTS RECEIVED
		if counter.value("# Total German Replacements Received") >= 0 then
			messageText = messageText .. "\n^Total German Replacements Received: " .. counter.value("# Total German Replacements Received") .. ""
		end	

		-- TOTAL # OF OBJECTIVES
		local conditionsAlliedVictory = 0
		local conditionsAlliedSuddenDeath = 0
		local conditionsAlliedSuddenDeath_GoL = 0
		if object.cROMA.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 4
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 4
		end
		if object.cBari.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cBastia.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cBologna.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end
		if object.cFirenze.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cFoggia.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cGenova.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cLivorno.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cMilano.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end
		if object.cNapoli.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end
		if object.cPalermo.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cPescara.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cReggioCalabria.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cRimini.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cVerona.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 1
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 1
		end
		if object.cTrieste.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 3
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 3
		end
		if object.cTurino.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end		
		if object.cValetta.owner == object.pAllies then
			conditionsAlliedVictory = conditionsAlliedVictory + 2
			conditionsAlliedSuddenDeath = conditionsAlliedSuddenDeath + 2
		end

		-- For 2nd Sudden Death Conditions along Gothic Line
		if object.ltownLucca.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownMassa.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownSanMarino.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownPistoia.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end
		if object.ltownLeccio.owner == object.pAllies then
			conditionsAlliedSuddenDeath_GoL = conditionsAlliedSuddenDeath_GoL + 1
		end			

		-- For Conquest of Sicily
		local sicilyIsConquered = 0
		if object.cCatania.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cMessina.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cPalermo.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cSiracusa.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end
		if object.cTrapani.owner == object.pAllies then
			sicilyIsConquered = sicilyIsConquered + 1
		end

		if flag.value("Sicily is Conquered") == false then
			messageText = messageText .. "\n^\r^Sicilian Cities controlled by Allies: " .. sicilyIsConquered .. " out of 5 to conquer the island"
		end

		-- if conditionsAlliedSuddenDeath >= 0 then
			-- if (civ.getTurn() >= 1 and civ.getTurn() <= 48) then
				-- messageText = messageText .. "\n^\r^Objectives controlled by Allies: " .. conditionsAlliedSuddenDeath .. " out of 14 to avoid Sudden Death by end of turn 49"
			-- elseif (civ.getTurn() >= 49 and civ.getTurn() <= 62) then
				-- messageText = messageText .. "\n^\r^Objectives controlled by Allies: " .. conditionsAlliedSuddenDeath .. " out of 19 to avoid Sudden Death by end of turn 62"
			-- end
		-- end	

		if flag.value("Sicily is Conquered") == true and flag.value("Allies completed first SD milestone") == false then
			messageText = messageText .. "\n^\r^Objectives Cities controlled by Allies: " .. conditionsAlliedSuddenDeath .. " out of 14 to avoid Sudden Death by end of turn 49"
		elseif flag.value("Allies completed first SD milestone") == true and flag.value("Allies completed second SD milestone") == false and conditionsAlliedVictory <= 18 then
			messageText = messageText .. "\n^\r^Objectives Points controlled by Allies: " .. conditionsAlliedSuddenDeath + conditionsAlliedSuddenDeath_GoL .. " out of 19 to avoid Sudden Death by end of turn 62"
		end

		-- if conditionsAlliedVictory >= 13 then
			-- messageText = messageText .. "\n^Objectives controlled by Allies: " .. conditionsAlliedVictory .. " out of 28 for Automatic Victory"
		-- end	

		if flag.value("Allies completed first SD milestone") == true and flag.value("Allies completed second SD milestone") == true then
			messageText = messageText .. "\n^\r^Objectives Cities controlled by Allies: " .. conditionsAlliedVictory .. " out of 28 for Automatic Victory"
		end	

		if flag.value("3rd Invasion Available") == true and flag.value("3rd Invasion Launched") == false then
			messageText = messageText .. "\n^\r^Maximum 3rd Invasion Launch Date: January IV, 1944 (turn #28)"
		end	
		
		-- Gustav Line Zones Breached
		local gustavLineIsBreached = 0
		if object.ltownCassino.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNE_Isernia.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNNE_Isernia.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNE_Formia.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNW_Cassino.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lfortNW_Ortona.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end
		if object.lPescara.owner == object.pAllies then
			gustavLineIsBreached = gustavLineIsBreached + 1
		end	
		
		if gustavLineIsBreached >= 0 and flag.value("Gustav Line is breached") == false and flag.value("2nd Invasion Available") == true then
			messageText = messageText .. "\n^Gustav Line controlled by Allies: " .. gustavLineIsBreached .. " out of 4 to Breach Line"
		end	

		-- Gothic Line Zones Breached
		local gothicLineIsBreached = 0
		if object.cBologna.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cForli.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cLaSpezia.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cModena.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cParma.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cRavenna.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end
		if object.cRimini.owner == object.pAllies then
			gothicLineIsBreached = gothicLineIsBreached + 1
		end	

		if gothicLineIsBreached >= 0 and flag.value("Gustav Line is breached") == true and flag.value("Gothic Line is breached") == false then
			messageText = messageText .. "\n^Gothic Line Cities controlled by Allies: " .. gothicLineIsBreached .. " out of 6 to Breach Line"
		end	


		-- Allied and Axis Losses Count	
		if counter.value("# Total Allied Killed") >= 0 then
			messageText = messageText .. "\n^\r^Total Allied Units Killed: " .. counter.value("# Total Allied Killed") .. ""
		end			
		if counter.value("# Total Germans Killed") >= 0 then
			messageText = messageText .. "\n^Total German Units Killed: " .. counter.value("# Total Germans Killed") .. ""
		end
		if counter.value("# Total Italians Killed") >= 0 then
			messageText = messageText .. "\n^Total Italian Units Killed: " .. counter.value("# Total Italians Killed") .. " (divided by 2 during loss calculation)"
		end			

		-- if counter.value("# Total Allied Killed") >= 0 then
			-- messageText = messageText .. "\n^Allied vs Axis Losses: " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% of maximum 70%"
		-- end	
		
		-- if civ.getTurn() >= 1 then
		-- -- if civ.getTurn() >= 4 and flag.value("Sicily is Conquered") == true then
			-- if (counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 70 then
				-- messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties70Plus .." supplies lost due to casualty rate"
			-- elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 60) or ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) < 70) then
				-- messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties60Plus .." supplies lost due to casualty rate"
			-- elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 40) or ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) < 60) then
				-- messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties40Plus .." supplies lost due to casualty rate"
			-- elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 26) or ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) < 40) then
				-- messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties26Plus .." supplies lost due to casualty rate"
			-- elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) < 26 then
				-- messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or 0 supplies lost due to casualty rate"
			-- end	
		-- end		
		
		if civ.getTurn() >= 1 and counter.value("# Total Allied Killed") >= 50 then
			if (counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 70 then
				messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties70Plus .." supplies lost due to casualty rate"
			elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 60) then
				messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties60Plus .." supplies lost due to casualty rate"
			elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 50) then
				messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties50Plus .." supplies lost due to casualty rate"
			elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 40)then
				messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties40Plus .." supplies lost due to casualty rate"
			elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) * 100) >= 26)then
				messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or " .. param.casualties26Plus .." supplies lost due to casualty rate"
			elseif ((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) < 26 then
				messageText = messageText .. "\n^Allied (".. counter.value("# Total Allied Killed") ..") vs Axis Losses (" .. (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2)) .. "): " .. math.floor((counter.value("# Total Allied Killed") / (counter.value("# Total Germans Killed") + (counter.value("# Total Italians Killed")/2))) * 100) .. "% or 0 supplies lost due to casualty rate"
			end	
		end	

		-- DISPLAY AND GAME OPTION STATUSES
		if flag.value("Allies at War") == true then
			messageText = messageText .. 
			"\n^\r^---------------------------------------------------------\n^DISPLAY AND GAME OPTION STATUSES:"..
			-- "\n^\r^Stacking Limit Option On? = ".. tostring(flag.value("Stacking Limit Options")) .. 
			-- "\n^Ships At Sea Penalty Option On? = ".. tostring(flag.value("Ships At Sea Penalty Option")) ..
			-- "\n^Display Combat Power Values On? = ".. tostring(flag.value("Display Attacker and Defender Values")) ..
			"\n^\r^Display Game Tip Pop-Ups On? = ".. tostring(flag.value("Display Game Tip Pop-Ups")) ..
			"\n^Display Out of Supply Pop-Ups On? = ".. tostring(flag.value("Display Out of Supply Pop-Ups")) .. ""
		end
		civ.ui.text(func.splitlines(messageText))
	end	

	-- KEY PRESS '2'
	-- STATUS REPORT THAT ALLOWS ALLIED PLAYER TO SEE HOW MANY OVERALL UNITS THEY HAVE IN EACH CATEGORY (designed to work in conjunction with procurement menu(keypress.three))
	if keyCode == keyboard.two then

	local commanderAFHQ_Base = onTurn.commanderAFHQ_1
	if civ.getTurn() >= 1 and civ.getTurn() <= 26 then
		commanderAFHQ_Base = onTurn.commanderAFHQ_1
	elseif civ.getTurn() >= 27 and civ.getTurn() <= 69 then
		commanderAFHQ_Base = onTurn.commanderAFHQ_2
	elseif civ.getTurn() >= 70 and civ.getTurn() <= 89 then
		commanderAFHQ_Base = onTurn.commanderAFHQ_3
	end

	local messageText = "************** " .. onTurn.displayWeekMonthYear() .. " **************\n^\r^General " .. commanderAFHQ_Base ..", here are the latest order of battle reports:\n^\r^"
		--if unit.owner == object.pAllies and 
		if flag.value("Allies at War") == true then
			local alliedUnitsDetails = getAlliedTotalsDetails ()
			local alliedUnitCount = 0
			for i = 1, 6 do
				if alliedUnitsDetails[i] ~= nil then
					alliedUnitCount = alliedUnitCount + alliedUnitsDetails[i]
					print("Count: " .. alliedUnitCount )
				end
			end
			if flag.value("Allies at War") == true then
			messageText = messageText .. 

			" Allied Total Units Currently Deployed in theater (*):\n^\r^US Fighters: " .. tostring((alliedUnitsDetails[1] or 0) + (alliedUnitsDetails[2] or 0) + (alliedUnitsDetails[3] or 0)) .. 
			" of " .. tostring(counter.value("# Total US Fighters")) .. "\n^US Bombers: " .. tostring((alliedUnitsDetails[4] or 0) + (alliedUnitsDetails[5] or 0) + (alliedUnitsDetails[6] or 0) + (alliedUnitsDetails[7] or 0) + (alliedUnitsDetails[8] or 0)) .. 
			" of " .. tostring(counter.value("# Total US Bombers")) .. "\n^US Armored: " .. tostring((alliedUnitsDetails[9] or 0) + (alliedUnitsDetails[10] or 0) + (alliedUnitsDetails[11] or 0) + (alliedUnitsDetails[12] or 0) + (alliedUnitsDetails[13] or 0) + (alliedUnitsDetails[14] or 0) + (alliedUnitsDetails[81] or 0) + (alliedUnitsDetails[84] or 0) + (alliedUnitsDetails[86] or 0)) .. 
			" of " .. tostring(counter.value("# Total US Armored")) .. " (**)\n^US Infantry: " .. tostring((alliedUnitsDetails[15] or 0) + (alliedUnitsDetails[16] or 0) + (alliedUnitsDetails[17] or 0) + (alliedUnitsDetails[18] or 0) + (alliedUnitsDetails[19] or 0) + (alliedUnitsDetails[20] or 0) + (alliedUnitsDetails[21] or 0) + (alliedUnitsDetails[100] or 0)) .. 
			" of " .. tostring(counter.value("# Total US Infantry")) .. "\n^US Artillery: " .. tostring((alliedUnitsDetails[22] or 0) + (alliedUnitsDetails[23] or 0) + (alliedUnitsDetails[24] or 0) + (alliedUnitsDetails[25] or 0) + (alliedUnitsDetails[26] or 0) + (alliedUnitsDetails[82] or 0)) .. 
			" of " .. tostring(counter.value("# Total US Artillery")) .. "\n^\r^UK Fighters: " .. tostring((alliedUnitsDetails[27] or 0) + (alliedUnitsDetails[28] or 0) + (alliedUnitsDetails[29] or 0) + (alliedUnitsDetails[87] or 0)) .. 
			
			" of " .. tostring(counter.value("# Total UK Fighters")) .. "\n^UK Bombers: " .. tostring((alliedUnitsDetails[30] or 0) + (alliedUnitsDetails[31] or 0) + (alliedUnitsDetails[32] or 0) + (alliedUnitsDetails[33] or 0) + (alliedUnitsDetails[34] or 0) + (alliedUnitsDetails[88] or 0)) .. 
			" of " .. tostring(counter.value("# Total UK Bombers")) .. "\n^UK Armored: " .. tostring((alliedUnitsDetails[35] or 0) + (alliedUnitsDetails[36] or 0) + (alliedUnitsDetails[37] or 0) + (alliedUnitsDetails[38] or 0) + (alliedUnitsDetails[39] or 0) + (alliedUnitsDetails[76] or 0) + (alliedUnitsDetails[77] or 0) + (alliedUnitsDetails[85] or 0) + (alliedUnitsDetails[99] or 0)) ..
			" of " .. tostring(counter.value("# Total UK Armored")) .. "\n^UK Infantry: " .. tostring((alliedUnitsDetails[40] or 0) + (alliedUnitsDetails[41] or 0) + (alliedUnitsDetails[42] or 0) + (alliedUnitsDetails[43] or 0) + (alliedUnitsDetails[44] or 0) + (alliedUnitsDetails[45] or 0) + (alliedUnitsDetails[98] or 0)) .. 
			" of " .. tostring(counter.value("# Total UK Infantry")) .. "\n^UK Artillery: " .. tostring((alliedUnitsDetails[46] or 0) + (alliedUnitsDetails[47] or 0) + (alliedUnitsDetails[48] or 0) + (alliedUnitsDetails[49] or 0) + (alliedUnitsDetails[83] or 0)) .. 
			" of " .. tostring(counter.value("# Total UK Artillery")) .. "\n^\r^" 
			end
			if flag.value("Allies at War") == true then
				messageText = messageText ..
				"Royal Navy and USN: " .. tostring((alliedUnitsDetails[89] or 0) + (alliedUnitsDetails[90] or 0) + (alliedUnitsDetails[91] or 0) + (alliedUnitsDetails[92] or 0) + (alliedUnitsDetails[93] or 0) + (alliedUnitsDetails[94] or 0) + (alliedUnitsDetails[95] or 0) + (alliedUnitsDetails[96] or 0) + (alliedUnitsDetails[97] or 0)) .. 
				" of " .. tostring(counter.value("# Total Naval Units")) .. "\n^"
			end	

			if flag.value("Canadian units withdrawn") == false then
				messageText = messageText ..
				"Canadian: " .. tostring((alliedUnitsDetails[50] or 0) + (alliedUnitsDetails[51] or 0) + (alliedUnitsDetails[52] or 0) + (alliedUnitsDetails[53] or 0) + (alliedUnitsDetails[54] or 0) + (alliedUnitsDetails[55] or 0)) .. 
				" of " .. tostring(counter.value("# Total Canadian")) .. "\n^"
			end			
			if flag.value("Indian Infantry units arrive") == true then
				messageText = messageText ..
				"Indian: " .. tostring((alliedUnitsDetails[56] or 0) + (alliedUnitsDetails[57] or 0) + (alliedUnitsDetails[58] or 0) + (alliedUnitsDetails[59] or 0)) .. 
				" of " .. tostring(counter.value("# Total Indian")) .. "\n^"
			end	
			if flag.value("New Zealand units arrive") == true then
				messageText = messageText ..
				"New Zealand: " .. tostring((alliedUnitsDetails[60] or 0) + (alliedUnitsDetails[61] or 0) + (alliedUnitsDetails[62] or 0)) .. 
				" of " .. tostring(counter.value("# Total New Zealand")) .. "\n^"
			end			
			if flag.value("South African units arrive") == true then
				messageText = messageText ..
				"South African: " .. tostring((alliedUnitsDetails[63] or 0) + (alliedUnitsDetails[64] or 0) + (alliedUnitsDetails[65] or 0) + (alliedUnitsDetails[66] or 0)) .. 
				" of " .. tostring(counter.value("# Total South African")) .. "\n^"
			end	
			if flag.value("Greek Infantry units arrive") == true then
				messageText = messageText ..
				"Brazilian et Al.: " .. tostring((alliedUnitsDetails[78] or 0) + (alliedUnitsDetails[79] or 0) + (alliedUnitsDetails[80] or 0)) .. 
				" of " .. tostring(counter.value("# Total Brazil et AL")) .. "\n^"
			end	
			if flag.value("Goumier Marocain units arrive") == true or flag.value("Tirailleur Algérien units arrive") == true or flag.value("Free French units arrive") == true then
				messageText = messageText ..
				"Free French: " .. tostring((alliedUnitsDetails[67] or 0) + (alliedUnitsDetails[68] or 0) + (alliedUnitsDetails[69] or 0) + (alliedUnitsDetails[70] or 0) + (alliedUnitsDetails[71] or 0) + (alliedUnitsDetails[72] or 0)) .. 
				" of " .. tostring(counter.value("# Total Free French")) .. "\n^"
			end				
			if flag.value("Polish units arrive") == true then
				messageText = messageText ..
				"Polish: " .. tostring((alliedUnitsDetails[73] or 0) + (alliedUnitsDetails[74] or 0) + (alliedUnitsDetails[75] or 0)) .. 
				" of " .. tostring(counter.value("# Total Polish")) .. "\n^"
			end	

			-- Naval Units: optional display
			if flag.value("Allied Naval unit count") == true then
				messageText = messageText ..
				"Naval Units: " .. tostring((alliedUnitsDetails[89] or 0) + (alliedUnitsDetails[90] or 0) + (alliedUnitsDetails[91] or 0) + (alliedUnitsDetails[92] or 0) + (alliedUnitsDetails[93] or 0) + (alliedUnitsDetails[94] or 0) + (alliedUnitsDetails[95] or 0)) .. 
				" of " .. tostring(counter.value("# Total Naval Units")) .. "\n^"
			end	
			
			messageText = messageText .. ("\n^Note *: You may go to 'Allied Replacement Procurement' menu (key '4')\n^to purchase available new units\n^Note **: Includes Engineer units")
		end
		civ.ui.text(func.splitlines(messageText))
	end


	-- KEY PRESS '3'
	-- ACTIVATES RADAR MENU (see radar.lua)
	
	-- KEY PRESS '4'
	-- GIVES HUMAN PLAYER ACCESS TO REPLACEMENTS MENU (see procurement.lua)
	if keyCode == keyboard.four then
		if flag.value("Allies at War") == true then
			-- procurement.onKeyPress()
			procurement.smallNestedMenu()
		end
	end	

	-- KEY PRESS '5'
	-- GIVES HUMAN PLAYER ACCESS TO COMMAND DECISION MENU (see commandDecision.lua)
	if keyCode == keyboard.five then
		if flag.value("Allies at War") == true then
			commandDecision.smallNestedMenu()
		end
	end	
	
	-- KEY PRESS '6'
	-- MENU FOR GAME TIPS AND SPECIAL SCENARIO RULES (see gameTips.lua)
	if keyCode == keyboard.six then
		if flag.value("Allies at War") == true then
			gameTips.smallNestedMenu()
		end
	end	

	-- KEY PRESS '7'
	-- GIVES DESIGNER ACCESS TO UTILITY TOOL TO TRACK THE CURRENT MAXIMUM ALLOWED UNITS PER TURN (see printparamList.lua)
	if keyCode == keyboard.seven then
		if flag.value("Allies at War") == true then
			printparamList.smallNestedMenu()
		end
	end		
	
	-- KEY PRESS '8'
	-- TESTING/CHECKING POLYGON ZONES
	if keyCode == keyboard.eight then
		-- civ.ui.text("The current tile is in Corsica: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.corsicaPolygon)))
		-- civ.ui.text("The current tile is in Sicily: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.sicilyPolygon)))
		-- civ.ui.text("The current tile is in Sardinia: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.sardiniaPolygon)))
		-- civ.ui.text("The current tile is in Italian Surrender Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.italianSurrenderPolygon)))
		-- civ.ui.text("The current tile is in the Gustav Line Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.gustavLinePolygon)))
		-- civ.ui.text("The current tile is in the Gothic Line Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.gothicLinePolygon)))
		-- civ.ui.text("The current tile is in Southern Italy: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.southernItalyPolygon)))
		-- civ.ui.text("The current tile is in Italy south of Gustav Line: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.italySouthGustavLinePolygon)))
		-- civ.ui.text("The current tile is in the Ortona Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.ortonaPolygon)))
		-- civ.ui.text("The current tile is in the Foggia Reaction Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.foggiaPolygonCoordinates)))
		-- civ.ui.text("Print Licata: " .. tostring(object.ltownLicata.owner))
		
		if (gen.inPolygon(civ.getCurrentTile(),tables.etnaPolygon)) then
			civ.ui.text("The current tile is in the Etna Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.etnaPolygon)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.corsicaPolygon)) then
			civ.ui.text("The current tile is in Corsica: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.corsicaPolygon)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.sardiniaPolygon)) then
			civ.ui.text("The current tile is in Sardinia: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.sardiniaPolygon)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.sicilyPolygon)) then
			civ.ui.text("The current tile is in Sicily: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.sicilyPolygon)))
		-- elseif (gen.inPolygon(civ.getCurrentTile(),tables.massaPolygonCoordinates)) then
			-- civ.ui.text("The current tile is in the Massa Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.massaPolygonCoordinates)))
		-- elseif (gen.inPolygon(civ.getCurrentTile(),tables.riminiPolygonCoordinates)) then
			-- civ.ui.text("The current tile is in the Rimini Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.riminiPolygonCoordinates)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.gothicLinePolygon)) then
			civ.ui.text("The current tile is in the Gothic Line Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.gothicLinePolygon)))
		-- elseif (gen.inPolygon(civ.getCurrentTile(),tables.gothicLineTotalPolygon)) then
			-- civ.ui.text("The current tile is in the Gothic Line Total Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.gothicLineTotalPolygon)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.gustavLinePolygon)) then
			civ.ui.text("The current tile is in the Gustav Line Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.gustavLinePolygon)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.italianSurrenderPolygon2)) then
			civ.ui.text("The current tile is in Italian Surrender Zone 2: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.italianSurrenderPolygon2)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.italySouthGustavLinePolygon)) then
			civ.ui.text("The current tile is in South of Gustav Line Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.italySouthGustavLinePolygon)))
		-- elseif (gen.inPolygon(civ.getCurrentTile(),tables.poValleyPolygon)) then
			-- civ.ui.text("The current tile is in Po Valley Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.poValleyPolygon)))
		elseif (gen.inPolygon(civ.getCurrentTile(),tables.triestePolygon)) then
			civ.ui.text("The current tile is in Trieste Zone: "..tostring(gen.inPolygon(civ.getCurrentTile(),tables.triestePolygon)))
		end
	end	

end
---&endAutoDoc

---&autoDoc onScenarioLoaded
-- On Scenario Loaded
function events.onScenarioLoaded()
    --civ.ui.text("Scenario Loaded consolidated event")

end
---&endAutoDoc

---&autoDoc onNegotiation
-- On Negotiation 
-- Return true if the talker can contact the listener,
-- and false otherwise.
-- This is combined with the discrete events and the
-- legacy events, as well as a separate onNegotiation.lua file
-- If any of these return false, then negotiation is prevented
function events.onNegotiation(talker,listener)

    return false
end
---&endAutoDoc

---&autoDoc onSchism
-- On Schism 
-- Return true (default) if the tribe can schism,
-- and false otherwise.
-- This is combined with the discrete events and the
-- legacy events, as well as a separate onSchism.lua file
-- If any of these return false, then schism is prevented
function events.onSchism(tribe)

    return false
end
---&endAutoDoc

---&autoDoc onTurn
-- Between Turns
function events.onTurn(turn)
    -- civ.ui.text("The turn is "..tostring(turn)..".")


end
---&endAutoDoc

---&autoDoc onUnitKilled
-- On unit killed in combat
function events.onUnitKilled(loser,winner,aggressor,victim,loserLocation,winnerVetStatus,loserVetStatus)
    --civ.ui.text("A "..loser.type.name.." has been defeated by a "..winner.type.name..".")

end
---&endAutoDoc




---&autoDoc onUnitDefeated
-- On unit defeated in combat or by some other event
function events.onUnitDefeated(loser,winner,aggressor,victim,loserLocation,winnerVetStatus,loserVetStatus)
    --civ.ui.text("unit defeated consolidated test")

end
---&endAutoDoc


---&autoDoc onCityProcessingComplete
-- After Production
function events.onCityProcessingComplete(turn,tribe)
    if _global.eventTesting then
        civ.ui.text("consolidated.onCityProcessingComplete for turn "..tostring(turn).." and tribe "..tribe.name)
    end

end
---&endAutoDoc


---&autoDoc onTribeTurnBegin
-- Before Production
function events.onTribeTurnBegin(turn,tribe)
    if _global.eventTesting then
        civ.ui.text("consolidated.onTribeTurnBegin for turn "..tostring(turn).." and tribe "..tribe.name)
    end

end
---&endAutoDoc

---&autoDoc onTribeTurnEnd
function events.onTribeTurnEnd(turn,tribe)
    if _global.eventTesting then
        civ.ui.text("consolidated.onTribeTurnEnd for turn "..tostring(turn).." and tribe "..tribe.name)
    end

end
---&endAutoDoc

---&autoDoc onGameEnds
-- On Game Ends
-- Return true if the game ends as normal,
-- and false otherwise.
-- This is combined with the discrete events and the
-- legacy events, as well as a separate onGameEnds.lua file
-- If any of these return false, then game end is prevented
-- Lua Function Reference Info:
--onGameEnds
--civ.scen.onGameEnds(function (reason) -> boolean) -> void
--
--Registers a function that is called when the game ends. `reason` is an integer between 1 and 6:
--1 and 2 - Space race victory. This does not trigger if `onCentauriArrival` has a callback registered.
--3 - Conquest victory
--4 - Defeat
--5 - Retirement
--6 - Macro ENDGAME action
--Return `true` to end the game, `false` to keep playing.
--
function events.onGameEnds(reason)

    return true
end
---&endAutoDoc

---&autoDoc onCanFoundCity
-- Checking if a unit can found a city
-- Return true if the unit can found a city
-- return false if it can't
-- If any one of the consolidated event, the discrete events, 
-- or the separate file event return false, then the city
-- can't be built
-- Notes: Returning true does NOT override any normal city
-- building condition (like no adjacent cities, or cities at sea)
-- Registers a function that is called to determine if `unit` can found 
-- a city at the unit's location. `advancedTribe` is `true` when picking 
-- up a hut with `unit` triggers an advanced tribe. 
-- Return `true` to allow, `false` to disallow.
function events.onCanFoundCity(unit,advancedTribe)
    if _global.eventTesting then
        civ.ui.text("consolidated.onCanFoundCity for "..unit.type.name)
    end
    return true
end
---&endAutoDoc

---&autoDoc onEnterTile
-- onEnterTile(unit,previousTile,previousDomainSpec)
-- executes when a unit successfully enters a tile (so not when it attacks
-- a unit or fails to enter a tile because it lacks movement points)
function events.onEnterTile(unit,previousTile,previousDomainSpec)
    if _global.eventTesting then
        civ.ui.text("consolidated.onEnterTile: "..unit.type.name.." has entered tile ("..text.coordinates(unit.location)..") from tile ("..text.coordinates(previousTile)..").")
    end
end
---&endAutoDoc

---&autoDoc onFinalOrderGiven
-- onFinalOrderGiven(unit)
-- executes when a unit has been given its final order for the turn.
-- that is, when a new unit is active and the previous unit has spent
-- all its movement points
function events.onFinalOrderGiven(unit)
    if _global.eventTesting then
        civ.ui.text("consolidated.onFinalOrderGiven: "..unit.type.name.." has been given its order.")
    end
end
---&endAutoDoc

-- ===============================================================================
--
--          End of File
--
-- ===============================================================================

-- ==============================
-- DELAYED ACTION FUNCTIONS
-- ==============================

-- ===================================
-- BUILDING DOCKS IN PORT CITIES
-- ===================================

local function DocksBuiltInBari(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Bari and the engineer returns to be assigned for other tasks!","Docks Built In Bari!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{123,125,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{123,125,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Bari")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInBari",DocksBuiltInBari)	

local function DocksBuiltInCatania(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Catania and the engineer returns to be assigned for other tasks!","Docks Built In Catania!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Catania")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInCatania",DocksBuiltInCatania)

local function DocksBuiltInLivorno(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Livorno and the engineer returns to be assigned for other tasks!","Docks Built In Livorno!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Livorno")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInLivorno",DocksBuiltInLivorno)		

local function DocksBuiltInMessina(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Messina and the engineer returns to be assigned for other tasks!","Docks Built In Messina!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{73,155,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{73,155,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Messina")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInMessina",DocksBuiltInMessina)	

local function DocksBuiltInNapoli(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Napoli and the engineer returns to be assigned for other tasks!","Docks Built In Napoli!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Napoli")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInNapoli",DocksBuiltInNapoli)	

local function DocksBuiltInPalermo(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Palermo and the engineer returns to be assigned for other tasks!","Docks Built In Palermo!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Palermo")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInPalermo",DocksBuiltInPalermo)	

local function DocksBuiltInTaranto(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The docks were built in the port city of Taranto and the engineer returns to be assigned for other tasks!","Docks Built In Taranto!",object.mDocksBuilt)
	gen.createUnit(object.uEngineer, object.pAllies, {{120,138,0}}, {count=1, randomize=false, veteran=false})
	gen.createUnit(object.uDocks, object.pAllies, {{120,138,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	flag.setTrue("Docks built in Taranto")
end 
delayedAction.makeFunctionDelayable("BuildingDocksInTaranto",DocksBuiltInTaranto)


-- ===================================
-- REMOVING SEA MINES FROM PORT CITIES
-- ===================================

local function SeaMinesRemovedInBari(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Bari and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Bari!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 123 and unit.location.y >= 123 and unit.location.x <= 125 and unit.location.y <= 127 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{123,125,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Bari")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInBari",SeaMinesRemovedInBari)	

local function SeaMinesRemovedInBrindisi(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Brindisi and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Brindisi!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 130 and unit.location.y >= 140 and unit.location.x <= 132 and unit.location.y <= 144 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{130,142,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Brindisi")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInBrindisi",SeaMinesRemovedInBrindisi)	

local function SeaMinesRemovedInCatania(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Catania and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Catania!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 62 and unit.location.y >= 160 and unit.location.x <= 64 and unit.location.y <= 164 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Catania")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInCatania",SeaMinesRemovedInCatania)	

local function SeaMinesRemovedInGenova(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Genova and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Genova!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 70 and unit.location.y >= 12 and unit.location.x <= 72 and unit.location.y <= 14 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{72,12,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Genova")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInGenova",SeaMinesRemovedInGenova)	

local function SeaMinesRemovedInLaSpezia(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of La Spezia and the engineer returns to be assigned for other tasks!","Sea Mines Removed In LaSpezia!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 75 and unit.location.y >= 23 and unit.location.x <= 77 and unit.location.y <= 25 and unit.location.z == 0)  then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{77,23,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from La Spezia")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInLaSpezia",SeaMinesRemovedInLaSpezia)	

local function SeaMinesRemovedInLivorno(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Livorno and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Livorno!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 73 and unit.location.y >= 33 and unit.location.x <= 75 and unit.location.y <= 37 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Livorno")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInLivorno",SeaMinesRemovedInLivorno)	


local function SeaMinesRemovedInNapoli(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Napoli and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Napoli!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 87 and unit.location.y >= 107 and unit.location.x <= 89 and unit.location.y <= 109 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Napoli")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInNapoli",SeaMinesRemovedInNapoli)	

local function SeaMinesRemovedInPalermo(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Palermo and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Palermo!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 45 and unit.location.y >= 139 and unit.location.x <= 47 and unit.location.y <= 141 and unit.location.z == 0)  then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Palermo")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInPalermo",SeaMinesRemovedInPalermo)	

local function SeaMinesRemovedInPescara(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Pescara and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Pescara!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 107 and unit.location.y >= 81 and unit.location.x <= 109 and unit.location.y <= 85 and unit.location.z == 0)  then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{107,83,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Pescara")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInPescara",SeaMinesRemovedInPescara)	

local function SeaMinesRemovedInRimini(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Rimini and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Rimini!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 107 and unit.location.y >= 43 and unit.location.x <= 109 and unit.location.y <= 45 and unit.location.z == 0) then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{106,44,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Rimini")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInRimini",SeaMinesRemovedInRimini)	

local function SeaMinesRemovedInSiracusa(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Siracusa and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Siracusa!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and unit.location.x == 61 and unit.location.y == 169 and unit.location.z == 0 then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{59,169,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Siracusa")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInSiracusa",SeaMinesRemovedInSiracusa)	

local function SeaMinesRemovedInTrapani(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Trapani and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Trapani!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 30 and unit.location.y >= 134 and unit.location.x <= 34 and unit.location.y <= 136 and unit.location.z == 0)  then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{32,136,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Trapani")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInTrapani",SeaMinesRemovedInTrapani)	

local function SeaMinesRemovedInVenezia(argTable)
	civ.playSound("Cheers2.wav")
	text.simple("The obstacles were removed from the port city of Venezia and the engineer returns to be assigned for other tasks!","Sea Mines Removed In Venezia!",object.monexone)
	for unit in civ.iterateUnits() do
		if unit.owner == object.pAxis and unit.type == object.uSeaMines and (unit.location.x >= 117 and unit.location.y >= 25 and unit.location.x <= 121 and unit.location.y <= 27 and unit.location.z == 0)  then
			civ.deleteUnit(unit)
			-- gen.chartTruthfully(unit.location,unit.owner)
			civ.ui.redrawTile(unit.location) 
		end
	end	
	gen.createUnit(object.uEngineer, object.pAllies, {{119,25,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# Engineer",1)
	counter.add("# Total US Armored",1)
	
	flag.setTrue("Sea Mines removed from Venezia")
end 
delayedAction.makeFunctionDelayable("SeaMinesBeingRemovedInVenezia",SeaMinesRemovedInVenezia)	

-- ===================================
-- BUILDING SUPPLY DEPOT IN PORT CITIES
-- ===================================

local function SupplyDepotBuiltInBari(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Bari and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Bari!",object.mDocksBuilt)
	civ.addImprovement(object.cBari,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{123,125,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Bari")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInBari",SupplyDepotBuiltInBari)		

local function SupplyDepotBuiltInBologna(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Bologna and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Bologna!",object.mDocksBuilt)
	civ.addImprovement(object.cBologna,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{100,30,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Bologna")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInBologna",SupplyDepotBuiltInBologna)		

local function SupplyDepotBuiltInCatania(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Catania and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Catania!",object.mDocksBuilt)
	civ.addImprovement(object.cCatania,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{62,162,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Catania")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInCatania",SupplyDepotBuiltInCatania)	

local function SupplyDepotBuiltInFirenze(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Firenze and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Firenze!",object.mDocksBuilt)
	civ.addImprovement(object.cFirenze,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{89,39,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Firenze")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInFirenze",SupplyDepotBuiltInFirenze)	

local function SupplyDepotBuiltInFoggia(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Foggia and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Foggia!",object.mDocksBuilt)
	civ.addImprovement(object.cFoggia,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{111,107,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Foggia")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInFoggia",SupplyDepotBuiltInFoggia)	

local function SupplyDepotBuiltInGenova(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Genova and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Genova!",object.mDocksBuilt)
	civ.addImprovement(object.cGenova,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{72,12,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Genova")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInGenova",SupplyDepotBuiltInGenova)	

local function SupplyDepotBuiltInLivorno(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Livorno and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Livorno!",object.mDocksBuilt)
	civ.addImprovement(object.cLivorno,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{75,35,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Livorno")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInLivorno",SupplyDepotBuiltInLivorno)	

local function SupplyDepotBuiltInMessina(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Messina and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Messina!",object.mDocksBuilt)
	civ.addImprovement(object.cMessina,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{73,155,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Messina")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInMessina",SupplyDepotBuiltInMessina)	

local function SupplyDepotBuiltInNapoli(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Napoli and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Napoli!",object.mDocksBuilt)
	civ.addImprovement(object.cNapoli,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{88,106,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Napoli")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInNapoli",SupplyDepotBuiltInNapoli)	

local function SupplyDepotBuiltInPalermo(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Palermo and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Palermo!",object.mDocksBuilt)
	civ.addImprovement(object.cPalermo,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{44,140,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Palermo")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInPalermo",SupplyDepotBuiltInPalermo)

local function SupplyDepotBuiltInPescara(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Pescara and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Pescara!",object.mDocksBuilt)
	civ.addImprovement(object.cPescara,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{107,83,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Pescara")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInPescara",SupplyDepotBuiltInPescara)	

local function SupplyDepotBuiltInRimini(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Rimini and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Rimini!",object.mDocksBuilt)
	civ.addImprovement(object.cRimini,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{106,44,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Rimini")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInRimini",SupplyDepotBuiltInRimini)	

local function SupplyDepotBuiltInRoma(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Roma and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Roma!",object.mDocksBuilt)
	civ.addImprovement(object.cROMA,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{80,76,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Roma")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInRoma",SupplyDepotBuiltInRoma)	

local function SupplyDepotBuiltInTaranto(argTable)
	civ.playSound("Bldspcsh.wav")
	text.simple("The Supply Depot was built in the port city of Taranto and the DUKW returns to be assigned for other tasks!","Supply Depot Built In Taranto!",object.mDocksBuilt)
	civ.addImprovement(object.cTaranto,object.iSupplyDepot)
	gen.createUnit(object.uDUKW, object.pAllies, {{120,138,0}}, {count=1, randomize=false, veteran=false})
	counter.add("# DUKW",1)
	counter.add("# Total Naval Units",1)
	flag.setTrue("Supply Depot built in Taranto")
end 
delayedAction.makeFunctionDelayable("BuildingSupplyDepotInTaranto",SupplyDepotBuiltInTaranto)		

gen.versionFunctions(events,versionNumber,fileModified,"consolidatedEvents.lua")
return consolidator.maintainEvents(events)

